@@ -22,7 +22,6 @@ from utils.integration_test_helpers_mixin import IntegrationTestHelpersMixin
 from utils.server import DebusineServer
 from utils.worker import Worker
 
-
 logger = logging.getLogger(__name__)
 
 
@@ -40,7 +39,7 @@ class IntegrationTaskMmDebstrapTests(
 
     TASK_NAME = "mmdebstrap"
 
-    def setUp(self):
+    def setUp(self) -> None:
         """Initialize test."""
         # If debusine-server or nginx was launched just before the
         # integration-tests-task-mmdebstrap.py is launched the debusine-server
@@ -55,7 +54,7 @@ class IntegrationTaskMmDebstrapTests(
 
         self.worker = Worker()
 
-    def test_mmdebstrap(self):
+    def test_mmdebstrap(self) -> None:
         """Create a workflow to run mmdebstrap."""
         # This asserts that the workflow succeeds.
         self.create_system_images_mmdebstrap(["bookworm", "trixie"])
