@@ -43,7 +43,7 @@ class IntegrationTaskPiupartsTests(
 
     TASK_NAME = "piuparts"
 
-    def setUp(self):
+    def setUp(self) -> None:
         """Initialize test."""
         # If debusine-server or nginx was launched just before the
         # integration-tests.py is launched the debusine-server might not be
@@ -60,9 +60,12 @@ class IntegrationTaskPiupartsTests(
             ["dpkg", "--print-architecture"], text=True
         ).strip()
 
-    def test_piuparts_input_source_artifact(self):
+    def test_piuparts_input_source_artifact(self) -> None:
         """Create piuparts job: input is "hello" source package."""
-        upload_artifact_id = self.create_artifact_upload(["hello"])
+        with self.apt_indexes("bookworm") as apt_path:
+            upload_artifact_id = self.create_artifact_upload(
+                apt_path, ["hello"]
+            )
 
         task_data = textwrap.dedent(
             f'''\
