@@ -40,5 +40,5 @@ class Waiter:
 
         return True
 
-    wait_for_success = partialmethod(_wait_for, True)
-    wait_for_failure = partialmethod(_wait_for, False)
+    wait_for_success: partialmethod[bool] = partialmethod(_wait_for, True)
+    wait_for_failure: partialmethod[bool] = partialmethod(_wait_for, False)
