@@ -25,7 +25,7 @@ class Worker:
     TOKEN_FILE = CONFIG_DIRECTORY / 'token'
 
     @classmethod
-    def set_up(cls):
+    def set_up(cls) -> None:
         """
         Set up worker and waits for the token file.
 
@@ -54,7 +54,7 @@ class Worker:
         cls.wait_for_token_file()
 
     @classmethod
-    def wait_for_token_file(cls):
+    def wait_for_token_file(cls) -> bool:
         """
         Wait for the /etc/debusine/worker/token file to appear.
 
@@ -63,7 +63,7 @@ class Worker:
         return Waiter.wait_for_success(30, cls.TOKEN_FILE.exists)
 
     @classmethod
-    def read_token(cls):
+    def read_token(cls) -> str:
         """Return token: contents of cls.TOKEN_FILE."""
         with open(cls.TOKEN_FILE) as token_file:
             return token_file.read()
