@@ -8,11 +8,12 @@
 # contained in the LICENSE file.
 """Debusine client."""
 
-from debusine.artifacts.local_artifact import (  # noqa: F401
+from debusine.artifacts.local_artifact import (
     AutopkgtestArtifact,
     BinaryPackage,
     BinaryPackages,
     BlhcArtifact,
+    DebDiffArtifact,
     DebianSystemTarballArtifact,
     LintianArtifact,
     LocalArtifact,
@@ -24,3 +25,21 @@ from debusine.artifacts.local_artifact import (  # noqa: F401
     Upload,
     WorkRequestDebugLogs,
 )
+
+__all__ = [
+    "AutopkgtestArtifact",
+    "BinaryPackage",
+    "BinaryPackages",
+    "BlhcArtifact",
+    "DebDiffArtifact",
+    "DebianSystemTarballArtifact",
+    "LintianArtifact",
+    "LocalArtifact",
+    "PackageBuildLog",
+    "SigningInputArtifact",
+    "SigningKeyArtifact",
+    "SigningOutputArtifact",
+    "SourcePackage",
+    "Upload",
+    "WorkRequestDebugLogs",
+]
