@@ -20,7 +20,7 @@ from collections.abc import Iterable, Sequence
 from datetime import datetime
 from json import JSONDecodeError
 from pathlib import Path
-from typing import Any, ClassVar, Generic, Self, TypeVar
+from typing import Any, ClassVar, Generic, Self, TypeVar, cast, overload
 
 from debian import deb822, debfile
 
@@ -77,7 +77,7 @@ class LocalArtifact(pydantic.BaseModel, Generic[AD], abc.ABC):
         str, type["LocalArtifact['Any']"]
     ] = {}
 
-    def __init__(self, **kwargs):
+    def __init__(self, **kwargs: Any) -> None:
         """Set category to _category by default."""
         kwargs.setdefault("category", self._category)
         if "data" in kwargs:
@@ -85,7 +85,7 @@ class LocalArtifact(pydantic.BaseModel, Generic[AD], abc.ABC):
                 kwargs["data"] = self.create_data(d)
         super().__init__(**kwargs)
 
-    def __init_subclass__(cls, **kwargs):
+    def __init_subclass__(cls, **kwargs: Any) -> None:
         """
         Register subclass into LocalArtifact._local_artifacts_category_to_class.
 
@@ -102,7 +102,7 @@ class LocalArtifact(pydantic.BaseModel, Generic[AD], abc.ABC):
     @classmethod
     def create_data(cls, data_dict: dict[str, Any]) -> AD:
         """Instantiate a data model from a dict."""
-        return cls._data_type(**data_dict)
+        return cast(AD, cls._data_type(**data_dict))
 
     @staticmethod
     def artifact_categories() -> list[str]:
@@ -128,7 +128,7 @@ class LocalArtifact(pydantic.BaseModel, Generic[AD], abc.ABC):
 
     def _check_existing_file(
         self, path_in_artifact: str, new_file: Path | FileRequest
-    ):
+    ) -> None:
         """Check whether the artifact already contains a given file."""
         all_files = self.files | self.remote_files
         if path_in_artifact in all_files:
@@ -144,7 +144,7 @@ class LocalArtifact(pydantic.BaseModel, Generic[AD], abc.ABC):
         *,
         artifact_base_dir: Path | None = None,
         override_name: str | None = None,
-    ):
+    ) -> None:
         """
         Add a local file in the artifact.
 
@@ -209,7 +209,7 @@ class LocalArtifact(pydantic.BaseModel, Generic[AD], abc.ABC):
         self._check_existing_file(path_in_artifact, file_request)
         self.remote_files[path_in_artifact] = file_request
 
-    def validate_model(self):
+    def validate_model(self) -> None:
         """Raise ValueError with an error if the model is not valid."""
         *_, error = pydantic.validate_model(self.__class__, self.__dict__)
 
@@ -369,7 +369,17 @@ class PackageBuildLog(LocalArtifact[data_models.DebianPackageBuildLog]):
         return super()._validate_files_end_in(files, [".build"])
 
 
-def deb822dict_to_dict(element):
+@overload
+def deb822dict_to_dict(
+    element: dict[Any, Any] | deb822.Deb822Dict
+) -> dict[Any, Any]: ...
+
+
+@overload
+def deb822dict_to_dict(element: Any) -> Any: ...
+
+
+def deb822dict_to_dict(element: Any) -> Any:
     """
     Traverse recursively element converting Deb822Dict to dict.
 
@@ -380,6 +390,7 @@ def deb822dict_to_dict(element):
     Reason is to simplify is that json module cannot encode Deb822Dict. To
     simplify let's convert Deb822Dict as soon as possible to dict.
     """
+    result: Any
     if isinstance(element, (dict, deb822.Deb822Dict)):
         result = {}
         for key, value in element.items():
@@ -487,7 +498,10 @@ class SourcePackage(LocalArtifact[data_models.DebianSourcePackage]):
         dsc_fields = {}
         for file in files:
             if file.suffix == ".dsc":
-                dsc_fields = deb822dict_to_dict(utils.read_dsc(file))
+                dsc = utils.read_dsc(file)
+                if dsc is None:
+                    raise ValueError(f"{file} is not a valid .dsc file")
+                dsc_fields = deb822dict_to_dict(dsc)
 
         data = data_models.DebianSourcePackage(
             name=name,
@@ -505,19 +519,22 @@ class SourcePackage(LocalArtifact[data_models.DebianSourcePackage]):
 
     @pydantic.validator("files")
     @classmethod
-    def files_contain_one_dsc(cls, files: dict[str, Path]):
+    def files_contain_one_dsc(cls, files: dict[str, Path]) -> dict[str, Path]:
         """Raise ValueError when files does not have exactly 1 .dsc file."""
         return cls._validate_exactly_one_file_ends_in(files, ".dsc")
 
     @pydantic.validator("files")
     @classmethod
-    def files_contains_files_in_dsc(cls, files: dict[str, Path]):
+    def files_contains_files_in_dsc(
+        cls, files: dict[str, Path]
+    ) -> dict[str, Path]:
         """
         Validate that set(files) == set(files_in_dsc_file).
 
         Exception: The .dsc file must be in files but not in the .dsc file.
         """
-        return files_in_meta_file_match_files(".dsc", deb822.Dsc, files)
+        files_in_meta_file_match_files(".dsc", deb822.Dsc, files)
+        return files
 
 
 class BinaryPackage(LocalArtifact[data_models.DebianBinaryPackage]):
@@ -772,6 +789,23 @@ class BlhcArtifact(LocalArtifact[data_models.EmptyArtifactData]):
         return artifact
 
 
+class DebDiffArtifact(LocalArtifact[data_models.DebDiff]):
+    """DebDiffArtifact: encapsulate result of the debdiff run."""
+
+    _category = data_models.ArtifactCategory.DEBDIFF
+
+    @classmethod
+    def create(cls, debdiff_output: Path, original: str, new: str) -> Self:
+        """Return a DebDiffArtifact with the files set."""
+        data = data_models.DebDiff(original=original, new=new)
+
+        artifact = cls(category=cls._category, data=data)
+
+        artifact.add_local_file(debdiff_output, override_name="debdiff.txt")
+
+        return artifact
+
+
 class DebianSystemImageArtifact(LocalArtifact[data_models.DebianSystemImage]):
     """
     Contains a image.tar.xz file with a bootable Debian system.
@@ -849,10 +883,14 @@ class SigningInputArtifact(LocalArtifact[data_models.DebusineSigningInput]):
         files: Iterable[Path],
         base_dir: Path,
         trusted_certs: list[str] | None = None,
+        binary_package_name: str | None = None,
     ) -> Self:
         """Return a new SigningInputArtifact."""
         data = data_models.DebusineSigningInput(
-            trusted_certs=None if trusted_certs is None else list(trusted_certs)
+            trusted_certs=(
+                None if trusted_certs is None else list(trusted_certs)
+            ),
+            binary_package_name=binary_package_name,
         )
         artifact = cls(category=cls._category, data=data)
         for file in files:
@@ -861,7 +899,9 @@ class SigningInputArtifact(LocalArtifact[data_models.DebusineSigningInput]):
 
     @pydantic.validator("files")
     @classmethod
-    def validate_at_least_one_file(cls, files: dict[str, Path]):
+    def validate_at_least_one_file(
+        cls, files: dict[str, Path]
+    ) -> dict[str, Path]:
         """Validate that artifact has at least one file."""
         if not files:
             raise ValueError("Expected at least one file")
