@@ -43,7 +43,9 @@ class DebusinePromise(pydantic.BaseModel):
 
     @pydantic.root_validator(pre=True)
     @classmethod
-    def check_no_extra_names_starting_promise(cls, values):
+    def check_no_extra_names_starting_promise(
+        cls, values: dict[str, Any]
+    ) -> dict[str, Any]:
         r"""Forbid extra field names starting with 'promise\\_'."""
         allowed_promise_fields = cls.__fields__.keys()
 
@@ -70,6 +72,7 @@ class ArtifactCategory(enum.StrEnum):
     AUTOPKGTEST = "debian:autopkgtest"
     SYSTEM_TARBALL = "debian:system-tarball"
     BLHC = "debian:blhc"
+    DEBDIFF = "debian:debdiff"
     SYSTEM_IMAGE = "debian:system-image"
     SIGNING_KEY = "debusine:signing-key"
     SIGNING_INPUT = "debusine:signing-input"
@@ -88,11 +91,18 @@ class CollectionCategory(enum.StrEnum):
     SUITE_SIGNING_KEYS = "debian:suite-signing-keys"
     PACKAGE_BUILD_LOGS = "debian:package-build-logs"
     WORKFLOW_INTERNAL = "debusine:workflow-internal"
+    TASK_HISTORY = "debusine:task-history"
 
     # Only implemented in tests.
     TEST = "debusine:test"
 
 
+SINGLETON_COLLECTION_CATEGORIES = {
+    CollectionCategory.PACKAGE_BUILD_LOGS,
+    CollectionCategory.TASK_HISTORY,
+}
+
+
 class BaseArtifactDataModel(pydantic.BaseModel):
     """Base pydantic model for artifact data and their components."""
 
@@ -102,7 +112,7 @@ class BaseArtifactDataModel(pydantic.BaseModel):
         validate_assignment = True
         extra = pydantic.Extra.forbid
 
-    def dict(self, **kwargs) -> dict[str, Any]:
+    def dict(self, **kwargs: Any) -> dict[str, Any]:
         """Use aliases by default when serializing."""
         kwargs.setdefault("by_alias", True)
         return super().dict(**kwargs)
@@ -266,8 +276,11 @@ class DebianUpload(ArtifactData):
         if files := self.changes_fields.get("Files"):
             for file in files:
                 if (name := file["name"]).endswith(".changes"):
-                    return name
-            return files[0]["name"]
+                    break
+            else:
+                name = files[0]["name"]
+            assert isinstance(name, str)
+            return name
         return None
 
     @staticmethod
@@ -282,7 +295,7 @@ class DebianUpload(ArtifactData):
 
     @pydantic.validator("changes_fields")
     @classmethod
-    def metadata_mandatory_fields(cls, data: dict[str, Any]):
+    def metadata_mandatory_fields(cls, data: dict[str, Any]) -> dict[str, Any]:
         """Validate that changes_fields contains Architecture and Files."""
         if "Architecture" not in data:
             raise ValueError("changes_fields must contain Architecture")
@@ -292,7 +305,9 @@ class DebianUpload(ArtifactData):
 
     @pydantic.validator("changes_fields")
     @classmethod
-    def metadata_contains_debs_if_binary(cls, data: dict[str, Any]):
+    def metadata_contains_debs_if_binary(
+        cls, data: dict[str, Any]
+    ) -> dict[str, Any]:
         """
         Validate that binary uploads reference binaries.
 
@@ -319,7 +334,9 @@ class DebianUpload(ArtifactData):
 
     @pydantic.validator("changes_fields")
     @classmethod
-    def metadata_contains_dsc_if_source(cls, data: dict[str, Any]):
+    def metadata_contains_dsc_if_source(
+        cls, data: dict[str, Any]
+    ) -> dict[str, Any]:
         """
         Validate that source uploads contain one and only one source package.
 
@@ -513,6 +530,7 @@ class DebusineSigningInput(ArtifactData):
     """Input to a Sign task."""
 
     trusted_certs: list[str] | None = None
+    binary_package_name: str | None = None
 
     def get_label(self) -> None:
         """Return a short human-readable label for the artifact."""
@@ -528,7 +546,7 @@ class SigningResult(BaseArtifactDataModel):
 
     @pydantic.root_validator(allow_reuse=True)
     @classmethod
-    def check_exactly_one_result(cls, values):
+    def check_exactly_one_result(cls, values: dict[str, Any]) -> dict[str, Any]:
         """Ensure exactly one of output_file and error_message is present."""
         if (values.get("output_file") is None) == (
             values.get("error_message") is None
@@ -549,3 +567,14 @@ class DebusineSigningOutput(ArtifactData):
     def get_label(self) -> None:
         """Return a short human-readable label for the artifact."""
         return None
+
+
+class DebDiff(ArtifactData):
+    """Data for debian:debdiff artifacts."""
+
+    original: str
+    new: str
+
+    def get_label(self) -> str:
+        """Return a short human-readable label for the artifact."""
+        return f"debdiff {self.original} {self.new}"
