@@ -9,12 +9,14 @@
 
 """Infrastructure to create mock artifact data."""
 
+import email.utils
 import hashlib
 import re
 import subprocess
 import tempfile
 import textwrap
 from collections.abc import Set
+from datetime import UTC, datetime
 from pathlib import Path, PurePath
 from typing import Any
 
@@ -24,6 +26,7 @@ from debusine.artifacts.local_artifact import (
     BinaryPackage,
     PackageBuildLog,
     SourcePackage,
+    Upload,
     deb822dict_to_dict,
 )
 
@@ -37,7 +40,9 @@ class ArtifactPlayground:
         destdir: Path,
         *,
         name: str = "hello",
+        binaries: list[str] | None = None,
         version: str = "1.0-1",
+        architectures: set[str] | None = None,
     ) -> SourcePackage:
         """Create a SourcePackage artifact."""
         dsc = destdir / f"{name}_{version}.dsc"
@@ -49,9 +54,15 @@ class ArtifactPlayground:
         else:
             sources.append(destdir / f"{name}_{version}.tar.gz")
         for source in sources:
-            (destdir / source).write_bytes(source.name.encode())
+            source.write_bytes(source.name.encode())
         cls.write_deb822_file(
-            deb822.Dsc, dsc, sources, source=name, version=version
+            deb822.Dsc,
+            dsc,
+            sources,
+            source=name,
+            binaries=binaries,
+            version=version,
+            architectures=architectures,
         )
         return SourcePackage.create(
             name=name, version=version, files=[dsc] + sources
@@ -71,6 +82,70 @@ class ArtifactPlayground:
         cls.write_deb_file(deb, source_name=name, source_version=version)
         return BinaryPackage.create(file=deb)
 
+    @classmethod
+    def create_upload(
+        cls,
+        destdir: Path,
+        *,
+        source: bool = True,
+        src_name: str = "hello",
+        version: str = "1.0-1",
+        binary: bool = False,
+        binaries: list[str] | None = None,
+        bin_architecture: str = "all",
+    ) -> Upload:
+        """Create an Upload artifact."""
+        architectures: set[str] = set()
+        files: list[Path] = []
+        if binaries is None:
+            binaries = [src_name]
+        if source:
+            architectures.add("source")
+            dsc = destdir / f"{src_name}_{version}.dsc"
+            src_files: list[Path] = [dsc]
+            v_parts = version.split("-", 1)
+            if len(v_parts) == 2:
+                src_files.append(
+                    destdir / f"{src_name}_{version}.debian.tar.xz"
+                )
+                src_files.append(
+                    destdir / f"{src_name}_{v_parts[0]}.orig.tar.gz"
+                )
+            else:
+                src_files.append(destdir / f"{src_name}_{version}.tar.gz")
+            for src_file in src_files:
+                src_file.write_bytes(src_file.name.encode())
+            cls.write_deb822_file(
+                deb822.Dsc,
+                dsc,
+                src_files,
+                source=src_name,
+                binaries=binaries,
+                version=version,
+            )
+            files += src_files
+        if binary:
+            architectures.add(bin_architecture)
+            for bin_name in binaries:
+                deb = destdir / f"{bin_name}_{version}_{bin_architecture}.deb"
+                cls.write_deb_file(
+                    deb, source_name=src_name, source_version=version
+                )
+                files.append(deb)
+        else:
+            binaries = None
+        arch_tag = "+".join(sorted(architectures))
+        changes = destdir / f"{src_name}_{version}_{arch_tag}.changes"
+        cls.write_deb822_file(
+            deb822.Changes,
+            changes,
+            files,
+            source=src_name,
+            binaries=binaries,
+            version=version,
+        )
+        return Upload.create(changes_file=changes)
+
     @classmethod
     def create_package_build_log(
         cls,
@@ -176,16 +251,20 @@ class ArtifactPlayground:
         path: Path,
         files: list[Path],
         *,
+        format_version: str | None = None,
         source: str = "hello-traditional",
+        binaries: list[str] | None = None,
         version: str,
+        architectures: set[str] | None = None,
         binnmu: bool = False,
         maintainer: str = "Example Maintainer <example@example.org>",
     ) -> dict[str, Any]:
         """Write dsc or changes file with files information."""
-        architectures = set()
+        architectures = set(architectures or set())
         match file_type:
             case deb822.Dsc:
                 architectures.add("any")
+                format_version = format_version or "3.0 (quilt)"
             case deb822.Changes:
                 for file in files:
                     name = file.name
@@ -194,6 +273,7 @@ class ArtifactPlayground:
                     elif name.endswith((".deb", ".udeb")):
                         if m := re.match(r".*_([a-z0-9-]+).u?deb$", name):
                             architectures.add(m.group(1))
+                format_version = format_version or "1.8"
             case _ as unreachable:
                 raise NotImplementedError(
                     f"{unreachable!r} not supported:"
@@ -204,26 +284,62 @@ class ArtifactPlayground:
             source += f" ({version})"
             version += "+b1"
 
-        changes_contents = (
-            textwrap.dedent(
-                f"""\
-                Format: 3.0 (quilt)
-                Source: {source}
-                Binary: {source}
-                Architecture: {' '.join(sorted(architectures))}
-                Version: {version}
-                Maintainer: {maintainer}
-                Homepage: http://www.gnu.org/software/{source}/
-                Standards-Version: 4.3.0
-                Package-List:
-                 {source} deb devel optional arch=any
-                """
-            )
-            + cls.hash_deb822_files(
-                file_type, {file.name: file.read_bytes() for file in files}
-            )
+        changes_contents = textwrap.dedent(
+            f"""\
+            Format: {format_version}
+            Source: {source}
+            Architecture: {' '.join(sorted(architectures))}
+            Version: {version}
+            Maintainer: {maintainer}
+            """
+        )
+        match file_type:
+            case deb822.Dsc:
+                if binaries is None:
+                    binaries = [source]
+                changes_contents += textwrap.dedent(
+                    f"""\
+                    Binary: {', '.join(binaries)}
+                    Homepage: http://www.gnu.org/software/{source}/
+                    Standards-Version: 4.3.0
+                    Package-List:
+                     {source} deb devel optional arch=any
+                    """
+                )
+            case deb822.Changes:
+                timestamp = datetime(
+                    2024, 12, 1, 0, 0, 0, tzinfo=UTC
+                ).timestamp()
+                changes_contents += textwrap.dedent(
+                    f"""\
+                    Date: {email.utils.formatdate(timestamp)}
+                    Distribution: unstable
+                    Urgency: medium
+                    Changes:
+                      {source} ({version}) unstable; urgency=medium
+                      .
+                        * Test upload.
+                    """
+                )
+                if binaries:
+                    changes_contents += (
+                        f"Binary: {' '.join(binaries)}\n"
+                        + "Description:\n"
+                        + "\n".join(
+                            f" {binary} - A Description" for binary in binaries
+                        )
+                        + "\n"
+                    )
+                # bookworm's coverage.py seems to get confused here
+                pass  # pragma: no cover
+            case _ as unreachable:
+                raise NotImplementedError(
+                    f"{unreachable!r} not supported:"
+                    " only Dsc and Changes are supported"
+                )
+        changes_contents += cls.hash_deb822_files(
+            file_type, {file.name: file.read_bytes() for file in files}
         )
-
         path.write_text(changes_contents)
 
         return deb822dict_to_dict(file_type(path.read_bytes()))
