@@ -29,6 +29,7 @@ from debusine.artifacts import (
     BinaryPackage,
     BinaryPackages,
     BlhcArtifact,
+    DebDiffArtifact,
     LintianArtifact,
     LocalArtifact,
     PackageBuildLog,
@@ -606,7 +607,6 @@ class UploadTests(TestCase):
             "debusine.artifacts.local_artifact.files_in_meta_file_match_files"
         )
         mocked = patcher.start()
-        mocked.return_value = files
         self.addCleanup(patcher.stop)
 
         self.assertEqual(
@@ -905,6 +905,14 @@ class SourcePackageTests(TestCase):
         self.assertEqual(package.files, expected_files)
         self.assertEqual(package.data, expected_data)
 
+    def test_create_invalid_dsc_raise_value_error(self) -> None:
+        """Raise ValueError: invalid .dsc file."""
+        file = self.create_temporary_file(suffix=".dsc")
+
+        expected_message = fr"{file.name} is not a valid \.dsc file"
+        with self.assertRaisesRegex(ValueError, expected_message):
+            SourcePackage.create(name="hello", version="1.0", files=[file])
+
     def test_files_no_contain_dsc_raise_value_error(self) -> None:
         """Raise ValueError: missing expected .dsc file."""
         file = self.create_temporary_file()
@@ -932,18 +940,15 @@ class SourcePackageTests(TestCase):
 
     def test_files_contains_files_in_dsc_call_implementation(self) -> None:
         """Test files_contains_files_in_dsc call the utils method."""
-        result = {"a": "b"}
+        files = {"a": Path("b")}
         patcher = mock.patch(
             "debusine.artifacts.local_artifact.files_in_meta_file_match_files"
         )
         mocked = patcher.start()
-        mocked.return_value = result
         self.addCleanup(patcher.stop)
 
-        files: dict[str, Path] = {}
-
         self.assertEqual(
-            SourcePackage.files_contains_files_in_dsc(files), result
+            SourcePackage.files_contains_files_in_dsc(files), files
         )
 
         mocked.assert_called_with(".dsc", deb822.Dsc, files)
@@ -1299,6 +1304,64 @@ class BlhcArtifactTests(TestCase):
         )
 
 
+class DebDiffArtifactTests(TestCase):
+    """Tests for DebDiffArtifact class."""
+
+    output_content: ClassVar[bytes]
+
+    @classmethod
+    def setUpClass(cls) -> None:
+        """Set up common data for the tests."""
+        cls.output_content = (
+            b"diff -Nru a/debian/patches/series b/debian/patches/series"
+            b"--- a/debian/patches/series       2024-03-23 10:37:14.00000 +0100"
+            b"+++ b/debian/patches/series       2024-08-22 12:58:30.00000 +0200"
+            b"@@ -5,3 +5,4 @@"
+            b" fix-ftbfs-with-gcc-12.patch"
+            b" fix-aptitude-changelog-parser.patch"
+            b" fix-ftbfs-with-t64.patch"
+            b"+0008-Add-missing-include-to-build-with-gcc-14.patch"
+        )
+
+    def test_create(self) -> None:
+        """Test _create()."""
+        output = self.create_temporary_file(
+            suffix=".txt", contents=self.output_content
+        )
+
+        package = DebDiffArtifact.create(
+            debdiff_output=output, original="original", new="new"
+        )
+
+        self.assertIsInstance(package, DebDiffArtifact)
+        self.assertEqual(
+            package.data, data_models.DebDiff(original="original", new="new")
+        )
+
+        self.assertEqual(
+            package.files,
+            {
+                "debdiff.txt": output,
+            },
+        )
+
+    def test_json_serializable(self) -> None:
+        """Artifact returned by create is JSON serializable."""
+        # Create file for the artifact
+        debdiff_output = self.create_temporary_file(
+            suffix=".txt", contents=b"debdiff a.deb b.deb\n"
+        )
+
+        artifact = DebDiffArtifact.create(
+            debdiff_output=debdiff_output, original="original", new="new"
+        )
+
+        # No exception is raised
+        json.dumps(
+            artifact.serialize_for_create_artifact(workspace="some-workspace")
+        )
+
+
 class DebianSystemImageArtifactTests(TestCase):
     """Tests for DebianSystemImageArtifactTests."""
 
@@ -1442,7 +1505,9 @@ class SigningInputArtifactTests(TestCase):
         artifact = SigningInputArtifact.create([unsigned], base_dir)
 
         artifact.validate_model()
-        self.assertEqual(artifact.data, {"trusted_certs": None})
+        self.assertEqual(
+            artifact.data, {"trusted_certs": None, "binary_package_name": None}
+        )
         self.assertEqual(artifact.files, {"package/dir/file": unsigned})
 
     def test_create_trusted_certs(self) -> None:
@@ -1460,7 +1525,10 @@ class SigningInputArtifactTests(TestCase):
         )
 
         artifact.validate_model()
-        self.assertEqual(artifact.data, {"trusted_certs": ["0" * 64]})
+        self.assertEqual(
+            artifact.data,
+            {"trusted_certs": ["0" * 64], "binary_package_name": None},
+        )
         self.assertEqual(
             artifact.files,
             {
@@ -1469,6 +1537,24 @@ class SigningInputArtifactTests(TestCase):
             },
         )
 
+    def test_create_binary_package_name(self) -> None:
+        """Create a valid SigningInputArtifact with a binary package name."""
+        base_dir = self.create_temporary_directory()
+        unsigned = base_dir / "package" / "dir" / "file"
+        unsigned.parent.mkdir(parents=True)
+        unsigned.write_text("contents")
+
+        artifact = SigningInputArtifact.create(
+            [unsigned], base_dir, binary_package_name="hello"
+        )
+
+        artifact.validate_model()
+        self.assertEqual(
+            artifact.data,
+            {"trusted_certs": None, "binary_package_name": "hello"},
+        )
+        self.assertEqual(artifact.files, {"package/dir/file": unsigned})
+
     def test_create_no_files(self) -> None:
         """A SigningInputArtifact must have at least one file."""
         base_dir = self.create_temporary_directory()
