@@ -519,3 +519,17 @@ class DebusineSigningOutputTest(TestCase):
         )
         label = data.get_label()  # type: ignore[func-returns-value]
         self.assertIsNone(label)
+
+
+class DebDiffTest(TestCase):
+    """Test the DebDiff model."""
+
+    def test_get_label(self) -> None:
+        """get_label returns the debdiff command."""
+        data = data_models.DebDiff(
+            original="original-package",
+            new="new-package",
+        )
+        self.assertEqual(
+            data.get_label(), "debdiff original-package new-package"
+        )
