@@ -9,18 +9,16 @@
 
 """Unit tests for utils.py."""
 
-from unittest import TestCase
-
 from debian import deb822
 from debusine.artifacts import utils
-from debusine.test import TestHelpersMixin
+from debusine.test import TestCase
 
 
-class UtilsTests(TestHelpersMixin, TestCase):
+class UtilsTests(TestCase):
     """Tests for functions in utils.py."""
 
-    def test_files_in_meta_file_match_files_return_files(self) -> None:
-        """Files in files match files in .dsc: return files."""
+    def test_files_in_meta_file_match_files_return_None(self) -> None:
+        """Files in files match files in .dsc: return None."""
         file_1 = self.create_temporary_file()
 
         dsc_file = self.create_temporary_file(suffix=".dsc")
@@ -32,10 +30,8 @@ class UtilsTests(TestHelpersMixin, TestCase):
             dsc_file.name: dsc_file,
         }
 
-        self.assertEqual(
-            utils.files_in_meta_file_match_files(".dsc", deb822.Dsc, files),
-            files,
-        )
+        # Does not raise an exception.
+        utils.files_in_meta_file_match_files(".dsc", deb822.Dsc, files)
 
     def test_files_in_meta_file_extra_file_in_files_raise_value_error(
         self,
