@@ -9,16 +9,18 @@
 
 """Utils for the artifacts module."""
 
+from collections.abc import Mapping
 from pathlib import Path
 
 from debian import deb822 as deb822
+from debusine.client.models import FileRequest
 
 
 def files_in_meta_file_match_files(
     metafile_extension: str,
     metafile_reader: type[deb822.Deb822],
-    files: dict[str, Path],
-):
+    files: Mapping[str, Path | FileRequest],
+) -> None:
     """
     Raise ValidationError if files in the metafile do not match files.
 
@@ -32,9 +34,10 @@ def files_in_meta_file_match_files(
     """
     metafile: Path | None = None
 
-    # Find the .dsc file.
+    # Find the metafile.
     for file_name, file_path in files.items():
         if file_name.endswith(metafile_extension):
+            assert isinstance(file_path, Path)
             metafile = file_path
             break
     # Always available because a previous validator verified it.
@@ -61,5 +64,3 @@ def files_in_meta_file_match_files(
             f"Files: {sorted(files.keys())} "
             f"Listed in {metafile_extension}: {sorted(files_in_metafile)}"
         )
-
-    return files
