@@ -14,7 +14,7 @@ import sys
 from debusine.client.cli import Cli
 
 
-def entry_point():
+def entry_point() -> None:
     """Entry point for the packaging."""
     main = Cli(sys.argv[1:])
     main.execute()
