@@ -20,12 +20,14 @@ import signal
 import subprocess
 import sys
 import tempfile
-from collections.abc import Callable
+import types
+from collections.abc import Callable, Generator
+from contextlib import contextmanager
 from datetime import datetime
 from functools import partial
 from itertools import islice
 from pathlib import Path
-from typing import Any
+from typing import Any, NoReturn
 
 import yaml
 from dateutil.parser import isoparse
@@ -46,6 +48,7 @@ from debusine.client.debusine import Debusine
 from debusine.client.exceptions import DebusineError
 from debusine.client.models import (
     CreateWorkflowRequest,
+    RelationType,
     WorkRequestExternalDebsignRequest,
     WorkRequestRequest,
     WorkRequestResponse,
@@ -64,16 +67,16 @@ class Cli:
         main.execute()
     """
 
-    def __init__(self, argv):
+    def __init__(self, argv: list[str]) -> None:
         """Initialize object."""
         self._argv = argv
 
     @staticmethod
-    def _exit(signum, frame):
+    def _exit(signum: int, frame: types.FrameType | None) -> NoReturn:
         signum, frame  # fake usage for vulture
         raise SystemExit(0)
 
-    def _parse_args(self):
+    def _parse_args(self) -> None:
         """Parse argv and store results in self.args."""
         parser = argparse.ArgumentParser(
             prog='debusine',
@@ -89,6 +92,14 @@ class Cli:
             ),
         )
 
+        parser.add_argument(
+            '--scope',
+            help=(
+                'Set scope to be used (use configuration file default '
+                'if not specified)'
+            ),
+        )
+
         parser.add_argument(
             '--config-file',
             default=ConfigHandler.DEFAULT_CONFIG_FILE_PATH,
@@ -355,7 +366,7 @@ class Cli:
 
         self.args = parser.parse_args(self._argv)
 
-    def _build_debusine_object(self):
+    def _build_debusine_object(self) -> Debusine:
         """Return the debusine object matching the command line parameters."""
         configuration = ConfigHandler(
             server_name=self.args.server,
@@ -378,11 +389,12 @@ class Cli:
 
         return Debusine(
             base_api_url=server_configuration['api-url'],
+            scope=self.args.scope or server_configuration['scope'],
             api_token=server_configuration['token'],
             logger=logger,
         )
 
-    def _setup_http_logging(self):
+    def _setup_http_logging(self) -> None:
         if self.args.debug:
             """Do setup urllib3&requests traces."""
             # https://docs.python-requests.org/en/latest/api/#api-changes
@@ -406,12 +418,12 @@ class Cli:
             requests_log.setLevel(logging.DEBUG)
             requests_log.addHandler(sh)
 
-            def print_to_log(*args):
+            def print_to_log(*args: str) -> None:
                 requests_log.debug(" ".join(args))
 
-            http.client.print = print_to_log
+            setattr(http.client, "print", print_to_log)
 
-    def execute(self):
+    def execute(self) -> None:
         """Execute the command requested by the user."""
         signal.signal(signal.SIGINT, self._exit)
         signal.signal(signal.SIGTERM, self._exit)
@@ -496,7 +508,7 @@ class Cli:
                     f"Unexpected sub-command name: {unreachable}"
                 )
 
-    def execute_create_work_request(self, debusine):
+    def execute_create_work_request(self, debusine: Debusine) -> None:
         """Execute the work request action requested by the user."""
         data = self.args.data.read()
         self.args.data.close()
@@ -517,39 +529,38 @@ class Cli:
         )
 
     @staticmethod
-    def _print_yaml(dictionary):
-        """Print dictionary to stdout as yaml."""
-        output = yaml.safe_dump(dictionary, sort_keys=False, width=math.inf)
+    def _print_yaml(data: Any) -> None:
+        """Print data to stdout as yaml."""
+        output = yaml.safe_dump(data, sort_keys=False, width=math.inf)
         print(output, end="")
 
     @staticmethod
-    def _fail(error, *, summary=None):
+    def _fail(error: object, *, summary: str | None = None) -> NoReturn:
         print(error, file=sys.stderr)
         if summary is not None:
             print(summary, file=sys.stderr)
         raise SystemExit(3)
 
-    def _list_work_requests(self, debusine: Debusine, limit: int):
+    def _list_work_requests(self, debusine: Debusine, limit: int) -> None:
         """List up to limit most recent work requests."""
-        work_requests = self._api_call_or_fail(
-            lambda: list(islice(debusine.work_request_iter(), limit))
-        )
+        with self._api_call_or_fail():
+            work_requests = list(islice(debusine.work_request_iter(), limit))
         result = {
             "results": [work_request.dict() for work_request in work_requests]
         }
         self._print_yaml(result)
 
-    def _show_work_request(self, debusine, work_request_id):
+    def _show_work_request(
+        self, debusine: Debusine, work_request_id: int
+    ) -> None:
         """Print the task information for work_request_id."""
-        work_request = self._api_call_or_fail(
-            debusine.work_request_get, work_request_id
-        )
+        with self._api_call_or_fail():
+            work_request = debusine.work_request_get(work_request_id)
 
         artifacts_information = []
         for artifact_id in work_request.artifacts:
-            artifact_information = self._api_call_or_fail(
-                debusine.artifact_get, artifact_id
-            )
+            with self._api_call_or_fail():
+                artifact_information = debusine.artifact_get(artifact_id)
             artifacts_information.append(
                 model_to_json_serializable_dict(artifact_information)
             )
@@ -564,23 +575,25 @@ class Cli:
         debusine: Debusine,
         task_name: str,
         workspace: str,
-        task_data,
-        event_reactions,
-    ):
-        task_data = self._parse_yaml_data(task_data)
-        event_reactions = self._parse_yaml_data(event_reactions)
+        task_data: str,
+        event_reactions: str,
+    ) -> None:
+        parsed_task_data = self._parse_yaml_data(task_data)
+        parsed_event_reactions = self._parse_yaml_data(event_reactions)
 
         work_request = WorkRequestRequest(
             task_name=task_name,
             workspace=workspace,
-            task_data=task_data,
-            event_reactions=event_reactions,
+            task_data=parsed_task_data,
+            event_reactions=parsed_event_reactions,
         )
 
+        output: dict[str, Any]
         try:
-            work_request_created = self._api_call_or_fail(
-                debusine.work_request_create, work_request
-            )
+            with self._api_call_or_fail():
+                work_request_created = debusine.work_request_create(
+                    work_request
+                )
         except DebusineError as err:
             output = {"result": "failure", "error": err.asdict()}
         else:
@@ -604,11 +617,10 @@ class Cli:
         if priority_adjustment is None:
             self._fail("Error: no changes specified")
 
-        self._api_call_or_fail(
-            debusine.work_request_update,
-            work_request_id,
-            priority_adjustment=priority_adjustment,
-        )
+        with self._api_call_or_fail():
+            debusine.work_request_update(
+                work_request_id, priority_adjustment=priority_adjustment
+            )
 
     def _retry_work_request(
         self,
@@ -616,10 +628,8 @@ class Cli:
         work_request_id: int,
     ) -> None:
         """Retry an aborted/failed work request."""
-        self._api_call_or_fail(
-            debusine.work_request_retry,
-            work_request_id,
-        )
+        with self._api_call_or_fail():
+            debusine.work_request_retry(work_request_id)
 
     def _provide_signature_debsign(
         self,
@@ -630,14 +640,15 @@ class Cli:
         """Provide a work request with an external signature using `debsign`."""
         # Get a version of the work request with its dynamic task data
         # resolved.
-        work_request = self._api_call_or_fail(
-            debusine.work_request_external_debsign_get, work_request.id
-        )
+        with self._api_call_or_fail():
+            work_request = debusine.work_request_external_debsign_get(
+                work_request.id
+            )
         assert work_request.dynamic_task_data is not None
-        unsigned = self._api_call_or_fail(
-            debusine.artifact_get,
-            work_request.dynamic_task_data["unsigned_id"],
-        )
+        with self._api_call_or_fail():
+            unsigned = debusine.artifact_get(
+                work_request.dynamic_task_data["unsigned_id"]
+            )
 
         with tempfile.TemporaryDirectory(
             prefix="debusine-debsign-"
@@ -653,9 +664,8 @@ class Cli:
                     or name.endswith(".dsc")
                     or name.endswith(".buildinfo")
                 ):
-                    self._api_call_or_fail(
-                        debusine.download_artifact_file, unsigned, name, path
-                    )
+                    with self._api_call_or_fail():
+                        debusine.download_artifact_file(unsigned, name, path)
             # Upload artifacts are guaranteed to have exactly one .changes
             # file.
             [changes_path] = [
@@ -673,29 +683,27 @@ class Cli:
             signed_local = Upload.create(
                 changes_file=changes_path, allow_remote=True
             )
-            signed_remote = self._api_call_or_fail(
-                debusine.upload_artifact,
-                signed_local,
-                workspace=unsigned.workspace,
-            )
+            with self._api_call_or_fail():
+                signed_remote = debusine.upload_artifact(
+                    signed_local, workspace=unsigned.workspace
+                )
 
             # Complete the work request using the signed artifact.
-            self._api_call_or_fail(
-                debusine.work_request_external_debsign_complete,
-                work_request.id,
-                WorkRequestExternalDebsignRequest(
-                    signed_artifact=signed_remote.id
-                ),
-            )
+            with self._api_call_or_fail():
+                debusine.work_request_external_debsign_complete(
+                    work_request.id,
+                    WorkRequestExternalDebsignRequest(
+                        signed_artifact=signed_remote.id
+                    ),
+                )
 
     def _provide_signature(
         self, debusine: Debusine, work_request_id: int, extra_args: list[str]
     ) -> None:
         """Provide a work request with an external signature."""
         # Find out what kind of work request we're dealing with.
-        work_request = self._api_call_or_fail(
-            debusine.work_request_get, work_request_id
-        )
+        with self._api_call_or_fail():
+            work_request = debusine.work_request_get(work_request_id)
         match (work_request.task_type, work_request.task_name):
             case "Wait", "externaldebsign":
                 self._provide_signature_debsign(
@@ -728,10 +736,12 @@ class Cli:
             priority=priority,
         )
 
+        output: dict[str, Any]
         try:
-            workflow_template_created = self._api_call_or_fail(
-                debusine.workflow_template_create, workflow_template
-            )
+            with self._api_call_or_fail():
+                workflow_template_created = debusine.workflow_template_create(
+                    workflow_template
+                )
         except DebusineError as err:
             output = {"result": "failure", "error": err.asdict()}
         else:
@@ -777,10 +787,10 @@ class Cli:
             task_data=parsed_task_data,
         )
 
+        output: dict[str, Any]
         try:
-            workflow_created = self._api_call_or_fail(
-                debusine.workflow_create, workflow
-            )
+            with self._api_call_or_fail():
+                workflow_created = debusine.workflow_create(workflow)
         except DebusineError as err:
             output = {"result": "failure", "error": err.asdict()}
         else:
@@ -808,9 +818,10 @@ class Cli:
             task_data=data,
         )
 
-    def _api_call_or_fail(self, method, *args, **kwargs):
+    @contextmanager
+    def _api_call_or_fail(self) -> Generator[None, None, None]:
         """
-        Call method with args and kwargs.
+        Context manager to handle failures from calling a method.
 
         :raises: exceptions.NotFoundError: server returned 404.
         :raises: UnexpectedResponseError: e.g. invalid JSON.
@@ -819,7 +830,7 @@ class Cli:
           reports an error.
         """
         try:
-            result = method(*args, **kwargs)
+            yield
         except exceptions.NotFoundError as exc:
             self._fail(exc)
         except exceptions.UnexpectedResponseError as exc:
@@ -829,8 +840,6 @@ class Cli:
         except exceptions.ClientConnectionError as client_error:
             self._fail(f'Error connecting to debusine: {client_error}')
 
-        return result
-
     @classmethod
     def _parse_yaml_data(cls, data_yaml: str | None) -> dict[str, Any]:
         if data_yaml is not None:
@@ -852,18 +861,19 @@ class Cli:
         else:
             data = {}
 
+        assert isinstance(data, dict)
         return data
 
     def _create_artifact(
         self,
-        debusine,
+        debusine: Debusine,
         artifact_type: str,
         workspace: str,
         upload_paths: list[str],
         upload_base_directory: str | None,
         expire_at: datetime | None,
-        data_yaml,
-    ):
+        data_yaml: str | None,
+    ) -> None:
         data = self._parse_yaml_data(data_yaml)
 
         artifact_cls = LocalArtifact.class_from_category(artifact_type)
@@ -882,13 +892,12 @@ class Cli:
         except ValueError as exc:
             self._fail(f"Cannot create artifact: {exc}")
 
+        output: dict[str, Any]
         try:
-            artifact_created = self._api_call_or_fail(
-                debusine.artifact_create,
-                local_artifact,
-                workspace=workspace,
-                expire_at=expire_at,
-            )
+            with self._api_call_or_fail():
+                artifact_created = debusine.artifact_create(
+                    local_artifact, workspace=workspace, expire_at=expire_at
+                )
         except DebusineError as err:
             output = {"result": "failure", "error": err.asdict()}
             self._print_yaml(output)
@@ -910,11 +919,8 @@ class Cli:
                 ]
 
             self._print_yaml(output)
-            self._api_call_or_fail(
-                debusine.upload_files,
-                artifact_created.id,
-                files_to_upload,
-            )
+            with self._api_call_or_fail():
+                debusine.upload_files(artifact_created.id, files_to_upload)
 
     def _prepare_changes_for_upload(
         self, changes: Path
@@ -1008,7 +1014,7 @@ class Cli:
         workspace: str,
         upload: str,
         expire_at: datetime | None,
-    ):
+    ) -> None:
         result: dict[str, Any] = {}
         uploaded = []
         try:
@@ -1027,12 +1033,10 @@ class Cli:
                         f"imported with this command. {path} is not supported."
                     )
                 for artifact in artifacts:
-                    remote_artifact = self._api_call_or_fail(
-                        debusine.upload_artifact,
-                        artifact,
-                        workspace=workspace,
-                        expire_at=expire_at,
-                    )
+                    with self._api_call_or_fail():
+                        remote_artifact = debusine.upload_artifact(
+                            artifact, workspace=workspace, expire_at=expire_at
+                        )
                     uploaded.append(remote_artifact)
             primary_artifact = uploaded[0]
             result = {
@@ -1045,21 +1049,21 @@ class Cli:
                 "artifact_id": primary_artifact.id,
             }
             for related_artifact in uploaded[1:]:
-                self._api_call_or_fail(
-                    debusine.relation_create,
-                    primary_artifact.id,
-                    related_artifact.id,
-                    "extends",
-                )
+                with self._api_call_or_fail():
+                    debusine.relation_create(
+                        primary_artifact.id,
+                        related_artifact.id,
+                        RelationType.EXTENDS,
+                    )
                 result.setdefault("extends", []).append(
                     {"artifact_id": related_artifact.id}
                 )
-                self._api_call_or_fail(
-                    debusine.relation_create,
-                    primary_artifact.id,
-                    related_artifact.id,
-                    "relates-to",
-                )
+                with self._api_call_or_fail():
+                    debusine.relation_create(
+                        primary_artifact.id,
+                        related_artifact.id,
+                        RelationType.RELATES_TO,
+                    )
                 result.setdefault("relates_to", []).append(
                     {"artifact_id": related_artifact.id}
                 )
@@ -1082,32 +1086,28 @@ class Cli:
         *,
         target_directory: Path,
         tarball: bool,
-    ):
+    ) -> None:
         if not os.access(target_directory, os.W_OK):
             self._fail(f"Error: Cannot write to {target_directory}")
 
-        self._api_call_or_fail(
-            debusine.download_artifact,
-            artifact_id,
-            destination=target_directory,
-            tarball=tarball,
-        )
+        with self._api_call_or_fail():
+            debusine.download_artifact(
+                artifact_id, destination=target_directory, tarball=tarball
+            )
 
-    def _show_artifact(self, debusine: Debusine, artifact_id: int):
-        artifact = self._api_call_or_fail(debusine.artifact_get, artifact_id)
+    def _show_artifact(self, debusine: Debusine, artifact_id: int) -> None:
+        with self._api_call_or_fail():
+            artifact = debusine.artifact_get(artifact_id)
         self._print_yaml(model_to_json_serializable_dict(artifact))
 
     def _show_artifact_relations(
         self, debusine: Debusine, artifact_id: int, reverse: bool
     ) -> None:
-        if reverse:
-            models = self._api_call_or_fail(
-                debusine.relation_list, target_id=artifact_id
-            )
-        else:
-            models = self._api_call_or_fail(
-                debusine.relation_list, artifact_id=artifact_id
-            )
+        with self._api_call_or_fail():
+            if reverse:
+                models = debusine.relation_list(target_id=artifact_id)
+            else:
+                models = debusine.relation_list(artifact_id=artifact_id)
 
         relations = [model_to_json_serializable_dict(model) for model in models]
         self._print_yaml(relations)
@@ -1119,7 +1119,7 @@ class Cli:
         workspaces: list[str],
         last_completed_at: Path | None,
         command: str,
-    ):
+    ) -> None:
         if shutil.which(command) is None:
             self._fail(
                 f'Error: "{command}" does not exist or is not executable'
