@@ -13,6 +13,7 @@ import mmap
 import os
 import time
 from pathlib import Path
+from typing import IO, TYPE_CHECKING
 
 import requests
 from requests import status_codes
@@ -21,15 +22,18 @@ from debusine.client import exceptions
 from debusine.client.utils import requests_put_or_connection_error
 from debusine.utils import parse_range_header
 
+if TYPE_CHECKING:
+    from _typeshed import SupportsRead
+
 
 class UnexpectedStatusCodeError(Exception):
     """Raised when the server did not return the expected status code."""
 
-    def __init__(self, response: requests.models.Response):
+    def __init__(self, response: requests.models.Response) -> None:
         """Initialize exception."""
         self._response = response
 
-    def __str__(self):
+    def __str__(self) -> str:
         """Return human-readable error message."""
         request = self._response.request
 
@@ -42,7 +46,7 @@ class UnexpectedStatusCodeError(Exception):
 
         error += (
             f"Received unexpected status code: {self._response.status_code} "
-            f"Response body:\n{self._response.content}"
+            f"Response body:\n{self._response.text}"
         )
 
         return error
@@ -78,11 +82,13 @@ class FileUploader:
     # client_max_body_size as well
     _MAX_CHUNK_SIZE_BYTES = 50 * 1024 * 1024
 
-    def __init__(self, token):
+    def __init__(self, token: str) -> None:
         """Initialize object."""
         self._headers = {"Token": token}
 
-    def _upload_chunk(self, url, content_range, file):
+    def _upload_chunk(
+        self, url: str, content_range: str, file: "SupportsRead[bytes]"
+    ) -> requests.models.Response:
         attempt = 0
 
         while True:
@@ -102,7 +108,9 @@ class FileUploader:
                 time.sleep(self._CHUNK_UPLOAD_RETRY_WAIT_SECS)
 
     @staticmethod
-    def _parse_range_or_raise_exception(response: requests.models.Response):
+    def _parse_range_or_raise_exception(
+        response: requests.models.Response,
+    ) -> dict[str, int]:
         try:
             parsed_range = parse_range_header(response.headers)
             if parsed_range is not None:
@@ -117,7 +125,7 @@ class FileUploader:
             f"{response.headers.get('content-range', None)}"
         )
 
-    def _upload_file(self, file, file_size, url):
+    def _upload_file(self, file: IO[bytes], file_size: int, url: str) -> bool:
         content_range = f"bytes */{file_size}"
 
         r = requests_put_or_connection_error(
@@ -164,7 +172,7 @@ class FileUploader:
             else:
                 raise UnexpectedStatusCodeError(r)
 
-    def upload(self, file_path: Path, url):
+    def upload(self, file_path: Path, url: str) -> None:
         """Upload the file used in this instance (see __init__())."""
         upload_attempt = 1
         success = False
