@@ -10,7 +10,7 @@
 """Models used by debusine client."""
 
 import json
-from collections.abc import Sequence
+from collections.abc import Iterator, Sequence
 from datetime import datetime
 from enum import StrEnum
 from pathlib import Path
@@ -71,7 +71,7 @@ class WorkRequestResponse(StrictBaseModel):
     artifacts: list[int]
     workspace: str
 
-    def __str__(self):
+    def __str__(self) -> str:
         """Return representation of the object."""
         return f'WorkRequest: {self.id}'
 
@@ -143,7 +143,11 @@ class FileRequest(StrictBaseModel):
         """Return a FileRequest for the file path."""
         return FileRequest(
             size=path.stat().st_size,
-            checksums={"sha256": calculate_hash(path, "sha256").hex()},
+            checksums={
+                "sha256": pydantic.parse_obj_as(
+                    StrMaxLength255, calculate_hash(path, "sha256").hex()
+                )
+            },
             type="file",
         )
 
@@ -193,7 +197,12 @@ class RemoteArtifact(StrictBaseModel):
     workspace: str
 
 
-RelationCreateRequestType = Literal["extends", "relates-to", "built-using"]
+class RelationType(StrEnum):
+    """Possible values for `RelationCreateRequest.type`."""
+
+    EXTENDS = "extends"
+    RELATES_TO = "relates-to"
+    BUILT_USING = "built-using"
 
 
 class RelationCreateRequest(StrictBaseModel):
@@ -201,7 +210,7 @@ class RelationCreateRequest(StrictBaseModel):
 
     artifact: int
     target: int
-    type: RelationCreateRequestType
+    type: RelationType
 
 
 class RelationResponse(RelationCreateRequest):
@@ -216,13 +225,23 @@ class RelationsResponse(StrictBaseModel):
     # This model parses a list, not an object/dict.
     __root__: Sequence[RelationResponse]
 
-    def __iter__(self):
+    def __iter__(self) -> Iterator[RelationResponse]:  # type: ignore[override]
         """Iterate over individual relation responses."""
         return iter(self.__root__)
 
 
-class CollectionItemType(StrEnum):
-    """A collection item type."""
+class LookupChildType(StrEnum):
+    """Possible values for `LookupDict.child_type` and `expect_type`."""
+
+    BARE = "bare"
+    ARTIFACT = "artifact"
+    ARTIFACT_OR_PROMISE = "artifact-or-promise"
+    COLLECTION = "collection"
+    ANY = "any"
+
+
+class LookupResultType(StrEnum):
+    """A collection item type returned by a lookup."""
 
     BARE = "b"
     ARTIFACT = "a"
@@ -234,7 +253,7 @@ class LookupSingleRequest(StrictBaseModel):
 
     lookup: int | str
     work_request: int
-    expect_type: CollectionItemType
+    expect_type: LookupChildType
     default_category: str | None = None
 
 
@@ -243,14 +262,14 @@ class LookupMultipleRequest(StrictBaseModel):
 
     lookup: list[int | str | dict[str, Any]]
     work_request: int
-    expect_type: CollectionItemType
+    expect_type: LookupChildType
     default_category: str | None = None
 
 
 class LookupSingleResponse(StrictBaseModel):
     """A response from the server with a single lookup result."""
 
-    result_type: CollectionItemType
+    result_type: LookupResultType
     collection_item: int | None = None
     artifact: int | None = None
     collection: int | None = None
@@ -263,7 +282,7 @@ class LookupSingleResponseArtifact(LookupSingleResponse):
     Used to assist type annotations.
     """
 
-    result_type: Literal[CollectionItemType.ARTIFACT]
+    result_type: Literal[LookupResultType.ARTIFACT]
     artifact: int
 
 
@@ -274,7 +293,7 @@ class LookupSingleResponseCollection(LookupSingleResponse):
     Used to assist type annotations.
     """
 
-    result_type: Literal[CollectionItemType.COLLECTION]
+    result_type: Literal[LookupResultType.COLLECTION]
     collection: int
 
 
@@ -287,13 +306,13 @@ class LookupMultipleResponse(StrictBaseModel, Generic[LSR]):
     # This model parses a list, not an object/dict.
     __root__: Sequence[LSR]
 
-    def __iter__(self):
+    def __iter__(self) -> Iterator[LSR]:  # type: ignore[override]
         """Iterate over individual results."""
         return iter(self.__root__)
 
 
 def model_to_json_serializable_dict(
-    model: pydantic.BaseModel,
+    model: pydantic.BaseModel, exclude_unset: bool = False
 ) -> dict[Any, Any]:
     """
     Similar to model.dict() but the returned dictionary is JSON serializable.
@@ -303,4 +322,6 @@ def model_to_json_serializable_dict(
 
     Replace with model_dump() in Pydantic 2.
     """
-    return json.loads(model.json())
+    serializable = json.loads(model.json(exclude_unset=exclude_unset))
+    assert isinstance(serializable, dict)
+    return serializable
