@@ -1,4 +1,4 @@
-# Copyright 2022 The Debusine Developers
+# Copyright 2022-2024 The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -12,13 +12,13 @@
 import io
 from configparser import ConfigParser
 from pathlib import Path
-from unittest import TestCase
+from typing import Any
 
 from debusine.client.config import ConfigHandler
-from debusine.test import TestHelpersMixin
+from debusine.test import TestCase
 
 
-class ConfigHandlerTests(TestHelpersMixin, TestCase):
+class ConfigHandlerTests(TestCase):
     """Tests for ConfigHandler class."""
 
     def setUp(self) -> None:
@@ -32,10 +32,12 @@ class ConfigHandlerTests(TestHelpersMixin, TestCase):
         configuration['General'] = {'default-server': 'debian'}
         configuration['server:debian'] = {
             'api-url': 'https://debusine.debian.org/api',
+            'scope': 'debian',
             'token': 'the-token-for-debian',
         }
         configuration['server:kali'] = {
             'api-url': 'https://debusine.kali.org/api',
+            'scope': 'kali',
             'token': 'the-token-for-kali',
         }
 
@@ -51,7 +53,7 @@ class ConfigHandlerTests(TestHelpersMixin, TestCase):
         return temp_file
 
     def build_config_handler(
-        self, config_parser, **config_kwargs
+        self, config_parser: ConfigParser, **config_kwargs: Any
     ) -> ConfigHandler:
         """Return ConfigHandler for config_parser."""
         temporary_config = self.write_configuration(config_parser)
@@ -112,6 +114,7 @@ class ConfigHandlerTests(TestHelpersMixin, TestCase):
             config._server_configuration(server_name),
             {
                 'api-url': server_config["api-url"],
+                'scope': server_config["scope"],
                 'token': server_config["token"],
             },
         )
@@ -132,6 +135,7 @@ class ConfigHandlerTests(TestHelpersMixin, TestCase):
             config_handler.server_configuration(),
             {
                 'api-url': default_server["api-url"],
+                'scope': default_server["scope"],
                 'token': default_server["token"],
             },
         )
@@ -151,6 +155,7 @@ class ConfigHandlerTests(TestHelpersMixin, TestCase):
             config_handler.server_configuration(),
             {
                 'api-url': config_server["api-url"],
+                'scope': config_server["scope"],
                 'token': config_server["token"],
             },
         )
@@ -181,7 +186,7 @@ class ConfigHandlerTests(TestHelpersMixin, TestCase):
         self.assertEqual(
             self.stderr.getvalue(),
             'Missing required keys in the section '
-            '[server:incomplete-server]: api-url, token in '
+            '[server:incomplete-server]: api-url, scope, token in '
             f'{config._config_file_path} .\n',
         )
 
