@@ -19,10 +19,11 @@ import os
 import stat
 import tarfile
 import textwrap
+import types
 import urllib
 from collections.abc import Callable
 from pathlib import Path
-from typing import Any
+from typing import Any, NoReturn
 from unittest import mock
 from unittest.mock import MagicMock, call
 from urllib.parse import urlencode
@@ -46,14 +47,16 @@ from debusine.artifacts.models import (
 from debusine.client.debusine import Debusine
 from debusine.client.models import (
     ArtifactResponse,
-    CollectionItemType,
     CreateWorkflowRequest,
     FileResponse,
     FilesResponseType,
+    LookupChildType,
     LookupMultipleResponse,
+    LookupResultType,
     LookupSingleResponse,
     RelationCreateRequest,
     RelationResponse,
+    RelationType,
     RelationsResponse,
     RemoteArtifact,
     StrMaxLength255,
@@ -676,7 +679,10 @@ class DebusineTests(TestCase):
         url = self.api_url + "/1.0/artifact-relation/"
 
         response_body = RelationResponse(
-            id=7, artifact=artifact_id, target=target_id, type="extends"
+            id=7,
+            artifact=artifact_id,
+            target=target_id,
+            type=RelationType.EXTENDS,
         )
         responses.add(
             responses.POST,
@@ -684,7 +690,9 @@ class DebusineTests(TestCase):
             json=dict(response_body),
         )
 
-        self.debusine.relation_create(artifact_id, target_id, "extends")
+        self.debusine.relation_create(
+            artifact_id, target_id, RelationType.EXTENDS
+        )
 
         responses.assert_call_count(url, 1)
 
@@ -694,7 +702,9 @@ class DebusineTests(TestCase):
         self.assertEqual(
             request_body,
             RelationCreateRequest(
-                artifact=artifact_id, target=target_id, type="extends"
+                artifact=artifact_id,
+                target=target_id,
+                type=RelationType.EXTENDS,
             ),
         )
 
@@ -717,7 +727,7 @@ class DebusineTests(TestCase):
         """relation_list() with an artifact ID makes an HTTP GET."""
         url = self.api_url + "/1.0/artifact-relation/?artifact=1"
         relation_response = RelationResponse(
-            id=7, artifact=1, target=2, type="extends"
+            id=7, artifact=1, target=2, type=RelationType.EXTENDS
         )
         responses.add(responses.GET, url, json=[dict(relation_response)])
 
@@ -733,7 +743,7 @@ class DebusineTests(TestCase):
         """relation_list() with a target ID makes an HTTP GET."""
         url = self.api_url + "/1.0/artifact-relation/?target_artifact=2"
         relation_response = RelationResponse(
-            id=7, artifact=1, target=2, type="extends"
+            id=7, artifact=1, target=2, type=RelationType.EXTENDS
         )
         responses.add(responses.GET, url, json=[dict(relation_response)])
 
@@ -745,7 +755,7 @@ class DebusineTests(TestCase):
         responses.assert_call_count(url, 1)
 
     def add_get_download_artifact_response(
-        self, files: int = 0, file_size: int = 10240
+        self, files: int = 0, file_size: int = 10240, directories: bool = False
     ) -> ArtifactResponse:
         """
         Call responses.add("GET", "/api/1.0/artifact) with an Artifact.
@@ -758,15 +768,17 @@ class DebusineTests(TestCase):
         )
         url_download_artifact = f"{artifact_base_url}?archive=tar.gz"
 
+        directory = "directory/sub-directory/" if directories else ""
+
         response_files = {
-            f"file{i}": FileResponse(
+            f"{directory}file{i}": FileResponse(
                 size=file_size,
                 checksums={
                     "sha256": pydantic.parse_obj_as(StrMaxLength255, "abc123")
                 },
                 type="file",
                 url=pydantic.parse_obj_as(
-                    pydantic.AnyUrl, f"{artifact_base_url}file{i}"
+                    pydantic.AnyUrl, f"{artifact_base_url}{directory}file{i}"
                 ),
             )
             for i in range(files)
@@ -932,6 +944,31 @@ class DebusineTests(TestCase):
             (downloaded_directory / file.name).read_bytes(),
         )
 
+    @responses.activate
+    def test_download_artifact_single_file_with_parent_directory(self) -> None:
+        """download_artifact() handles directories in single-file downloads."""
+        contents = b"file contents"
+        downloaded_directory = self.create_temporary_directory()
+        artifact_response = self.add_get_download_artifact_response(
+            files=1, directories=True
+        )
+        destination = downloaded_directory / "directory/sub-directory/file0"
+
+        responses.add(
+            responses.GET,
+            artifact_response.files["directory/sub-directory/file0"].url,
+            contents,
+        )
+
+        self.debusine.download_artifact(
+            artifact_response.id, downloaded_directory
+        )
+
+        self.assertEqual(destination.read_bytes(), contents)
+        self.logger.info.assert_called_with(
+            "Downloading artifact files into %s", downloaded_directory
+        )
+
     def test_should_download_as_tarball_single_file(self) -> None:
         """_should_download_as_tarball() single file artifact."""
         artifact_response = self.add_get_download_artifact_response(files=1)
@@ -964,7 +1001,7 @@ class DebusineTests(TestCase):
         )
 
     def add_get_download_artifact_file_response(
-        self, contents
+        self, contents: bytes
     ) -> ArtifactResponse:
         """
         Call responses.add("GET", "/api/1.0/artifact) with an Artifact.
@@ -1074,7 +1111,7 @@ class DebusineTests(TestCase):
             responses.POST,
             f"{self.debusine.api_url}/lookup/single/",
             json={
-                "result_type": CollectionItemType.ARTIFACT,
+                "result_type": LookupResultType.ARTIFACT,
                 "collection_item": None,
                 "artifact": 1,
                 "collection": None,
@@ -1084,14 +1121,14 @@ class DebusineTests(TestCase):
         response = self.debusine.lookup_single(
             lookup="collection/artifact",
             work_request=1,
-            expect_type=CollectionItemType.ARTIFACT,
+            expect_type=LookupChildType.ARTIFACT,
             default_category=CollectionCategory.TEST,
         )
 
         self.assertEqual(
             response,
             LookupSingleResponse(
-                result_type=CollectionItemType.ARTIFACT, artifact=1
+                result_type=LookupResultType.ARTIFACT, artifact=1
             ),
         )
         assert responses.calls[0].request.body is not None
@@ -1101,7 +1138,7 @@ class DebusineTests(TestCase):
                 "lookup": "collection/artifact",
                 "work_request": 1,
                 "default_category": CollectionCategory.TEST,
-                "expect_type": CollectionItemType.ARTIFACT,
+                "expect_type": LookupChildType.ARTIFACT,
             },
         )
 
@@ -1113,7 +1150,7 @@ class DebusineTests(TestCase):
             f"{self.debusine.api_url}/lookup/multiple/",
             json=[
                 {
-                    "result_type": CollectionItemType.ARTIFACT,
+                    "result_type": LookupResultType.ARTIFACT,
                     "collection_item": artifact_id,
                     "artifact": artifact_id,
                     "collection": None,
@@ -1125,7 +1162,7 @@ class DebusineTests(TestCase):
         response = self.debusine.lookup_multiple(
             lookup=LookupMultiple.parse_obj({"collection": "collection"}),
             work_request=1,
-            expect_type=CollectionItemType.ARTIFACT,
+            expect_type=LookupChildType.ARTIFACT,
             default_category=CollectionCategory.TEST,
         )
 
@@ -1134,7 +1171,7 @@ class DebusineTests(TestCase):
             LookupMultipleResponse.parse_obj(
                 [
                     LookupSingleResponse(
-                        result_type=CollectionItemType.ARTIFACT,
+                        result_type=LookupResultType.ARTIFACT,
                         collection_item=artifact_id,
                         artifact=artifact_id,
                     )
@@ -1154,11 +1191,12 @@ class DebusineTests(TestCase):
                         "collection": "collection",
                         "data_matchers": [],
                         "name_matcher": None,
+                        "lookup_filters": [],
                     }
                 ],
                 "work_request": 1,
                 "default_category": CollectionCategory.TEST,
-                "expect_type": CollectionItemType.ARTIFACT,
+                "expect_type": LookupChildType.ARTIFACT,
             },
         )
 
@@ -1228,7 +1266,7 @@ class DebusineAsyncTests(server.DebusineAioHTTPTestCase, TestCase):
         """A WorkRequest finished and command is executed."""
         await self.execute_wait_and_execute(
             command_modifier=lambda cmd: cmd,
-            working_directory=lambda _: Path.cwd(),  # noqa: U101
+            working_directory=lambda _: Path.cwd(),
             on_completed_work_requests=1,
         )
         self.assertEqual(self.requests[-1].headers["Token"], self.token)
@@ -1237,7 +1275,7 @@ class DebusineAsyncTests(server.DebusineAioHTTPTestCase, TestCase):
         """Two WorkRequests finished: two commands are executed."""
         await self.execute_wait_and_execute(
             command_modifier=lambda cmd: cmd,
-            working_directory=lambda _: Path.cwd(),  # noqa: U101
+            working_directory=lambda _: Path.cwd(),
             on_completed_work_requests=2,
         )
 
@@ -1356,14 +1394,19 @@ class DebusineAsyncTests(server.DebusineAioHTTPTestCase, TestCase):
         exc = aiohttp.client_exceptions.ClientError(exc_message)
 
         # Mock ws_connect to raise the ClientError exception
-        def mock_ws_connect(*args, **kwargs):
-            class MockWebSocketResponse:
-                async def __aenter__(self):
-                    raise exc
+        class MockWebSocketResponse:
+            async def __aenter__(self) -> NoReturn:
+                raise exc
 
-                async def __aexit__(self, exc_type, exc_val, exc_tb):
-                    pass  # pragma: no cover
+            async def __aexit__(
+                self,
+                exc_type: type[BaseException] | None,
+                exc_val: BaseException | None,
+                exc_tb: types.TracebackType | None,
+            ) -> None:
+                pass  # pragma: no cover
 
+        def mock_ws_connect(*args: Any, **kwargs: Any) -> MockWebSocketResponse:
             return MockWebSocketResponse()
 
         patcher = mock.patch.object(
@@ -1400,7 +1443,7 @@ class DebusineAsyncTests(server.DebusineAioHTTPTestCase, TestCase):
         self.debusine = Debusine(self.api_url, None, logger=self.logger)
         await self.execute_wait_and_execute(
             command_modifier=lambda cmd: cmd,
-            working_directory=lambda _: Path.cwd(),  # noqa: U101
+            working_directory=lambda _: Path.cwd(),
             on_completed_work_requests=1,
         )
         self.assertNotIn("Token", self.requests[-1].headers)
