@@ -17,7 +17,7 @@ from debusine.client.exceptions import DebusineError
 class DebusineErrorTests(TestCase):
     """Tests for DebusineError exception."""
 
-    def test_asdict(self):
+    def test_asdict(self) -> None:
         """Test asdict() returns the dictionary."""
         data = {
             "title": "The title of the error",
