@@ -9,12 +9,13 @@
 
 """Unit tests for the FileUploader class."""
 
+import io
 import mmap
 from collections.abc import Callable
-from unittest import TestCase, mock
+from typing import Any
+from unittest import mock
 
 import requests
-
 import responses
 
 from debusine.client import exceptions
@@ -23,16 +24,16 @@ from debusine.client.file_uploader import (
     FileUploader,
     UnexpectedStatusCodeError,
 )
-from debusine.test import TestHelpersMixin
+from debusine.test import TestCase
 from debusine.utils import parse_range_header
 
 
 def body_matcher(
-    expected_body,
+    expected_body: bytes | None,
 ) -> Callable[..., tuple[bool, str]]:  # pragma: no cover
     """Matcher for the body's content of a request. Used by responses.add."""
 
-    def match(request) -> tuple[bool, str]:
+    def match(request: requests.models.PreparedRequest) -> tuple[bool, str]:
         """Return True if request_body matches expected_body."""
         if isinstance(request.body, mmap.mmap):
             position = request.body.tell()
@@ -68,7 +69,7 @@ class UnexpectedStatusCodeErrorTests(TestCase):
         )
         self.mocked_response.request = mocked_request
         self.mocked_response.status_code = requests.codes.bad_request
-        self.mocked_response.content = "Error from the server"
+        self.mocked_response.text = "Error from the server"
 
     def test_str_without_content_range(self) -> None:
         """Test for the human-readable string: content-range was not sent."""
@@ -81,7 +82,7 @@ class UnexpectedStatusCodeErrorTests(TestCase):
             str(status_code_error),
             f"Client HTTP {mocked_request.method} {mocked_request.url} "
             f"Received unexpected status code: {mocked_response.status_code} "
-            f"Response body:\n{mocked_response.content}",
+            f"Response body:\n{mocked_response.text}",
         )
 
     def test_str_with_content_range(self) -> None:
@@ -98,11 +99,11 @@ class UnexpectedStatusCodeErrorTests(TestCase):
             f"Client HTTP {mocked_request.method} {mocked_request.url} "
             f"with Content-Range {mocked_request.headers['Content-Range']} "
             f"Received unexpected status code: {mocked_response.status_code} "
-            f"Response body:\n{mocked_response.content}",
+            f"Response body:\n{mocked_response.text}",
         )
 
 
-class FileUploaderTests(TestHelpersMixin, TestCase):
+class FileUploaderTests(TestCase):
     """Tests for the debusine command line interface."""
 
     def setUp(self) -> None:
@@ -238,9 +239,12 @@ class FileUploaderTests(TestHelpersMixin, TestCase):
     def test_file_upload_conflict_uploading_file(self) -> None:
         """Client upload file, there is a conflict and uploads it again."""
 
-        def succeeds_after_one_attempt(request):  # noqa: U100
+        def succeeds_after_one_attempt(
+            request: requests.PreparedRequest,  # noqa: U100
+        ) -> tuple[int, dict[str, str], None]:
             first_attempt = getattr(succeeds_after_one_attempt, "first", True)
 
+            status_code: int
             if first_attempt:
                 status_code = requests.codes.conflict
             else:
@@ -418,9 +422,12 @@ class FileUploaderTests(TestHelpersMixin, TestCase):
             requests.exceptions.RequestException(request=request)
         )
 
-        with self.assertRaises(exceptions.ClientConnectionError):
+        with (
+            self.assertRaises(exceptions.ClientConnectionError),
+            open(self.file_path, "rb") as file,
+        ):
             self.file_uploader._upload_file(
-                self.file_path, len(self.file_content), "https://test.com"
+                file, len(self.file_content), "https://test.com"
             )
 
     def patch_sleep(self) -> mock.MagicMock:
@@ -432,7 +439,7 @@ class FileUploaderTests(TestHelpersMixin, TestCase):
 
         return mocked_sleep
 
-    def patch_requests_put(self, side_effect) -> mock.MagicMock:
+    def patch_requests_put(self, side_effect: Any) -> mock.MagicMock:
         """Patch requests.put(), return its mock."""
         patcher = mock.patch("requests.put", autospec=True)
         mocked_put = patcher.start()
@@ -458,7 +465,7 @@ class FileUploaderTests(TestHelpersMixin, TestCase):
         )
 
         r = self.file_uploader._upload_chunk(
-            "some-url", "bytes=1-2/2", "some-file"
+            "some-url", "bytes=1-2/2", io.BytesIO(b"some-data")
         )
         self.assertEqual(mocked_put.call_count, 2)
         self.assertEqual(r, return_value)
@@ -480,7 +487,7 @@ class FileUploaderTests(TestHelpersMixin, TestCase):
 
         with self.assertRaises(exceptions.ClientConnectionError):
             self.file_uploader._upload_chunk(
-                "some-url", "bytes=1-2/2", "some-file"
+                "some-url", "bytes=1-2/2", io.BytesIO(b"some-data")
             )
 
         self.assertEqual(
