@@ -17,7 +17,7 @@ from debusine.client.__main__ import entry_point
 class ConfigHandlerTests(TestCase):
     """Tests for ConfigHandler class."""
 
-    def test_main_entry_point(self):
+    def test_main_entry_point(self) -> None:
         """Ensure client's main entry point calls Cli().parse()."""
         with mock.patch(
             "debusine.client.cli.Cli.execute", autospec=True
