@@ -16,11 +16,12 @@ from collections.abc import Generator, Iterable
 from contextlib import contextmanager
 from pathlib import Path
 from tempfile import TemporaryDirectory
-from typing import TypeAlias, Union
+from typing import Any, TypeAlias, Union
 from urllib.parse import urljoin, urlparse, urlunparse
 
 import requests
 from debian.deb822 import Changes, Deb822, Dsc
+from requests_file import FileAdapter
 
 from debusine.client import exceptions
 
@@ -30,7 +31,7 @@ DownloadedFileStats: TypeAlias = dict[str, Union[str, int]]
 
 
 def requests_put_or_connection_error(
-    *args, **kwargs
+    *args: Any, **kwargs: Any
 ) -> requests.models.Response:
     r"""Return requests.put(\*args, \*\*kwargs) or ClientConnectionError."""
     try:
@@ -74,10 +75,17 @@ def download_file(
     return stats
 
 
-def get_url_contents_sha256sum(url: str, max_size: int) -> tuple[bytes, str]:
+def get_url_contents_sha256sum(
+    url: str, max_size: int, allow_file: bool = False
+) -> tuple[bytes, str]:
     """
     Fetch URL, return contents and sha256sum.
 
+    :param url: the URL to fetch.
+    :param max_size: the maximum number of bytes to read from the URL.
+    :param allow_file: if True, allow reading `file://` URLs; the caller
+      must previously have checked the URL path to ensure that it is in an
+      allowed part of the file system.
     :raise exceptions.ClientContentTooLargeError: content is bigger than
       max_size.
 
@@ -86,17 +94,20 @@ def get_url_contents_sha256sum(url: str, max_size: int) -> tuple[bytes, str]:
     sha256sum = hashlib.sha256()
     contents: list[bytes] = []
     size = 0
-    with requests.get(url, stream=True) as r:
-        r.raise_for_status()
-        for chunk in r.iter_content(chunk_size=min(1024 * 1024, max_size)):
-            sha256sum.update(chunk)
-            contents.append(chunk)
-            size += len(chunk)
-            if size >= max_size:
-                raise exceptions.ContentTooLargeError(
-                    "Content size exceeds maximum "
-                    f"allowed size of {max_size} bytes."
-                )
+    with requests.Session() as session:
+        if allow_file:
+            session.mount("file://", FileAdapter())
+        with session.get(url, stream=True) as r:
+            r.raise_for_status()
+            for chunk in r.iter_content(chunk_size=min(1024 * 1024, max_size)):
+                sha256sum.update(chunk)
+                contents.append(chunk)
+                size += len(chunk)
+                if size >= max_size:
+                    raise exceptions.ContentTooLargeError(
+                        "Content size exceeds maximum "
+                        f"allowed size of {max_size} bytes."
+                    )
 
     return b"".join(contents), sha256sum.hexdigest()
 
@@ -116,7 +127,7 @@ class DGet:
     queue: list[str]
     workdir: Path
 
-    def __init__(self, url: str, workdir: Path):
+    def __init__(self, url: str, workdir: Path) -> None:
         """Set up for a single set of downloads."""
         parsed_url = urlparse(url)
         # If we are downloading multiple files, we need a logical structure
@@ -135,12 +146,12 @@ class DGet:
         self.queue = [url_path.name]
         self.workdir = workdir
 
-    def download(self):
+    def download(self) -> None:
         """Download all queued files."""
         while self.queue:
             self.download_file(self.queue.pop(0))
 
-    def download_file(self, filename: str):
+    def download_file(self, filename: str) -> None:
         """Download one filename into workdir."""
         if filename in self.downloaded:  # pragma: no cover
             return
@@ -151,7 +162,9 @@ class DGet:
         if destination.suffix in (".dsc", ".changes"):
             self._add_referenced_files(destination)
 
-    def _record_download(self, filename: str, stats: DownloadedFileStats):
+    def _record_download(
+        self, filename: str, stats: DownloadedFileStats
+    ) -> None:
         self.downloaded.add(filename)
         self.known_files.setdefault(filename, {})
         for stat_name, value in stats.items():
@@ -163,7 +176,7 @@ class DGet:
         filename: str,
         stat_name: str,
         value: str | int,
-    ):
+    ) -> None:
         stats = self.known_files[filename]
         if stat_name in stats:
             if stats[stat_name] != value:
@@ -174,7 +187,7 @@ class DGet:
         else:
             stats[stat_name] = value
 
-    def _add_referenced_files(self, path: Path):
+    def _add_referenced_files(self, path: Path) -> None:
         for name, size, hash_name, hash_digest in self._iter_referenced_files(
             path
         ):
@@ -218,7 +231,7 @@ class DGet:
                 yield name, size, hash_name, hash_digest
 
 
-def dget(url: str, workdir: Path):
+def dget(url: str, workdir: Path) -> None:
     """Execute a download with DGet."""
     DGet(url, workdir).download()
 
