@@ -22,7 +22,7 @@ from debusine.tasks.models import TaskTypes
 
 
 def _dict_has_path(d: dict[str, Any], path: Sequence[str]) -> bool:
-    """Does `d[path[0]][path[1]][...]` exist?"""
+    """Return whether `d[path[0]][path[1]][...]` exists."""
     for key in path[:-1]:
         d = d.get(key, {})
     return path[-1] in d
@@ -61,7 +61,7 @@ def make_data_field_renamer(
         filters: dict[str, Any],
         renames: list[tuple[str, str]],
         apps: StateApps,
-        schema_editor: BaseDatabaseSchemaEditor,
+        schema_editor: BaseDatabaseSchemaEditor,  # noqa: U100
     ) -> None:
         Model = apps.get_model("db", model_name)
         rename_paths = [
@@ -134,7 +134,7 @@ def make_work_request_task_renamer(
         old_task_name: str,
         new_task_name: str,
         apps: StateApps,
-        schema_editor: BaseDatabaseSchemaEditor,
+        schema_editor: BaseDatabaseSchemaEditor,  # noqa: U100
     ) -> None:
         WorkRequest = apps.get_model("db", "WorkRequest")
         WorkRequest.objects.filter(
