@@ -10,8 +10,10 @@
 """Unit tests for the migrations."""
 
 import hashlib
-from typing import Any
+from datetime import timedelta
+from typing import Any, cast
 
+from django.conf import settings
 from django.contrib.auth.hashers import is_password_usable
 from django.db import connection
 from django.db.migrations.exceptions import IrreversibleError
@@ -24,12 +26,16 @@ from debusine.artifacts.models import (
     ArtifactCategory,
     BareDataCategory,
     CollectionCategory,
+    KeyPurpose,
 )
+from debusine.client.models import LookupChildType
 from debusine.db.models import (
     DEFAULT_FILE_STORE_NAME,
     DEFAULT_WORKSPACE_NAME,
     FileStore,
     SYSTEM_USER_NAME,
+    User,
+    WorkRequest,
     default_file_store,
     default_workspace,
 )
@@ -64,7 +70,7 @@ class MigrationTests(TestCase):
     """Test migrations."""
 
     @staticmethod
-    def get_test_user_for_apps(apps: StateApps):
+    def get_test_user_for_apps(apps: StateApps) -> User:
         """
         Return a test user.
 
@@ -72,13 +78,14 @@ class MigrationTests(TestCase):
         """
         user_model = apps.get_model("db", "User")
         try:
-            return user_model.objects.get(username="usertest")
+            user = user_model.objects.get(username="usertest")
         except user_model.DoesNotExist:
-            return user_model.objects.create_user(
+            user = user_model.objects.create_user(
                 username="usertest",
                 password="userpassword",
                 email="usertest@example.org",
             )
+        return cast(User, user)
 
     def test_default_store_created(self) -> None:
         """Assert Default FileStore has been created."""
@@ -1199,3 +1206,560 @@ class MigrationTests(TestCase):
         ):
             executor.loader.build_graph()
             executor.migrate(migrate_from)
+
+    def test_sign_multiple_unsigned(self) -> None:
+        """Single unsigned lookups in Sign are migrated to multiple lookups."""
+        migrate_from = [("db", "0090_workspacerole_and_more")]
+        migrate_to = [("db", "0091_sign_multiple_unsigned")]
+
+        executor = MigrationExecutor(
+            connection, progress_callback=_migrate_check_constraints
+        )
+        executor.migrate(migrate_from)
+        old_apps = executor.loader.project_state(migrate_from).apps
+        old_work_request_model = old_apps.get_model("db", "WorkRequest")
+        default_workspace = old_apps.get_model("db", "Workspace").objects.get(
+            name=DEFAULT_WORKSPACE_NAME
+        )
+        test_user = self.get_test_user_for_apps(old_apps)
+        old_unsigned = [2, "internal@collections/name:unsigned"]
+        old_dynamic_task_data = [{"unsigned_id": 2, "key_id": 1}, None]
+        new_unsigned = [[2], ["internal@collections/name:unsigned"]]
+        new_dynamic_task_data = [{"unsigned_ids": [2], "key_id": 1}, None]
+        for unsigned, dynamic_task_data in zip(
+            old_unsigned, old_dynamic_task_data
+        ):
+            old_work_request_model.objects.create(
+                task_type=TaskTypes.SIGNING,
+                task_name="sign",
+                created_by_id=test_user.id,
+                workspace=default_workspace,
+                task_data={
+                    "purpose": KeyPurpose.UEFI,
+                    "unsigned": unsigned,
+                    "key": 1,
+                },
+                dynamic_task_data=dynamic_task_data,
+            )
+
+        executor.loader.build_graph()
+        executor.migrate(migrate_to)
+
+        new_apps = executor.loader.project_state(migrate_to).apps
+        new_work_requests = new_apps.get_model(
+            "db", "WorkRequest"
+        ).objects.order_by("id")
+        self.assertEqual(
+            [
+                work_request.task_data["unsigned"]
+                for work_request in new_work_requests
+            ],
+            new_unsigned,
+        )
+        self.assertEqual(
+            [
+                work_request.dynamic_task_data
+                for work_request in new_work_requests
+            ],
+            new_dynamic_task_data,
+        )
+
+        executor.loader.build_graph()
+        executor.migrate(migrate_from)
+
+        old_work_requests = old_apps.get_model(
+            "db", "WorkRequest"
+        ).objects.order_by("id")
+        self.assertEqual(
+            [
+                work_request.task_data["unsigned"]
+                for work_request in old_work_requests
+            ],
+            old_unsigned,
+        )
+        self.assertEqual(
+            [
+                work_request.dynamic_task_data
+                for work_request in old_work_requests
+            ],
+            old_dynamic_task_data,
+        )
+
+    def test_sign_multiple_unsigned_unsigned_too_complex(self) -> None:
+        """Migration is irreversible if task_data.unsigned is too complex."""
+        migrate_from = [("db", "0091_sign_multiple_unsigned")]
+        migrate_to = [("db", "0090_workspacerole_and_more")]
+
+        executor = MigrationExecutor(
+            connection, progress_callback=_migrate_check_constraints
+        )
+        executor.migrate(migrate_from)
+        old_apps = executor.loader.project_state(migrate_from).apps
+        default_workspace = old_apps.get_model("db", "Workspace").objects.get(
+            name=DEFAULT_WORKSPACE_NAME
+        )
+        test_user = self.get_test_user_for_apps(old_apps)
+        work_request = old_apps.get_model("db", "WorkRequest").objects.create(
+            task_type=TaskTypes.SIGNING,
+            task_name="sign",
+            created_by_id=test_user.id,
+            workspace=default_workspace,
+            task_data={
+                "purpose": KeyPurpose.UEFI,
+                "unsigned": {
+                    "collection": "internal@collections",
+                    "child_type": LookupChildType.ARTIFACT,
+                    "category": ArtifactCategory.SIGNING_INPUT,
+                },
+                "key": 1,
+            },
+        )
+
+        with self.assertRaisesRegex(
+            IrreversibleError,
+            f"Work request {work_request.id}: {{.*}} cannot be converted to a "
+            f"single lookup",
+        ):
+            executor.loader.build_graph()
+            executor.migrate(migrate_to)
+
+    def test_sign_multiple_unsigned_unsigned_ids_too_complex(self) -> None:
+        """Migration is irreversible with multiple unsigned_ids."""
+        migrate_from = [("db", "0091_sign_multiple_unsigned")]
+        migrate_to = [("db", "0090_workspacerole_and_more")]
+
+        executor = MigrationExecutor(
+            connection, progress_callback=_migrate_check_constraints
+        )
+        executor.migrate(migrate_from)
+        old_apps = executor.loader.project_state(migrate_from).apps
+        default_workspace = old_apps.get_model("db", "Workspace").objects.get(
+            name=DEFAULT_WORKSPACE_NAME
+        )
+        test_user = self.get_test_user_for_apps(old_apps)
+        work_request = old_apps.get_model("db", "WorkRequest").objects.create(
+            task_type=TaskTypes.SIGNING,
+            task_name="sign",
+            created_by_id=test_user.id,
+            workspace=default_workspace,
+            task_data={
+                "purpose": KeyPurpose.UEFI,
+                "unsigned": [2],
+                "key": 1,
+            },
+            dynamic_task_data={
+                # This should be impossible since task_data.unsigned has
+                # exactly one lookup, but it's here for the sake of test
+                # coverage.
+                "unsigned_ids": [2, 3],
+                "key_id": 1,
+            },
+        )
+
+        with self.assertRaisesRegex(
+            IrreversibleError,
+            fr"Work request {work_request.id}: \[2, 3\] does not have exactly "
+            fr"one element",
+        ):
+            executor.loader.build_graph()
+            executor.migrate(migrate_to)
+
+    def test_default_workspace_public(self) -> None:
+        """The default workspace is made public."""
+        migrate_from = [("db", "0091_sign_multiple_unsigned")]
+        migrate_to = [("db", "0092_default_workspace_public")]
+
+        executor = MigrationExecutor(
+            connection, progress_callback=_migrate_check_constraints
+        )
+        executor.migrate(migrate_from)
+        old_apps = executor.loader.project_state(migrate_from).apps
+        self.assertFalse(
+            old_apps.get_model("db", "Workspace")
+            .objects.get(
+                scope__name=settings.DEBUSINE_DEFAULT_SCOPE,
+                name=DEFAULT_WORKSPACE_NAME,
+            )
+            .public
+        )
+
+        executor.loader.build_graph()
+        executor.migrate(migrate_to)
+        new_apps = executor.loader.project_state(migrate_to).apps
+        self.assertTrue(
+            new_apps.get_model("db", "Workspace")
+            .objects.get(
+                scope__name=settings.DEBUSINE_DEFAULT_SCOPE,
+                name=DEFAULT_WORKSPACE_NAME,
+            )
+            .public
+        )
+
+    def test_create_singleton_collections(self) -> None:
+        """Singleton collections are created in the default workspace."""
+        migrate_from = [
+            ("db", "0094_collection_db_collection_name_not_reserved")
+        ]
+        migrate_to = [("db", "0095_create_singleton_collections")]
+        singleton_collection_categories = (
+            CollectionCategory.PACKAGE_BUILD_LOGS,
+            CollectionCategory.TASK_HISTORY,
+        )
+
+        executor = MigrationExecutor(
+            connection, progress_callback=_migrate_check_constraints
+        )
+        executor.migrate(migrate_from)
+        old_apps = executor.loader.project_state(migrate_from).apps
+        self.assertQuerySetEqual(
+            old_apps.get_model("db", "Collection")
+            .objects.filter(
+                name="_",
+                category__in=singleton_collection_categories,
+                workspace__scope__name=settings.DEBUSINE_DEFAULT_SCOPE,
+                workspace__name=DEFAULT_WORKSPACE_NAME,
+            )
+            .values_list("category", flat=True),
+            [],
+        )
+
+        executor.loader.build_graph()
+        executor.migrate(migrate_to)
+        new_apps = executor.loader.project_state(migrate_to).apps
+        self.assertQuerySetEqual(
+            new_apps.get_model("db", "Collection")
+            .objects.filter(
+                name="_",
+                category__in=singleton_collection_categories,
+                workspace__scope__name=settings.DEBUSINE_DEFAULT_SCOPE,
+                workspace__name=DEFAULT_WORKSPACE_NAME,
+            )
+            .values_list("category", flat=True),
+            singleton_collection_categories,
+            ordered=False,
+        )
+
+    def test_delete_dynamic_data_workflows(self) -> None:
+        """Dynamic data from workflows is set to None."""
+        migrate_from = [("db", "0098_artifact_original_artifact")]
+        migrate_to = [("db", "0099_delete_dynamic_data_workflows")]
+
+        executor = MigrationExecutor(
+            connection, progress_callback=_migrate_check_constraints
+        )
+        executor.migrate(migrate_from)
+        old_apps = executor.loader.project_state(migrate_from).apps
+
+        work_request = old_apps.get_model("db", "WorkRequest").objects.create(
+            task_type=TaskTypes.WORKFLOW,
+            task_name="piuparts",
+            created_by_id=self.get_test_user_for_apps(old_apps).id,
+            workspace=old_apps.get_model("db", "Workspace").objects.get(
+                name=DEFAULT_WORKSPACE_NAME
+            ),
+            task_data={
+                "purpose": KeyPurpose.UEFI,
+                "unsigned": [2],
+                "key": 1,
+            },
+            dynamic_task_data={
+                "source_artifact_id": 10,
+                "binary_artifacts_ids": [11, 12, 13],
+            },
+        )
+
+        work_request_id = work_request.id
+
+        executor.loader.build_graph()
+        executor.migrate(migrate_to)
+        new_apps = executor.loader.project_state(migrate_to).apps
+
+        work_request = new_apps.get_model("db", "WorkRequest").objects.get(
+            id=work_request_id
+        )
+
+        # Migration removed dynamic_task_data
+        self.assertIsNone(work_request.dynamic_task_data)
+
+        executor.loader.build_graph()
+
+        # Cannot reverse migrate if some WorkRequest is task_type == WORKFLOW
+        with self.assertRaises(IrreversibleError):
+            executor.migrate(migrate_from)
+
+        work_request.task_type = TaskTypes.WORKER
+        work_request.dynamic_task_data = {"source_artifact_id": 10}
+        work_request.save()
+
+        # If no WorkRequest of type WORKFLOW: reverse migration does nothing
+        executor.migrate(migrate_from)
+
+        work_request.refresh_from_db()
+
+        self.assertEqual(
+            work_request.dynamic_task_data, {"source_artifact_id": 10}
+        )
+
+    def test_wait_needs_input(self) -> None:
+        """`needs_input` is added to WAIT tasks."""
+        migrate_from = [("db", "0104_workspacerole_contributor")]
+        migrate_to = [("db", "0105_wait_needs_input")]
+
+        executor = MigrationExecutor(
+            connection, progress_callback=_migrate_check_constraints
+        )
+        executor.migrate(migrate_from)
+        old_apps = executor.loader.project_state(migrate_from).apps
+        workspace = old_apps.get_model("db", "Workspace").objects.get(
+            scope__name=settings.DEBUSINE_DEFAULT_SCOPE,
+            name=DEFAULT_WORKSPACE_NAME,
+        )
+        now = timezone.now()
+
+        old_work_requests = [
+            (TaskTypes.WORKER, "delay", {}, {}),
+            (
+                TaskTypes.WAIT,
+                "delay",
+                {"delay_until": now.isoformat()},
+                {"step": "soon"},
+            ),
+            (
+                TaskTypes.WAIT,
+                "delay",
+                {"delay_until": (now + timedelta(days=1)).isoformat()},
+                {"needs_input": True},
+            ),
+            (TaskTypes.WAIT, "externaldebsign", {"unsigned": 1}, {}),
+            (TaskTypes.WAIT, "waitnoop", {}, {}),
+        ]
+        new_work_requests = [
+            # Unchanged: not a WAIT task.
+            (TaskTypes.WORKER, "delay", {}, {}),
+            (
+                TaskTypes.WAIT,
+                "delay",
+                {"delay_until": now.isoformat()},
+                {"needs_input": False, "step": "soon"},
+            ),
+            # Unchanged: needs_input already set.
+            (
+                TaskTypes.WAIT,
+                "delay",
+                {"delay_until": (now + timedelta(days=1)).isoformat()},
+                {"needs_input": True},
+            ),
+            (
+                TaskTypes.WAIT,
+                "externaldebsign",
+                {"unsigned": 1},
+                {"needs_input": True},
+            ),
+            # Unchanged: not "delay" or "externaldebsign".
+            (TaskTypes.WAIT, "waitnoop", {}, {}),
+        ]
+
+        self.maxDiff = None
+        for task_type, task_name, task_data, workflow_data in old_work_requests:
+            old_apps.get_model("db", "WorkRequest").objects.create(
+                workspace=workspace,
+                created_by_id=self.get_test_user_for_apps(old_apps).id,
+                task_type=task_type,
+                task_name=task_name,
+                task_data=task_data,
+                workflow_data_json=workflow_data,
+            )
+
+        executor.loader.build_graph()
+        executor.migrate(migrate_to)
+        new_apps = executor.loader.project_state(migrate_to).apps
+
+        self.assertQuerySetEqual(
+            new_apps.get_model("db", "WorkRequest")
+            .objects.order_by("id")
+            .values_list(
+                "task_type", "task_name", "task_data", "workflow_data_json"
+            ),
+            new_work_requests,
+        )
+
+        executor.loader.build_graph()
+        executor.migrate(migrate_from)
+        old_apps = executor.loader.project_state(migrate_from).apps
+
+        self.assertQuerySetEqual(
+            old_apps.get_model("db", "WorkRequest")
+            .objects.order_by("id")
+            .values_list(
+                "task_type", "task_name", "task_data", "workflow_data_json"
+            ),
+            old_work_requests,
+        )
+
+    def test_fill_scope_label(self) -> None:
+        """Scope labels are initialized from scope names."""
+        migrate_from = [("db", "0107_add_scope_label_icon")]
+        migrate_to = [("db", "0108_fill_scope_label")]
+
+        executor = MigrationExecutor(
+            connection, progress_callback=_migrate_check_constraints
+        )
+        executor.migrate(migrate_from)
+        old_apps = executor.loader.project_state(migrate_from).apps
+        OldScope = old_apps.get_model("db", "Scope")
+
+        old_default_scope = OldScope.objects.create(name="unlabeled")
+        self.assertEqual(old_default_scope.label, "")
+
+        old_labeled_scope = OldScope.objects.create(
+            name="labeled", label="Labeled Scope"
+        )
+
+        executor.loader.build_graph()
+        executor.migrate(migrate_to)
+        new_apps = executor.loader.project_state(migrate_to).apps
+        NewScope = new_apps.get_model("db", "Scope")
+
+        self.assertEqual(
+            NewScope.objects.get(pk=old_default_scope.pk).label, "Unlabeled"
+        )
+        self.assertEqual(
+            NewScope.objects.get(pk=old_labeled_scope.pk).label, "Labeled Scope"
+        )
+
+        executor.loader.build_graph()
+        executor.migrate(migrate_from)
+
+        # Backwards migration does not reset labels that do not match
+        # automatically generated ones
+        self.assertEqual(
+            OldScope.objects.get(pk=old_default_scope.pk).label, ""
+        )
+        self.assertEqual(
+            OldScope.objects.get(pk=old_labeled_scope.pk).label, "Labeled Scope"
+        )
+
+    def test_workflow_last_activity_at(self) -> None:
+        """Test workflow last_activity_at is updated on migration."""
+        migrate_from = [("db", "0109_scope_label_required_and_unique")]
+        migrate_to = [("db", "0110_workrequest_workflow_last_activity_at")]
+
+        executor = MigrationExecutor(
+            connection, progress_callback=_migrate_check_constraints
+        )
+        executor.migrate(migrate_from)
+        old_apps = executor.loader.project_state(migrate_from).apps
+
+        OldWorkRequest = old_apps.get_model("db", "WorkRequest")
+
+        Workspace = old_apps.get_model("db", "Workspace")
+        Scope = old_apps.get_model("db", "Scope")
+        FileStore = old_apps.get_model("db", "FileStore")
+        User = old_apps.get_model("db", "User")
+
+        workspace = Workspace.objects.create(
+            name="Test",
+            scope=Scope.objects.first(),
+            default_file_store=FileStore.objects.first(),
+        )
+        user = User.objects.first()
+
+        old_workflow_root = OldWorkRequest.objects.create(
+            task_type=TaskTypes.WORKFLOW, workspace=workspace, created_by=user
+        )
+        OldWorkRequest.objects.create(
+            parent=old_workflow_root,
+            started_at=timezone.now(),
+            workspace=workspace,
+            created_by=user,
+        )
+        OldWorkRequest.objects.create(
+            parent=old_workflow_root,
+            started_at=timezone.now(),
+            completed_at=timezone.now(),
+            workspace=workspace,
+            created_by=user,
+        )
+        old_workflow_child_workflow = OldWorkRequest.objects.create(
+            parent=old_workflow_root,
+            started_at=timezone.now(),
+            completed_at=timezone.now(),
+            task_type=TaskTypes.WORKFLOW,
+            workspace=workspace,
+            created_by=user,
+        )
+        old_workflow_child_child = OldWorkRequest.objects.create(
+            parent=old_workflow_child_workflow,
+            started_at=timezone.now(),
+            completed_at=timezone.now(),
+            workspace=workspace,
+            created_by=user,
+        )
+
+        executor.loader.build_graph()
+        executor.migrate(migrate_to)
+        new_apps = executor.loader.project_state(migrate_to).apps
+        NewWorkRequest = new_apps.get_model("db", "WorkRequest")
+
+        self.assertEqual(
+            NewWorkRequest.objects.get(
+                pk=old_workflow_root.pk
+            ).workflow_last_activity_at,
+            old_workflow_child_child.completed_at,
+        )
+
+    def test_workflow_runtime_status(self) -> None:
+        """Test workflow_runtime_status is updated at migration."""
+        migrate_from = [("db", "0110_workrequest_workflow_last_activity_at")]
+        migrate_to = [("db", "0111_workrequest_workflow_runtime_status")]
+
+        executor = MigrationExecutor(
+            connection, progress_callback=_migrate_check_constraints
+        )
+        executor.migrate(migrate_from)
+        old_apps = executor.loader.project_state(migrate_from).apps
+
+        OldWorkRequest = old_apps.get_model("db", "WorkRequest")
+
+        Workspace = old_apps.get_model("db", "Workspace")
+        Scope = old_apps.get_model("db", "Scope")
+        FileStore = old_apps.get_model("db", "FileStore")
+        User = old_apps.get_model("db", "User")
+
+        workspace = Workspace.objects.create(
+            name="Test",
+            scope=Scope.objects.first(),
+            default_file_store=FileStore.objects.first(),
+        )
+        user = User.objects.first()
+
+        old_workflow_root = OldWorkRequest.objects.create(
+            task_type=TaskTypes.WORKFLOW, workspace=workspace, created_by=user
+        )
+        work_request = OldWorkRequest.objects.create(
+            parent=old_workflow_root,
+            started_at=timezone.now(),
+            workspace=workspace,
+            created_by=user,
+            status=WorkRequest.Statuses.BLOCKED,
+        )
+
+        executor.loader.build_graph()
+        executor.migrate(migrate_to)
+        new_apps = executor.loader.project_state(migrate_to).apps
+        NewWorkRequest = new_apps.get_model("db", "WorkRequest")
+
+        # Workflow's workflow_runtime_status is BLOCKED
+        self.assertEqual(
+            NewWorkRequest.objects.get(
+                pk=old_workflow_root.pk
+            ).workflow_runtime_status,
+            WorkRequest.RuntimeStatuses.BLOCKED,
+        )
+
+        # The children's workflow_runtime_status is None
+        self.assertIsNone(
+            NewWorkRequest.objects.get(
+                pk=work_request.pk
+            ).workflow_runtime_status
+        )
