@@ -9,24 +9,43 @@
 
 """Data models for db collections."""
 
+import re
 from datetime import datetime
 from functools import cached_property
-from typing import Any, Optional, TYPE_CHECKING, TypeAlias
+from typing import (
+    Any,
+    ClassVar,
+    Generic,
+    Optional,
+    TYPE_CHECKING,
+    TypeAlias,
+    TypeVar,
+)
 
 import jsonpath_rw
 from django.conf import settings
 from django.contrib.postgres.constraints import ExclusionConstraint
 from django.contrib.postgres.fields import RangeOperators
+from django.core.exceptions import ValidationError
 from django.db import models
 from django.db.models import CheckConstraint, F, Q, QuerySet, UniqueConstraint
+from django.db.models.fields.json import KT
 from django.urls import reverse
 
 from debusine.artifacts.models import (
+    ArtifactCategory,
     BareDataCategory,
     CollectionCategory,
     DebusinePromise,
+    SINGLETON_COLLECTION_CATEGORIES,
 )
 from debusine.db.constraints import JsonDataUniqueConstraint
+from debusine.db.context import context
+from debusine.db.models.permissions import (
+    PermissionUser,
+    permission_check,
+    permission_filter,
+)
 from debusine.db.models.workspaces import Workspace
 
 if TYPE_CHECKING:
@@ -39,6 +58,24 @@ if TYPE_CHECKING:
 else:
     TypedModelMeta = object
 
+A = TypeVar("A")
+
+#: Regexp matching the structure of collection names
+collection_name_regex = re.compile(r"^[A-Za-z][A-Za-z0-9+._-]*$")
+
+
+def is_valid_collection_name(value: str) -> bool:
+    """Check if value is a valid scope name."""
+    return bool(collection_name_regex.match(value))
+
+
+def validate_collection_name(value: str) -> None:
+    """Validate collection names."""
+    if not is_valid_collection_name(value):
+        raise ValidationError(
+            "%(value)r is not a valid collection name", params={"value": value}
+        )
+
 
 class _CollectionRetainsArtifacts(models.TextChoices):
     """Choices for Collection.retains_artifacts."""
@@ -48,16 +85,64 @@ class _CollectionRetainsArtifacts(models.TextChoices):
     ALWAYS = "always", "Always"
 
 
+class CollectionQuerySet(QuerySet["Collection", A], Generic[A]):
+    """Custom QuerySet for Collection."""
+
+    def in_current_scope(self) -> "CollectionQuerySet[A]":
+        """Filter to collections in the current scope."""
+        return self.filter(workspace__scope=context.require_scope())
+
+    def in_current_workspace(self) -> "CollectionQuerySet[A]":
+        """Filter to collections in the current workspace."""
+        return self.filter(workspace=context.require_workspace())
+
+    @permission_filter
+    def can_display(self, user: PermissionUser) -> "CollectionQuerySet[A]":
+        """Keep only Collections that can be displayed."""
+        # Delegate to workspace can_display check
+        return self.filter(workspace__in=Workspace.objects.can_display(user))
+
+
+class CollectionManager(models.Manager["Collection"]):
+    """Manager for Collection model."""
+
+    def get_queryset(self) -> CollectionQuerySet[Any]:
+        """Use the custom QuerySet."""
+        return CollectionQuerySet(self.model, using=self._db)
+
+    @staticmethod
+    def get_or_create_singleton(
+        category: CollectionCategory,
+        workspace: Workspace,
+        *,
+        data: dict[str, Any] | None = None,
+    ) -> "tuple[Collection, bool]":
+        """Create a singleton collection."""
+        if category not in SINGLETON_COLLECTION_CATEGORIES:
+            raise ValueError(
+                f"'{category}' is not a singleton collection category"
+            )
+        return Collection.objects.get_or_create(
+            name="_", category=category, workspace=workspace, data=data or {}
+        )
+
+
 class Collection(models.Model):
     """Model representing a collection."""
 
+    objects = CollectionManager.from_queryset(CollectionQuerySet)()
+
     RetainsArtifacts: TypeAlias = _CollectionRetainsArtifacts
 
-    name = models.CharField(max_length=255)
+    name = models.CharField(
+        max_length=255, validators=[validate_collection_name]
+    )
     category = models.CharField(max_length=255)
     full_history_retention_period = models.DurationField(null=True, blank=True)
     metadata_only_retention_period = models.DurationField(null=True, blank=True)
-    workspace = models.ForeignKey(Workspace, on_delete=models.PROTECT)
+    workspace = models.ForeignKey(
+        Workspace, on_delete=models.PROTECT, related_name="collections"
+    )
     retains_artifacts = models.CharField(
         max_length=8,
         choices=RetainsArtifacts.choices,
@@ -89,6 +174,16 @@ class Collection(models.Model):
             CheckConstraint(
                 check=~Q(name=""), name="%(app_label)s_%(class)s_name_not_empty"
             ),
+            CheckConstraint(
+                check=(
+                    (
+                        ~Q(category__in=SINGLETON_COLLECTION_CATEGORIES)
+                        & ~Q(name__startswith="_")
+                    )
+                    | Q(category__in=SINGLETON_COLLECTION_CATEGORIES, name="_")
+                ),
+                name="%(app_label)s_%(class)s_name_not_reserved",
+            ),
             CheckConstraint(
                 check=~Q(category=""),
                 name="%(app_label)s_%(class)s_category_not_empty",
@@ -114,9 +209,36 @@ class Collection(models.Model):
             },
         )
 
+    @permission_check("{user} cannot display collection {resource}")
+    def can_display(self, user: PermissionUser) -> bool:
+        """Check if the collection can be displayed."""
+        return self.workspace.can_display(user)
+
     def __str__(self) -> str:
         """Return id, name, category."""
-        return f"Id: {self.id} Name: {self.name} Category: {self.category}"
+        # Stringify using a valid lookup syntax
+        return f"{self.name}@{self.category}"
+
+
+class CollectionItemQuerySet(QuerySet["CollectionItem"]):
+    """Custom QuerySet for CollectionItem."""
+
+    def artifacts_in_collection(
+        self, parent_collection: Collection, category: ArtifactCategory
+    ) -> "CollectionItemQuerySet":
+        """
+        Filter to artifacts in a given parent collection.
+
+        This is tuned to use the indexes on :py:class:`CollectionItem`.
+        """
+        return self.filter(
+            parent_collection=parent_collection,
+            # Technically redundant with parent_collection, but clues
+            # PostgreSQL into using the correct index.
+            parent_category=parent_collection.category,
+            child_type=CollectionItem.Types.ARTIFACT,
+            category=category,
+        )
 
 
 class CollectionItemManager(models.Manager["CollectionItem"]):
@@ -190,7 +312,7 @@ class CollectionItemManager(models.Manager["CollectionItem"]):
             created_by_user=created_by_user,
         )
 
-    def drop_full_history(self, at: datetime):
+    def drop_full_history(self, at: datetime) -> None:
         """
         Drop artifacts from collections after full_history_retention_period.
 
@@ -238,8 +360,10 @@ class _CollectionItemTypes(models.TextChoices):
 class CollectionItem(models.Model):
     """CollectionItem model."""
 
-    objects = CollectionItemManager()
-    active_objects = CollectionItemActiveManager()
+    objects = CollectionItemManager.from_queryset(CollectionItemQuerySet)()
+    active_objects = CollectionItemActiveManager.from_queryset(
+        CollectionItemQuerySet
+    )()
 
     name = models.CharField(max_length=255)
 
@@ -303,9 +427,13 @@ class CollectionItem(models.Model):
 
     @staticmethod
     def expand_variables(
-        variables: dict[str, str], artifact: "Artifact"
+        variables: dict[str, str], reference_data: dict[Any, Any]
     ) -> dict[str, str]:
-        """Expand JSONPath variables against an Artifact's data."""
+        """
+        Expand JSONPath variables against some data.
+
+        The data will normally come from an Artifact.
+        """
         jsonpaths = {}
         for name, path in variables.items():
             if name.startswith("$"):
@@ -316,17 +444,17 @@ class CollectionItem(models.Model):
 
         expanded_variables = {}
         for name, jsonpath in jsonpaths.items():
-            matches = jsonpath.find(artifact.data)
+            matches = jsonpath.find(reference_data)
             if len(matches) == 1:
                 expanded_variables[name] = matches[0].value
             elif len(matches) > 1:
                 raise ValueError(
                     "Too many values expanding",
                     variables[f"${name}"],
-                    artifact.data,
+                    reference_data,
                 )
             else:
-                raise KeyError(variables[f"${name}"], artifact.data)
+                raise KeyError(variables[f"${name}"], reference_data)
 
         for name, value in variables.items():
             if not name.startswith("$"):
@@ -438,6 +566,30 @@ class CollectionItem(models.Model):
                 ),
             ),
         ]
+        indexes = [
+            models.Index(
+                F("parent_collection"),
+                KT("data__package"),
+                KT("data__version"),
+                name="%(app_label)s_ci_suite_source_idx",
+                condition=Q(
+                    parent_category=CollectionCategory.SUITE,
+                    child_type=_CollectionItemTypes.ARTIFACT,
+                    category=ArtifactCategory.SOURCE_PACKAGE,
+                ),
+            ),
+            models.Index(
+                F("parent_collection"),
+                KT("data__srcpkg_name"),
+                KT("data__srcpkg_version"),
+                name="%(app_label)s_ci_suite_binary_source_idx",
+                condition=Q(
+                    parent_category=CollectionCategory.SUITE,
+                    child_type=_CollectionItemTypes.ARTIFACT,
+                    category=ArtifactCategory.BINARY_PACKAGE,
+                ),
+            ),
+        ]
 
     def save(self, *args: Any, **kwargs: Any) -> None:
         """Populate `parent_category` on save."""
@@ -455,7 +607,9 @@ class CollectionItemMatchConstraint(models.Model):
     :py:attr:`value`.
     """
 
-    objects = models.Manager["CollectionItemMatchConstraint"]()
+    objects: ClassVar[models.Manager["CollectionItemMatchConstraint"]] = (
+        models.Manager["CollectionItemMatchConstraint"]()
+    )
 
     collection = models.ForeignKey(
         Collection,
