@@ -14,7 +14,7 @@ import os
 from django.core.exceptions import ImproperlyConfigured
 
 
-def read_secret_key(secret_key_file):
+def read_secret_key(secret_key_file: str | os.PathLike[str]) -> str:
     """
     Return the content of secret_key_file.
 
