@@ -20,15 +20,15 @@ class ServerConfig(AppConfig):
 
     name = 'debusine.server'
 
-    def ready(self):
+    def ready(self) -> None:
         """Mark workers as disconnected. Runs on Django startup."""
         # Import the scheduler so that its signal receiver is registered.
         import debusine.server.scheduler  # noqa: F401
 
-        # Import server-side and signing tasks so that they are registered.
+        # Import various tasks so that they are registered.
         import debusine.server.tasks  # noqa: F401
+        import debusine.server.tasks.wait  # noqa: F401
         import debusine.signing.tasks  # noqa: F401
-
         from debusine.db.models import Worker
 
         # The workers are disconnected (because the server is just loading).
