@@ -13,7 +13,6 @@ import logging
 from contextlib import ExitStack
 
 from celery import shared_task
-
 from django.db import transaction
 
 from debusine.db.models import WorkRequest
@@ -21,7 +20,6 @@ from debusine.server.tasks import BaseServerTask
 from debusine.tasks import TaskConfigError
 from debusine.tasks.models import TaskTypes
 
-
 logger = logging.getLogger(__name__)
 
 
@@ -29,7 +27,13 @@ class CeleryWorkerRequiresServerTask(Exception):
     """Only server tasks can run on Celery workers."""
 
 
-@shared_task
+class WorkRequestNotPending(Exception):
+    """We only run pending work requests."""
+
+
+# mypy complains that celery.shared_task is untyped, which is true, but we
+# can't fix that here.
+@shared_task  # type: ignore[misc]
 def run_server_task(work_request_id: int) -> bool:
     """Run a :class:`BaseTask` via Celery."""
     try:
@@ -37,6 +41,15 @@ def run_server_task(work_request_id: int) -> bool:
     except WorkRequest.DoesNotExist:
         logger.error("Work request %d does not exist", work_request_id)
         raise
+
+    if work_request.status != WorkRequest.Statuses.PENDING:
+        logger.error(
+            "Work request %d is in status %s, not pending",
+            work_request_id,
+            work_request.status,
+        )
+        raise WorkRequestNotPending
+
     task_name = work_request.task_name
     work_request.mark_running()
 
