@@ -14,6 +14,9 @@ from abc import ABC
 from collections.abc import Iterable
 from typing import Any, assert_never
 
+from django.contrib.auth.models import AnonymousUser
+from django.db.models import Q
+
 from debusine.artifacts.models import (
     ArtifactCategory,
     BareDataCategory,
@@ -25,7 +28,9 @@ from debusine.db.models import (
     CollectionItem,
     User,
     WorkRequest,
+    Workspace,
 )
+from debusine.tasks.models import LookupMultiple, LookupSingle
 
 
 class ItemAdditionError(Exception):
@@ -195,8 +200,10 @@ class CollectionManagerInterface(ABC):
         collection: Collection,
         *,
         user: User,
+        workflow: WorkRequest | None = None,
         variables: dict[str, Any] | None = None,
         name: str | None = None,
+        replace: bool = False,
     ) -> CollectionItem:
         """
         Add the collection to the managed collection.
@@ -207,11 +214,13 @@ class CollectionManagerInterface(ABC):
 
         :param collection: collection to add
         :param user: user adding the collection
+        :param workflow: workflow adding the collection to the collection
         :param variables: additional variables passed down to the specific
           collection's :py:meth:`do_add_collection`, which may be used to
           compute per-item data
         :param name: set name of new collection item (implementations may
           ignore this if they compute their own names)
+        :param replace: if True, replace an existing similar item
         """
         if (
             self.VALID_COLLECTION_CATEGORIES is not None
@@ -223,7 +232,12 @@ class CollectionManagerInterface(ABC):
             )
 
         return self.do_add_collection(
-            collection, user=user, variables=variables, name=name
+            collection,
+            user=user,
+            workflow=workflow,
+            variables=variables,
+            name=name,
+            replace=replace,
         )
 
     def remove_bare_data(
@@ -313,6 +327,36 @@ class CollectionManagerInterface(ABC):
                 return None
         return self.do_lookup(query)
 
+    def lookup_filter(
+        self,
+        key: str,
+        value: LookupSingle | LookupMultiple,
+        *,
+        workspace: Workspace,
+        user: User | AnonymousUser,
+        workflow_root: WorkRequest | None = None,
+    ) -> Q:
+        """
+        Return :py:class:`CollectionItem` conditions for a lookup filter.
+
+        :param key: name of the lookup filter
+        :param value: parameter passed to the specific lookup filter
+        :param workspace: workspace to perform the lookup in
+        :param user: user performing the lookup
+        :param workflow_root: root of the workflow performing the lookup, if
+          any
+        :return: a Django :py:class:`Q` object with conditions for this lookup
+          filter.
+        :raise LookupError: the query contains an unknown lookup filter type.
+        """
+        return self.do_lookup_filter(
+            key,
+            value,
+            workspace=workspace,
+            user=user,
+            workflow_root=workflow_root,
+        )
+
     def do_add_bare_data(
         self,
         category: BareDataCategory,  # noqa: U100
@@ -400,8 +444,10 @@ class CollectionManagerInterface(ABC):
         collection: Collection,  # noqa: U100
         *,
         user: User,  # noqa: U100
+        workflow: WorkRequest | None = None,  # noqa: U100
         variables: dict[str, Any] | None = None,  # noqa: U100
         name: str | None = None,  # noqa: U100
+        replace: bool = False,  # noqa: U100
     ) -> CollectionItem:
         """
         Add the collection into the managed collection.
@@ -444,6 +490,24 @@ class CollectionManagerInterface(ABC):
         """
         raise LookupError(f'Unexpected lookup format: "{query}"')
 
+    def do_lookup_filter(
+        self,
+        key: str,
+        value: LookupSingle | LookupMultiple,  # noqa: U100
+        *,
+        workspace: Workspace,  # noqa: U100
+        user: User | AnonymousUser,  # noqa: U100
+        workflow_root: WorkRequest | None = None,  # noqa: U100
+    ) -> Q:
+        """
+        Return :py:class:`CollectionItem` conditions for a lookup filter.
+
+        Called by lookup_filter().
+
+        :raise LookupError: the query contains an unknown lookup filter type.
+        """
+        raise LookupError(f'Unexpected lookup filter format: "{key}"')
+
     def do_remove_child_types(
         self,
         child_types: Iterable[CollectionItem.Types],
