@@ -14,11 +14,12 @@ from collections.abc import Sequence
 from dataclasses import dataclass
 from typing import Literal, assert_never, overload
 
-from django.db.models import Q, TextField
-from django.db.models.fields.json import KeyTextTransform
-from django.db.models.functions import Cast
+from django.contrib.auth.models import AnonymousUser
+from django.db.models import Q
+from django.db.models.fields.json import KT
 
-from debusine.artifacts.models import CollectionCategory
+from debusine.artifacts.models import BareDataCategory, CollectionCategory
+from debusine.client.models import LookupChildType
 from debusine.db.models import (
     Artifact,
     Collection,
@@ -28,7 +29,6 @@ from debusine.db.models import (
     Workspace,
 )
 from debusine.tasks.models import (
-    LookupChildType,
     LookupDict,
     LookupMultiple,
     LookupSingle,
@@ -83,7 +83,7 @@ def _lookup_single_first(
     lookup: str,
     workspace: Workspace,
     *,
-    user: User | None,
+    user: User | AnonymousUser,
     default_category: CollectionCategory | None = None,
     workflow_root: WorkRequest | None = None,
 ) -> LookupResult:
@@ -95,18 +95,22 @@ def _lookup_single_first(
     """
     try:
         if m := re.match(r"^([0-9]+)@artifacts$", lookup):
-            workspace_filter = Q(workspace=workspace) | Q(
-                workspace__public=True
-            )
+            # TODO: filter using the current scope?
+            # TODO: support user is None to mean the user in the current
+            #       context?
+            workspaces = Workspace.objects.can_display(user)
+            workspace_filter = Q(workspace__in=workspaces)
             visible_artifacts = Artifact.objects.filter(workspace_filter)
             return LookupResult(
                 result_type=CollectionItem.Types.ARTIFACT,
                 artifact=visible_artifacts.get(id=int(m.group(1))),
             )
         elif m := re.match(r"^([0-9]+)@collections$", lookup):
-            workspace_filter = Q(workspace=workspace) | Q(
-                workspace__public=True
-            )
+            # TODO: filter using the current scope?
+            # TODO: support user is None to mean the user in the current
+            #       context?
+            workspaces = Workspace.objects.can_display(user)
+            workspace_filter = Q(workspace__in=workspaces)
             visible_collections = Collection.objects.filter(workspace_filter)
             return LookupResult(
                 result_type=CollectionItem.Types.COLLECTION,
@@ -146,10 +150,10 @@ def lookup_single(
     lookup: LookupSingle,
     workspace: Workspace,
     *,
-    user: User | None,
+    user: User | AnonymousUser,
     default_category: CollectionCategory | None = None,
     workflow_root: WorkRequest | None = None,
-    expect_type: Literal[CollectionItem.Types.ARTIFACT],
+    expect_type: Literal[LookupChildType.ARTIFACT],
 ) -> LookupResultArtifact: ...
 
 
@@ -158,10 +162,10 @@ def lookup_single(
     lookup: LookupSingle,
     workspace: Workspace,
     *,
-    user: User | None,
+    user: User | AnonymousUser,
     default_category: CollectionCategory | None = None,
     workflow_root: WorkRequest | None = None,
-    expect_type: Literal[CollectionItem.Types.COLLECTION],
+    expect_type: Literal[LookupChildType.COLLECTION],
 ) -> LookupResultCollection: ...
 
 
@@ -170,10 +174,10 @@ def lookup_single(
     lookup: LookupSingle,
     workspace: Workspace,
     *,
-    user: User | None,
+    user: User | AnonymousUser,
     default_category: CollectionCategory | None = None,
     workflow_root: WorkRequest | None = None,
-    expect_type: CollectionItem.Types | None = None,
+    expect_type: LookupChildType = LookupChildType.ANY,
 ) -> LookupResult: ...
 
 
@@ -181,10 +185,10 @@ def lookup_single(
     lookup: LookupSingle,
     workspace: Workspace,
     *,
-    user: User | None,
+    user: User | AnonymousUser,
     default_category: CollectionCategory | None = None,
     workflow_root: WorkRequest | None = None,
-    expect_type: CollectionItem.Types | None = None,
+    expect_type: LookupChildType = LookupChildType.ANY,
 ) -> LookupResult:
     """
     Look up a collection item using a single lookup syntax.
@@ -195,9 +199,12 @@ def lookup_single(
     :raises LookupError: if the lookup is invalid in some way.
     """
     if isinstance(lookup, int):
-        if expect_type == CollectionItem.Types.ARTIFACT:
+        if expect_type in (
+            LookupChildType.ARTIFACT,
+            LookupChildType.ARTIFACT_OR_PROMISE,
+        ):
             lookup = f"{lookup}@artifacts"
-        elif expect_type == CollectionItem.Types.COLLECTION:
+        elif expect_type == LookupChildType.COLLECTION:
             lookup = f"{lookup}@collections"
         else:
             raise LookupError(
@@ -224,7 +231,11 @@ def lookup_single(
     for i, segment in enumerate(segments[1:], start=1):
         container_name = "/".join(segments[:i])
         if result.result_type != CollectionItem.Types.COLLECTION:
-            raise LookupError(f"{container_name!r} is not of type 'collection'")
+            raise LookupError(
+                f"{container_name!r} is of type"
+                f" {result.result_type.name.lower()!r}"
+                " instead of expected 'collection'"
+            )
         container = result.collection
         assert container is not None
         if ":" not in segment:
@@ -239,10 +250,21 @@ def lookup_single(
             collection=item.collection,
         )
 
-    if expect_type is not None and result.result_type != expect_type:
-        raise LookupError(
-            f"{lookup!r} is not of type {expect_type.name.lower()!r}"
-        )
+    if expect_type != LookupChildType.ANY:
+        expected_result_types = {
+            LookupChildType.BARE: [CollectionItem.Types.BARE],
+            LookupChildType.ARTIFACT: [CollectionItem.Types.ARTIFACT],
+            LookupChildType.ARTIFACT_OR_PROMISE: [
+                CollectionItem.Types.ARTIFACT,
+                CollectionItem.Types.BARE,
+            ],
+            LookupChildType.COLLECTION: [CollectionItem.Types.COLLECTION],
+        }[expect_type]
+        if result.result_type not in expected_result_types:
+            raise LookupError(
+                f"{lookup!r} is of type {result.result_type.name.lower()!r}"
+                f" instead of expected {expect_type.name.lower()!r}"
+            )
 
     return result
 
@@ -251,47 +273,65 @@ def _lookup_dict(
     lookup: LookupDict,
     workspace: Workspace,
     *,
-    user: User | None,
+    user: User | AnonymousUser,
     default_category: CollectionCategory | None = None,
     workflow_root: WorkRequest | None = None,
-    expect_type: CollectionItem.Types | None = None,
+    expect_type: LookupChildType = LookupChildType.ANY,
 ) -> list[LookupResult]:
     """
     Look up collection items using the dictionary lookup syntax.
 
     See :ref:`lookup-dict`.
     """
+    if (
+        (expect_type == LookupChildType.ANY)
+        or (expect_type == lookup.child_type)
+        or (
+            expect_type == LookupChildType.ARTIFACT_OR_PROMISE
+            and lookup.child_type
+            in (LookupChildType.ARTIFACT, LookupChildType.BARE)
+        )
+    ):
+        # Valid request
+        pass
+    else:
+        raise LookupError(
+            f"Only lookups for type {expect_type.name.lower()!r} are allowed "
+            f"here"
+        )
+
     # Find the containing collection.
-    try:
-        collection = lookup_single(
-            lookup.collection,
-            workspace,
-            user=user,
-            default_category=default_category,
-            workflow_root=workflow_root,
-            expect_type=CollectionItem.Types.COLLECTION,
-        ).collection
-    except KeyError:
-        return []
+    collection = lookup_single(
+        lookup.collection,
+        workspace,
+        user=user,
+        default_category=default_category,
+        workflow_root=workflow_root,
+        expect_type=LookupChildType.COLLECTION,
+    ).collection
     objects = CollectionItem.active_objects.filter(parent_collection=collection)
 
     # Prepare query conditions.  We don't need to check the workspace here;
     # it's good enough if the item is in a collection we can see.
     match lookup.child_type:
         case LookupChildType.BARE:
-            child_type = CollectionItem.Types.BARE
+            objects = objects.filter(child_type=CollectionItem.Types.BARE)
         case LookupChildType.ARTIFACT:
-            child_type = CollectionItem.Types.ARTIFACT
+            objects = objects.filter(child_type=CollectionItem.Types.ARTIFACT)
+        case LookupChildType.ARTIFACT_OR_PROMISE:
+            objects = objects.filter(
+                Q(child_type=CollectionItem.Types.ARTIFACT)
+                | Q(
+                    child_type=CollectionItem.Types.BARE,
+                    category=BareDataCategory.PROMISE,
+                )
+            )
         case LookupChildType.COLLECTION:
-            child_type = CollectionItem.Types.COLLECTION
+            objects = objects.filter(child_type=CollectionItem.Types.COLLECTION)
+        case LookupChildType.ANY:
+            pass
         case _ as unreachable:
             assert_never(unreachable)
-    if expect_type is not None and expect_type != child_type:
-        raise LookupError(
-            f"Only lookups for type {expect_type.name.lower()!r} are allowed "
-            f"here"
-        )
-    objects = objects.filter(child_type=child_type)
     if lookup.category is not None:
         objects = objects.filter(category=lookup.category)
     if lookup.name_matcher is not None:
@@ -300,9 +340,19 @@ def _lookup_dict(
         )
     for key, matcher in lookup.data_matchers:
         annotation = f"data_text_{key}"
-        objects = objects.annotate(
-            **{annotation: Cast(KeyTextTransform(key, "data"), TextField())}
-        ).filter(**{f"{annotation}__{matcher.kind}": matcher.value})
+        objects = objects.annotate(**{annotation: KT(f"data__{key}")}).filter(
+            **{f"{annotation}__{matcher.kind}": matcher.value}
+        )
+    for key, value in lookup.lookup_filters:
+        objects = objects.filter(
+            collection.manager.lookup_filter(
+                key,
+                value,
+                workspace=workspace,
+                user=user,
+                workflow_root=workflow_root,
+            )
+        )
     objects = objects.select_related("artifact", "collection")
 
     # Execute the query.
@@ -322,10 +372,10 @@ def lookup_multiple(
     lookup: LookupMultiple,
     workspace: Workspace,
     *,
-    user: User | None,
+    user: User | AnonymousUser,
     default_category: CollectionCategory | None = None,
     workflow_root: WorkRequest | None = None,
-    expect_type: Literal[CollectionItem.Types.ARTIFACT],
+    expect_type: Literal[LookupChildType.ARTIFACT],
 ) -> Sequence[LookupResultArtifact]: ...
 
 
@@ -334,10 +384,10 @@ def lookup_multiple(
     lookup: LookupMultiple,
     workspace: Workspace,
     *,
-    user: User | None,
+    user: User | AnonymousUser,
     default_category: CollectionCategory | None = None,
     workflow_root: WorkRequest | None = None,
-    expect_type: Literal[CollectionItem.Types.COLLECTION],
+    expect_type: Literal[LookupChildType.COLLECTION],
 ) -> Sequence[LookupResultCollection]: ...
 
 
@@ -346,10 +396,10 @@ def lookup_multiple(
     lookup: LookupMultiple,
     workspace: Workspace,
     *,
-    user: User | None,
+    user: User | AnonymousUser,
     default_category: CollectionCategory | None = None,
     workflow_root: WorkRequest | None = None,
-    expect_type: CollectionItem.Types | None = None,
+    expect_type: LookupChildType = LookupChildType.ANY,
 ) -> Sequence[LookupResult]: ...
 
 
@@ -357,35 +407,33 @@ def lookup_multiple(
     lookup: LookupMultiple,
     workspace: Workspace,
     *,
-    user: User | None,
+    user: User | AnonymousUser,
     default_category: CollectionCategory | None = None,
     workflow_root: WorkRequest | None = None,
-    expect_type: CollectionItem.Types | None = None,
+    expect_type: LookupChildType = LookupChildType.ANY,
 ) -> Sequence[LookupResult]:
     """
     Look up collection items using a multiple lookup syntax.
 
     See :ref:`lookup-multiple`.
 
+    :raises KeyError: if any of the lookups does not resolve to an item.
     :raises LookupError: if the lookup is invalid in some way.
     """
     results: list[LookupResult] = []
-    for alternative in lookup.__root__:
+    for alternative in lookup:
         match alternative:
             case int() | str():
-                try:
-                    results.append(
-                        lookup_single(
-                            alternative,
-                            workspace,
-                            user=user,
-                            default_category=default_category,
-                            workflow_root=workflow_root,
-                            expect_type=expect_type,
-                        )
+                results.append(
+                    lookup_single(
+                        alternative,
+                        workspace,
+                        user=user,
+                        default_category=default_category,
+                        workflow_root=workflow_root,
+                        expect_type=expect_type,
                     )
-                except KeyError:
-                    pass
+                )
             case LookupDict():
                 results.extend(
                     _lookup_dict(
