@@ -8,7 +8,6 @@
 # contained in the LICENSE file.
 
 """Unit tests for the CollectionManagerInterface."""
-from typing import Any
 from unittest.mock import patch
 
 from django.contrib.auth import get_user_model
@@ -18,11 +17,10 @@ from debusine.artifacts.models import (
     BareDataCategory,
     CollectionCategory,
 )
+from debusine.db.context import context
 from debusine.db.models import (
-    Artifact,
     Collection,
     CollectionItem,
-    User,
     WorkRequest,
     default_workspace,
 )
@@ -34,78 +32,11 @@ from debusine.server.collections import (
 from debusine.test.django import TestCase
 
 
-class TestInstance(CollectionManagerInterface):
-    """Instance used for testing."""
+class TestManager(CollectionManagerInterface):
+    """Collection manager used for testing."""
 
     COLLECTION_CATEGORY = CollectionCategory.TEST
 
-    def do_add_bare_data(
-        self,
-        category: BareDataCategory,
-        *,
-        user: User,
-        workflow: WorkRequest | None = None,
-        data: dict[str, Any] | None = None,
-        name: str | None = None,
-        replace: bool = False,
-    ) -> CollectionItem:
-        """Raise NotImplementedError."""
-        raise NotImplementedError()
-
-    def do_remove_bare_data(
-        self,
-        name: str,
-        *,
-        user: User | None = None,
-        workflow: WorkRequest | None = None,
-    ) -> None:
-        """Raise NotImplementedError."""
-        raise NotImplementedError()
-
-    def do_add_artifact(
-        self,
-        artifact: Artifact,
-        *,
-        user: User,
-        workflow: WorkRequest | None = None,
-        variables: dict[str, Any] | None = None,
-        name: str | None = None,
-        replace: bool = False,
-    ) -> CollectionItem:
-        """Raise NotImplementedError."""
-        raise NotImplementedError()
-
-    def do_remove_artifact(
-        self,
-        artifact: Artifact,
-        *,
-        user: User | None = None,
-        workflow: WorkRequest | None = None,
-    ) -> None:
-        """Raise NotImplementedError."""
-        raise NotImplementedError()
-
-    def do_add_collection(
-        self,
-        collection: Collection,
-        *,
-        user: User,
-        variables: dict[str, Any] | None = None,
-        name: str | None = None,
-    ) -> CollectionItem:
-        """Raise NotImplementedError."""
-        raise NotImplementedError
-
-    def do_remove_collection(
-        self,
-        collection: Collection,
-        *,
-        user: User | None = None,
-        workflow: WorkRequest | None = None,
-    ) -> None:
-        """Raise NotImplementedError."""
-        raise NotImplementedError
-
 
 class CollectionManagerInterfaceTests(TestCase):
     """Tests for CollectionManagerInterface."""
@@ -130,7 +61,7 @@ class CollectionManagerInterfaceTests(TestCase):
             category=CollectionCategory.TEST,
             workspace=self.workspace,
         )
-        self.manager = TestInstance(self.collection)
+        self.manager = TestManager(self.collection)
 
     def test_add_bare_data(self) -> None:
         """Test add_bare_data() call do_add_bare_data()."""
@@ -183,6 +114,7 @@ class CollectionManagerInterfaceTests(TestCase):
             replace=False,
         )
 
+    @context.disable_permission_checks()
     def test_add_artifact(self) -> None:
         """Test add_artifact() call do_add_artifact()."""
         category = ArtifactCategory.TEST
@@ -202,6 +134,7 @@ class CollectionManagerInterfaceTests(TestCase):
             replace=False,
         )
 
+    @context.disable_permission_checks()
     def test_add_artifact_invalid_category_raise_item_addition_error(
         self,
     ) -> None:
@@ -223,6 +156,7 @@ class CollectionManagerInterfaceTests(TestCase):
             user=self.user,
         )
 
+    @context.disable_permission_checks()
     def test_add_artifact_any_category_valid(self) -> None:
         """If VALID_ARTIFACT_CATEGORIES is None, any category is valid."""
         category = ArtifactCategory.TEST
@@ -255,7 +189,12 @@ class CollectionManagerInterfaceTests(TestCase):
             self.manager.add_collection(collection, user=self.user)
 
         mocked.assert_called_with(
-            collection, user=self.user, variables=None, name=None
+            collection,
+            user=self.user,
+            workflow=None,
+            variables=None,
+            name=None,
+            replace=False,
         )
 
     def test_add_collection_invalid_category_raise_item_addition_error(
@@ -291,7 +230,12 @@ class CollectionManagerInterfaceTests(TestCase):
             self.manager.add_collection(collection, user=self.user)
 
         mocked.assert_called_with(
-            collection, user=self.user, variables=None, name=None
+            collection,
+            user=self.user,
+            workflow=None,
+            variables=None,
+            name=None,
+            replace=False,
         )
 
     def test_remove_bare_data(self) -> None:
@@ -307,6 +251,7 @@ class CollectionManagerInterfaceTests(TestCase):
             "test", user=self.user, workflow=self.workflow
         )
 
+    @context.disable_permission_checks()
     def test_remove_artifact(self) -> None:
         """Test remove_artifact() call do_remove_artifact()."""
         artifact, _ = self.create_artifact()
@@ -333,6 +278,7 @@ class CollectionManagerInterfaceTests(TestCase):
             collection, user=self.user, workflow=self.workflow
         )
 
+    @context.disable_permission_checks()
     def test_do_remove_child_types_artifact(self) -> None:
         """Test do_remove_child_types() call do_remove_artifact()."""
         artifact, _ = self.create_artifact()
@@ -409,6 +355,18 @@ class CollectionManagerInterfaceTests(TestCase):
         """`lookup` returns None if there are no matches."""
         self.assertIsNone(self.manager.lookup("name:nonexistent"))
 
+    def test_lookup_filter_unexpected_format_raise_lookup_error(self) -> None:
+        """`lookup_filter` raises `LookupError`: invalid format."""
+        with self.assertRaisesRegex(
+            LookupError, '^Unexpected lookup filter format: "nonsense"$'
+        ):
+            self.manager.lookup_filter(
+                "nonsense",
+                "foo",
+                workspace=self.playground.get_default_workspace(),
+                user=self.playground.get_default_user(),
+            )
+
     def test_get_manager_for(self) -> None:
         """Test getting specialized manager."""
         collection = Collection.objects.create(
