@@ -11,25 +11,25 @@
 
 from datetime import datetime
 from operator import itemgetter
-from typing import Any
+from typing import Any, ClassVar
 
 from django.contrib.auth import get_user_model
-from django.test import TestCase
 
 from debusine.artifacts.models import ArtifactCategory, CollectionCategory
+from debusine.db.context import context
 from debusine.db.models import (
     Artifact,
     Collection,
     CollectionItem,
     CollectionItemMatchConstraint,
-    File,
     FileInArtifact,
+    User,
+    Workspace,
     default_workspace,
 )
-from debusine.db.tests.utils import _calculate_hash_from_data
 from debusine.server.collections import DebianSuiteManager, ItemAdditionError
 from debusine.server.collections.debian_suite import make_pool_filename
-from debusine.test import TestHelpersMixin
+from debusine.test.django import TestCase
 
 
 class MakePoolFilenameTests(TestCase):
@@ -50,24 +50,32 @@ class MakePoolFilenameTests(TestCase):
         )
 
 
-class DebianSuiteManagerTests(TestHelpersMixin, TestCase):
+class DebianSuiteManagerTests(TestCase):
     """Tests for DebianSuiteManager."""
 
-    def setUp(self) -> None:
+    user: ClassVar[User]
+    workspace: ClassVar[Workspace]
+    collection: ClassVar[Collection]
+    manager: ClassVar[DebianSuiteManager]
+
+    @classmethod
+    def setUpTestData(cls) -> None:
         """Set up tests."""
-        self.user = get_user_model().objects.create_user(
+        super().setUpTestData()
+
+        cls.user = get_user_model().objects.create_user(
             username="John", email="john@example.org"
         )
 
-        self.workspace = default_workspace()
+        cls.workspace = default_workspace()
 
-        self.collection = Collection.objects.create(
+        cls.collection = Collection.objects.create(
             name="Debian",
             category=CollectionCategory.SUITE,
-            workspace=self.workspace,
+            workspace=cls.workspace,
         )
 
-        self.manager = DebianSuiteManager(collection=self.collection)
+        cls.manager = DebianSuiteManager(collection=cls.collection)
 
     def create_source_package(
         self, name: str, version: str, paths: list[str]
@@ -154,6 +162,7 @@ class DebianSuiteManagerTests(TestHelpersMixin, TestCase):
         with self.assertRaisesRegex(ValueError, msg):
             DebianSuiteManager(collection)
 
+    @context.disable_permission_checks()
     def test_add_source_package(self) -> None:
         """`add_source_package` adds a source package to the suite."""
         source_package_artifact = self.create_source_package(
@@ -202,6 +211,7 @@ class DebianSuiteManagerTests(TestHelpersMixin, TestCase):
             ],
         )
 
+    @context.disable_permission_checks()
     def test_add_source_package_wrong_category(self) -> None:
         """`add_source_package` requires `debian:source-package` artifacts."""
         artifact, _ = self.create_artifact(
@@ -217,7 +227,8 @@ class DebianSuiteManagerTests(TestHelpersMixin, TestCase):
                 artifact, user=self.user, component="main", section="devel"
             )
 
-    def test_add_source_package_replace(self):
+    @context.disable_permission_checks()
+    def test_add_source_package_replace(self) -> None:
         """`add_source_package` can replace an existing artifact."""
         self.collection.data["may_reuse_versions"] = True
         source_package_artifact = self.create_source_package(
@@ -248,7 +259,8 @@ class DebianSuiteManagerTests(TestHelpersMixin, TestCase):
         self.assertEqual(collection_item2.artifact, source_package_artifact2)
         self.assertIsNone(collection_item2.removed_at)
 
-    def test_add_source_package_replace_nonexistent(self):
+    @context.disable_permission_checks()
+    def test_add_source_package_replace_nonexistent(self) -> None:
         """Replacing a nonexistent source package is allowed."""
         source_package_artifact = self.create_source_package(
             "hello", "1.0", ["hello_1.0.dsc", "hello_1.0.tar.xz"]
@@ -265,6 +277,7 @@ class DebianSuiteManagerTests(TestHelpersMixin, TestCase):
         self.assertEqual(collection_item.name, "hello_1.0")
         self.assertEqual(collection_item.artifact, source_package_artifact)
 
+    @context.disable_permission_checks()
     def test_add_binary_package(self) -> None:
         """`add_binary_package` adds a binary package to the suite."""
         binary_package_artifact = self.create_binary_package(
@@ -323,6 +336,7 @@ class DebianSuiteManagerTests(TestHelpersMixin, TestCase):
             ],
         )
 
+    @context.disable_permission_checks()
     def test_add_binary_package_wrong_category(self) -> None:
         """`add_binary_package` requires `debian:binary-package` artifacts."""
         artifact, _ = self.create_artifact(
@@ -342,7 +356,8 @@ class DebianSuiteManagerTests(TestHelpersMixin, TestCase):
                 priority="optional",
             )
 
-    def test_add_binary_package_replace(self):
+    @context.disable_permission_checks()
+    def test_add_binary_package_replace(self) -> None:
         """`add_binary_package` can replace an existing artifact."""
         self.collection.data["may_reuse_versions"] = True
         binary_package_artifact = self.create_binary_package(
@@ -385,7 +400,8 @@ class DebianSuiteManagerTests(TestHelpersMixin, TestCase):
         self.assertEqual(collection_item2.artifact, binary_package_artifact2)
         self.assertIsNone(collection_item2.removed_at)
 
-    def test_add_binary_package_replace_nonexistent(self):
+    @context.disable_permission_checks()
+    def test_add_binary_package_replace_nonexistent(self) -> None:
         """Replacing a nonexistent binary package is allowed."""
         binary_package_artifact = self.create_binary_package(
             "hello",
@@ -408,6 +424,7 @@ class DebianSuiteManagerTests(TestHelpersMixin, TestCase):
         self.assertEqual(collection_item.name, "libhello1_1:1.0_amd64")
         self.assertEqual(collection_item.artifact, binary_package_artifact)
 
+    @context.disable_permission_checks()
     def test_do_add_artifact_source_package_requires_component(self) -> None:
         """`do_add_artifact` requires `component` for source packages."""
         source_package_artifact = self.create_source_package("hello", "1.0", [])
@@ -419,6 +436,7 @@ class DebianSuiteManagerTests(TestHelpersMixin, TestCase):
                 source_package_artifact, user=self.user
             )
 
+    @context.disable_permission_checks()
     def test_do_add_artifact_source_package_requires_section(self) -> None:
         """`do_add_artifact` requires `section` for source packages."""
         source_package_artifact = self.create_source_package("hello", "1.0", [])
@@ -432,6 +450,7 @@ class DebianSuiteManagerTests(TestHelpersMixin, TestCase):
                 variables={"component": "main"},
             )
 
+    @context.disable_permission_checks()
     def test_do_add_artifact_binary_package_requires_component(self) -> None:
         """`do_add_artifact` requires `component` for binary packages."""
         binary_package_artifact = self.create_binary_package(
@@ -445,6 +464,7 @@ class DebianSuiteManagerTests(TestHelpersMixin, TestCase):
                 binary_package_artifact, user=self.user
             )
 
+    @context.disable_permission_checks()
     def test_do_add_artifact_binary_package_requires_section(self) -> None:
         """`do_add_artifact` requires `section` for binary packages."""
         binary_package_artifact = self.create_binary_package(
@@ -460,6 +480,7 @@ class DebianSuiteManagerTests(TestHelpersMixin, TestCase):
                 variables={"component": "main"},
             )
 
+    @context.disable_permission_checks()
     def test_do_add_artifact_binary_package_requires_priority(self) -> None:
         """`do_add_artifact` requires `priority` for binary packages."""
         binary_package_artifact = self.create_binary_package(
@@ -476,6 +497,7 @@ class DebianSuiteManagerTests(TestHelpersMixin, TestCase):
                 variables={"component": "main", "section": "devel"},
             )
 
+    @context.disable_permission_checks()
     def test_do_add_artifact_overlapping_files_same_contents(self) -> None:
         """`do_add_artifact` allows same paths with same contents."""
         source_package_artifacts: list[Artifact] = []
@@ -488,14 +510,11 @@ class DebianSuiteManagerTests(TestHelpersMixin, TestCase):
                 "hello_1.0.orig.tar.xz",
             ):
                 contents = f"data for {path}".encode()
-                digest = _calculate_hash_from_data(contents)
-                file, _ = File.get_or_create(
-                    hash_digest=digest, size=len(contents)
-                )
+                file = self.create_file(contents)
                 FileInArtifact.objects.create(
                     artifact=artifact, path=path, file=file
                 )
-                path_digests.append((path, digest.hex()))
+                path_digests.append((path, file.hash_digest.hex()))
             source_package_artifacts.append(artifact)
 
         collection_items = [
@@ -538,6 +557,7 @@ class DebianSuiteManagerTests(TestHelpersMixin, TestCase):
             ],
         )
 
+    @context.disable_permission_checks()
     def test_do_add_artifact_overlapping_files_different_contents(self) -> None:
         """`do_add_artifact` forbids same paths with different contents."""
         source_package_artifacts: list[Artifact] = []
@@ -549,10 +569,7 @@ class DebianSuiteManagerTests(TestHelpersMixin, TestCase):
                 "hello_1.0.orig.tar.xz",
             ):
                 contents = f"data for {path} at {version}".encode()
-                digest = _calculate_hash_from_data(contents)
-                file, _ = File.get_or_create(
-                    hash_digest=digest, size=len(contents)
-                )
+                file = self.create_file(contents)
                 FileInArtifact.objects.create(
                     artifact=artifact, path=path, file=file
                 )
@@ -574,6 +591,7 @@ class DebianSuiteManagerTests(TestHelpersMixin, TestCase):
                 section="devel",
             )
 
+    @context.disable_permission_checks()
     def test_do_remove_artifact(self) -> None:
         """`do_remove_artifact` removes the artifact."""
         source_package_artifact = self.create_source_package(
@@ -622,6 +640,7 @@ class DebianSuiteManagerTests(TestHelpersMixin, TestCase):
             ],
         )
 
+    @context.disable_permission_checks()
     def test_do_remove_artifact_reuse_versions(self) -> None:
         """If `may_reuse_versions=False`, versions may be reused."""
         self.collection.data["may_reuse_versions"] = True
@@ -661,6 +680,7 @@ class DebianSuiteManagerTests(TestHelpersMixin, TestCase):
         self.assertIsNone(self.manager.lookup("binary:hello_amd64"))
         self.assertIsNone(self.manager.lookup("name:nonexistent"))
 
+    @context.disable_permission_checks()
     def test_lookup_source(self) -> None:
         """`lookup` returns a matching source package item."""
         items: list[CollectionItem] = []
@@ -722,6 +742,7 @@ class DebianSuiteManagerTests(TestHelpersMixin, TestCase):
         self.assertEqual(self.manager.lookup("name:base-files_1.0"), items[2])
         self.assertIsNone(self.manager.lookup("name:hello_1.2"))
 
+    @context.disable_permission_checks()
     def test_lookup_binary(self) -> None:
         """`lookup` returns a matching binary package item."""
         items: list[CollectionItem] = []
