@@ -17,10 +17,10 @@ from debusine.server.collections import (
     DebianSuiteLintianManager,
     DebianSuiteManager,
 )
-from debusine.test import TestHelpersMixin
+from debusine.test.django import TestCase
 
 
-class CollectionTestMixin(TestHelpersMixin):
+class CollectionTestMixin(TestCase):
     """Utility functions to create test collections."""
 
     def create_source_package(
@@ -45,7 +45,7 @@ class CollectionTestMixin(TestHelpersMixin):
         self, manager: DebianSuiteManager, name: str, version: str
     ) -> CollectionItem:
         """Create a minimal source package collection item."""
-        user = self.get_test_user()
+        user = self.playground.get_default_user()
         return manager.add_source_package(
             self.create_source_package(name, version),
             user=user,
@@ -90,7 +90,7 @@ class CollectionTestMixin(TestHelpersMixin):
         architecture: str,
     ) -> CollectionItem:
         """Create a minimal source package collection item."""
-        user = self.get_test_user()
+        user = self.playground.get_default_user()
         return manager.add_binary_package(
             self.create_binary_package(
                 srcpkg_name, srcpkg_version, name, version, architecture
@@ -125,7 +125,7 @@ class CollectionTestMixin(TestHelpersMixin):
     ) -> CollectionItem:
         """Create a minimal Lintian collection item."""
         assert related_item.artifact is not None
-        user = self.get_test_user()
+        user = self.playground.get_default_user()
         return manager.add_artifact(
             self.create_lintian_artifact(related_item.artifact),
             user=user,
