@@ -10,23 +10,28 @@
 """Tests for ExternalDebianSuiteFileBackend."""
 
 from pathlib import Path
-
-from django.test import TestCase
+from typing import ClassVar
 
 import responses
 
 from debusine.db.models import FileStore
 from debusine.server.file_backend.external import ExternalDebianSuiteFileBackend
-from debusine.test import TestHelpersMixin
+from debusine.test.django import TestCase
 from debusine.utils import NotSupportedError
 
 
-class ExternalDebianSuiteFileBackendTests(TestHelpersMixin, TestCase):
+class ExternalDebianSuiteFileBackendTests(TestCase):
     """Tests for ExternalDebianSuiteFileBackend."""
 
-    def setUp(self) -> None:
+    db_store: ClassVar[FileStore]
+    file_backend: ClassVar[ExternalDebianSuiteFileBackend]
+
+    @classmethod
+    def setUpTestData(cls) -> None:
         """Initialize file backend to be tested."""
-        self.db_store = FileStore.objects.create(
+        super().setUpTestData()
+
+        cls.db_store = FileStore.objects.create(
             name="ExternalDebianSuite",
             backend=FileStore.BackendChoices.EXTERNAL_DEBIAN_SUITE,
             configuration={
@@ -36,7 +41,7 @@ class ExternalDebianSuiteFileBackendTests(TestHelpersMixin, TestCase):
             },
         )
 
-        self.file_backend = ExternalDebianSuiteFileBackend(self.db_store)
+        cls.file_backend = ExternalDebianSuiteFileBackend(cls.db_store)
 
     @responses.activate
     def test_remote_file(self) -> None:  # pragma: no cover
