@@ -11,29 +11,41 @@
 
 import logging
 from pathlib import Path
+from typing import ClassVar
 from unittest import mock
 
 from django.conf import settings
-from django.test import TestCase
 
-from debusine.db.models import File, FileInStore, FileStore, Workspace
+from debusine.db.models import (
+    Collection,
+    File,
+    FileInStore,
+    FileStore,
+    Workspace,
+)
 from debusine.db.tests.utils import _calculate_hash_from_data
 from debusine.server.file_backend.local import LocalFileBackend
-from debusine.test import TestHelpersMixin
+from debusine.test.django import TestCase
 
 
-class LocalFileBackendTests(TestHelpersMixin, TestCase):
+class LocalFileBackendTests(TestCase):
     """Tests for LocalFileBackend."""
 
+    default_file_store: ClassVar[FileStore]
+    file_backend: ClassVar[LocalFileBackend]
+
     playground_memory_file_store = False
 
-    def setUp(self) -> None:
+    @classmethod
+    def setUpTestData(cls) -> None:
         """Initialize file backend to be tested."""
-        self.default_file_store = FileStore.default()
-        self.file_backend = LocalFileBackend(self.default_file_store)
+        super().setUpTestData()
+        cls.default_file_store = FileStore.default()
+        cls.file_backend = LocalFileBackend(cls.default_file_store)
 
     def test_init_set_base_directory_default(self) -> None:
         """Test LocalFileBackend use setting's directory for Default store."""
+        Collection.objects.filter(name="_").delete()
         Workspace.objects.all().delete()
         self.default_file_store.delete()
 
