@@ -11,9 +11,10 @@
 
 import argparse
 import sys
+from typing import Any, NoReturn
 
-from django.core.management import CommandError
-from django.db import IntegrityError
+from django.core.exceptions import ValidationError
+from django.core.management import CommandError, CommandParser
 
 from debusine.db.models import Collection, DEFAULT_WORKSPACE_NAME, Workspace
 from debusine.django.management.debusine_base_command import DebusineBaseCommand
@@ -24,7 +25,7 @@ class Command(DebusineBaseCommand):
 
     help = "Create a new collection"
 
-    def add_arguments(self, parser):
+    def add_arguments(self, parser: CommandParser) -> None:
         """Add CLI arguments for the create_collection command."""
         parser.add_argument("name", help="Name")
         parser.add_argument("category", help="Category")
@@ -41,12 +42,12 @@ class Command(DebusineBaseCommand):
             default="-",
         )
 
-    def cleanup_arguments(self, *args, **options):
+    def cleanup_arguments(self, *args: Any, **options: Any) -> None:
         """Clean up objects created by parsing arguments."""
         if options["data"] != sys.stdin:
             options["data"].close()
 
-    def handle(self, *args, **options):
+    def handle(self, *args: Any, **options: Any) -> NoReturn:
         """Create the collection."""
         name = options["name"]
         category = options["category"]
@@ -61,12 +62,14 @@ class Command(DebusineBaseCommand):
             )
 
         try:
-            Collection.objects.create(
+            collection = Collection(
                 name=name, category=category, workspace=workspace, data=data
             )
-        except IntegrityError:
+            collection.full_clean()
+            collection.save()
+        except ValidationError as exc:
             raise CommandError(
-                "A collection with this name and category already exists",
+                "Error creating collection: " + "\n".join(exc.messages),
                 returncode=3,
             )
 
