@@ -11,9 +11,10 @@
 
 import argparse
 import sys
+from typing import Any, NoReturn
 
 from django.core.exceptions import ValidationError
-from django.core.management import CommandError
+from django.core.management import CommandError, CommandParser
 
 from debusine.db.models import NotificationChannel
 from debusine.django.management.debusine_base_command import DebusineBaseCommand
@@ -24,7 +25,7 @@ class Command(DebusineBaseCommand):
 
     help = "Create a new notification channel"
 
-    def add_arguments(self, parser):
+    def add_arguments(self, parser: CommandParser) -> None:
         """Add CLI arguments for the create_notification_channel command."""
         methods = dict(NotificationChannel.Methods.choices).keys()
         parser.add_argument("name", help="Notification channel name")
@@ -39,22 +40,28 @@ class Command(DebusineBaseCommand):
             default="-",
         )
 
-    def cleanup_arguments(self, *args, **options):
+    def cleanup_arguments(self, *args: Any, **options: Any) -> None:
         """Clean up objects created by parsing arguments."""
         if options["data"] != sys.stdin:
             options["data"].close()
 
-    def handle(self, *args, **options):
+    def handle(self, *args: Any, **options: Any) -> NoReturn:
         """Create NotificationChannel."""
         name = options["name"]
         method = options["method"]
         data = self.parse_yaml_data(options["data"].read())
 
         try:
-            NotificationChannel.objects.create(
+            channel = NotificationChannel.objects.create(
                 name=name, method=method, data=data
             )
+            channel.full_clean()
+            channel.save()
         except ValidationError as exc:
-            raise CommandError(f"Error creating: {exc}", returncode=3)
+            raise CommandError(
+                "Error creating notification channel: "
+                + "\n".join(exc.messages),
+                returncode=3,
+            )
 
         raise SystemExit(0)
