@@ -9,8 +9,10 @@
 
 """debusine-admin command to create tokens."""
 
+from typing import Any, NoReturn
+
 from django.contrib.auth import get_user_model
-from django.core.management import CommandError
+from django.core.management import CommandError, CommandParser
 
 from debusine.db.models import Token, User
 from debusine.django.management.debusine_base_command import DebusineBaseCommand
@@ -21,7 +23,7 @@ class Command(DebusineBaseCommand):
 
     help = "Create a new, enabled token to be used by a Debusine client"
 
-    def add_arguments(self, parser):
+    def add_arguments(self, parser: CommandParser) -> None:
         """Add CLI arguments for the create_token command."""
         parser.add_argument(
             "username", help="Username of the user associated to the token"
@@ -32,7 +34,7 @@ class Command(DebusineBaseCommand):
             help='Reason for the creation of this token',
         )
 
-    def handle(self, *args, **options):
+    def handle(self, *args: Any, **options: Any) -> NoReturn:
         """Create the token."""
         username = options["username"]
 
