@@ -17,10 +17,11 @@ automation.
 
 import argparse
 import sys
+from typing import Any, NoReturn
 
 import yaml
 from django.contrib.auth import get_user_model
-from django.core.management import CommandError
+from django.core.management import CommandError, CommandParser
 from django.db import transaction
 
 from debusine.db.models import (
@@ -39,7 +40,7 @@ class Command(DebusineBaseCommand):
 
     help = "Create a new work request"
 
-    def add_arguments(self, parser):
+    def add_arguments(self, parser: CommandParser) -> None:
         """Add CLI arguments for the create_work_request command."""
         parser.add_argument(
             "task_type",
@@ -74,14 +75,14 @@ class Command(DebusineBaseCommand):
             help="File path in YAML format requesting notifications.",
         )
 
-    def cleanup_arguments(self, *args, **options):
+    def cleanup_arguments(self, *args: Any, **options: Any) -> None:
         """Clean up objects created by parsing arguments."""
         if options["data"] != sys.stdin:
             options["data"].close()
         if options["event_reactions"] not in {None, sys.stdin}:
             options["event_reactions"].close()
 
-    def handle(self, *args, **options):
+    def handle(self, *args: Any, **options: Any) -> NoReturn:
         """Create the work request."""
         task_type = TaskTypes[options["task_type"].upper()]
         task_name = options["task_name"]
