@@ -10,11 +10,15 @@
 """debusine-admin command to delete expired artifacts."""
 
 import contextlib
+import io
+from collections.abc import Generator
 from datetime import datetime
-from io import TextIOWrapper
+from typing import Any, NoReturn
 
 import django
+from django.core.management import CommandParser
 from django.db import connection, transaction
+from django.db.backends.utils import CursorWrapper
 from django.db.models import Exists, OuterRef, ProtectedError, Q
 from django.utils import timezone
 
@@ -36,7 +40,7 @@ class Command(DebusineBaseCommand):
 
     help = "Delete expired artifacts if no other artifact depends on them"
 
-    def add_arguments(self, parser):
+    def add_arguments(self, parser: CommandParser) -> None:
         """Add CLI arguments for the delete_expired command."""
         parser.add_argument(
             "--dry-run",
@@ -44,7 +48,7 @@ class Command(DebusineBaseCommand):
             action="store_true",
         )
 
-    def handle(self, *args, **options):
+    def handle(self, *args: Any, **options: Any) -> NoReturn:
         """Delete expired artifacts."""
         with DeleteOperation(
             out=self.stdout,
@@ -71,11 +75,11 @@ class DeleteOperation:
     def __init__(
         self,
         *,
-        out: TextIOWrapper,
-        err: TextIOWrapper,
+        out: io.TextIOBase,
+        err: io.TextIOBase,
         dry_run: bool = True,
         verbosity: int = 0,
-    ):
+    ) -> None:
         """
         Initialize object.
 
@@ -87,8 +91,8 @@ class DeleteOperation:
         :param verbosity: If True, the method will provide detailed information
             about the execution progress and results.
         """
-        self._out: TextIOWrapper = out
-        self._err: TextIOWrapper = err
+        self._out: io.TextIOBase = out
+        self._err: io.TextIOBase = err
 
         self._verbosity: int = verbosity
         self.dry_run: bool = dry_run
@@ -103,7 +107,7 @@ class DeleteOperation:
             self.verbose("dry-run mode: no changes will be made\n")
         return self
 
-    def __exit__(self, *args) -> None:
+    def __exit__(self, *args: Any) -> None:
         """Context manager end."""
         if self.dry_run:
             self.verbose("dry-run mode: no changes were made\n")
@@ -142,7 +146,7 @@ class DeleteExpiredArtifacts:
         self._lock_timeout_secs: float = 5
 
     @staticmethod
-    def _tables_to_lock():
+    def _tables_to_lock() -> list[str]:
         return [
             Artifact._meta.db_table,
             ArtifactRelation._meta.db_table,
@@ -152,7 +156,9 @@ class DeleteExpiredArtifacts:
         ]
 
     @contextlib.contextmanager
-    def lock_tables(self, table_names: list[str]):
+    def lock_tables(
+        self, table_names: list[str]
+    ) -> Generator[CursorWrapper, None, None]:
         """
         Lock the tables or raise LockTableFailed.
 
@@ -266,7 +272,7 @@ class DeleteExpiredArtifacts:
 
         return delete_filesobjs
 
-    def _delete_files_from_stores(self, file_objs: set[File]):
+    def _delete_files_from_stores(self, file_objs: set[File]) -> None:
         """
         Delete the files from stores.
 
@@ -322,7 +328,7 @@ class DeleteExpiredArtifacts:
 
     def _traverse_and_mark_from_artifact(
         self, artifact_id: int, marked_to_keep: set[int], visited: set[int]
-    ):
+    ) -> None:
         if artifact_id in visited:
             return
 
