@@ -9,10 +9,12 @@
 
 """debusine-admin command to delete a notification channel."""
 
-from django.core.management import CommandError
+from typing import Any, NoReturn
+
+from django.core.management import CommandError, CommandParser
 
 from debusine.db.models import NotificationChannel
-from debusine.server.management.debusine_base_command import DebusineBaseCommand
+from debusine.django.management.debusine_base_command import DebusineBaseCommand
 
 
 class Command(DebusineBaseCommand):
@@ -20,11 +22,11 @@ class Command(DebusineBaseCommand):
 
     help = "Delete a notification channel"
 
-    def add_arguments(self, parser):
+    def add_arguments(self, parser: CommandParser) -> None:
         """Add CLI arguments for the create_notification_channel command."""
         parser.add_argument("name", help="Notification channel name")
 
-    def handle(self, *args, **options):
+    def handle(self, *args: Any, **options: Any) -> NoReturn:
         """Create NotificationChannel."""
         name = options["name"]
 
