@@ -9,11 +9,13 @@
 
 """debusine-admin command to delete tokens. Allows filtering."""
 
-from django.core.management.base import CommandError
+from typing import Any
+
+from django.core.management import CommandError, CommandParser
 
 from debusine.db.models import Token
 from debusine.django.management.debusine_base_command import DebusineBaseCommand
-from debusine.server.management.utils import print_tokens
+from debusine.server.management.utils import Tokens
 
 
 class Command(DebusineBaseCommand):
@@ -24,7 +26,7 @@ class Command(DebusineBaseCommand):
         "before deletion"
     )
 
-    def add_arguments(self, parser):
+    def add_arguments(self, parser: CommandParser) -> None:
         """Add CLI arguments for the remove_tokens command."""
         parser.add_argument(
             '--yes', action='store_true', help='Skips confirmation of deletion'
@@ -39,7 +41,7 @@ class Command(DebusineBaseCommand):
             '--username', help='Deletes tokens owned by USERNAME'
         )
 
-    def handle(self, *args, **options):
+    def handle(self, *args: Any, **options: Any) -> None:
         """Delete (if selected and confirmed) tokens."""
         tokens = Token.objects.get_tokens(
             username=options['username'], key=options['token']
@@ -53,8 +55,7 @@ class Command(DebusineBaseCommand):
             else:
                 raise CommandError(error_message, returncode=3)
 
-        print_tokens(self.stdout, tokens)
-        self.stdout.write()
+        Tokens(yaml=False).print(tokens, self.stdout)
 
         deletion_confirmed = False
         deletion_forced = options['yes']
