@@ -9,15 +9,15 @@
 
 """debusine-admin command to edit workers metadata."""
 
+import io
 import os
 import shutil
 import subprocess
 import tempfile
-from typing import Any
-
-from django.core.management import CommandError
+from typing import Any, NoReturn
 
 import yaml
+from django.core.management import CommandError, CommandParser
 
 from debusine.db.models import Worker
 from debusine.django.management.debusine_base_command import DebusineBaseCommand
@@ -26,7 +26,13 @@ from debusine.django.management.debusine_base_command import DebusineBaseCommand
 class WorkerStaticMetadataEditor:
     """Edit Worker static metadata."""
 
-    def __init__(self, worker, yaml_file, stdout, stderr):
+    def __init__(
+        self,
+        worker: Worker,
+        yaml_file: str | None,
+        stdout: io.TextIOBase,
+        stderr: io.TextIOBase,
+    ) -> None:
         """
         Initialize fields of WorkerStaticMetadataEditor.
 
@@ -41,7 +47,7 @@ class WorkerStaticMetadataEditor:
         self._stdout = stdout
         self._stderr = stderr
 
-    def edit(self):
+    def edit(self) -> bool:
         """Edit the Worker's static_metadata."""
         if self._yaml_file:
             metadata = self._read_yaml_file(self._yaml_file)
@@ -60,7 +66,7 @@ class WorkerStaticMetadataEditor:
         else:
             return False
 
-    def _read_yaml_file(self, yaml_path) -> dict[Any, Any] | None:
+    def _read_yaml_file(self, yaml_path: str) -> dict[Any, Any] | None:
         """
         Read yaml_path and returns it as dictionary (or None if invalid).
 
@@ -88,19 +94,25 @@ class WorkerStaticMetadataEditor:
             raise SystemExit(3)
 
         if contents is None:
-            # Empty YAML is normalised to an empty dictionary
+            # Empty YAML is normalized to an empty dictionary
             contents = {}
         elif not isinstance(contents, dict):
             self._stderr.write('Worker metadata must be a dictionary or empty')
             return None
 
+        assert isinstance(contents, dict)
         return contents
 
     @staticmethod
-    def _input():
+    def _input() -> str:
         return input()  # pragma: no cover
 
-    def _edit_worker_metadata(self, metadata, stdout, stderr):
+    def _edit_worker_metadata(
+        self,
+        metadata: dict[str, Any] | None,
+        stdout: io.TextIOBase,
+        stderr: io.TextIOBase,
+    ) -> dict[str, Any] | None:
         edit_metadata = tempfile.NamedTemporaryFile(
             prefix='debusine-edit_worker_metadata_',
             suffix='.tmp.yaml',
@@ -140,7 +152,7 @@ class WorkerStaticMetadataEditor:
                     return None
 
     @staticmethod
-    def _open_editor(file_path):
+    def _open_editor(file_path: str) -> None:
         # sensible-editor is in sensible-utils package
         subprocess.run(['sensible-editor', file_path])  # pragma: no cover
 
@@ -150,7 +162,7 @@ class Command(DebusineBaseCommand):
 
     help = "Edit worker's metadata"
 
-    def add_arguments(self, parser):
+    def add_arguments(self, parser: CommandParser) -> None:
         """Add CLI arguments for the edit_worker_metadata command."""
         parser.add_argument(
             '--set', help='Filename with the metadata in YAML', metavar='PATH'
@@ -160,7 +172,7 @@ class Command(DebusineBaseCommand):
             help='Name of the worker of which the metadata will be edited',
         )
 
-    def handle(self, *args, **options):
+    def handle(self, *args: Any, **options: Any) -> NoReturn:
         """Edit the Worker based on the command options."""
         worker_name = options['worker_name']
 
