@@ -9,9 +9,13 @@
 
 """debusine-admin command to list notification channels."""
 
+from typing import Any, NoReturn
+
+from django.core.management import CommandParser
+
 from debusine.db.models import NotificationChannel
 from debusine.django.management.debusine_base_command import DebusineBaseCommand
-from debusine.server.management.utils import print_notification_channels
+from debusine.server.management.utils import NotificationChannels
 
 
 class Command(DebusineBaseCommand):
@@ -19,8 +23,16 @@ class Command(DebusineBaseCommand):
 
     help = "List notification channels"
 
-    def handle(self, *args, **options):
+    def add_arguments(self, parser: CommandParser) -> None:
+        """Add CLI arguments for the list_tokens command."""
+        parser.add_argument(
+            '--yaml', action="store_true", help="Machine readable YAML output"
+        )
+
+    def handle(self, *args: Any, **options: Any) -> NoReturn:
         """List notification channels."""
         notification_channels = NotificationChannel.objects.all()
-        print_notification_channels(self.stdout, notification_channels)
+        NotificationChannels(options["yaml"]).print(
+            notification_channels, self.stdout
+        )
         raise SystemExit(0)
