@@ -9,9 +9,13 @@
 
 """debusine-admin command to list tokens. Allows filtering."""
 
+from typing import Any
+
+from django.core.management import CommandParser
+
 from debusine.db.models import Token
-from debusine.server.management.debusine_base_command import DebusineBaseCommand
-from debusine.server.management.utils import print_tokens
+from debusine.django.management.debusine_base_command import DebusineBaseCommand
+from debusine.server.management.utils import Tokens
 
 
 class Command(DebusineBaseCommand):
@@ -19,14 +23,18 @@ class Command(DebusineBaseCommand):
 
     help = "List tokens. See the options to filter which tokens will be listed"
 
-    def add_arguments(self, parser):
+    def add_arguments(self, parser: CommandParser) -> None:
         """Add CLI arguments for the list_tokens command."""
         parser.add_argument('--token', help='List only this token')
         parser.add_argument('--username', help='List tokens owner by USERNAME')
+        parser.add_argument(
+            '--yaml', action="store_true", help="Machine readable YAML output"
+        )
 
-    def handle(self, *args, **options):
+    def handle(self, *args: Any, **options: Any) -> None:
         """List the tokens."""
         tokens = Token.objects.get_tokens(
             username=options['username'], key=options['token']
         )
-        print_tokens(self.stdout, tokens=tokens)
+        Tokens(options["yaml"]).print(tokens, self.stdout)
+        raise SystemExit(0)
