@@ -9,10 +9,13 @@
 
 """debusine-admin command to list work requests."""
 
+from typing import Any, NoReturn
+
 from django.contrib.auth import get_user_model
+from django.core.management import CommandParser
 
 from debusine.django.management.debusine_base_command import DebusineBaseCommand
-from debusine.server.management.utils import print_users
+from debusine.server.management.utils import Users
 
 
 class Command(DebusineBaseCommand):
@@ -20,8 +23,14 @@ class Command(DebusineBaseCommand):
 
     help = "List users sorted by date joined."
 
-    def handle(self, *args, **options):
+    def add_arguments(self, parser: CommandParser) -> None:
+        """Add CLI arguments for the list_tokens command."""
+        parser.add_argument(
+            '--yaml', action="store_true", help="Machine readable YAML output"
+        )
+
+    def handle(self, *args: Any, **options: Any) -> NoReturn:
         """List users."""
         users = get_user_model().objects.filter(is_system=False)
-        print_users(self.stdout, users)
+        Users(options["yaml"]).print(users, self.stdout)
         raise SystemExit(0)
