@@ -9,9 +9,13 @@
 
 """debusine-admin command to list work requests."""
 
+from typing import Any
+
+from django.core.management import CommandParser
+
 from debusine.db.models import WorkRequest
 from debusine.django.management.debusine_base_command import DebusineBaseCommand
-from debusine.server.management.utils import print_work_requests
+from debusine.server.management.utils import WorkRequests
 
 
 class Command(DebusineBaseCommand):
@@ -19,7 +23,14 @@ class Command(DebusineBaseCommand):
 
     help = "List work requests. Sorted by 'created_at'"
 
-    def handle(self, *args, **options):
+    def add_arguments(self, parser: CommandParser) -> None:
+        """Add CLI arguments for the list_tokens command."""
+        parser.add_argument(
+            '--yaml', action="store_true", help="Machine readable YAML output"
+        )
+
+    def handle(self, *args: Any, **options: Any) -> None:
         """List the work requests."""
         work_requests = WorkRequest.objects.all().order_by('created_at')
-        print_work_requests(self.stdout, work_requests)
+        WorkRequests(options["yaml"]).print(work_requests, self.stdout)
+        raise SystemExit(0)
