@@ -9,9 +9,13 @@
 
 """debusine-admin command to list workers."""
 
+from typing import Any
+
+from django.core.management import CommandParser
+
 from debusine.db.models import Worker
 from debusine.django.management.debusine_base_command import DebusineBaseCommand
-from debusine.server.management.utils import print_workers
+from debusine.server.management.utils import Workers
 
 
 class Command(DebusineBaseCommand):
@@ -19,7 +23,14 @@ class Command(DebusineBaseCommand):
 
     help = "List all the workers"
 
-    def handle(self, *args, **options):
+    def add_arguments(self, parser: CommandParser) -> None:
+        """Add CLI arguments for the list_tokens command."""
+        parser.add_argument(
+            '--yaml', action="store_true", help="Machine readable YAML output"
+        )
+
+    def handle(self, *args: Any, **options: Any) -> None:
         """List the workers."""
         workers = Worker.objects.all()
-        print_workers(self.stdout, workers)
+        Workers(options["yaml"]).print(workers, self.stdout)
+        raise SystemExit(0)
