@@ -9,9 +9,15 @@
 
 """debusine-admin command to list workspaces."""
 
+from typing import Any
+
+from django.core.management import CommandParser
+
 from debusine.db.models import Workspace
-from debusine.server.management.debusine_base_command import DebusineBaseCommand
-from debusine.server.management.utils import print_workspaces
+from debusine.django.management.debusine_base_command import DebusineBaseCommand
+from debusine.server.management.utils import Workspaces
+
+# TODO: this is deprecated in favor of the 'workspace' command
 
 
 class Command(DebusineBaseCommand):
@@ -19,7 +25,14 @@ class Command(DebusineBaseCommand):
 
     help = "List all the workspaces"
 
-    def handle(self, *args, **options):
+    def add_arguments(self, parser: CommandParser) -> None:
+        """Add CLI arguments for the list_tokens command."""
+        parser.add_argument(
+            '--yaml', action="store_true", help="Machine readable YAML output"
+        )
+
+    def handle(self, *args: Any, **options: Any) -> None:
         """List the workspaces."""
         workspaces = Workspace.objects.all()
-        print_workspaces(self.stdout, workspaces)
+        Workspaces(options["yaml"]).print(workspaces, self.stdout)
+        raise SystemExit(0)
