@@ -10,9 +10,11 @@
 """debusine-admin command to manage a channel notification."""
 
 import argparse
+import sys
+from typing import Any, NoReturn
 
 from django.core.exceptions import ValidationError
-from django.core.management import CommandError
+from django.core.management import CommandError, CommandParser
 
 from debusine.db.models import NotificationChannel
 from debusine.django.management.debusine_base_command import DebusineBaseCommand
@@ -23,7 +25,7 @@ class Command(DebusineBaseCommand):
 
     help = "Manage channel notification"
 
-    def add_arguments(self, parser):
+    def add_arguments(self, parser: CommandParser) -> None:
         """Add CLI arguments for manage_channel_notification."""
         subparsers = parser.add_subparsers(dest="action")
 
@@ -43,7 +45,12 @@ class Command(DebusineBaseCommand):
             default="-",
         )
 
-    def handle(self, *args, **options):
+    def cleanup_arguments(self, *args: Any, **options: Any) -> None:
+        """Clean up objects created by parsing arguments."""
+        if options["action"] == "change-data" and options["data"] != sys.stdin:
+            options["data"].close()
+
+    def handle(self, *args: Any, **options: Any) -> NoReturn:
         """Manage the notification."""
         name = options["name"]
         try:
