@@ -9,8 +9,10 @@
 
 """debusine-admin command to manage users."""
 
+from typing import Any, NoReturn
+
 from django.contrib.auth import get_user_model
-from django.core.management import CommandError
+from django.core.management import CommandError, CommandParser
 
 from debusine.django.management.debusine_base_command import DebusineBaseCommand
 
@@ -20,28 +22,23 @@ class Command(DebusineBaseCommand):
 
     help = "Manage users"
 
-    def add_arguments(self, parser):
+    def add_arguments(self, parser: CommandParser) -> None:
         """Add CLI arguments for the manage_user command."""
         subparsers = parser.add_subparsers(dest="action", required=True)
 
         # change email
         change_email = subparsers.add_parser("change-email")
-
-        # Workaround for https://code.djangoproject.com/ticket/34259#ticket
-        # To be removed when using Django 4.2 with the fix
-        change_email.called_from_command_line = parser.called_from_command_line
-
         change_email.add_argument("username", help="Username to modify")
         change_email.add_argument("email", help="New email to set")
 
-    def handle(self, *args, **options):
+    def handle(self, *args: Any, **options: Any) -> None:
         """Manage the user."""
         if options["action"] == "change-email":
             self._change_email(options["username"], options["email"])
         else:  # pragma: no cover
             pass  # pragma: no cover
 
-    def _change_email(self, username: str, email: str):
+    def _change_email(self, username: str, email: str) -> NoReturn:
         try:
             user = get_user_model().objects.get(
                 username=username, is_system=False
