@@ -10,21 +10,28 @@
 """debusine-admin command to manage workspaces."""
 
 from datetime import timedelta
+from typing import Any, NoReturn
 
-from django.core.management import CommandError
+from django.core.management import CommandError, CommandParser
 
-from debusine.db.models import FileStore, Workspace
+from debusine.db.models import FileStore, Scope, Workspace
 from debusine.django.management.debusine_base_command import DebusineBaseCommand
 
+# TODO: this is deprecated in favor of the 'workspace' command
+
 
 class Command(DebusineBaseCommand):
     """Command to manage workspaces."""
 
     help = "Manage workspaces"
 
-    def add_arguments(self, parser):
+    def add_arguments(self, parser: CommandParser) -> None:
         """Add CLI arguments for the manage_workspace command."""
-        parser.add_argument("name", help="Name to modify")
+        parser.add_argument(
+            "scope_workspace",
+            metavar="scope/name",
+            help="scope/name of workspace to modify",
+        )
         public = parser.add_mutually_exclusive_group()
         public.add_argument("--public", action="store_true", default=None)
         public.add_argument(
@@ -47,13 +54,24 @@ class Command(DebusineBaseCommand):
             help="Name of the default file store to use for this workspace",
         )
 
-    def handle(self, *args, **options):
+    def handle(
+        self, scope_workspace: str, *args: Any, **options: Any
+    ) -> NoReturn:
         """Manage the workspace."""
-        name = options["name"]
+        scope_name, workspace_name = scope_workspace.split("/", 1)
         try:
-            workspace = Workspace.objects.get(name=name)
+            scope = Scope.objects.get(name=scope_name)
+        except Scope.DoesNotExist:
+            raise CommandError(
+                f'Scope "{scope_name}" not found',
+                returncode=3,
+            )
+        try:
+            workspace = Workspace.objects.get(name=workspace_name, scope=scope)
         except Workspace.DoesNotExist:
-            raise CommandError(f'Workspace "{name}" not found', returncode=3)
+            raise CommandError(
+                f'Workspace "{workspace_name}" not found', returncode=3
+            )
 
         if options["public"] is not None:
             workspace.public = options["public"]
