@@ -9,26 +9,30 @@
 
 """Tests for the management command create_notification_channel."""
 import io
+from typing import ClassVar
 
 import yaml
 from django.core.management import CommandError
-from django.test import TestCase
 
 from debusine.db.models import NotificationChannel
 from debusine.django.management.tests import call_command
-from debusine.test import TestHelpersMixin
+from debusine.test.django import TestCase
 
 
-class CreateNotificationChannelCommandTests(TestHelpersMixin, TestCase):
+class CreateNotificationChannelCommandTests(TestCase):
     """Tests for the create_notification_channel command."""
 
-    def setUp(self):
+    yaml_data: ClassVar[str]
+
+    @classmethod
+    def setUpTestData(cls) -> None:
         """Set up test."""
-        self.yaml_data = yaml.safe_dump(
+        super().setUpTestData()
+        cls.yaml_data = yaml.safe_dump(
             {"from": "debusine@debusine.com", "to": ["lts@debusine.com"]}
         )
 
-    def test_create_notification_channel_from_file(self):
+    def test_create_notification_channel_from_file(self) -> None:
         """create_notification_channel create a notification (data in file)."""
         name = "lts"
         method = NotificationChannel.Methods.EMAIL
@@ -53,7 +57,7 @@ class CreateNotificationChannelCommandTests(TestHelpersMixin, TestCase):
             ).exists()
         )
 
-    def test_create_notification_channel_from_stdin(self):
+    def test_create_notification_channel_from_stdin(self) -> None:
         """create_notification_channel create a notification (data in stdin)."""
         name = "lts"
         method = NotificationChannel.Methods.EMAIL
@@ -75,7 +79,7 @@ class CreateNotificationChannelCommandTests(TestHelpersMixin, TestCase):
             ).exists()
         )
 
-    def test_create_notification_channel_invalid_data_yaml(self):
+    def test_create_notification_channel_invalid_data_yaml(self) -> None:
         """create_notification_channel return error: cannot parse YAML data."""
         data_file = self.create_temporary_file(contents=b":")
         with self.assertRaisesRegex(
@@ -91,11 +95,11 @@ class CreateNotificationChannelCommandTests(TestHelpersMixin, TestCase):
 
         self.assertEqual(exc.exception.returncode, 3)
 
-    def test_create_notification_invalid_data(self):
+    def test_create_notification_invalid_data(self) -> None:
         """create_notification_channel return error: data is invalid."""
         # it is invalid because does not have the required fields
         data_file = self.create_temporary_file(contents=b"")
-        with self.assertRaisesRegex(CommandError, "^Error creating:") as exc:
+        with self.assertRaisesRegex(CommandError, "^Error creating") as exc:
             call_command(
                 "create_notification_channel",
                 "test",
@@ -106,7 +110,22 @@ class CreateNotificationChannelCommandTests(TestHelpersMixin, TestCase):
 
         self.assertEqual(exc.exception.returncode, 3)
 
-    def test_create_duplicated_notification_raise_command_error(self):
+    def test_create_notification_invalid_name(self) -> None:
+        """create_notification_channel return error: name is invalid."""
+        # it is invalid because does not have the required fields
+        data_file = self.create_temporary_file(contents=b"{}")
+        with self.assertRaisesRegex(CommandError, "^Error creating") as exc:
+            call_command(
+                "create_notification_channel",
+                "a: b",
+                "email",
+                "--data",
+                str(data_file),
+            )
+
+        self.assertEqual(exc.exception.returncode, 3)
+
+    def test_create_duplicated_notification_raise_command_error(self) -> None:
         """create_notification_channel raise CommandError: duplicated name."""
         name = "lts"
         method = NotificationChannel.Methods.EMAIL
@@ -122,7 +141,7 @@ class CreateNotificationChannelCommandTests(TestHelpersMixin, TestCase):
             str(data_file),
         )
 
-        with self.assertRaisesRegex(CommandError, "^Error creating: ") as exc:
+        with self.assertRaisesRegex(CommandError, "^Error creating") as exc:
             call_command(
                 "create_notification_channel",
                 name,
