@@ -14,6 +14,7 @@ import io
 import yaml
 from django.core.management import CommandError
 
+from debusine.db.context import context
 from debusine.db.models import WorkRequest, default_workspace, system_user
 from debusine.django.management.tests import call_command
 from debusine.tasks.models import (
@@ -28,7 +29,7 @@ from debusine.test.django import TestCase
 class CreateWorkRequestCommandTests(TestCase):
     """Tests for the create_work_request command."""
 
-    def test_data_from_file(self):
+    def test_data_from_file(self) -> None:
         """`create_work_request` accepts data from a file."""
         data = {"result": True}
         data_file = self.create_temporary_file(
@@ -55,7 +56,7 @@ class CreateWorkRequestCommandTests(TestCase):
         self.assertEqual(stderr, "")
         self.assertEqual(exit_code, 0)
 
-    def test_data_from_stdin(self):
+    def test_data_from_stdin(self) -> None:
         """`create_work_request` accepts data from stdin."""
         data = {"result": True}
         stdout, stderr, exit_code = call_command(
@@ -78,7 +79,7 @@ class CreateWorkRequestCommandTests(TestCase):
         self.assertEqual(stderr, "")
         self.assertEqual(exit_code, 0)
 
-    def test_event_reactions(self):
+    def test_event_reactions(self) -> None:
         """`create_work_request` handles event reactions."""
         data = {"result": True}
         event_reactions = {
@@ -112,7 +113,7 @@ class CreateWorkRequestCommandTests(TestCase):
         self.assertEqual(stderr, "")
         self.assertEqual(exit_code, 0)
 
-    def test_server(self):
+    def test_server(self) -> None:
         """`create_work_request` can create server work requests."""
         data = {"result": True}
         event_reactions = {
@@ -158,7 +159,7 @@ class CreateWorkRequestCommandTests(TestCase):
         self.assertEqual(stderr, "")
         self.assertEqual(exit_code, 0)
 
-    def test_signing(self):
+    def test_signing(self) -> None:
         """`create_work_request` can create signing work requests."""
         data = {"purpose": "uefi", "description": "A UEFI key"}
         event_reactions = {
@@ -204,7 +205,7 @@ class CreateWorkRequestCommandTests(TestCase):
         self.assertEqual(stderr, "")
         self.assertEqual(exit_code, 0)
 
-    def test_empty_data(self):
+    def test_empty_data(self) -> None:
         """`create_work_request` defaults data to {}."""
         stdout, stderr, exit_code = call_command(
             "create_work_request", "worker", "noop", stdin=io.StringIO()
@@ -220,9 +221,9 @@ class CreateWorkRequestCommandTests(TestCase):
         self.assertEqual(stderr, "")
         self.assertEqual(exit_code, 0)
 
-    def test_different_created_by(self):
+    def test_different_created_by(self) -> None:
         """`create_work_request` can use a different created-by user."""
-        user = self.get_test_user()
+        user = self.playground.get_default_user()
         stdout, stderr, exit_code = call_command(
             "create_work_request",
             "worker",
@@ -242,10 +243,11 @@ class CreateWorkRequestCommandTests(TestCase):
         self.assertEqual(stderr, "")
         self.assertEqual(exit_code, 0)
 
-    def test_different_workspace(self):
+    def test_different_workspace(self) -> None:
         """`create_work_request` can use a non-default workspace."""
         workspace_name = "test-workspace"
-        self.playground.create_workspace(name=workspace_name)
+        with context.disable_permission_checks():
+            self.playground.create_workspace(name=workspace_name)
         stdout, stderr, exit_code = call_command(
             "create_work_request",
             "worker",
@@ -265,7 +267,7 @@ class CreateWorkRequestCommandTests(TestCase):
         self.assertEqual(stderr, "")
         self.assertEqual(exit_code, 0)
 
-    def test_invalid_data_yaml(self):
+    def test_invalid_data_yaml(self) -> None:
         """`create_work_request` returns error: cannot parse data."""
         with self.assertRaisesRegex(
             CommandError, r"^Error parsing YAML:"
@@ -276,7 +278,7 @@ class CreateWorkRequestCommandTests(TestCase):
 
         self.assertEqual(exc.exception.returncode, 3)
 
-    def test_invalid_event_reactions_yaml(self):
+    def test_invalid_event_reactions_yaml(self) -> None:
         """`create_work_request` returns error: cannot parse data."""
         event_reactions_file = self.create_temporary_file(contents=b":")
 
@@ -294,7 +296,7 @@ class CreateWorkRequestCommandTests(TestCase):
 
         self.assertEqual(exc.exception.returncode, 3)
 
-    def test_bad_task_type(self):
+    def test_bad_task_type(self) -> None:
         """`create_work_request` returns error: bad task type."""
         with self.assertRaisesRegex(
             CommandError, r"task_type: invalid choice: 'nonexistent'"
@@ -308,7 +310,7 @@ class CreateWorkRequestCommandTests(TestCase):
 
         self.assertEqual(exc.exception.returncode, 1)
 
-    def test_user_not_found(self):
+    def test_user_not_found(self) -> None:
         """`create_work_request` returns error: user not found."""
         with self.assertRaisesRegex(
             CommandError, r'^User "nonexistent" not found'
@@ -324,7 +326,7 @@ class CreateWorkRequestCommandTests(TestCase):
 
         self.assertEqual(exc.exception.returncode, 3)
 
-    def test_workspace_not_found(self):
+    def test_workspace_not_found(self) -> None:
         """`create_work_request` returns error: workspace not found."""
         with self.assertRaisesRegex(
             CommandError, r'^Workspace "nonexistent" not found'
@@ -340,7 +342,7 @@ class CreateWorkRequestCommandTests(TestCase):
 
         self.assertEqual(exc.exception.returncode, 3)
 
-    def test_task_name_not_found(self):
+    def test_task_name_not_found(self) -> None:
         """`create_work_request` returns error: task name not found."""
         with self.assertRaisesRegex(
             CommandError,
@@ -355,7 +357,7 @@ class CreateWorkRequestCommandTests(TestCase):
 
         self.assertEqual(exc.exception.returncode, 3)
 
-    def test_bad_task_data(self):
+    def test_bad_task_data(self) -> None:
         """`create_work_request` returns error: bad task data."""
         with self.assertRaisesRegex(
             CommandError, r"invalid Worker task data"
