@@ -26,6 +26,7 @@ from debusine.artifacts.models import (
     BareDataCategory,
     CollectionCategory,
 )
+from debusine.db.context import context
 from debusine.db.models import (
     Artifact,
     ArtifactRelation,
@@ -58,12 +59,15 @@ from debusine.test.utils import tomorrow
 class DeleteExpiredCommandTests(TestCase):
     """Tests for delete_expired command."""
 
-    def setUp(self):
-        """Set up common objects for the tests."""
-        artifact_1, _ = self.create_artifact(expiration_delay=1)
+    @classmethod
+    @context.disable_permission_checks()
+    def setUpTestData(cls) -> None:
+        """Set up common data for tests."""
+        super().setUpTestData()
+        artifact_1, _ = cls.create_artifact(expiration_delay=1)
         artifact_1.created_at = timezone.now() - timedelta(days=2)
         artifact_1.save()
-        artifact_2, _ = self.create_artifact(expiration_delay=1)
+        artifact_2, _ = cls.create_artifact(expiration_delay=1)
         artifact_2.created_at = timezone.now() - timedelta(days=2)
         artifact_2.save()
 
@@ -73,7 +77,7 @@ class DeleteExpiredCommandTests(TestCase):
             type=ArtifactRelation.Relations.BUILT_USING,
         )
 
-    def test_delete_expired(self):
+    def test_delete_expired(self) -> None:
         """Test delete_expired command delete artifacts."""
         self.assertEqual(Artifact.objects.count(), 2)
         self.assertEqual(ArtifactRelation.objects.count(), 1)
@@ -108,31 +112,32 @@ class DeleteExpiredCommandTests(TestCase):
         self.assertEqual(stderr, "")
         self.assertEqual(retval, 0)
 
-    def test_delete_expired_dry_run_verbose(self):
+    def test_delete_expired_dry_run_verbose(self) -> None:
         """Test delete_expired artifacts dry-run: no deletion."""
-        artifact_id_1, artifact_id_2 = Artifact.objects.all()
+        with context.disable_permission_checks():
+            artifact_id_1, artifact_id_2 = Artifact.objects.all()
 
-        # Add an artifact with a file
-        artifact_id_3, _ = self.create_artifact(
-            paths=["README"], create_files=True, expiration_delay=1
-        )
-        artifact_id_3.created_at = timezone.now() - timedelta(days=2)
-        artifact_id_3.save()
+            # Add an artifact with a file
+            artifact_id_3, _ = self.create_artifact(
+                paths=["README"], create_files=True, expiration_delay=1
+            )
+            artifact_id_3.created_at = timezone.now() - timedelta(days=2)
+            artifact_id_3.save()
 
-        collection_1 = self.playground.create_collection(
-            "internal", CollectionCategory.WORKFLOW_INTERNAL
-        )
-        work_request_1 = self.create_work_request(
-            created_at=timezone.now() - timedelta(days=365),
-            expiration_delay=timedelta(days=1),
-            internal_collection=collection_1,
-        )
+            collection_1 = self.playground.create_collection(
+                "internal", CollectionCategory.WORKFLOW_INTERNAL
+            )
+            work_request_1 = self.playground.create_work_request(
+                created_at=timezone.now() - timedelta(days=365),
+                expiration_delay=timedelta(days=1),
+                internal_collection=collection_1,
+            )
 
-        self.assertEqual(Artifact.objects.count(), 3)
-        self.assertEqual(ArtifactRelation.objects.count(), 1)
-        self.assertEqual(FileInStore.objects.all().count(), 1)
-        self.assertEqual(WorkRequest.objects.all().count(), 1)
-        self.assertEqual(Collection.objects.all().count(), 1)
+        num_artifacts = Artifact.objects.count()
+        num_artifact_relations = ArtifactRelation.objects.count()
+        num_files_in_store = FileInStore.objects.all().count()
+        num_work_requests = WorkRequest.objects.all().count()
+        num_collections = Collection.objects.all().count()
 
         stdout, stderr, retval = call_command(
             "delete_expired", "--dry-run", verbosity=2
@@ -149,24 +154,26 @@ class DeleteExpiredCommandTests(TestCase):
         )
 
         # Artifacts, ArtifactRelation didn't get deleted
-        self.assertEqual(Artifact.objects.count(), 3)
-        self.assertEqual(ArtifactRelation.objects.count(), 1)
+        self.assertEqual(Artifact.objects.count(), num_artifacts)
+        self.assertEqual(
+            ArtifactRelation.objects.count(), num_artifact_relations
+        )
 
         # Files didn't get deleted
-        self.assertEqual(FileInStore.objects.all().count(), 1)
+        self.assertEqual(FileInStore.objects.all().count(), num_files_in_store)
 
         # Work requests and their internal collections didn't get deleted
-        self.assertEqual(WorkRequest.objects.all().count(), 1)
-        self.assertEqual(Collection.objects.all().count(), 1)
+        self.assertEqual(WorkRequest.objects.all().count(), num_work_requests)
+        self.assertEqual(Collection.objects.all().count(), num_collections)
 
-    def test_delete_expired_dry_run(self):
+    def test_delete_expired_dry_run(self) -> None:
         """Test --dry-run without verbose does not write to stdout."""
         stdout, stderr, retval = call_command("delete_expired", "--dry-run")
 
         # dry-run without verbose: no messages are written to stdout.
         self.assertEqual(stdout, "")
 
-    def test_delete_expired_verbose(self):
+    def test_delete_expired_verbose(self) -> None:
         """Test delete_expired verbose: print information."""
         artifact_id_1 = Artifact.objects.all()[0]
         artifact_id_2 = Artifact.objects.all()[1]
@@ -174,18 +181,22 @@ class DeleteExpiredCommandTests(TestCase):
             "internal", CollectionCategory.WORKFLOW_INTERNAL
         )
         collection_1.manager.add_artifact(
-            artifact_id_1, user=self.get_test_user(), name="artifact1"
+            artifact_id_1,
+            user=self.playground.get_default_user(),
+            name="artifact1",
         )
         collection_1.manager.add_artifact(
-            artifact_id_2, user=self.get_test_user(), name="artifact2"
+            artifact_id_2,
+            user=self.playground.get_default_user(),
+            name="artifact2",
         )
-        work_request_1 = self.create_work_request(
+        work_request_1 = self.playground.create_work_request(
             created_at=timezone.now() - timedelta(days=365),
             expiration_delay=timedelta(days=1),
             internal_collection=collection_1,
         )
         self.assertEqual(WorkRequest.objects.count(), 1)
-        self.assertEqual(Collection.objects.count(), 1)
+        num_collections = Collection.objects.count()
 
         stdout, stderr, retval = call_command("delete_expired", verbosity=2)
 
@@ -197,9 +208,9 @@ class DeleteExpiredCommandTests(TestCase):
         self.assertEqual(Artifact.objects.count(), 0)
         self.assertEqual(ArtifactRelation.objects.count(), 0)
         self.assertEqual(WorkRequest.objects.count(), 0)
-        self.assertEqual(Collection.objects.count(), 0)
+        self.assertEqual(Collection.objects.count(), num_collections - 1)
 
-    def test_delete_expired_verbose_nothing_deleted(self):
+    def test_delete_expired_verbose_nothing_deleted(self) -> None:
         """Test delete_expired verbose: print no artifact expired."""
         Artifact.objects.update(created_at=tomorrow())
         stdout, stderr, retval = call_command("delete_expired", verbosity=2)
@@ -216,7 +227,8 @@ class DeleteExpiredArtifactsTests(TestCase):
 
     playground_memory_file_store = False
 
-    def setUp(self):
+    @context.disable_permission_checks()
+    def setUp(self) -> None:
         """Initialize test."""
         self.out = io.StringIO()
         self.err = io.StringIO()
@@ -224,19 +236,22 @@ class DeleteExpiredArtifactsTests(TestCase):
         self.operation.initial_time = timezone.now()
         self.delete_expired = DeleteExpiredArtifacts(self.operation)
 
-    def test_mark_to_keep_artifact_no_expiration_date(self):
+    @context.disable_permission_checks()
+    def test_mark_to_keep_artifact_no_expiration_date(self) -> None:
         """One artifact, not expired: keep it."""
         artifact, _ = self.create_artifact(expiration_delay=0)
 
         self.assertEqual(self.delete_expired._mark_to_keep(), {artifact.id})
 
-    def test_mark_to_keep_artifact_expires_tomorrow(self):
+    @context.disable_permission_checks()
+    def test_mark_to_keep_artifact_expires_tomorrow(self) -> None:
         """One artifact, not expired (expires tomorrow): keep it."""
         artifact, _ = self.create_artifact(expiration_delay=1)
 
         self.assertEqual(self.delete_expired._mark_to_keep(), {artifact.id})
 
-    def test_mark_to_keep_only_artifact_is_expired(self):
+    @context.disable_permission_checks()
+    def test_mark_to_keep_only_artifact_is_expired(self) -> None:
         """One artifact, expired yesterday. No artifacts are kept."""
         artifact_1, _ = self.create_artifact(expiration_delay=1)
         artifact_1.created_at = timezone.now() - timedelta(days=2)
@@ -254,12 +269,13 @@ class DeleteExpiredArtifactsTests(TestCase):
             parent_collection=collection_parent_1,
             child_type=CollectionItem.Types.ARTIFACT,
             artifact=artifact_1,
-            created_by_user=self.get_test_user(),
+            created_by_user=self.playground.get_default_user(),
         )
 
         self.assertEqual(self.delete_expired._mark_to_keep(), set())
 
-    def test_mark_to_keep_only_artifact_is_retained(self):
+    @context.disable_permission_checks()
+    def test_mark_to_keep_only_artifact_is_retained(self) -> None:
         """One artifact, expired yesterday. retains_artifacts is set."""
         artifact_1, _ = self.create_artifact(expiration_delay=1)
         artifact_1.created_at = timezone.now() - timedelta(days=2)
@@ -276,11 +292,12 @@ class DeleteExpiredArtifactsTests(TestCase):
             parent_collection=collection_parent_1,
             child_type=CollectionItem.Types.ARTIFACT,
             artifact=artifact_1,
-            created_by_user=self.get_test_user(),
+            created_by_user=self.playground.get_default_user(),
         )
 
         self.assertEqual(self.delete_expired._mark_to_keep(), {artifact_1.id})
 
+    @context.disable_permission_checks()
     def test_mark_to_keep_artifacts_retained_by_workflow(self) -> None:
         """RetainsArtifacts.WORKFLOWS retains while the workflow is active."""
         template = WorkflowTemplate.objects.create(
@@ -290,7 +307,9 @@ class DeleteExpiredArtifactsTests(TestCase):
         artifacts: dict[WorkRequest.Statuses, Artifact] = {}
         for status in WorkRequest.Statuses:
             workflow = WorkRequest.objects.create_workflow(
-                template=template, data={}, created_by=self.get_test_user()
+                template=template,
+                data={},
+                created_by=self.playground.get_default_user(),
             )
             workflow.status = status
             workflow.save()
@@ -304,7 +323,9 @@ class DeleteExpiredArtifactsTests(TestCase):
             artifacts[status].created_at = timezone.now() - timedelta(days=2)
             artifacts[status].save()
             collections[status].manager.add_artifact(
-                artifacts[status], user=self.get_test_user(), name=str(status)
+                artifacts[status],
+                user=self.playground.get_default_user(),
+                name=str(status),
             )
 
         self.assertEqual(
@@ -316,7 +337,8 @@ class DeleteExpiredArtifactsTests(TestCase):
             },
         )
 
-    def test_mark_to_keep_all_artifacts_non_expired_or_targeted(self):
+    @context.disable_permission_checks()
+    def test_mark_to_keep_all_artifacts_non_expired_or_targeted(self) -> None:
         """Keep both artifacts: the expired is targeted by the not-expired."""
         artifact_expired, _ = self.create_artifact(expiration_delay=1)
         artifact_expired.created_at = timezone.now() - timedelta(days=2)
@@ -330,7 +352,8 @@ class DeleteExpiredArtifactsTests(TestCase):
             {artifact_expired.id, artifact_not_expired.id},
         )
 
-    def test_mark_to_keep_no_artifact_all_expired_two(self):
+    @context.disable_permission_checks()
+    def test_mark_to_keep_no_artifact_all_expired_two(self) -> None:
         """Two artifacts (related), both expired."""
         artifact_expired_1, _ = self.create_artifact(expiration_delay=1)
         artifact_expired_1.created_at = timezone.now() - timedelta(days=2)
@@ -343,7 +366,8 @@ class DeleteExpiredArtifactsTests(TestCase):
 
         self.assertEqual(self.delete_expired._mark_to_keep(), set())
 
-    def test_mark_to_keep_no_artifact_all_expired_three(self):
+    @context.disable_permission_checks()
+    def test_mark_to_keep_no_artifact_all_expired_three(self) -> None:
         """All artifacts are expired."""
         artifact_1, _ = self.create_artifact(expiration_delay=1)
         artifact_1.created_at = timezone.now() - timedelta(days=2)
@@ -360,7 +384,8 @@ class DeleteExpiredArtifactsTests(TestCase):
 
         self.assertEqual(self.delete_expired._mark_to_keep(), set())
 
-    def test_mark_to_keep_two_artifacts_non_expired_and_targeted(self):
+    @context.disable_permission_checks()
+    def test_mark_to_keep_two_artifacts_non_expired_and_targeted(self) -> None:
         """Three artifacts, keep two (non-expired and target of non-expired)."""
         artifact_1, _ = self.create_artifact(expiration_delay=1)
         artifact_1.created_at = timezone.now() - timedelta(days=2)
@@ -378,7 +403,8 @@ class DeleteExpiredArtifactsTests(TestCase):
             {artifact_1.id, artifact_2.id},
         )
 
-    def test_mark_to_keep_isolated(self):
+    @context.disable_permission_checks()
+    def test_mark_to_keep_isolated(self) -> None:
         """Two expired and one non-expired isolated."""
         artifact_1, _ = self.create_artifact(expiration_delay=1)
         artifact_1.created_at = timezone.now() - timedelta(days=2)
@@ -393,7 +419,8 @@ class DeleteExpiredArtifactsTests(TestCase):
 
         self.assertEqual(self.delete_expired._mark_to_keep(), {artifact_3.id})
 
-    def test_mark_to_keep_artifacts_circular_dependency(self):
+    @context.disable_permission_checks()
+    def test_mark_to_keep_artifacts_circular_dependency(self) -> None:
         """Artifacts are not expired (keep), have a circular dependency."""
         artifact_1, _ = self.create_artifact(expiration_delay=0)
         artifact_2, _ = self.create_artifact(expiration_delay=0)
@@ -406,7 +433,8 @@ class DeleteExpiredArtifactsTests(TestCase):
             {artifact_1.id, artifact_2.id},
         )
 
-    def test_sweep_delete_artifacts(self):
+    @context.disable_permission_checks()
+    def test_sweep_delete_artifacts(self) -> None:
         """Sweep() delete the artifacts and print progress."""
         artifact_1, _ = self.create_artifact(expiration_delay=1)
         artifact_1.created_at = timezone.now() - timedelta(days=2)
@@ -437,32 +465,34 @@ class DeleteExpiredArtifactsTests(TestCase):
             _format_deleted_artifacts({artifact_1, artifact_2}),
         )
 
-    def test_sweep_delete_artifacts_no_retains(self):
+    @context.disable_permission_checks()
+    def test_sweep_delete_artifacts_no_retains(self) -> None:
         """Sweep() delete the artifacts from the collections."""
-        artifact, _ = self.create_artifact(
-            category=ArtifactCategory.SYSTEM_TARBALL,
-            data={
-                "codename": "bookworm",
-                "architecture": "amd64",
-                "variant": "apt",
-                "with_dev": True,
-            },
-            expiration_delay=1,
-        )
-        artifact.created_at = timezone.now() - timedelta(days=2)
-        artifact.save()
+        with context.disable_permission_checks():
+            artifact, _ = self.create_artifact(
+                category=ArtifactCategory.SYSTEM_TARBALL,
+                data={
+                    "codename": "bookworm",
+                    "architecture": "amd64",
+                    "variant": "apt",
+                    "with_dev": True,
+                },
+                expiration_delay=1,
+            )
+            artifact.created_at = timezone.now() - timedelta(days=2)
+            artifact.save()
 
-        collection = Collection.objects.create(
-            name="Debian",
-            category=CollectionCategory.ENVIRONMENTS,
-            workspace=default_workspace(),
-            retains_artifacts=Collection.RetainsArtifacts.NEVER,
-        )
+            collection = Collection.objects.create(
+                name="Debian",
+                category=CollectionCategory.ENVIRONMENTS,
+                workspace=default_workspace(),
+                retains_artifacts=Collection.RetainsArtifacts.NEVER,
+            )
 
         manager = DebianEnvironmentsManager(collection=collection)
         manager.add_artifact(
             artifact,
-            user=self.get_test_user(),
+            user=self.playground.get_default_user(),
             variables={"codename": "trixie"},
         )
 
@@ -478,7 +508,8 @@ class DeleteExpiredArtifactsTests(TestCase):
             _format_deleted_artifacts({artifact}),
         )
 
-    def test_delete_artifact_all_related_files(self):
+    @context.disable_permission_checks()
+    def test_delete_artifact_all_related_files(self) -> None:
         """_delete_artifact() deletes all the artifact's related models."""
         path_in_artifact = "README"
         artifact, _ = self.create_artifact(
@@ -491,12 +522,13 @@ class DeleteExpiredArtifactsTests(TestCase):
         self.assertEqual(File.objects.all().count(), 1)
 
         # Retrieve the store_backend for the file
-        fileobj = File.objects.all().first()
-        file_in_store = FileInStore.objects.all().first()
+        fileobj = File.objects.latest("id")
+        file_in_store = FileInStore.objects.latest("id")
         store_backend = file_in_store.store.get_backend_object()
 
         # The file exists
         filepath = store_backend.get_local_path(fileobj)
+        assert filepath is not None
         self.assertTrue(filepath.exists())
 
         # The Artifact is going to be deleted and the only file with it
@@ -514,7 +546,8 @@ class DeleteExpiredArtifactsTests(TestCase):
 
         self.assertTrue(filepath.exists())
 
-    def test_delete_artifact_not_related_files(self):
+    @context.disable_permission_checks()
+    def test_delete_artifact_not_related_files(self) -> None:
         """
         Two files created, _delete_artifact() delete the Artifact.
 
@@ -550,7 +583,9 @@ class DeleteExpiredArtifactsTests(TestCase):
         ).store.get_backend_object()
 
         file_to_keep_path = store_backend.get_local_path(file_to_keep)
+        assert file_to_keep_path is not None
         file_to_delete_path = store_backend.get_local_path(file_to_delete)
+        assert file_to_delete_path is not None
 
         # Both files exist on disk
         self.assertTrue(file_to_keep_path.exists())
@@ -569,7 +604,8 @@ class DeleteExpiredArtifactsTests(TestCase):
         self.assertTrue(file_to_keep_path.exists())
         self.assertTrue(file_to_delete_path.exists())
 
-    def test_delete_artifact_dry_run(self):
+    @context.disable_permission_checks()
+    def test_delete_artifact_dry_run(self) -> None:
         """_delete_artifact() does nothing: running in dry-run."""
         artifact, _ = self.create_artifact(expiration_delay=1)
         artifact.created_at = timezone.now() - timedelta(days=2)
@@ -586,7 +622,7 @@ class DeleteExpiredArtifactsTests(TestCase):
         # No callables to delete files after the transaction returned
         self.assertEqual(len(callables), 0)
 
-    def test_delete_files_from_stores(self):
+    def test_delete_files_from_stores(self) -> None:
         """_delete_files_from_stores delete the file."""
         fileobj = self.create_file()
         FileInStore.objects.create(store=default_file_store(), file=fileobj)
@@ -595,7 +631,7 @@ class DeleteExpiredArtifactsTests(TestCase):
 
         self.assertFalse(File.objects.filter(id=fileobj.id).exists())
 
-    def test_delete_files_from_multiple_stores(self):
+    def test_delete_files_from_multiple_stores(self) -> None:
         """_delete_files_from_stores delete files from more than one store."""
         fileobj = self.create_file()
 
@@ -616,14 +652,15 @@ class DeleteExpiredArtifactsTests(TestCase):
         # And the file is gone...
         self.assertFalse(File.objects.filter(id=fileobj.id).exists())
 
-    def test_delete_files_from_stores_file_was_re_added(self):
+    @context.disable_permission_checks()
+    def test_delete_files_from_stores_file_was_re_added(self) -> None:
         """
         _delete_files_from_stores does not delete the file.
 
         The file exist in another Artifact: cannot be deleted.
         """
         fileobj = self.create_file()
-        files_to_delete = [fileobj]
+        files_to_delete = {fileobj}
 
         artifact, _ = self.create_artifact()
         FileInArtifact.objects.create(artifact=artifact, file=fileobj)
@@ -638,29 +675,33 @@ class DeleteExpiredArtifactsTransactionTests(TransactionTestCase):
 
     playground_memory_file_store = False
 
-    def setUp(self):
+    @context.disable_permission_checks()
+    def setUp(self) -> None:
         """Initialize test."""
         super().setUp()
-        self.workspace = self.playground.get_default_workspace()
+        with context.disable_permission_checks():
+            self.workspace = self.playground.get_default_workspace()
 
-        self.artifact_1, _ = self.playground.create_artifact(
-            expiration_delay=1, paths=["README"], create_files=True
-        )
-        self.artifact_1.created_at = timezone.now() - timedelta(days=2)
-        self.artifact_1.save()
-        self.artifact_2, _ = self.playground.create_artifact(expiration_delay=1)
-        self.artifact_2.created_at = timezone.now() - timedelta(days=2)
-        self.artifact_2.save()
+            self.artifact_1, _ = self.playground.create_artifact(
+                expiration_delay=1, paths=["README"], create_files=True
+            )
+            self.artifact_1.created_at = timezone.now() - timedelta(days=2)
+            self.artifact_1.save()
+            self.artifact_2, _ = self.playground.create_artifact(
+                expiration_delay=1
+            )
+            self.artifact_2.created_at = timezone.now() - timedelta(days=2)
+            self.artifact_2.save()
 
-        self.artifact_relation = self.playground.create_artifact_relation(
-            self.artifact_1, self.artifact_2
-        )
+            self.artifact_relation = self.playground.create_artifact_relation(
+                self.artifact_1, self.artifact_2
+            )
 
-        self.out = io.StringIO()
-        self.err = io.StringIO()
+            self.out = io.StringIO()
+            self.err = io.StringIO()
 
-        self.operation = DeleteOperation(out=self.out, err=self.err)
-        self.delete_expired = DeleteExpiredArtifacts(self.operation)
+            self.operation = DeleteOperation(out=self.out, err=self.err)
+            self.delete_expired = DeleteExpiredArtifacts(self.operation)
 
     def assert_delete_expired_run_failed(
         self,
@@ -669,7 +710,7 @@ class DeleteExpiredArtifactsTransactionTests(TransactionTestCase):
         expected_artifact_relations: int,
         timeout: float,
         err: io.StringIO,
-    ):
+    ) -> None:
         """Assert result of deletion of artifacts when lock table failed."""
         self.assertEqual(Artifact.objects.all().count(), expected_artifacts)
         self.assertEqual(
@@ -681,7 +722,7 @@ class DeleteExpiredArtifactsTransactionTests(TransactionTestCase):
             f"Lock timed out ({timeout} seconds). Try again.\n",
         )
 
-    def test_run_cannot_lock_artifact_relation_return_false(self):
+    def test_run_cannot_lock_artifact_relation_return_false(self) -> None:
         """
         run() cannot lock the tables of Artifact or ArtifactRelation.
 
@@ -702,9 +743,10 @@ class DeleteExpiredArtifactsTransactionTests(TransactionTestCase):
                 expected_relations = ArtifactRelation.objects.all().count()
 
                 manager = instance._meta.default_manager
+                assert manager is not None
 
                 thread = RunInParallelTransaction(
-                    lambda: manager.select_for_update().get(id=instance.id)
+                    lambda: manager.select_for_update().get(pk=instance.pk)
                 )
 
                 thread.start_transaction()
@@ -724,7 +766,7 @@ class DeleteExpiredArtifactsTransactionTests(TransactionTestCase):
                 )
 
     @staticmethod
-    def check_table_locked_for(manager: Manager[Any]):
+    def check_table_locked_for(manager: Manager[Any]) -> bool:
         """
         Call manager.first(). Expect timeout.
 
@@ -743,14 +785,14 @@ class DeleteExpiredArtifactsTransactionTests(TransactionTestCase):
 
         return table_was_locked
 
-    def test_tables_locked_on_sweep(self):
+    def test_tables_locked_on_sweep(self) -> None:
         """Check relevant tables are locked when _sweep() is called."""
 
-        def wait_for_check(*args, **kwargs):
+        def wait_for_check(*args: Any, **kwargs: Any) -> set[File]:
             """Notify that _sweep() is called, wait for the check."""
             sweep_is_called.set()
             wait_for_check_tables_locked.wait()
-            return []
+            return set()
 
         sweep_is_called = threading.Event()
         wait_for_check_tables_locked = threading.Event()
@@ -784,7 +826,7 @@ class DeleteExpiredArtifactsTransactionTests(TransactionTestCase):
 
             delete_expired_run.join()
 
-    def test_on_commit_cleanup_delete_files(self):
+    def test_on_commit_cleanup_delete_files(self) -> None:
         """File is deleted from the store if the transaction is committed."""
         artifact, _ = self.playground.create_artifact(
             paths=["README"], create_files=True, expiration_delay=1
@@ -802,6 +844,7 @@ class DeleteExpiredArtifactsTransactionTests(TransactionTestCase):
         store_backend = file_in_store.store.get_backend_object()
 
         file_path = store_backend.get_local_path(fileobj)
+        assert file_path is not None
         self.assertTrue(file_path.exists())
 
         with self.operation:
@@ -819,7 +862,7 @@ class DeleteExpiredArtifactsTransactionTests(TransactionTestCase):
             message_artifacts_deleted + "Deleting files from the store\n",
         )
 
-    def test_deleting_files_artifact_table_is_locked(self):
+    def test_deleting_files_artifact_table_is_locked(self) -> None:
         """Files get deleted while the Artifact table is still locked."""
         # Delete a relation and an artifact, and create an artifact
         # that expires tomorrow. This is to simplify the test
@@ -848,7 +891,7 @@ class DeleteExpiredArtifactsTransactionTests(TransactionTestCase):
 
         delete_files_mocked.assert_called()
 
-    def test_delete_files_from_stores_raise_artifact_is_deleted(self):
+    def test_delete_files_from_stores_raise_artifact_is_deleted(self) -> None:
         """Artifact is deleted even if the file store deletion fail."""
         exception = RuntimeError
         patcher = mock.patch.object(
@@ -878,7 +921,7 @@ class DeleteExpiredArtifactsTransactionTests(TransactionTestCase):
 class DeleteExpiredWorkRequestsTests(TestCase):
     """Tests for the DeleteExpiredWorkRequests class."""
 
-    def setUp(self):
+    def setUp(self) -> None:
         """Initialize test."""
         self.out = io.StringIO()
         self.err = io.StringIO()
@@ -932,7 +975,7 @@ class DeleteExpiredWorkRequestsTests(TestCase):
 
     def test_one_expired(self) -> None:
         """One expired work request."""
-        wr1 = self.create_work_request(expired=True)
+        wr1 = self.playground.create_work_request(expired=True)
         with (
             self.mock_perform_deletions() as perform_deletions,
             self.mock_verbose() as verbose,
@@ -949,9 +992,9 @@ class DeleteExpiredWorkRequestsTests(TestCase):
 
     def test_multiple_expired(self) -> None:
         """Multiple expired work requests."""
-        wr1 = self.create_work_request(expired=True)
-        wr2 = self.create_work_request(expired=True)
-        wr3 = self.create_work_request(parent=wr2, expired=True)
+        wr1 = self.playground.create_work_request(expired=True)
+        wr2 = self.playground.create_work_request(expired=True)
+        wr3 = self.playground.create_work_request(parent=wr2, expired=True)
         with (
             self.mock_perform_deletions() as perform_deletions,
             self.mock_verbose() as verbose,
@@ -982,13 +1025,13 @@ class DeleteExpiredWorkRequestsTests(TestCase):
             "internal-2", CollectionCategory.WORKFLOW_INTERNAL
         )
 
-        work_request_keep = self.create_work_request(
+        work_request_keep = self.playground.create_work_request(
             created_at=timezone.now() - timedelta(days=365),
             expiration_delay=timedelta(days=1),
             internal_collection=collection_2,
         )
 
-        work_request_delete = self.create_work_request(
+        work_request_delete = self.playground.create_work_request(
             created_at=timezone.now() - timedelta(days=365),
             expiration_delay=timedelta(days=1),
             internal_collection=collection_1,
@@ -1009,7 +1052,7 @@ class DeleteExpiredWorkRequestsTests(TestCase):
         # WorkRequest.internal_collection != collection_1
         collection_1.manager.add_bare_data(
             name="Item-1",
-            user=self.get_test_user(),
+            user=self.playground.get_default_user(),
             category=BareDataCategory.TEST,
             workflow=work_request_keep,
         )
@@ -1019,7 +1062,7 @@ class DeleteExpiredWorkRequestsTests(TestCase):
         # WorkRequest.internal-collection == collection_2
         collection_1.manager.add_bare_data(
             name="Item-2",
-            user=self.get_test_user(),
+            user=self.playground.get_default_user(),
             category=BareDataCategory.TEST,
             workflow=work_request_delete,
         )
@@ -1034,7 +1077,9 @@ class DeleteExpiredWorkRequestsTests(TestCase):
         # work_request_keep still kept: now is referenced by a
         # CollectionItem.removed_by_workflow
         collection_1.manager.remove_bare_data(
-            name="Item-1", user=self.get_test_user(), workflow=work_request_keep
+            name="Item-1",
+            user=self.playground.get_default_user(),
+            workflow=work_request_keep,
         )
 
         with self.mock_perform_deletions():
@@ -1057,9 +1102,9 @@ class DeleteExpiredWorkRequestsTests(TestCase):
 
     def test_expired_but_valid_parent(self) -> None:
         """Keep work requests with non-expired parent."""
-        wr1 = self.create_work_request()
-        self.create_work_request(parent=wr1, expired=True)
-        wr3 = self.create_work_request(expired=True)
+        wr1 = self.playground.create_work_request()
+        self.playground.create_work_request(parent=wr1, expired=True)
+        wr3 = self.playground.create_work_request(expired=True)
         with (
             self.mock_perform_deletions() as perform_deletions,
             self.mock_verbose() as verbose,
@@ -1079,7 +1124,9 @@ class DeleteExpiredWorkRequestsTests(TestCase):
         c1 = self.playground.create_collection(
             "Collection-1", CollectionCategory.TEST
         )
-        wr1 = self.create_work_request(internal_collection=c1, expired=True)
+        wr1 = self.playground.create_work_request(
+            internal_collection=c1, expired=True
+        )
         with (
             self.mock_perform_deletions() as perform_deletions,
             self.mock_verbose() as verbose,
