@@ -14,6 +14,7 @@ import io
 from django.core.management import CommandError
 
 from debusine.artifacts.models import ArtifactCategory, CollectionCategory
+from debusine.db.context import context
 from debusine.db.models import (
     ArtifactRelation,
     WorkRequest,
@@ -21,6 +22,7 @@ from debusine.db.models import (
     Workspace,
     default_workspace,
 )
+from debusine.db.playground import Playground, scenarios
 from debusine.django.management.tests import call_command
 from debusine.test.django import TestCase
 
@@ -28,10 +30,12 @@ from debusine.test.django import TestCase
 class DeleteWorkspaceCommandTests(TestCase):
     """Tests for the delete_workspace command."""
 
-    def create_test_workspace(self):
+    @context.disable_permission_checks()
+    def create_test_workspace(self) -> Workspace:
         """Create a test workspace."""
-        workspace = self.playground.create_workspace(name="Test")
-        work_request = self.create_work_request(
+        scope = self.playground.get_or_create_scope("scope")
+        workspace = self.playground.create_workspace(name="Test", scope=scope)
+        work_request = self.playground.create_work_request(
             workspace=workspace, task_name="noop"
         )
         collection = self.playground.create_collection(
@@ -80,11 +84,13 @@ class DeleteWorkspaceCommandTests(TestCase):
             ArtifactRelation.Relations.RELATES_TO,
         )
         collection.manager.add_artifact(
-            artifact_hello, user=self.get_test_user(), name="hello"
+            artifact_hello,
+            user=self.playground.get_default_user(),
+            name="hello",
         )
         collection.manager.add_artifact(
             artifact_hello_traditional,
-            user=self.get_test_user(),
+            user=self.playground.get_default_user(),
             name="hello-traditional",
         )
         WorkflowTemplate.objects.create(
@@ -120,57 +126,58 @@ class DeleteWorkspaceCommandTests(TestCase):
         workflow.mark_running()
         workflow.save()
 
-        self.assertIsNotNone(workflow.internal_collection)
+        assert workflow.internal_collection is not None
         self.assertEqual(workflow.internal_collection.workflow, workflow)
 
         # TODO: add more kinds of elements to the workspace, to make
         # sure deletion catches them
         return workspace
 
-    def test_delete_missing_workspace(self):
+    def test_delete_missing_workspace(self) -> None:
         """Workspace does not exist."""
+        self.playground.get_or_create_scope("scope")
         with self.assertRaisesRegex(
-            CommandError, r"^Workspace Test does not exist$"
+            CommandError, r"^Workspace Test does not exist in scope scope$"
         ) as exc:
             call_command(
                 'delete_workspace',
-                'Test',
+                "scope/Test",
             )
         self.assertEqual(exc.exception.returncode, 3)
 
         with self.assertRaisesRegex(
-            CommandError, r"^Workspace Test does not exist$"
+            CommandError, r"^Workspace Test does not exist in scope scope$"
         ) as exc:
             call_command(
                 'delete_workspace',
+                "scope/Test",
                 '--yes',
-                'Test',
             )
         self.assertEqual(exc.exception.returncode, 3)
 
         stdout, stderr, _ = call_command(
             'delete_workspace',
+            "scope/Test",
             '--force',
-            'Test',
         )
         self.assertEqual('', stdout)
 
         stdout, stderr, _ = call_command(
             'delete_workspace',
+            "scope/Test",
             '--force',
             "--yes",
-            'Test',
         )
         self.assertEqual('', stdout)
 
-    def test_delete_workspace(self):
+    def test_delete_workspace(self) -> None:
         """Delete an existing workspace."""
         workspace = self.create_test_workspace()
 
         stdout, stderr, _ = call_command(
             'delete_workspace',
+            "scope/Test",
             "--yes",
-            'Test',
         )
         self.assertEqual('', stdout)
 
@@ -180,21 +187,50 @@ class DeleteWorkspaceCommandTests(TestCase):
         with self.assertRaises(Workspace.DoesNotExist):
             Workspace.objects.get(name="Test")
 
-    def test_delete_workspace_confirmation(self):
+    def test_delete_workspace_playground_ui_scenario(self) -> None:
+        """Delete a workspace with the playground UI scenario."""
+        playground = Playground(default_workspace_name="Playground")
+        scenario = scenarios.UIPlayground()
+        playground.build_scenario(scenario)
+
+        stdout, stderr, _ = call_command(
+            'delete_workspace',
+            str(scenario.workspace),
+            "--yes",
+        )
+        self.assertEqual('', stdout)
+
+        with self.assertRaises(Workspace.DoesNotExist):
+            Workspace.objects.get(id=scenario.workspace.id)
+
+        with self.assertRaises(Workspace.DoesNotExist):
+            Workspace.objects.get(
+                scope=scenario.scope, name=scenario.workspace.name
+            )
+
+    def test_delete_workspace_confirmation(self) -> None:
         """delete_workspace doesn't delete (user does not confirm)."""
         workspace = self.create_test_workspace()
 
-        call_command("delete_workspace", "Test", stdin=io.StringIO("N\n"))
-        self.assertQuerysetEqual(
+        call_command(
+            "delete_workspace",
+            "scope/Test",
+            stdin=io.StringIO("N\n"),
+        )
+        self.assertQuerySetEqual(
             Workspace.objects.filter(name="Test"), [workspace]
         )
 
-        call_command("delete_workspace", "Test", stdin=io.StringIO("\n"))
-        self.assertQuerysetEqual(
+        call_command(
+            "delete_workspace",
+            "scope/Test",
+            stdin=io.StringIO("\n"),
+        )
+        self.assertQuerySetEqual(
             Workspace.objects.filter(name="Test"), [workspace]
         )
 
-    def test_delete_default_workspace(self):
+    def test_delete_default_workspace(self) -> None:
         """Default workspace does not get deleted."""
         workspace = default_workspace()
         with self.assertRaisesRegex(
@@ -203,8 +239,17 @@ class DeleteWorkspaceCommandTests(TestCase):
         ) as exc:
             call_command(
                 "delete_workspace",
-                workspace.name,
+                str(workspace),
                 "--yes",
                 "--force",
             )
         self.assertEqual(exc.exception.returncode, 3)
+
+    def test_delete_workspace_with_scope_not_found(self) -> None:
+        """delete_workspace: scope not found."""
+        with self.assertRaisesRegex(
+            CommandError, 'Scope "nonexistent" not found'
+        ) as exc:
+            call_command("delete_workspace", "nonexistent/test")
+
+        self.assertEqual(exc.exception.returncode, 3)
