@@ -11,27 +11,31 @@
 
 import copy
 import io
+from typing import Any, ClassVar
 
 import yaml
 from django.core.management import CommandError
-from django.test import TestCase
 
 from debusine.db.models import NotificationChannel
 from debusine.django.management.tests import call_command
-from debusine.test import TestHelpersMixin
+from debusine.test.django import TestCase
 
 
-class ManageNotificationChannelTests(TestHelpersMixin, TestCase):
+class ManageNotificationChannelTests(TestCase):
     """Tests for the manage_notification_channel command."""
 
-    def setUp(self):
+    email_data: ClassVar[dict[str, Any]]
+
+    @classmethod
+    def setUpTestData(cls) -> None:
         """Set up test."""
-        self.email_data = {
+        super().setUpTestData()
+        cls.email_data = {
             "from": "debusine@debusine.com",
             "to": ["user@example.com"],
         }
 
-    def test_change_name(self):
+    def test_change_name(self) -> None:
         """Verify change name of a notification channel."""
         old_name = "lts"
         new_name = "new-lts"
@@ -57,7 +61,7 @@ class ManageNotificationChannelTests(TestHelpersMixin, TestCase):
             NotificationChannel.objects.filter(name=new_name).exists()
         )
 
-    def test_change_data_from_file(self):
+    def test_change_data_from_file(self) -> None:
         """Verify change data of a notification channel from file."""
         new_data = copy.deepcopy(self.email_data)
         new_data["from"] = "a-new-email@example.com"
@@ -90,7 +94,7 @@ class ManageNotificationChannelTests(TestHelpersMixin, TestCase):
             NotificationChannel.objects.get(name=name).data, new_data
         )
 
-    def test_change_data_from_stdin(self):
+    def test_change_data_from_stdin(self) -> None:
         """Verify change data of a notification channel from stdin."""
         new_data = copy.deepcopy(self.email_data)
         new_data["from"] = "a-new-email@example.com"
@@ -118,7 +122,7 @@ class ManageNotificationChannelTests(TestHelpersMixin, TestCase):
             NotificationChannel.objects.get(name=name).data, new_data
         )
 
-    def test_change_data_invalid_data(self):
+    def test_change_data_invalid_data(self) -> None:
         """Command raise CommandError: invalid data."""
         name = "deblts-email"
         NotificationChannel.objects.create(
@@ -142,7 +146,7 @@ class ManageNotificationChannelTests(TestHelpersMixin, TestCase):
 
         self.assertEqual(exc.exception.returncode, 3)
 
-    def test_change_name_name_does_not_exist(self):
+    def test_change_name_name_does_not_exist(self) -> None:
         """Name cannot be changed: NotificationChannel does not exist."""
         name = "does-not-exist"
         expected_error = f'NotificationChannel "{name}" does not exist'
