@@ -13,7 +13,8 @@ from typing import ClassVar
 
 from django.core.management import CommandError
 
-from debusine.db.models import FileStore, Workspace
+from debusine.db.context import context
+from debusine.db.models import FileStore, Scope, Workspace
 from debusine.django.management.tests import call_command
 from debusine.test.django import TestCase
 
@@ -21,28 +22,33 @@ from debusine.test.django import TestCase
 class ManageWorkspaceCommandTests(TestCase):
     """Tests for manage_workspace management command."""
 
+    scope: ClassVar[Scope]
     workspace: ClassVar[Workspace]
 
     @classmethod
-    def setUpTestData(cls):
+    @context.disable_permission_checks()
+    def setUpTestData(cls) -> None:
         """Set up test fixture."""
         super().setUpTestData()
-        cls.workspace = cls.playground.create_workspace(name="test")
+        cls.scope = cls.playground.get_or_create_scope(name="scope")
+        cls.workspace = cls.playground.create_workspace(
+            scope=cls.scope, name="test"
+        )
 
-    def test_no_default_reset(self):
+    def test_no_default_reset(self) -> None:
         """Don't confuse no-option with reset-to-default."""
         # Non-default values
         default_expiration_delay = 7
         stdout, stderr, exit_code = call_command(
             "manage_workspace",
-            self.workspace.name,
+            str(self.workspace),
             "--public",
             f"--default-expiration-delay={default_expiration_delay}",
         )
 
         stdout, stderr, exit_code = call_command(
             "manage_workspace",
-            self.workspace.name,
+            str(self.workspace),
         )
 
         self.workspace.refresh_from_db()
@@ -52,13 +58,13 @@ class ManageWorkspaceCommandTests(TestCase):
             timedelta(default_expiration_delay),
         )
 
-    def test_change_public(self):
+    def test_change_public(self) -> None:
         """manage_workspace changes the public status."""
         for public in (True, False):
             stdout, stderr, exit_code = call_command(
                 "manage_workspace",
                 "--public" if public else "--private",
-                self.workspace.name,
+                str(self.workspace),
             )
 
             self.assertEqual(stdout, "")
@@ -68,7 +74,7 @@ class ManageWorkspaceCommandTests(TestCase):
             self.workspace.refresh_from_db()
             self.assertEqual(self.workspace.public, public)
 
-    def test_change_public_not_found(self):
+    def test_change_public_not_found(self) -> None:
         """manage_workspace workspace not found."""
         name = "non-existing-workspace"
 
@@ -77,12 +83,12 @@ class ManageWorkspaceCommandTests(TestCase):
             call_command(
                 "manage_workspace",
                 "--public",
-                name,
+                self.scope.name + "/" + name,
             )
 
         self.assertEqual(exc.exception.returncode, 3)
 
-    def test_change_default_expiration_delay(self):
+    def test_change_default_expiration_delay(self) -> None:
         """manage_workspace changes the default expiration delay."""
         default_expiration_delay = 123456
 
@@ -91,14 +97,14 @@ class ManageWorkspaceCommandTests(TestCase):
                 "manage_workspace",
                 "--default-expiration-delay",
                 "letters",
-                self.workspace.name,
+                str(self.workspace),
             )
 
         stdout, stderr, exit_code = call_command(
             "manage_workspace",
             "--default-expiration-delay",
-            default_expiration_delay,
-            self.workspace.name,
+            str(default_expiration_delay),
+            str(self.workspace),
         )
 
         self.assertEqual(stdout, "")
@@ -117,8 +123,8 @@ class ManageWorkspaceCommandTests(TestCase):
         call_command(
             "manage_workspace",
             "--default-expiration-delay",
-            default_expiration_delay,
-            self.workspace.name,
+            str(default_expiration_delay),
+            str(self.workspace),
         )
 
         self.workspace.refresh_from_db()
@@ -127,7 +133,7 @@ class ManageWorkspaceCommandTests(TestCase):
             timedelta(default_expiration_delay),
         )
 
-    def test_change_default_file_store(self):
+    def test_change_default_file_store(self) -> None:
         """manage_workspace changes the default file store."""
         default_file_store = FileStore.objects.create(
             name="test", backend=FileStore.BackendChoices.LOCAL
@@ -137,7 +143,7 @@ class ManageWorkspaceCommandTests(TestCase):
             "manage_workspace",
             "--default-file-store",
             "test",
-            self.workspace.name,
+            str(self.workspace),
         )
 
         self.assertEqual(stdout, "")
@@ -147,7 +153,7 @@ class ManageWorkspaceCommandTests(TestCase):
         self.workspace.refresh_from_db()
         self.assertEqual(self.workspace.default_file_store, default_file_store)
 
-    def test_change_default_file_store_not_found(self):
+    def test_change_default_file_store_not_found(self) -> None:
         """manage_workspace: new default file store not found."""
         with self.assertRaisesRegex(
             CommandError, 'File store "nonexistent" not found'
@@ -156,12 +162,12 @@ class ManageWorkspaceCommandTests(TestCase):
                 "manage_workspace",
                 "--default-file-store",
                 "nonexistent",
-                self.workspace.name,
+                str(self.workspace),
             )
 
         self.assertEqual(exc.exception.returncode, 3)
 
-    def test_change_all(self):
+    def test_change_all(self) -> None:
         """manage_workspace changes multiple fields at once."""
         default_expiration_delay = 7
         default_file_store = FileStore.objects.create(
@@ -173,7 +179,7 @@ class ManageWorkspaceCommandTests(TestCase):
             "--public",
             f"--default-expiration-delay={default_expiration_delay}",
             "--default-file-store=test",
-            self.workspace.name,
+            str(self.workspace),
         )
 
         self.assertEqual(stdout, "")
@@ -187,3 +193,12 @@ class ManageWorkspaceCommandTests(TestCase):
             timedelta(default_expiration_delay),
         )
         self.assertEqual(self.workspace.default_file_store, default_file_store)
+
+    def test_delete_workspace_with_scope_not_found(self) -> None:
+        """manage_workspace: scope not found."""
+        with self.assertRaisesRegex(
+            CommandError, 'Scope "nonexistent" not found'
+        ) as exc:
+            call_command("manage_workspace", "nonexistent/test")
+
+        self.assertEqual(exc.exception.returncode, 3)
