@@ -12,24 +12,34 @@
 import os
 from datetime import datetime, timedelta
 from pathlib import Path
+from typing import TypedDict
 from unittest import mock
 
-from django.test import TestCase
 from django.utils import timezone
 
+from debusine.db.context import context
 from debusine.db.models import Artifact, FileInArtifact, FileStore, FileUpload
 from debusine.django.management.tests import call_command
 from debusine.project import settings
 from debusine.server.management.commands.monthly_cleanup import Command
-from debusine.test import TestHelpersMixin
+from debusine.test.django import TestCase
 
 
-class MonthlyCleanupCommandTests(TestHelpersMixin, TestCase):
+class _FilesAndDirectories(TypedDict):
+    """Dictionary containing files and directories for testing."""
+
+    root: Path
+    files: set[Path]
+    directories: set[Path]
+
+
+class MonthlyCleanupCommandTests(TestCase):
     """Tests for monthly_cleanup command."""
 
     playground_memory_file_store = False
 
-    def setUp(self):
+    @context.disable_permission_checks()
+    def setUp(self) -> None:
         """Set common objects and settings."""
         # Using override_settings does not work (perhaps because it's a
         # lambda calculated setting)
@@ -51,25 +61,28 @@ class MonthlyCleanupCommandTests(TestHelpersMixin, TestCase):
         self.original_upload_directory = settings.DEBUSINE_STORE_DIRECTORY
         settings.DEBUSINE_UPLOAD_DIRECTORY = self.debusine_upload_directory
 
-    def tearDown(self):
+    def tearDown(self) -> None:
         """Restore settings."""
         settings.DEBUSINE_STORE_DIRECTORY = (
             self.original_debusine_store_directory
         )
         settings.DEBUSINE_UPLOAD_DIRECTORY = self.original_upload_directory
 
-    def test_delete_file_in_disk_not_in_store_dry_run(self):
+    def test_delete_file_in_disk_not_in_store_dry_run(self) -> None:
         """
         Test monthly_cleanup find file in disk not in DB.
 
         Also test that is not finding a file part of an artifact.
         """
-        artifact, _ = self.create_artifact(paths=["README"], create_files=True)
+        with context.disable_permission_checks():
+            artifact, _ = self.create_artifact(
+                paths=["README"], create_files=True
+            )
 
-        orphan_file = self.create_temporary_file(
-            directory=self.debusine_store_directory
-        )
-        self._set_modified_time_two_days_ago(orphan_file)
+            orphan_file = self.create_temporary_file(
+                directory=self.debusine_store_directory
+            )
+            self._set_modified_time_two_days_ago(orphan_file)
 
         stdout, stderr, exitcode = call_command(
             "monthly_cleanup", "--dry-run", verbosity=2
@@ -100,21 +113,22 @@ class MonthlyCleanupCommandTests(TestHelpersMixin, TestCase):
         assert path is not None
         return path
 
-    def test_delete_file_in_disk_not_in_db(self):
+    def test_delete_file_in_disk_not_in_db(self) -> None:
         """
         Test monthly_cleanup find and delete file that is in disk but not in DB.
 
         Also test that file part of an artifact is not deleted.
         """
-        path_in_artifact = "README"
-        artifact, _ = self.create_artifact(
-            paths=[path_in_artifact], create_files=True
-        )
+        with context.disable_permission_checks():
+            path_in_artifact = "README"
+            artifact, _ = self.create_artifact(
+                paths=[path_in_artifact], create_files=True
+            )
 
-        temporary_file = self.create_temporary_file(
-            directory=self.debusine_store_directory
-        )
-        self._set_modified_time_two_days_ago(temporary_file)
+            temporary_file = self.create_temporary_file(
+                directory=self.debusine_store_directory
+            )
+            self._set_modified_time_two_days_ago(temporary_file)
         stdout, stderr, exitcode = call_command("monthly_cleanup")
 
         # The file was deleted
@@ -129,7 +143,7 @@ class MonthlyCleanupCommandTests(TestHelpersMixin, TestCase):
         self.assertEqual(stderr, "")
         self.assertEqual(exitcode, 0)
 
-    def test_file_not_deleted_too_new(self):
+    def test_file_not_deleted_too_new(self) -> None:
         """Do not delete the file: it's too new."""
         temporary_file = self.create_temporary_file(
             directory=self.debusine_store_directory
@@ -139,13 +153,14 @@ class MonthlyCleanupCommandTests(TestHelpersMixin, TestCase):
         # The file was not deleted
         self.assertTrue(temporary_file.exists())
 
-    def test_delete_empty_directory_dry_run(self):
+    def test_delete_empty_directory_dry_run(self) -> None:
         """Report delete an empty directory (but dry-run: does not delete)."""
-        empty_directory = self.create_temporary_directory(
-            directory=self.debusine_store_directory
-        )
-        self._set_modified_time_two_days_ago(empty_directory)
-        self.create_artifact(paths=["README"], create_files=True)
+        with context.disable_permission_checks():
+            empty_directory = self.create_temporary_directory(
+                directory=self.debusine_store_directory
+            )
+            self._set_modified_time_two_days_ago(empty_directory)
+            self.create_artifact(paths=["README"], create_files=True)
 
         stdout, stderr, exitcode = call_command(
             "monthly_cleanup", "--dry-run", verbosity=2
@@ -167,18 +182,19 @@ class MonthlyCleanupCommandTests(TestHelpersMixin, TestCase):
         # Not deleted: --dry-run was used
         self.assertTrue(empty_directory.is_dir())
 
-    def test_delete_empty_directory(self):
+    def test_delete_empty_directory(self) -> None:
         """Delete an empty directory (and leave a non-empty directory)."""
-        empty_dir = self.create_temporary_directory(
-            directory=self.debusine_store_directory
-        )
-        self._set_modified_time_two_days_ago(empty_dir)
-        path_in_artifact = "README"
-        artifact, _ = self.create_artifact(
-            paths=[path_in_artifact],
-            create_files=True,
-            skip_add_files_in_store=False,
-        )
+        with context.disable_permission_checks():
+            empty_dir = self.create_temporary_directory(
+                directory=self.debusine_store_directory
+            )
+            self._set_modified_time_two_days_ago(empty_dir)
+            path_in_artifact = "README"
+            artifact, _ = self.create_artifact(
+                paths=[path_in_artifact],
+                create_files=True,
+                skip_add_files_in_store=False,
+            )
 
         stdout, stderr, exitcode = call_command("monthly_cleanup")
 
@@ -193,7 +209,7 @@ class MonthlyCleanupCommandTests(TestHelpersMixin, TestCase):
         local_file_path = self.get_local_file_path(artifact, path_in_artifact)
         self.assertTrue(local_file_path.parent.is_dir())
 
-    def test_delete_empty_directory_too_new(self):
+    def test_delete_empty_directory_too_new(self) -> None:
         """Directory is not deleted: it's too new."""
         empty_dir = self.create_temporary_directory(
             directory=self.debusine_store_directory
@@ -202,7 +218,7 @@ class MonthlyCleanupCommandTests(TestHelpersMixin, TestCase):
 
         self.assertTrue(empty_dir.is_dir())
 
-    def test_delete_directory_fails(self):
+    def test_delete_directory_fails(self) -> None:
         """directory.rmdir() fails (file was probably added after the check)."""
         empty_dir = self.create_temporary_directory(
             directory=self.debusine_store_directory
@@ -223,20 +239,23 @@ class MonthlyCleanupCommandTests(TestHelpersMixin, TestCase):
 
         self.assertTrue(empty_dir.is_dir())
 
-    def test_delete_unfinished_uploads_no_delete_dry_run(self):
+    def test_delete_unfinished_uploads_no_delete_dry_run(self) -> None:
         """Unfinished, old FileUpload is not deleted: dry run."""
-        artifact, _ = self.create_artifact(paths=["README"], create_files=True)
-        file_in_artifact = FileInArtifact.objects.get(artifact=artifact)
+        with context.disable_permission_checks():
+            artifact, _ = self.create_artifact(
+                paths=["README"], create_files=True
+            )
+            file_in_artifact = FileInArtifact.objects.get(artifact=artifact)
 
-        temporary_file = self.create_temporary_file(
-            directory=self.debusine_upload_directory
-        )
+            temporary_file = self.create_temporary_file(
+                directory=self.debusine_upload_directory
+            )
 
-        file_upload = FileUpload.objects.create(
-            file_in_artifact=file_in_artifact, path=temporary_file.name
-        )
-        file_upload.last_activity_at = timezone.now() - timedelta(days=2)
-        file_upload.save()
+            file_upload = FileUpload.objects.create(
+                file_in_artifact=file_in_artifact, path=temporary_file.name
+            )
+            file_upload.last_activity_at = timezone.now() - timedelta(days=2)
+            file_upload.save()
 
         stdout, stderr, exitcode = call_command(
             "monthly_cleanup", "--dry-run", verbosity=2
@@ -256,21 +275,25 @@ class MonthlyCleanupCommandTests(TestHelpersMixin, TestCase):
         self.assertEqual(exitcode, 0)
 
         # Check that the FileUpload row was not deleted
-        self.assertQuerysetEqual(
+        self.assertQuerySetEqual(
             FileUpload.objects.filter(id=file_upload.id), [file_upload]
         )
 
-    def test_delete_unfinished_upload_deleted(self):
+    def test_delete_unfinished_upload_deleted(self) -> None:
         """Unfinished and old FileUpload and its file is deleted."""
-        artifact, _ = self.create_artifact(paths=["README"], create_files=True)
-        file_in_artifact = FileInArtifact.objects.get(artifact=artifact)
-
-        file_upload = FileUpload.objects.create(
-            file_in_artifact=file_in_artifact, path="some-file-path"
-        )
-        file_upload.last_activity_at = timezone.now() - timedelta(days=2)
-        file_upload.path = Path(self.debusine_upload_directory) / "the-file"
-        file_upload.save()
+        with context.disable_permission_checks():
+            artifact, _ = self.create_artifact(
+                paths=["README"], create_files=True
+            )
+            file_in_artifact = FileInArtifact.objects.get(artifact=artifact)
+            file_upload = FileUpload.objects.create(
+                file_in_artifact=file_in_artifact, path="some-file-path"
+            )
+            file_upload.last_activity_at = timezone.now() - timedelta(days=2)
+            file_upload.path = (
+                Path(self.debusine_upload_directory) / "the-file"
+            ).as_posix()
+            file_upload.save()
 
         stdout, stderr, exitcode = call_command("monthly_cleanup")
 
@@ -279,33 +302,39 @@ class MonthlyCleanupCommandTests(TestHelpersMixin, TestCase):
         self.assertEqual(exitcode, 0)
 
         # Check that the FileUpload row was not deleted
-        self.assertQuerysetEqual(
+        self.assertQuerySetEqual(
             FileUpload.objects.filter(id=file_upload.id), []
         )
 
         # The file also got deleted
         self.assertFalse(Path(file_upload.path).exists())
 
-    def test_delete_unfinished_upload_not_deleted_recent_activity(self):
+    def test_delete_unfinished_upload_not_deleted_recent_activity(self) -> None:
         """Unfinished FileUpload is not deleted: recent activity."""
-        artifact, _ = self.create_artifact(paths=["README"], create_files=True)
-        file_in_artifact = FileInArtifact.objects.get(artifact=artifact)
+        with context.disable_permission_checks():
+            artifact, _ = self.create_artifact(
+                paths=["README"], create_files=True
+            )
+            file_in_artifact = FileInArtifact.objects.get(artifact=artifact)
 
-        upload_temporary_file = self.create_temporary_file(
-            directory=self.debusine_upload_directory
-        )
-        file_upload = FileUpload.objects.create(
-            file_in_artifact=file_in_artifact, path=upload_temporary_file.name
-        )
+            upload_temporary_file = self.create_temporary_file(
+                directory=self.debusine_upload_directory
+            )
+            file_upload = FileUpload.objects.create(
+                file_in_artifact=file_in_artifact,
+                path=upload_temporary_file.name,
+            )
 
         call_command("monthly_cleanup")
 
         # FileUpload did not get deleted: recent activity
-        self.assertQuerysetEqual(
+        self.assertQuerySetEqual(
             FileUpload.objects.filter(id=file_upload.id), [file_upload]
         )
 
-    def test_delete_orphan_file_in_upload_directory_no_delete_dry_run(self):
+    def test_delete_orphan_file_in_upload_directory_no_delete_dry_run(
+        self,
+    ) -> None:
         """File is orphan but is not deleted: --dry-run."""
         orphan_file = self.create_temporary_file(
             directory=self.debusine_upload_directory
@@ -330,7 +359,7 @@ class MonthlyCleanupCommandTests(TestHelpersMixin, TestCase):
         self.assertEqual(stderr, "")
         self.assertEqual(exitcode, 0)
 
-    def test_delete_orphan_file_too_new(self):
+    def test_delete_orphan_file_too_new(self) -> None:
         """File is orphan but not deleted: too new."""
         orphan_file = self.create_temporary_file(
             directory=self.debusine_upload_directory
@@ -343,7 +372,7 @@ class MonthlyCleanupCommandTests(TestHelpersMixin, TestCase):
         self.assertEqual(stderr, "")
         self.assertEqual(exitcode, 0)
 
-    def test_delete_orphan_file(self):
+    def test_delete_orphan_file(self) -> None:
         """File is orphan and deleted."""
         orphan_file = self.create_temporary_file(
             directory=self.debusine_upload_directory
@@ -353,15 +382,16 @@ class MonthlyCleanupCommandTests(TestHelpersMixin, TestCase):
 
         self.assertFalse(orphan_file.exists())
 
-    def test_file_in_store_db_missing_from_disk(self):
+    def test_file_in_store_db_missing_from_disk(self) -> None:
         """File is referenced by the DB but not in disk in the store."""
-        path_in_artifact = "README"
-        artifact, _ = self.create_artifact(
-            paths=[path_in_artifact], create_files=True
-        )
+        with context.disable_permission_checks():
+            path_in_artifact = "README"
+            artifact, _ = self.create_artifact(
+                paths=[path_in_artifact], create_files=True
+            )
 
-        file_path = self.get_local_file_path(artifact, path_in_artifact)
-        file_path.unlink()
+            file_path = self.get_local_file_path(artifact, path_in_artifact)
+            file_path.unlink()
 
         stdout, stderr, exitcode = call_command("monthly_cleanup")
 
@@ -369,15 +399,18 @@ class MonthlyCleanupCommandTests(TestHelpersMixin, TestCase):
         self.assertEqual(stderr, f"Dangling file: {file_path}\n")
         self.assertEqual(exitcode, 1)
 
-    def test_file_in_upload_db_missing_from_disk(self):
+    def test_file_in_upload_db_missing_from_disk(self) -> None:
         """File is referenced by the DB but not in disk for uploads."""
-        artifact, _ = self.create_artifact(paths=["README"], create_files=True)
-        file_in_artifact = FileInArtifact.objects.get(artifact=artifact)
+        with context.disable_permission_checks():
+            artifact, _ = self.create_artifact(
+                paths=["README"], create_files=True
+            )
+            file_in_artifact = FileInArtifact.objects.get(artifact=artifact)
 
-        file_path = "missing-file"
-        FileUpload.objects.create(
-            file_in_artifact=file_in_artifact, path=file_path
-        )
+            file_path = "missing-file"
+            FileUpload.objects.create(
+                file_in_artifact=file_in_artifact, path=file_path
+            )
 
         stdout, stderr, exitcode = call_command("monthly_cleanup")
 
@@ -390,7 +423,7 @@ class MonthlyCleanupCommandTests(TestHelpersMixin, TestCase):
         )
         self.assertEqual(exitcode, 1)
 
-    def _create_files_and_directories(self):
+    def _create_files_and_directories(self) -> _FilesAndDirectories:
         root = self.create_temporary_directory()
 
         files = {
@@ -403,7 +436,7 @@ class MonthlyCleanupCommandTests(TestHelpersMixin, TestCase):
 
         return {"root": root, "files": files, "directories": {directory}}
 
-    def test_list_directory_files_and_directories_raise_error(self):
+    def test_list_directory_files_and_directories_raise_error(self) -> None:
         """_list_directory raise ValueError for incompatible parameters."""
         with self.assertRaisesRegex(
             ValueError,
@@ -411,7 +444,7 @@ class MonthlyCleanupCommandTests(TestHelpersMixin, TestCase):
         ):
             Command._list_directory("/", files_only=True, directories_only=True)
 
-    def test_files_and_directories(self):
+    def test_files_and_directories(self) -> None:
         """_list_directory without filters: return all files and directories."""
         files_and_directories = self._create_files_and_directories()
 
@@ -421,7 +454,7 @@ class MonthlyCleanupCommandTests(TestHelpersMixin, TestCase):
             | files_and_directories["directories"],
         )
 
-    def test_files_only(self):
+    def test_files_only(self) -> None:
         """_list_directory(files_only) return only files (no directories)."""
         files_and_directories = self._create_files_and_directories()
 
@@ -432,7 +465,7 @@ class MonthlyCleanupCommandTests(TestHelpersMixin, TestCase):
             files_and_directories["files"],
         )
 
-    def test_directories_only(self):
+    def test_directories_only(self) -> None:
         """_list_directory(directories_only) return only directories."""
         files_and_directories = self._create_files_and_directories()
 
@@ -443,7 +476,7 @@ class MonthlyCleanupCommandTests(TestHelpersMixin, TestCase):
             files_and_directories["directories"],
         )
 
-    def test_filter_nothing(self):
+    def test_filter_nothing(self) -> None:
         """_list_directory with exclude all filter: return nothing."""
         files_and_directories = self._create_files_and_directories()
 
@@ -455,7 +488,7 @@ class MonthlyCleanupCommandTests(TestHelpersMixin, TestCase):
             set(),
         )
 
-    def test_filter_all(self):
+    def test_filter_all(self) -> None:
         """_list_directory (only files, include all filter): return files."""
         files_and_directories = self._create_files_and_directories()
 
@@ -469,6 +502,6 @@ class MonthlyCleanupCommandTests(TestHelpersMixin, TestCase):
         )
 
     @staticmethod
-    def _set_modified_time_two_days_ago(path: Path):
+    def _set_modified_time_two_days_ago(path: Path) -> None:
         two_days_ago = (datetime.now() - timedelta(days=2)).timestamp()
         os.utime(path, (two_days_ago, two_days_ago))
