@@ -10,9 +10,10 @@
 """Mapping database models to REST Api support."""
 
 from collections.abc import Callable
-from typing import Any, Never
+from typing import Any, Never, TYPE_CHECKING, TypeAlias
 
-from django.urls import reverse
+from django.db.models import QuerySet
+from django.http import HttpRequest
 
 try:
     import pydantic.v1 as pydantic
@@ -20,35 +21,102 @@ except ImportError:
     import pydantic as pydantic  # type: ignore
 
 from rest_framework import serializers
-from rest_framework.exceptions import ValidationError
-from rest_framework.request import Request
+from rest_framework.exceptions import ErrorDetail, ValidationError
 
 from debusine.artifacts.models import ArtifactCategory
+from debusine.client.models import (
+    LookupChildType,
+    model_to_json_serializable_dict,
+)
 from debusine.db.models import (
     Artifact,
     ArtifactRelation,
     Collection,
     CollectionItem,
+    DEFAULT_WORKSPACE_NAME,
     File,
     TaskDatabase,
     WorkRequest,
     WorkflowTemplate,
     Workspace,
-    default_workspace,
 )
 from debusine.server.collections.lookup import LookupResult
+from debusine.server.exceptions import (
+    DebusineAPIException,
+    raise_workspace_not_found,
+)
 from debusine.server.workflows import Workflow
 from debusine.server.workflows.models import WorkRequestManualUnblockAction
 from debusine.tasks.models import EventReactions, WorkerType
 
+if TYPE_CHECKING:
+    SerializerMixinBase: TypeAlias = serializers.Serializer[Any]
+    SlugRelatedFieldBase = serializers.SlugRelatedField
+else:
+    SerializerMixinBase = object
 
-class WorkRequestSerializer(serializers.ModelSerializer[WorkRequest]):
+    # SlugRelatedField doesn't support generic types at run-time yet.
+    class _SlugRelatedFieldBase:
+        def __class_getitem__(*args):
+            return serializers.SlugRelatedField
+
+    SlugRelatedFieldBase = _SlugRelatedFieldBase
+
+
+class DebusineSerializerMixin(SerializerMixinBase):
+    """Adjust a serializer with debusine-specific behaviour."""
+
+    validation_error_title: str
+
+    def is_valid(self, *, raise_exception: bool = False) -> bool:
+        """Raise a suitable API exception when deserialization fails."""
+        valid = super().is_valid()
+        if raise_exception and not valid:
+            if (workspace_errors := self.errors.get("workspace", [])) and all(
+                isinstance(error, ErrorDetail)
+                and error.code == "does_not_exist"
+                for error in workspace_errors
+            ):
+                raise_workspace_not_found(self.data["workspace"])
+            else:
+                raise DebusineAPIException(
+                    title=self.validation_error_title,
+                    validation_errors=self.errors,
+                )
+        return valid
+
+
+class WorkspaceInCurrentScopeField(SlugRelatedFieldBase[Workspace]):
+    """A field for workspaces in the current scope."""
+
+    def __init__(self, **kwargs: Any) -> None:
+        """Construct the field."""
+        kwargs.setdefault("default", self._get_default_workspace)
+        return super().__init__(**kwargs)
+
+    def get_queryset(self) -> QuerySet[Workspace]:
+        """Limit to the current scope."""
+        return Workspace.objects.all().in_current_scope()
+
+    def _get_default_workspace(self) -> Workspace:
+        """Get the default workspace."""
+        try:
+            return self.get_queryset().get(name=DEFAULT_WORKSPACE_NAME)
+        except Workspace.DoesNotExist:
+            # The default workspace name might not exist in the current
+            # scope.  If so, the client must provide a workspace.
+            self.fail("required")
+
+
+class WorkRequestSerializer(
+    DebusineSerializerMixin, serializers.ModelSerializer[WorkRequest]
+):
     """Serializer of a WorkRequest."""
 
+    validation_error_title = "Cannot deserialize work request"
+
     artifacts = serializers.SerializerMethodField("artifacts_for_work_request")
-    workspace = serializers.SlugRelatedField(
-        queryset=Workspace.objects.all(), slug_field="name"
-    )
+    workspace = WorkspaceInCurrentScopeField(slug_field="name")
     workflow_data = serializers.JSONField(
         source="workflow_data_json", required=False
     )
@@ -56,7 +124,7 @@ class WorkRequestSerializer(serializers.ModelSerializer[WorkRequest]):
         source='event_reactions_json', required=False
     )
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         """
         Initialize object.
 
@@ -76,7 +144,7 @@ class WorkRequestSerializer(serializers.ModelSerializer[WorkRequest]):
             .values_list("id", flat=True)
         )
 
-    def validate(self, data):
+    def validate(self, data: Any) -> Any:
         """
         Add extra validation in the WorkRequestSerializer. Used by .is_valid().
 
@@ -88,9 +156,8 @@ class WorkRequestSerializer(serializers.ModelSerializer[WorkRequest]):
             wanted_fields = set(self._only_fields)
 
             if unwanted_fields := (fields_in_data - wanted_fields):
-                unwanted_fields = ', '.join(sorted(unwanted_fields))
                 raise serializers.ValidationError(
-                    f'Invalid fields: {unwanted_fields}'
+                    f"Invalid fields: {', '.join(sorted(unwanted_fields))}"
                 )
 
         return data
@@ -131,25 +198,31 @@ class WorkRequestSerializer(serializers.ModelSerializer[WorkRequest]):
 class WorkRequestUpdateSerializer(WorkRequestSerializer):
     """Serializer for updating work requests."""
 
+    validation_error_title = "Cannot deserialize work request update"
+
     class Meta:
         model = WorkRequest
         fields = ['priority_adjustment']
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         """Ensure that only expected fields are passed."""
         kwargs["only_fields"] = self.Meta.fields
         super().__init__(*args, **kwargs)
 
 
-class WorkRequestUnblockSerializer(serializers.Serializer[Never]):
+class WorkRequestUnblockSerializer(
+    DebusineSerializerMixin, serializers.Serializer[Never]
+):
     """Serializer for the data used to edit a manual unblock."""
 
+    validation_error_title = "Cannot deserialize work request unblock"
+
     notes = serializers.CharField(max_length=65536, required=False)
     action = serializers.ChoiceField(
         choices=WorkRequestManualUnblockAction, required=False
     )
 
-    def validate(self, data):
+    def validate(self, data: Any) -> Any:
         """Ensure at least one of notes and action is present."""
         if not data.get("notes") and data.get("action") is None:
             raise serializers.ValidationError(
@@ -178,7 +251,11 @@ class WorkRequestExternalDebsignSerializer(WorkRequestSerializer):
                 TaskDatabase(instance)
             )
             data["dynamic_task_data"] = (
-                None if dynamic_task_data is None else dynamic_task_data.dict()
+                None
+                if dynamic_task_data is None
+                else model_to_json_serializable_dict(
+                    dynamic_task_data, exclude_unset=True
+                )
             )
         except Exception as exc:
             raise ValidationError(detail=f"Cannot process task data: {exc}")
@@ -187,10 +264,12 @@ class WorkRequestExternalDebsignSerializer(WorkRequestSerializer):
 
 
 class WorkRequestExternalDebsignRequestSerializer(
-    serializers.Serializer[Never]
+    DebusineSerializerMixin, serializers.Serializer[Never]
 ):
     """Serializer for the data used to complete an `ExternalDebsign` task."""
 
+    validation_error_title = "Cannot deserialize work request"
+
     signed_artifact = serializers.PrimaryKeyRelatedField(
         queryset=Artifact.objects
     )
@@ -204,6 +283,13 @@ class WorkRequestExternalDebsignRequestSerializer(
                     f"{ArtifactCategory.UPLOAD}; got {value.category}"
                 )
             )
+        if value.created_by_work_request is not None:
+            raise ValidationError(
+                detail=(
+                    "Signed artifact must not have been created by a work "
+                    "request"
+                )
+            )
         return value
 
 
@@ -259,11 +345,15 @@ class FileSerializerResponse(FileSerializer):
     url = serializers.URLField()
 
 
-class ArtifactSerializer(serializers.Serializer[Artifact]):
+class ArtifactSerializer(
+    DebusineSerializerMixin, serializers.Serializer[Artifact]
+):
     """Serializer to deserialize the client request to create an artifact."""
 
+    validation_error_title = "Cannot deserialize artifact"
+
     category = serializers.CharField(max_length=255)
-    workspace = serializers.CharField(max_length=255)
+    workspace = WorkspaceInCurrentScopeField(slug_field="name")
     files = serializers.DictField(child=FileSerializer())
     # Deliberately incompatible with Serializer.data.
     data = serializers.DictField()  # type: ignore[assignment]
@@ -355,37 +445,36 @@ class ArtifactSerializerResponse(serializers.ModelSerializer[Artifact]):
 
         return files
 
-    def _download_tar_gz_url(self, obj) -> str:  # noqa: U100
+    def _download_tar_gz_url(self, obj: Artifact) -> str:  # noqa: U100
         """Based on self._base_download_url return URL to download .tar.gz."""
         return self._base_download_url + "?archive=tar.gz"
 
     @staticmethod
-    def _build_absolute_download_url(artifact_id: int, request: Request) -> str:
+    def _build_absolute_download_url(
+        artifact: Artifact, request: HttpRequest
+    ) -> str:
         """Return URL to download the artifact (without the archive format)."""
-        return request.build_absolute_uri(
-            reverse(
-                "artifacts:download",
-                kwargs={"artifact_id": artifact_id},
-            )
-        )
+        return request.build_absolute_uri(artifact.get_absolute_url_download())
 
     @classmethod
     def from_artifact(
-        cls, artifact: Artifact, request: Request
+        cls, artifact: Artifact, request: HttpRequest
     ) -> "ArtifactSerializerResponse":
         """Given an artifact and request return ArtifactSerializerResponse."""
         serialized = ArtifactSerializerResponse(artifact)
         serialized._base_download_url = cls._build_absolute_download_url(
-            artifact.id, request
+            artifact, request
         )
         return serialized
 
 
 class ArtifactRelationResponseSerializer(
-    serializers.ModelSerializer[ArtifactRelation]
+    DebusineSerializerMixin, serializers.ModelSerializer[ArtifactRelation]
 ):
     """Serializer for relations between artifacts."""
 
+    validation_error_title = "Cannot deserialize artifact relation"
+
     media_type = "application/json"
 
     class Meta:
@@ -399,6 +488,29 @@ class ArtifactRelationResponseSerializer(
         validators: list[Callable[[ArtifactRelation], None]] = []
 
 
+class LookupExpectTypeChoiceField(serializers.ChoiceField):
+    """
+    A field for `Lookup*.expect_type`.
+
+    This handles values of `LookupChildType`, but also accepts values of
+    `CollectionItem.Types` for compatibility with old clients.
+    """
+
+    def __init__(self, **kwargs: Any) -> None:
+        """Initialize the field."""
+        return super().__init__(choices=LookupChildType)
+
+    def to_internal_value(self, data: Any) -> str:
+        """Accept values of `CollectionItemTypes` for compatibility."""
+        if data in CollectionItem.Types:
+            data = {
+                CollectionItem.Types.BARE: LookupChildType.BARE,
+                CollectionItem.Types.ARTIFACT: LookupChildType.ARTIFACT,
+                CollectionItem.Types.COLLECTION: LookupChildType.COLLECTION,
+            }[data]
+        return super().to_internal_value(data)
+
+
 class LookupSingleSerializer(serializers.Serializer[Never]):
     """Serializer for the data used to look up a single collection item."""
 
@@ -409,7 +521,7 @@ class LookupSingleSerializer(serializers.Serializer[Never]):
     default_category = serializers.CharField(
         max_length=255, required=False, allow_null=True
     )
-    expect_type = serializers.ChoiceField(choices=CollectionItem.Types)
+    expect_type = LookupExpectTypeChoiceField()
 
 
 class LookupMultipleSerializer(serializers.Serializer[Never]):
@@ -422,7 +534,7 @@ class LookupMultipleSerializer(serializers.Serializer[Never]):
     default_category = serializers.CharField(
         max_length=255, required=False, allow_null=True
     )
-    expect_type = serializers.ChoiceField(choices=CollectionItem.Types)
+    expect_type = LookupExpectTypeChoiceField()
 
 
 class LookupResponseSerializer(serializers.Serializer[LookupResult]):
@@ -442,16 +554,16 @@ class LookupResponseSerializer(serializers.Serializer[LookupResult]):
     )
 
 
-class WorkflowTemplateSerializer(serializers.ModelSerializer[WorkflowTemplate]):
+class WorkflowTemplateSerializer(
+    DebusineSerializerMixin, serializers.ModelSerializer[WorkflowTemplate]
+):
     """Serializer for workflow templates."""
 
+    validation_error_title = "Cannot deserialize workflow template"
+
     media_type = "application/json"
 
-    workspace = serializers.SlugRelatedField(
-        queryset=Workspace.objects.all(),
-        slug_field="name",
-        default=default_workspace,
-    )
+    workspace = WorkspaceInCurrentScopeField(slug_field="name")
 
     class Meta:
         model = WorkflowTemplate
@@ -464,7 +576,7 @@ class WorkflowTemplateSerializer(serializers.ModelSerializer[WorkflowTemplate]):
             "priority",
         ]
 
-    def validate_task_name(self, value):
+    def validate_task_name(self, value: str) -> str:
         """
         Ensure that task_name is valid.
 
@@ -479,17 +591,17 @@ class WorkflowTemplateSerializer(serializers.ModelSerializer[WorkflowTemplate]):
         return value
 
 
-class CreateWorkflowRequestSerializer(serializers.ModelSerializer[WorkRequest]):
+class CreateWorkflowRequestSerializer(
+    DebusineSerializerMixin, serializers.ModelSerializer[WorkRequest]
+):
     """Serializer for requests to create workflows."""
 
+    validation_error_title = "Cannot deserialize workflow creation request"
+
     media_type = "application/json"
 
     template_name = serializers.CharField(max_length=255)
-    workspace = serializers.SlugRelatedField(
-        queryset=Workspace.objects.all(),
-        slug_field="name",
-        default=default_workspace,
-    )
+    workspace = WorkspaceInCurrentScopeField(slug_field="name")
 
     class Meta:
         model = WorkRequest
