@@ -20,6 +20,7 @@ from typing import Protocol, cast, runtime_checkable
 import django.http
 from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured, MiddlewareNotUsed
+from django.utils.module_loading import import_string
 
 from debusine.server.signon.signon import Signon
 
@@ -34,19 +35,37 @@ class RequestSignonProtocol(Protocol):
 class SignonMiddleware:
     """Authenticate via external signon providers."""
 
+    signon_class: type[Signon]
+
     def __init__(
         self,
         get_response: Callable[
             [django.http.HttpRequest], django.http.HttpResponse
         ],
-    ):
+    ) -> None:
         """Middleware API entry point."""
         self.providers = getattr(settings, "SIGNON_PROVIDERS", ())
         if not self.providers:
             raise MiddlewareNotUsed()
+
+        # Find the Signon class to use. This allows customizing behaviour by
+        # subclassing Signon
+        if (
+            signon_class_path := getattr(settings, "SIGNON_CLASS", None)
+        ) is None:
+            self.signon_class = Signon
+        else:
+            self.signon_class = import_string(signon_class_path)
+            if not issubclass(self.signon_class, Signon):
+                raise ImproperlyConfigured(
+                    f"{signon_class_path} is not a subclass of Signon"
+                )
+
         self.get_response = get_response
 
-    def __call__(self, request: django.http.HttpRequest):
+    def __call__(
+        self, request: django.http.HttpRequest
+    ) -> django.http.HttpResponse:
         """Middleware API entry point."""
         # AuthenticationMiddleware is required so that request.user exists.
         if not hasattr(request, 'user'):
@@ -58,6 +77,6 @@ class SignonMiddleware:
             )
 
         # Add request.signon
-        cast(RequestSignonProtocol, request).signon = Signon(request)
+        cast(RequestSignonProtocol, request).signon = self.signon_class(request)
 
         return self.get_response(request)
