@@ -11,7 +11,7 @@
 """Logic to authenticate a request using signon Providers."""
 import functools
 import logging
-from collections.abc import Iterable
+from collections.abc import Iterable, Sequence
 
 import django.http
 from django.conf import settings
@@ -38,10 +38,12 @@ class Signon:
     every request. Everything else is loaded only when needed.
     """
 
-    def __init__(self, request: django.http.HttpRequest):
+    def __init__(self, request: django.http.HttpRequest) -> None:
         """Create a Signon object for a request."""
         self.request = request
-        self.providers = getattr(settings, "SIGNON_PROVIDERS", ())
+        self.providers: Sequence[providers.Provider] = getattr(
+            settings, "SIGNON_PROVIDERS", ()
+        )
 
     def status(
         self,
@@ -61,7 +63,7 @@ class Signon:
         """Lazily populate self.identities."""
         return self._compute_identities()
 
-    def logout_identities(self):
+    def logout_identities(self) -> None:
         """Deactivate all active external identities."""
         for provider in self.providers:
             bound = provider.bind(self.request)
@@ -93,7 +95,7 @@ class Signon:
 
         return identities
 
-    def activate_identity(self, identity: Identity, *options: str):
+    def activate_identity(self, identity: Identity, *options: str) -> None:
         """Activate the given identity and update authentication accordingly."""
         if self.request.user.is_authenticated:
             if "bind" not in options:
@@ -121,7 +123,7 @@ class Signon:
 
         self.request.session[f"signon_identity_{identity.issuer}"] = identity.pk
 
-    def _remove_invalid_signon_user(self):
+    def _remove_invalid_signon_user(self) -> None:
         """
         Log out an externally authenticated user.
 
@@ -144,7 +146,7 @@ class Signon:
     def _map_identity_to_user(self, identity: Identity) -> User | None:
         """Create a user model from the given identity, and bind it to it."""
         # Validate identity claims
-        if not (provider := self._provider_for_identity(identity)):
+        if not (provider := self.get_provider_for_identity(identity)):
             return None
         if claim_errors := provider.validate_claims(identity):
             for error in claim_errors:
@@ -157,7 +159,7 @@ class Signon:
             log.info("%s: user matched to identity %s", user, identity)
         else:
             # Else try to create a new user
-            user = self._create_user_from_identity(identity)
+            user = self.create_user_from_identity(identity)
             if user is not None:
                 log.info("%s: auto created from identity %s", user, identity)
             else:
@@ -169,7 +171,7 @@ class Signon:
         log.info("%s: bound to identity %s", user, identity)
         return user
 
-    def _provider_for_identity(
+    def get_provider_for_identity(
         self, identity: Identity
     ) -> providers.Provider | None:
         """Find the Provider for an identity."""
@@ -190,8 +192,13 @@ class Signon:
         except User.DoesNotExist:
             return None
 
-    def _create_user_from_identity(self, identity: Identity) -> User | None:
-        """Lookup or create a user from the data in an Identity."""
+    def create_user_from_identity(self, identity: Identity) -> User | None:
+        """
+        Lookup or create a user from the data in an Identity.
+
+        This is a default implementation. It can be customized by subclassing
+        Signon and using settings.SIGNON_CLASS
+        """
         User = auth.get_user_model()
         first_name, last_name = split_full_name(identity.claims["name"])
 
