@@ -21,9 +21,10 @@ from django.contrib.auth.models import AnonymousUser
 from django.core.exceptions import PermissionDenied
 from django.test import RequestFactory, TestCase, override_settings
 
-from debusine.db.models import Identity
+from debusine.db.models import Identity, User
 from debusine.server.signon import providers
 from debusine.server.signon.auth import SignonAuthBackend
+from debusine.server.signon.middleware import RequestSignonProtocol
 from debusine.server.signon.signon import Signon
 
 
@@ -33,25 +34,26 @@ class MockSession(django.contrib.sessions.backends.base.SessionBase):
     # TODO: is there a more standard way to provide a mock session?
     # Would SessionStore be enough?
 
-    # def exists(self):
+    # def exists(self, session_key: str) -> bool:
     #     """Mock SessionBase API endpoint."""
     #     return True
 
-    def create(self):
+    def create(self) -> None:
         """Mock SessionBase API endpoint."""
 
-    # def save(self):
+    # def save(self, must_create: bool = False) -> None:
     #     """Mock SessionBase API endpoint."""
     #     pass
 
-    def delete(self, *args):
+    def delete(self, *args: Any) -> None:
         """Mock SessionBase API endpoint."""
 
-    # def load(self):
+    # def load(self) -> dict[str, Any]:
     #     """Mock SessionBase API endpoint."""
     #     pass
 
-    # def clear_expired(self):
+    # @classmethod
+    # def clear_expired(cls) -> None:
     #     """Mock SessionBase API endpoint."""
     #     pass
 
@@ -73,21 +75,21 @@ class MockSession(django.contrib.sessions.backends.base.SessionBase):
 class TestAuthentication(TestCase):
     """Test Signon."""
 
-    def setUp(self):
+    def setUp(self) -> None:
         """Provide a mock unauthenticated request for tests."""
         super().setUp()
         self.factory = RequestFactory()
         self.request = self.factory.get("/")
         self.request.session = MockSession()
         self.request.user = AnonymousUser()
-        self.request.signon = Signon(self.request)
+        setattr(self.request, "signon", Signon(self.request))
 
     def _make_identity(
         self,
-        user=None,
+        user: User | None = None,
         issuer: str = "salsa",
         claims: dict[str, Any] | None = None,
-    ):
+    ) -> Identity:
         """Create a test identity."""
         subject = f"{issuer}@debian.org"
         if claims is None:
@@ -99,8 +101,9 @@ class TestAuthentication(TestCase):
             claims=claims,
         )
 
-    def test_no_active_identities(self):
+    def test_no_active_identities(self) -> None:
         """No active external identities leave request unauthenticated."""
+        assert isinstance(self.request, RequestSignonProtocol)
         self.assertEqual(self.request.signon.identities, {})
         self.assertFalse(self.request.user.is_authenticated)
 
@@ -111,8 +114,9 @@ class TestAuthentication(TestCase):
         self.assertEqual(status[1][0].name, "salsa")
         self.assertIsNone(status[1][1])
 
-    def test_one_active_unbound_identity_autocreates(self):
+    def test_one_active_unbound_identity_autocreates(self) -> None:
         """One active but unbound identity tries to autocreate a user."""
+        assert isinstance(self.request, RequestSignonProtocol)
         identity = self._make_identity(
             claims={
                 "name": "Test User",
@@ -135,8 +139,7 @@ class TestAuthentication(TestCase):
         self.assertIsNotNone(identity.user)
         user = identity.user
 
-        # FIXME: use assertQuerySetEqual from Django 4.2
-        self.assertQuerysetEqual(user.identities.all(), [identity])
+        self.assertQuerySetEqual(user.identities.all(), [identity])
         self.assertEqual(user.username, "test@example.org")
         self.assertEqual(user.email, "test@example.org")
         self.assertEqual(user.first_name, "Test")
@@ -159,8 +162,9 @@ class TestAuthentication(TestCase):
         self.assertEqual(status[1][0].name, "salsa")
         self.assertEqual(status[1][1], identity)
 
-    def test_one_active_bound_identity_logs_in(self):
+    def test_one_active_bound_identity_logs_in(self) -> None:
         """One active bound identity is enough to authenticate."""
+        assert isinstance(self.request, RequestSignonProtocol)
         user = get_user_model().objects.create_user("test")
         identity = self._make_identity(user=user)
         with self.assertNoLogs("debusine.server.signon"):
@@ -180,10 +184,11 @@ class TestAuthentication(TestCase):
             SignonAuthBackend,
         )
 
-    def test_add_aligned_bound_identity(self):
+    def test_add_aligned_bound_identity(self) -> None:
         """Adding a bound matching identity raises PermissionDenied."""
         # Multiple active bound identities pointing to the same user
         # authenticate successfully
+        assert isinstance(self.request, RequestSignonProtocol)
         user = get_user_model().objects.create_user(
             "test", email="test@example.org"
         )
@@ -207,8 +212,9 @@ class TestAuthentication(TestCase):
         )
         self.assertEqual(self.request.user, user)
 
-    def test_add_unbound_identity(self):
+    def test_add_unbound_identity(self) -> None:
         """Adding an unbound identity raises PermissionDenied."""
+        assert isinstance(self.request, RequestSignonProtocol)
         user = get_user_model().objects.create_user("test")
         ident1 = self._make_identity(user=user, issuer="debsso")
         ident2 = self._make_identity(issuer="salsa")
@@ -230,8 +236,9 @@ class TestAuthentication(TestCase):
         ident2.refresh_from_db()
         self.assertIsNone(ident2.user)
 
-    def test_add_conflicting_bound_identity(self):
+    def test_add_conflicting_bound_identity(self) -> None:
         """Adding a bound conflicting identity raises PermissionDenied."""
+        assert isinstance(self.request, RequestSignonProtocol)
         user1 = get_user_model().objects.create_user(
             "test1", email="test1@example.org"
         )
@@ -258,8 +265,9 @@ class TestAuthentication(TestCase):
         ident2.refresh_from_db()
         self.assertEqual(ident2.user, user2)
 
-    def test_one_active_missing_identity(self):
+    def test_one_active_missing_identity(self) -> None:
         """One identity present in session but not the DB is removed."""
+        assert isinstance(self.request, RequestSignonProtocol)
         self.request.session["signon_identity_salsa"] = 1
         self.request.signon._compute_identities()
 
@@ -275,8 +283,9 @@ class TestAuthentication(TestCase):
         self.assertEqual(status[1][0].name, "salsa")
         self.assertIsNone(status[1][1])
 
-    def test_bind_intention_succeeds(self):
+    def test_bind_intention_succeeds(self) -> None:
         """Bind intention binds to current user."""
+        assert isinstance(self.request, RequestSignonProtocol)
         user = get_user_model().objects.create_user("test")
         self.request.user = user
 
@@ -298,8 +307,9 @@ class TestAuthentication(TestCase):
             ],
         )
 
-    def test_bind_intention_succeeds_other_backend(self):
+    def test_bind_intention_succeeds_other_backend(self) -> None:
         """Bind intention binds to current user from a different backend."""
+        assert isinstance(self.request, RequestSignonProtocol)
         user = get_user_model().objects.create_user("test")
 
         # Simulate a valid existing login from a different auth backend
@@ -329,8 +339,9 @@ class TestAuthentication(TestCase):
         ident.refresh_from_db()
         self.assertEqual(ident.user, user)
 
-    def test_add_unbound_identities_binds(self):
+    def test_add_unbound_identities_binds(self) -> None:
         """One active bound identity and one unbound, when binding, binds."""
+        assert isinstance(self.request, RequestSignonProtocol)
         user = get_user_model().objects.create_user("test")
         ident1 = self._make_identity(user=user, issuer="debsso")
         with self.assertNoLogs("debusine.server.signon"):
@@ -362,8 +373,9 @@ class TestAuthentication(TestCase):
             ],
         )
 
-    def test_conflicting_bound_identities_bind(self):
+    def test_conflicting_bound_identities_bind(self) -> None:
         """Binding a second previously bound identity, rebinds."""
+        assert isinstance(self.request, RequestSignonProtocol)
         user1 = get_user_model().objects.create_user(
             "test1", email="test1@example.org"
         )
@@ -400,9 +412,10 @@ class TestAuthentication(TestCase):
             ],
         )
 
-    def test_logout_identities_logs_our_user_out(self):
+    def test_logout_identities_logs_our_user_out(self) -> None:
         """logout_identities() deactivates identities."""
         # Log in using an external identity
+        assert isinstance(self.request, RequestSignonProtocol)
         user = get_user_model().objects.create_user("test")
         ident1 = self._make_identity(user=user)
         with self.assertNoLogs("debusine.server.signon"):
@@ -417,9 +430,10 @@ class TestAuthentication(TestCase):
         # out
         self.assertFalse(self.request.user.is_authenticated)
 
-    def test_logout_identities_keep_user_from_other_backends(self):
+    def test_logout_identities_keep_user_from_other_backends(self) -> None:
         """logout_identities() deactivates identities."""
         # Log in using ModelBackend
+        assert isinstance(self.request, RequestSignonProtocol)
         user = get_user_model().objects.create_user("test")
         self.request.user = user
         self.request.session[auth.BACKEND_SESSION_KEY] = (
@@ -441,8 +455,9 @@ class TestAuthentication(TestCase):
         self.assertTrue(self.request.user.is_authenticated)
         self.assertEqual(self.request.user, user)
 
-    def test_map_user_email(self):
+    def test_map_user_email(self) -> None:
         """Map identity to user based on email."""
+        assert isinstance(self.request, RequestSignonProtocol)
         user = get_user_model().objects.create_user(
             "test", email="test@example.org"
         )
@@ -471,8 +486,9 @@ class TestAuthentication(TestCase):
             ],
         )
 
-    def test_map_user_unverified_email(self):
+    def test_map_user_unverified_email(self) -> None:
         """Do not map an identity with unverified email."""
+        assert isinstance(self.request, RequestSignonProtocol)
         get_user_model().objects.create_user("test", email="test@example.org")
         ident = self._make_identity(
             claims={
@@ -497,8 +513,9 @@ class TestAuthentication(TestCase):
             ],
         )
 
-    def test_map_user_invalid_email(self):
+    def test_map_user_invalid_email(self) -> None:
         """Test mapping an identity with an invalid email."""
+        assert isinstance(self.request, RequestSignonProtocol)
         ident = self._make_identity(
             claims={
                 "name": "Test User",
@@ -523,8 +540,9 @@ class TestAuthentication(TestCase):
         )
         self.assertEqual(len(log.output), 1)
 
-    def test_auto_create_user_fails(self):
+    def test_auto_create_user_fails(self) -> None:
         """Do not autocreate users with unverified emails."""
+        assert isinstance(self.request, RequestSignonProtocol)
         ident = self._make_identity(
             claims={
                 "name": "Test User",
@@ -548,8 +566,9 @@ class TestAuthentication(TestCase):
             ],
         )
 
-    def test_map_identity_invalid_provider(self):
+    def test_map_identity_invalid_provider(self) -> None:
         """An invalid provider in identity fails autocreation."""
+        assert isinstance(self.request, RequestSignonProtocol)
         ident = self._make_identity(
             issuer="invalid",
             claims={
