@@ -14,7 +14,7 @@ import contextlib
 import datetime
 import html
 import re
-from collections.abc import Sequence
+from collections.abc import Generator, Sequence
 from typing import Any, cast
 from unittest import mock
 from urllib.parse import parse_qs, urlparse
@@ -26,11 +26,13 @@ from django.test import RequestFactory, TestCase, override_settings
 from django.urls import resolve, reverse
 from django.utils import timezone
 
-from debusine.db.models import Identity
+from debusine.db.models import Identity, User
 from debusine.server.signon import providers
 from debusine.server.signon.middleware import RequestSignonProtocol
+from debusine.server.signon.providers import BoundOIDCProvider
 from debusine.server.signon.signon import Signon
 from debusine.server.signon.tests.test_signon import MockSession
+from debusine.test.django import TestResponseType
 
 
 @override_settings(
@@ -47,7 +49,9 @@ from debusine.server.signon.tests.test_signon import MockSession
 class SalsaSignonViews(TestCase):
     """Test Signon views."""
 
-    def _collect_urls(self, response, match_re: str) -> list[str]:
+    def _collect_urls(
+        self, response: TestResponseType, match_re: str
+    ) -> list[str]:
         """Collect external URLs in the response contents."""
         urls: list[str] = []
         re_url = re.compile(b'<a href="([^"]+)"')
@@ -64,11 +68,11 @@ class SalsaSignonViews(TestCase):
 
     def _make_identity(
         self,
-        user=None,
+        user: User | None = None,
         issuer: str = "salsa",
         subject: str | None = None,
-        **kwargs,
-    ):
+        **kwargs: Any,
+    ) -> Identity:
         """Create a test identity."""
         if subject is None:
             subject = f"{issuer}@debian.org"
@@ -80,18 +84,22 @@ class SalsaSignonViews(TestCase):
             **kwargs,
         )
 
-    def _make_request(self, url: str) -> HttpRequest:
+    def _make_post_request(self, url: str) -> HttpRequest:
         """Create a request that can be manipulated before invoking a view."""
         factory = RequestFactory()
-        request = factory.get(url)
+        request = factory.post(url)
         request.user = AnonymousUser()
         request.session = MockSession()
         cast(RequestSignonProtocol, request).signon = Signon(request)
+        # Bypass CSRF checks for logout.  django.test.Client does this, but
+        # we're bypassing that in order to manipulate the request, so for
+        # now we have to set a private attribute.
+        setattr(request, "_dont_enforce_csrf_checks", True)
         return request
 
-    def test_login_links(self):
+    def test_login_links(self) -> None:
         """Login form shows link to authenticate with salsa."""
-        response = self.client.get(reverse("accounts:login"))
+        response = self.client.get(reverse("login"))
 
         # Collect urls to the salsa provider
         external_urls = self._collect_urls(
@@ -112,20 +120,21 @@ class SalsaSignonViews(TestCase):
             query["redirect_uri"],
             [
                 "http://testserver"
-                + reverse("accounts:oidc_callback", kwargs={"name": "salsa"})
+                + reverse("signon:oidc_callback", kwargs={"name": "salsa"})
             ],
         )
         self.assertIn("client_id", query)
         self.assertIn("state", query)
 
-    def test_login_links_when_signed_in(self):
+    def test_login_links_when_signed_in(self) -> None:
         """Active identities don't show a login link."""
         user = get_user_model().objects.create_user("test")
         identity = self._make_identity(user=user)
 
         # Simulate a session with a logged in user
-        url = reverse("accounts:logout")
-        request = self._make_request(url)
+        url = reverse("user:logout")
+        request = self._make_post_request(url)
+        assert isinstance(request, RequestSignonProtocol)
         request.signon.activate_identity(identity)
         self.assertEqual(request.user, user)
 
@@ -139,14 +148,15 @@ class SalsaSignonViews(TestCase):
         )
         self.assertEqual(len(external_urls), 0)
 
-    def test_logout(self):
+    def test_logout(self) -> None:
         """Logout also deactivates signon identities."""
         user = get_user_model().objects.create_user("test")
         identity = self._make_identity(user=user)
 
         # Simulate a session with a logged in user
-        url = reverse("accounts:logout")
-        request = self._make_request(url)
+        url = reverse("user:logout")
+        request = self._make_post_request(url)
+        assert isinstance(request, RequestSignonProtocol)
         request.signon.activate_identity(identity)
         self.assertEqual(request.user, user)
 
@@ -162,6 +172,7 @@ class SalsaSignonViews(TestCase):
 
         # Identities have been deactivated
         self.assertIsNone(request.session.get("signon_identity_salsa"))
+        assert isinstance(request, RequestSignonProtocol)
         self.assertEqual(request.signon.identities, {})
 
     @contextlib.contextmanager
@@ -170,7 +181,7 @@ class SalsaSignonViews(TestCase):
         identity: Identity | None = None,
         options: Sequence[str] = (),
         **claims: Any,
-    ):
+    ) -> Generator[None, None, None]:
         """
         Mock remote authentication.
 
@@ -179,7 +190,7 @@ class SalsaSignonViews(TestCase):
         if identity is not None:
             claims.setdefault("sub", identity.subject)
 
-        def load_tokens(self):
+        def load_tokens(self: BoundOIDCProvider) -> None:
             self.id_token_claims = claims
             self.options = options
 
@@ -190,11 +201,11 @@ class SalsaSignonViews(TestCase):
         ):
             yield
 
-    def test_oidc_callback_create_identity(self):
+    def test_oidc_callback_create_identity(self) -> None:
         """The OIDC callback creates a missing identity."""
         self.assertEqual(Identity.objects.count(), 0)
 
-        cb_url = reverse("accounts:oidc_callback", kwargs={"name": "salsa"})
+        cb_url = reverse("signon:oidc_callback", kwargs={"name": "salsa"})
 
         start_time = timezone.now()
 
@@ -221,6 +232,7 @@ class SalsaSignonViews(TestCase):
         )
 
         # The identity is activated
+        assert isinstance(response.wsgi_request, RequestSignonProtocol)
         self.assertEqual(
             response.wsgi_request.signon.identities, {"salsa": identities[0]}
         )
@@ -228,7 +240,7 @@ class SalsaSignonViews(TestCase):
         # The user is not authenticated, because the identity is not bound
         self.assertFalse(response.wsgi_request.user.is_authenticated)
 
-    def test_oidc_callback_login(self):
+    def test_oidc_callback_login(self) -> None:
         """The OIDC callback logs in an existing bound identity."""
         self.assertEqual(Identity.objects.count(), 0)
 
@@ -241,9 +253,7 @@ class SalsaSignonViews(TestCase):
             last_used=timezone.now() - datetime.timedelta(days=1),
         )
 
-        cb_url = reverse(
-            "accounts:oidc_callback", kwargs={"name": ident.issuer}
-        )
+        cb_url = reverse("signon:oidc_callback", kwargs={"name": ident.issuer})
 
         start_time = timezone.now()
         with self.setup_mock_auth(identity=ident, profile="profile_url"):
@@ -264,6 +274,7 @@ class SalsaSignonViews(TestCase):
         self.assertEqual(self.client.session["signon_identity_salsa"], ident.pk)
 
         # The identity is activated
+        assert isinstance(response.wsgi_request, RequestSignonProtocol)
         self.assertEqual(
             response.wsgi_request.signon.identities, {"salsa": ident}
         )
@@ -272,7 +283,7 @@ class SalsaSignonViews(TestCase):
         self.assertTrue(response.wsgi_request.user.is_authenticated)
         self.assertEqual(response.wsgi_request.user, user)
 
-    def test_oidc_callback_login_twice(self):
+    def test_oidc_callback_login_twice(self) -> None:
         """The OIDC callback logs in an existing bound identity."""
         self.assertEqual(Identity.objects.count(), 0)
 
@@ -286,9 +297,7 @@ class SalsaSignonViews(TestCase):
             last_used=last_used,
         )
 
-        cb_url = reverse(
-            "accounts:oidc_callback", kwargs={"name": ident.issuer}
-        )
+        cb_url = reverse("signon:oidc_callback", kwargs={"name": ident.issuer})
 
         self.client.force_login(user)
 
@@ -302,18 +311,19 @@ class SalsaSignonViews(TestCase):
         self.assertEqual(Identity.objects.count(), 1)
 
         ident.refresh_from_db()
-        # last_used has not been updated
+        # last_used has been updated
         self.assertGreaterEqual(ident.last_used, start_time)
-        # Claims in ident have not been filled
+        # Claims in ident have been filled
         self.assertEqual(ident.claims, {"profile": "profile_url", "sub": "123"})
 
         # Check that the Identity is attached to the session
         self.assertNotIn("signon_identity_salsa", self.client.session)
 
         # The identity is activated
+        assert isinstance(response.wsgi_request, RequestSignonProtocol)
         self.assertEqual(response.wsgi_request.signon.identities, {})
 
-    def test_oidc_callback_bind(self):
+    def test_oidc_callback_bind(self) -> None:
         """The OIDC callback performs bind if requested."""
         self.assertEqual(Identity.objects.count(), 0)
 
@@ -325,9 +335,7 @@ class SalsaSignonViews(TestCase):
             last_used=timezone.now() - datetime.timedelta(days=1),
         )
 
-        cb_url = reverse(
-            "accounts:oidc_callback", kwargs={"name": ident.issuer}
-        )
+        cb_url = reverse("signon:oidc_callback", kwargs={"name": ident.issuer})
 
         self.client.force_login(user)
 
@@ -353,15 +361,16 @@ class SalsaSignonViews(TestCase):
         self.assertEqual(response.wsgi_request.user, user)
 
         # Check that the Identity is attached to the session
+        assert isinstance(response.wsgi_request, RequestSignonProtocol)
         self.assertEqual(
             response.wsgi_request.signon.identities, {"salsa": ident}
         )
 
-    def test_oidc_callback_wrong_provider(self):
+    def test_oidc_callback_wrong_provider(self) -> None:
         """The OIDC callback with a wrong provider."""
         self.assertEqual(Identity.objects.count(), 0)
 
-        cb_url = reverse("accounts:oidc_callback", kwargs={"name": "wrong"})
+        cb_url = reverse("signon:oidc_callback", kwargs={"name": "wrong"})
 
         response = self.client.get(cb_url)
         self.assertEqual(response.status_code, 404)
@@ -369,7 +378,7 @@ class SalsaSignonViews(TestCase):
         self.assertEqual(Identity.objects.count(), 0)
         self.assertFalse(response.wsgi_request.user.is_authenticated)
 
-    def test_bindidentity(self):
+    def test_bindidentity(self) -> None:
         """Bind identity redirects correctly."""
         user = get_user_model().objects.create_user(
             "test", email="test@example.org"
@@ -377,7 +386,7 @@ class SalsaSignonViews(TestCase):
         self.client.force_login(user)
         session = self.client.session
         response = self.client.get(
-            reverse("accounts:bind_identity", kwargs={"name": "salsa"})
+            reverse("signon:bind_identity", kwargs={"name": "salsa"})
         )
         self.assertEqual(response.status_code, 302)
         url = response.headers["Location"]
@@ -390,21 +399,21 @@ class SalsaSignonViews(TestCase):
         self.assertEqual(session["signon_state_salsa"], state)
         self.assertEqual(session["signon_state_salsa_options"], ["bind"])
 
-    def test_bindidentity_no_user(self):
+    def test_bindidentity_no_user(self) -> None:
         """Bind identity fails if not logged in."""
         response = self.client.get(
-            reverse("accounts:bind_identity", kwargs={"name": "salsa"})
+            reverse("signon:bind_identity", kwargs={"name": "salsa"})
         )
         self.assertEqual(response.status_code, 403)
 
-    def test_bindidentity_wrong_provider(self):
+    def test_bindidentity_wrong_provider(self) -> None:
         """Bind identity fails for a nonexisting provider."""
         user = get_user_model().objects.create_user(
             "test", email="test@example.org"
         )
         self.client.force_login(user)
         response = self.client.get(
-            reverse("accounts:bind_identity", kwargs={"name": "wrong"})
+            reverse("signon:bind_identity", kwargs={"name": "wrong"})
         )
         self.assertEqual(response.status_code, 403)
 
@@ -415,14 +424,14 @@ class SalsaSignonViews(TestCase):
     # For now we only support a single authentication provider, and we have no
     # urgent need of this
 
-    # def test_bind_view(self):
+    # def test_bind_view(self) -> None:
     #     """Test user profile view to bind external providers."""
     #     raise NotImplementedError()
 
-    # def test_oidc_callback_bind_intent(self):
+    # def test_oidc_callback_bind_intent(self) -> None:
     #     """Test callback when authenticated with bind intent."""
     #     raise NotImplementedError()
 
-    # def test_oidc_callback_no_bind_intent(self):
+    # def test_oidc_callback_no_bind_intent(self) -> None:
     #     """Test callback when authenticated with no bind intent."""
     #     raise NotImplementedError()
