@@ -21,6 +21,9 @@ from typing import Any
 from django import http
 from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured, PermissionDenied
+from django.core.handlers.exception import response_for_exception
+from django.db import transaction
+from django.http import HttpRequest, HttpResponse, HttpResponseBase
 from django.shortcuts import redirect
 from django.utils import timezone
 from django.utils.decorators import method_decorator
@@ -29,16 +32,20 @@ from django.views.generic import View
 
 from debusine.db.models import Identity
 from debusine.server.signon import providers
+from debusine.server.signon.middleware import RequestSignonProtocol
 
 
 class SignonLogoutMixin:
     """Mixin to logout external signon providers in a logout view."""
 
     @method_decorator(never_cache)
-    def dispatch(self, request, *args, **kwargs):
+    def dispatch(
+        self, request: HttpRequest, *args: Any, **kwargs: Any
+    ) -> HttpResponse:
         """Wrap the normal logout to also log out identifiers."""
         if signon := getattr(request, "signon", None):
             signon.logout_identities()
+        assert isinstance(self, View)
         return super().dispatch(request, *args, **kwargs)
 
 
@@ -50,7 +57,9 @@ class BindIdentityView(View):
     success the identity is bound to the current user
     """
 
-    def get(self, request, *args, **kwargs):
+    def get(
+        self, request: HttpRequest, *args: Any, **kwargs: Any
+    ) -> HttpResponseBase:
         """Check, setup, and redirect to the external identity provider."""
         if not request.user.is_authenticated:
             raise PermissionDenied
@@ -92,7 +101,9 @@ class OIDCAuthenticationCallbackView(View):
 
         return bound_provider.id_token_claims, bound_provider.options
 
-    def get(self, request, *args, **kwargs):
+    def get(
+        self, request: HttpRequest, *args: Any, **kwargs: Any
+    ) -> HttpResponseBase:
         """
         Notify successful authentication from from the external OIDC server.
 
@@ -119,7 +130,14 @@ class OIDCAuthenticationCallbackView(View):
         identity.last_used = timezone.now()
         identity.save()
 
-        request.signon.activate_identity(identity, *options)
+        with transaction.atomic():
+            # Handle the exception ourselves, since we want to save the
+            # identity's last_used state even if this fails.
+            try:
+                assert isinstance(request, RequestSignonProtocol)
+                request.signon.activate_identity(identity, *options)
+            except Exception as exc:
+                return response_for_exception(request, exc)
 
         return redirect(
             getattr(settings, "SIGNON_DEFAULT_REDIRECT", "homepage:homepage")
