@@ -23,7 +23,6 @@ from textwrap import dedent
 from typing import Generic, TypeVar
 
 from debian.deb822 import Deb822, Packages, Sources
-
 from django.db import transaction
 
 from debusine.artifacts.local_artifact import BinaryPackage, SourcePackage
@@ -38,7 +37,7 @@ from debusine.server.collections.debian_suite import DebianSuiteManager
 from debusine.server.tasks import BaseServerTask
 from debusine.server.tasks.models import APTMirrorData
 from debusine.tasks import TaskConfigError
-
+from debusine.tasks.models import BaseDynamicTaskData
 
 T = TypeVar("T", bound=Deb822)
 
@@ -90,7 +89,7 @@ def _run_and_log_errors(
         raise
 
 
-class APTMirror(BaseServerTask[APTMirrorData]):
+class APTMirror(BaseServerTask[APTMirrorData, BaseDynamicTaskData]):
     """Task that mirrors an external APT suite into a debusine collection."""
 
     TASK_VERSION = 1
@@ -312,7 +311,7 @@ class APTMirror(BaseServerTask[APTMirrorData]):
         name: str,
         binary: Packages,
         component: str,
-    ):
+    ) -> None:
         """Download a binary package and add it to the collection."""
         assert self.work_request is not None
         assert self.workspace is not None
@@ -495,3 +494,7 @@ class APTMirror(BaseServerTask[APTMirrorData]):
             self.update_sources(temp_path, self.plan_sources(temp_path))
             self.update_binaries(temp_path, self.plan_binaries(temp_path))
         return True
+
+    def get_label(self) -> str:
+        """Return the task label."""
+        return f"mirror {self.data.collection} from {self.data.url}"
