@@ -14,6 +14,7 @@ from pathlib import Path
 from typing import TypeVar
 
 from debusine.artifacts import WorkRequestDebugLogs
+from debusine.db.context import context
 from debusine.db.models import (
     Artifact,
     ArtifactRelation,
@@ -22,12 +23,13 @@ from debusine.db.models import (
 )
 from debusine.tasks import BaseTask
 from debusine.tasks.models import BaseDynamicTaskData, BaseTaskData, TaskTypes
-
+from debusine.worker.system_information import host_architecture
 
 TD = TypeVar("TD", bound=BaseTaskData)
+DTD = TypeVar("DTD", bound=BaseDynamicTaskData)
 
 
-class BaseServerTask(BaseTask[TD, BaseDynamicTaskData], metaclass=ABCMeta):
+class BaseServerTask(BaseTask[TD, DTD], metaclass=ABCMeta):
     """Base class for tasks that run on server-side Celery workers."""
 
     TASK_TYPE = TaskTypes.SERVER
@@ -45,12 +47,28 @@ class BaseServerTask(BaseTask[TD, BaseDynamicTaskData], metaclass=ABCMeta):
         self.work_request = work_request
         self.workspace = work_request.workspace
 
-        # BaseTask attributes; less useful for server tasks since we have
-        # the model instances, but we set them anyway for compatibility.
+        # BaseTask attributes; work_request_id and workspace_name are less
+        # useful for server tasks since we have the model instances, but we
+        # set them anyway for compatibility.
         self.work_request_id = work_request.id
         self.workspace_name = work_request.workspace.name
+        self.worker_host_architecture = host_architecture()
 
-    def _upload_work_request_debug_logs(self):
+    def execute(self) -> bool:
+        """Execute task, setting up a suitable context."""
+        assert self.work_request is not None
+        assert self.workspace is not None
+
+        context.reset()
+        context.set_scope(self.workspace.scope)
+        context.set_user(self.work_request.created_by)
+        self.workspace.set_current()
+        try:
+            return super().execute()
+        finally:
+            context.reset()
+
+    def _upload_work_request_debug_logs(self) -> None:
         """
         Create a WorkRequestDebugLogs artifact and upload the logs.
 
@@ -67,6 +85,7 @@ class BaseServerTask(BaseTask[TD, BaseDynamicTaskData], metaclass=ABCMeta):
             files=Path(self._debug_log_files_directory.name).glob("*")
         )
 
+        assert self.workspace is not None
         artifact = Artifact.objects.create_from_local_artifact(
             work_request_debug_logs,
             self.workspace,
