@@ -13,6 +13,7 @@ import hashlib
 import logging
 import os
 import re
+import shutil
 from collections.abc import Generator
 from contextlib import contextmanager
 from pathlib import Path
@@ -22,27 +23,27 @@ from typing import Any
 from unittest.mock import MagicMock, patch
 
 from debian.deb822 import Packages, Sources
-from django.test import TestCase
 
 from debusine.artifacts.models import ArtifactCategory, CollectionCategory
+from debusine.db.context import context
 from debusine.db.models import Artifact, ArtifactRelation, Collection, Workspace
 from debusine.db.tests.utils import _calculate_hash_from_data
 from debusine.server.collections.debian_suite import (
     DebianSuiteManager,
     make_pool_filename,
 )
+from debusine.server.tasks import APTMirror
 from debusine.server.tasks.aptmirror import (
-    APTMirror,
     InconsistentMirrorError,
     Plan,
     PlanAdd,
     PlanReplace,
 )
 from debusine.tasks import TaskConfigError
-from debusine.test import TestHelpersMixin
+from debusine.test.django import TestCase
 
 
-class APTMirrorTests(TestHelpersMixin, TestCase):
+class APTMirrorTests(TestCase):
     """Test the :py:class:`APTMirror` task."""
 
     def create_suite_collection(self, name: str) -> Collection:
@@ -204,7 +205,7 @@ class APTMirrorTests(TestHelpersMixin, TestCase):
     ) -> Generator[MagicMock, None, None]:
         """Temporarily patch the output of `apt-get indextargets`."""
 
-        def fake_run(args: list[str], **kwargs: Any) -> CompletedProcess[bytes]:
+        def fake_run(args: list[str], **kwargs: Any) -> CompletedProcess[str]:
             return CompletedProcess(
                 args=args,
                 returncode=0,
@@ -212,9 +213,9 @@ class APTMirrorTests(TestHelpersMixin, TestCase):
                     "".join(
                         f"Filename: {path}\nComponent: {component}\n\n"
                         for path, component in paths_and_components
-                    ).encode()
+                    )
                 ),
-                stderr=b"",
+                stderr="",
             )
 
         with patch("subprocess.run", side_effect=fake_run) as mock_run:
@@ -243,7 +244,7 @@ class APTMirrorTests(TestHelpersMixin, TestCase):
 
         def fake_run(
             args: list[str], *, cwd: Path, **kwargs: Any
-        ) -> CompletedProcess[bytes]:
+        ) -> CompletedProcess[str]:
             for name, contents in last_arg_to_files[args[-1]].items():
                 (cwd / name).write_bytes(contents)
             return CompletedProcess(args=args, returncode=0)
@@ -506,7 +507,7 @@ class APTMirrorTests(TestHelpersMixin, TestCase):
         temp_path = self.create_temporary_directory()
         collection = self.create_suite_collection("bookworm")
         manager = DebianSuiteManager(collection)
-        work_request = self.create_work_request()
+        work_request = self.playground.create_work_request()
         task = self.create_apt_mirror_task("bookworm")
         task.set_work_request(work_request)
 
@@ -582,7 +583,7 @@ class APTMirrorTests(TestHelpersMixin, TestCase):
         temp_path = self.create_temporary_directory()
         collection = self.create_suite_collection("bookworm")
         manager = DebianSuiteManager(collection)
-        work_request = self.create_work_request()
+        work_request = self.playground.create_work_request()
         task = self.create_apt_mirror_task("bookworm")
         task.set_work_request(work_request)
 
@@ -639,11 +640,12 @@ class APTMirrorTests(TestHelpersMixin, TestCase):
                 temp_path,
             )
 
+    @context.disable_permission_checks()
     def test_add_source(self) -> None:
         """`add_source` downloads and adds a source package."""
         temp_path = self.create_temporary_directory()
         collection = self.create_suite_collection("bookworm")
-        work_request = self.create_work_request()
+        work_request = self.playground.create_work_request()
         task = self.create_apt_mirror_task("bookworm")
         task.set_work_request(work_request)
 
@@ -696,6 +698,7 @@ class APTMirrorTests(TestHelpersMixin, TestCase):
         self.assertEqual(item.data["component"], "main")
         self.assertEqual(item.data["section"], "devel")
 
+    @context.disable_permission_checks()
     def test_update_sources(self) -> None:
         """`update_sources` executes a plan to update sources."""
         temp_path = self.create_temporary_directory()
@@ -703,7 +706,7 @@ class APTMirrorTests(TestHelpersMixin, TestCase):
         collection.data["may_reuse_versions"] = True
         collection.save()
         manager = DebianSuiteManager(collection)
-        work_request = self.create_work_request()
+        work_request = self.playground.create_work_request()
         task = self.create_apt_mirror_task("bookworm")
         task.set_work_request(work_request)
 
@@ -859,7 +862,7 @@ class APTMirrorTests(TestHelpersMixin, TestCase):
         temp_path = self.create_temporary_directory()
         collection = self.create_suite_collection("bookworm")
         manager = DebianSuiteManager(collection)
-        work_request = self.create_work_request()
+        work_request = self.playground.create_work_request()
         task = self.create_apt_mirror_task("bookworm")
         task.set_work_request(work_request)
 
@@ -928,7 +931,7 @@ class APTMirrorTests(TestHelpersMixin, TestCase):
         temp_path = self.create_temporary_directory()
         collection = self.create_suite_collection("bookworm")
         manager = DebianSuiteManager(collection)
-        work_request = self.create_work_request()
+        work_request = self.playground.create_work_request()
         task = self.create_apt_mirror_task("bookworm")
         task.set_work_request(work_request)
 
@@ -1022,11 +1025,12 @@ class APTMirrorTests(TestHelpersMixin, TestCase):
                 temp_path,
             )
 
+    @context.disable_permission_checks()
     def test_add_binary(self) -> None:
         """`add_binary` downloads and adds a binary package."""
         temp_path = self.create_temporary_directory()
         collection = self.create_suite_collection("bookworm")
-        work_request = self.create_work_request()
+        work_request = self.playground.create_work_request()
         task = self.create_apt_mirror_task("bookworm")
         task.set_work_request(work_request)
 
@@ -1087,11 +1091,12 @@ class APTMirrorTests(TestHelpersMixin, TestCase):
         self.assertEqual(item.data["section"], "libs")
         self.assertEqual(item.data["priority"], "optional")
 
+    @context.disable_permission_checks()
     def test_add_binary_renames(self) -> None:
         """`add_binary` renames downloaded binary packages if necessary."""
         temp_path = self.create_temporary_directory()
         collection = self.create_suite_collection("bookworm")
-        work_request = self.create_work_request()
+        work_request = self.playground.create_work_request()
         task = self.create_apt_mirror_task("bookworm")
         task.set_work_request(work_request)
 
@@ -1151,12 +1156,13 @@ class APTMirrorTests(TestHelpersMixin, TestCase):
         self.assertEqual(item.data["section"], "libs")
         self.assertEqual(item.data["priority"], "optional")
 
+    @context.disable_permission_checks()
     def test_add_binary_relates_to_source(self) -> None:
         """`add_binary` adds a relation to a matching source package."""
         temp_path = self.create_temporary_directory()
         collection = self.create_suite_collection("bookworm")
         manager = DebianSuiteManager(collection)
-        work_request = self.create_work_request()
+        work_request = self.playground.create_work_request()
         task = self.create_apt_mirror_task("bookworm")
         task.set_work_request(work_request)
 
@@ -1207,6 +1213,7 @@ class APTMirrorTests(TestHelpersMixin, TestCase):
         self.assertEqual(relation.target, source_package_artifacts[0])
         self.assertEqual(relation.type, ArtifactRelation.Relations.BUILT_USING)
 
+    @context.disable_permission_checks()
     def test_add_binary_no_source_relation_if_may_reuse_versions(self) -> None:
         """`add_binary` doesn't add source relations if `may_reuse_versions`."""
         temp_path = self.create_temporary_directory()
@@ -1214,7 +1221,7 @@ class APTMirrorTests(TestHelpersMixin, TestCase):
         collection.data["may_reuse_versions"] = True
         collection.save()
         manager = DebianSuiteManager(collection)
-        work_request = self.create_work_request()
+        work_request = self.playground.create_work_request()
         task = self.create_apt_mirror_task("bookworm")
         task.set_work_request(work_request)
 
@@ -1259,6 +1266,7 @@ class APTMirrorTests(TestHelpersMixin, TestCase):
         assert binary_item.artifact is not None
         self.assertFalse(binary_item.artifact.relations.exists())
 
+    @context.disable_permission_checks()
     def test_update_binaries(self) -> None:
         """`update_binaries` executes a plan to update binaries."""
         temp_path = self.create_temporary_directory()
@@ -1266,7 +1274,7 @@ class APTMirrorTests(TestHelpersMixin, TestCase):
         collection.data["may_reuse_versions"] = True
         collection.save()
         manager = DebianSuiteManager(collection)
-        work_request = self.create_work_request()
+        work_request = self.playground.create_work_request()
         task = self.create_apt_mirror_task("bookworm")
         task.set_work_request(work_request)
 
@@ -1352,30 +1360,86 @@ class APTMirrorTests(TestHelpersMixin, TestCase):
         """
         Executing the task runs through the full sequence.
 
-        Most of the details are tested elsewhere; we mock them for brevity.
+        Most of the details are tested elsewhere, but we do enough to check
+        that the task can add to its collection.
         """
-        work_request = self.create_work_request()
+        temp_path = self.create_temporary_directory()
+        with open(temp_path / "Sources", "w") as sources:
+            sources.write(
+                self.write_sample_source_package(
+                    temp_path, "hello", "1.0"
+                ).dump()
+                + "\n"
+            )
+        with open(temp_path / "Packages", "w") as packages:
+            packages.write(
+                self.write_sample_binary_package(
+                    temp_path, "hello", "1.0", "amd64"
+                ).dump()
+                + "\n"
+            )
+
+        collection = self.create_suite_collection("bookworm")
+        work_request = self.playground.create_work_request()
         task = self.create_apt_mirror_task("bookworm")
         task.set_work_request(work_request)
 
-        with (
-            patch.object(task, "fetch_indexes") as mock_fetch_indexes,
-            patch.object(task, "plan_sources") as mock_plan_sources,
-            patch.object(task, "update_sources") as mock_update_sources,
-            patch.object(task, "plan_binaries") as mock_plan_binaries,
-            patch.object(task, "update_binaries") as mock_update_binaries,
-        ):
-            self.assertTrue(task._execute())
+        def fake_run(
+            args: list[str],
+            cwd: str | os.PathLike[str] | None,
+            env: dict[str, str],
+            **kwargs: Any,
+        ) -> CompletedProcess[str]:
+            apt_lists_path = (
+                Path(env["APT_CONFIG"]).parent.parent.parent
+                / "var/lib/apt/lists"
+            )
+            stdout = ""
+            match args:
+                case ["apt-get", "update"]:
+                    for name in ("Sources", "Packages"):
+                        shutil.copy(temp_path / name, apt_lists_path / name)
+                case ["apt-get", "indextargets", "Identifier: Sources"]:
+                    stdout = dedent(
+                        f"""\
+                        Filename: {apt_lists_path}/Sources
+                        Component: main
 
-        mock_fetch_indexes.assert_called_once()
-        temp_path = mock_fetch_indexes.call_args[0][0]
-        mock_plan_sources.assert_called_once_with(temp_path)
-        mock_update_sources.assert_called_once_with(
-            temp_path, mock_plan_sources.return_value
-        )
-        mock_plan_binaries.assert_called_once_with(temp_path)
-        mock_update_binaries.assert_called_once_with(
-            temp_path, mock_plan_binaries.return_value
+                        """
+                    )
+                case ["apt-get", "indextargets", "Identifier: Packages"]:
+                    stdout = dedent(
+                        f"""\
+                        Filename: {apt_lists_path}/Packages
+                        Component: main
+
+                        """
+                    )
+                case ["apt-get", *_, "source", "hello=1.0"]:
+                    assert cwd is not None
+                    for name in ("hello_1.0.dsc", "hello_1.0.tar.xz"):
+                        shutil.copy(temp_path / name, Path(cwd) / name)
+                case ["apt-get", "download", "hello:amd64=1.0"]:
+                    assert cwd is not None
+                    shutil.copy(
+                        temp_path / "hello_1.0_amd64.deb",
+                        Path(cwd) / "hello_1.0_amd64.deb",
+                    )
+                case _ as unreachable:
+                    raise AssertionError(
+                        f"Unexpected subprocess arguments: {unreachable}"
+                    )
+            return CompletedProcess(
+                args=args, returncode=0, stdout=stdout, stderr=""
+            )
+
+        with patch("subprocess.run", side_effect=fake_run):
+            self.assertTrue(task.execute())
+
+        self.assertQuerySetEqual(
+            collection.child_items.values_list("name", flat=True),
+            ["hello_1.0", "hello_1.0_amd64"],
+            ordered=False,
         )
 
     def test_label(self) -> None:
