@@ -10,16 +10,16 @@
 """Unit tests for base server-side task classes."""
 
 from pathlib import Path
-
-from django.test import TestCase
+from typing import ClassVar
+from unittest import mock
 
 from debusine.artifacts.local_artifact import WorkRequestDebugLogs
+from debusine.db.context import context
 from debusine.db.models import Artifact, ArtifactRelation, WorkRequest
 from debusine.server.tasks.base import BaseServerTask
 from debusine.tasks.models import BaseDynamicTaskData, BaseTaskData, WorkerType
 from debusine.tasks.tests.helper_mixin import TestTaskMixin
-from debusine.tasks.tests.test_task import TestBaseTask2Data
-from debusine.test import TestHelpersMixin
+from debusine.test.django import TestCase
 from debusine.worker.system_information import host_architecture
 
 
@@ -33,8 +33,16 @@ class TestBaseServerTask(
         raise NotImplementedError()
 
 
+class TestBaseServerTask2Data(BaseTaskData):
+    """Data representation for TestBaseServerTask2."""
+
+    __test__ = False
+
+    foo: str
+
+
 class TestBaseServerTask2(
-    TestTaskMixin, BaseServerTask[TestBaseTask2Data, BaseDynamicTaskData]
+    TestTaskMixin, BaseServerTask[TestBaseServerTask2Data, BaseDynamicTaskData]
 ):
     """Test BaseServerTask class with jsonschema validation."""
 
@@ -45,22 +53,28 @@ class TestBaseServerTask2(
         raise NotImplementedError()
 
 
-class BaseServerTaskTests(TestHelpersMixin, TestCase):
+class BaseServerTaskTests(TestCase):
     """Unit tests for :class:`BaseServerTask`."""
 
-    def setUp(self) -> None:
-        """Create the shared attributes."""
-        self.task = TestBaseServerTask({})
-        self.task2 = TestBaseServerTask2({"foo": "bar"})
-        self.worker_metadata = {"system:worker_type": WorkerType.CELERY}
+    task: ClassVar[TestBaseServerTask]
+    task2: ClassVar[TestBaseServerTask2]
+    worker_metadata: ClassVar[dict[str, WorkerType]]
 
-    def test_can_run_on_no_version(self):
+    @classmethod
+    def setUpTestData(cls) -> None:
+        """Create the shared attributes."""
+        super().setUpTestData()
+        cls.task = TestBaseServerTask({})
+        cls.task2 = TestBaseServerTask2({"foo": "bar"})
+        cls.worker_metadata = {"system:worker_type": WorkerType.CELERY}
+
+    def test_can_run_on_no_version(self) -> None:
         """Ensure can_run_on returns True if no version is specified."""
         self.assertIsNone(self.task.TASK_VERSION)
         metadata = {**self.worker_metadata, **self.task.analyze_worker()}
         self.assertEqual(self.task.can_run_on(metadata), True)
 
-    def test_can_run_on_with_different_versions(self):
+    def test_can_run_on_with_different_versions(self) -> None:
         """Ensure can_run_on returns False if versions differ."""
         self.assertIsNone(self.task.TASK_VERSION)
         metadata = {**self.worker_metadata, **self.task.analyze_worker()}
@@ -69,7 +83,7 @@ class BaseServerTaskTests(TestHelpersMixin, TestCase):
 
     def test_set_work_request(self) -> None:
         """set_work_request sets appropriate attributes."""
-        work_request = self.create_work_request()
+        work_request = self.playground.create_work_request()
 
         self.task.set_work_request(work_request)
 
@@ -81,16 +95,36 @@ class BaseServerTaskTests(TestHelpersMixin, TestCase):
             self.task.worker_host_architecture, host_architecture()
         )
 
+    def test_execute_sets_up_context(self) -> None:
+        """`execute` sets up a suitable context."""
+        work_request = self.playground.create_work_request()
+        self.task.set_work_request(work_request)
+
+        def fake_execute() -> bool:
+            self.assertEqual(context.scope, work_request.workspace.scope)
+            self.assertEqual(context.user, work_request.created_by)
+            self.assertEqual(context.workspace, work_request.workspace)
+            return True
+
+        with mock.patch.object(
+            TestBaseServerTask, "_execute", side_effect=fake_execute
+        ):
+            self.task.execute()
+
+        self.assertIsNone(context.scope)
+        self.assertIsNone(context.user)
+        self.assertIsNone(context.workspace)
+
     def setup_upload_work_request_debug_logs(
         self, source_artifacts: list[Artifact] | None = None
-    ):
+    ) -> list[tuple[str, bytes]]:
         """Setup for upload_work_request_debug_logs tests."""  # noqa: D401
         # Add a file to be uploaded
         with self.task.open_debug_log_file("test.log") as file:
             file.write("log")
         assert self.task._debug_log_files_directory is not None
 
-        self.task.set_work_request(self.create_work_request())
+        self.task.set_work_request(self.playground.create_work_request())
         self.task._source_artifacts_ids = [
             artifact.id for artifact in source_artifacts or []
         ]
@@ -131,7 +165,8 @@ class BaseServerTaskTests(TestHelpersMixin, TestCase):
 
         return artifact
 
-    def test_upload_work_request_debug_logs_with_relation(self):
+    @context.disable_permission_checks()
+    def test_upload_work_request_debug_logs_with_relation(self) -> None:
         """
         Artifact is created and uploaded. Relation is created.
 
@@ -141,6 +176,7 @@ class BaseServerTaskTests(TestHelpersMixin, TestCase):
         expected_files = self.setup_upload_work_request_debug_logs(
             [source_artifact]
         )
+        assert self.task.work_request is not None
 
         self.task._upload_work_request_debug_logs()
 
@@ -155,9 +191,11 @@ class BaseServerTaskTests(TestHelpersMixin, TestCase):
         self.assertEqual(relation.target, source_artifact)
         self.assertEqual(relation.type, ArtifactRelation.Relations.RELATES_TO)
 
-    def test_upload_work_request_debug_logs(self):
+    @context.disable_permission_checks()
+    def test_upload_work_request_debug_logs(self) -> None:
         """Artifact is created and uploaded."""
         expected_files = self.setup_upload_work_request_debug_logs()
+        assert self.task.work_request is not None
 
         self.task._upload_work_request_debug_logs()
 
@@ -165,7 +203,7 @@ class BaseServerTaskTests(TestHelpersMixin, TestCase):
             self.task.work_request, expected_files
         )
 
-    def test_upload_work_request_no_log_files(self):
+    def test_upload_work_request_no_log_files(self) -> None:
         """No log files: no artifact created."""
         self.task._upload_work_request_debug_logs()
 
