@@ -19,7 +19,7 @@ from debusine.test.django import TestCase
 class ServerConfigTests(TestCase):
     """Tests for ServerConfig class."""
 
-    def setUp(self):
+    def setUp(self) -> None:
         """Set up test objects."""
         self.worker = self.playground.create_worker()
         self.worker.mark_connected()
@@ -28,7 +28,7 @@ class ServerConfigTests(TestCase):
         module = importlib.import_module(app_name)
         self.server_config = ServerConfig(app_name, module)
 
-    def test_ready_mark_workers_as_disconnected(self):
+    def test_ready_mark_workers_as_disconnected(self) -> None:
         """
         Test ready() method mark workers as disconnected.
 
@@ -40,7 +40,7 @@ class ServerConfigTests(TestCase):
         self.worker.refresh_from_db()
         self.assertFalse(self.worker.connected())
 
-    def test_ready_celery_workers_not_marked_as_disconnected(self):
+    def test_ready_celery_workers_not_marked_as_disconnected(self) -> None:
         """
         Test ready() method does not mark Celery workers as disconnected.
 
@@ -56,7 +56,7 @@ class ServerConfigTests(TestCase):
         worker.refresh_from_db()
         self.assertTrue(worker.connected())
 
-    def test_ready_workers_not_marked_as_disconnected(self):
+    def test_ready_workers_not_marked_as_disconnected(self) -> None:
         """
         Test ready() method does not mark workers as disconnected.
 
