@@ -13,18 +13,18 @@ import io
 import tarfile
 from pathlib import Path
 
-from django.test import TestCase
-
+from debusine.db.context import context
 from debusine.server.tar import TarArtifact
-from debusine.test import TestHelpersMixin
+from debusine.test.django import TestCase
 
 
-class TarArtifactTests(TestHelpersMixin, TestCase):
+class TarArtifactTests(TestCase):
     """Tests for TarArtifact class."""
 
     playground_memory_file_store = False
 
-    def test_tar_small_chunks(self):
+    @context.disable_permission_checks()
+    def test_tar_small_chunks(self) -> None:
         """Assert that TarArtifact return data in small chunks."""
         file_paths = ["README"]
 
@@ -41,7 +41,8 @@ class TarArtifactTests(TestHelpersMixin, TestCase):
 
         self.assertTrue(tar_artifact._tar_file.closed)
 
-    def test_tar_subdirectory(self):
+    @context.disable_permission_checks()
+    def test_tar_subdirectory(self) -> None:
         """Test creating a tar for an artifact, only files from a subdir."""
         file_paths = ["README", "doc/README", "doc/README2", "src/main.c"]
 
@@ -62,7 +63,8 @@ class TarArtifactTests(TestHelpersMixin, TestCase):
 
         self.assertTrue(tar_artifact._tar_file.closed)
 
-    def test_tar(self):
+    @context.disable_permission_checks()
+    def test_tar(self) -> None:
         """Test creating a tar for an artifact."""
         file_paths = ["README", "README2", "doc/README3"]
 
@@ -93,10 +95,9 @@ class TarArtifactTests(TestHelpersMixin, TestCase):
 
         # Assert file's contents are correct
         for path in file_paths:
-            self.assertEqual(
-                tar_file.extractfile(path).read(),
-                files_contents[path],
-            )
+            file = tar_file.extractfile(path)
+            assert file is not None
+            self.assertEqual(file.read(), files_contents[path])
 
         # Assert file names are ordered
         self.assertEqual(
@@ -118,7 +119,8 @@ class TarArtifactTests(TestHelpersMixin, TestCase):
 
         self.assertTrue(tar_artifact._tar_file.closed)
 
-    def test_next_raise_stop_iteration_file_not_found(self):
+    @context.disable_permission_checks()
+    def test_next_raise_stop_iteration_file_not_found(self) -> None:
         """Cannot read a file: exc logged, StopIteration raised."""
         file_paths = ["README"]
 
@@ -130,16 +132,18 @@ class TarArtifactTests(TestHelpersMixin, TestCase):
             artifact.workspace.default_file_store.get_backend_object()
         )
 
-        fileobj = artifact.fileinartifact_set.all().first().file
+        fileobj = artifact.fileinartifact_set.all()[0].file
 
         local_path = file_backend.get_local_path(fileobj)
+        assert local_path is not None
         local_path.unlink()
 
         tar_artifact = TarArtifact(artifact=artifact)
 
-        with self.assertLogsContains(
-            str(local_path), expected_count=2
-        ), self.assertRaises(StopIteration):
+        with (
+            self.assertLogsContains(str(local_path), expected_count=2),
+            self.assertRaises(StopIteration),
+        ):
             tar_artifact.__next__()
 
         self.assertTrue(tar_artifact._tar_file.closed)
