@@ -10,7 +10,6 @@
 """Views for the server application."""
 
 from debusine.server.views.base import (
-    ArtifactInPublicWorkspace,
     IsTokenAuthenticated,
     IsUserAuthenticated,
     ProblemResponse,
@@ -18,9 +17,8 @@ from debusine.server.views.base import (
 )
 
 __all__ = [
-    "ArtifactInPublicWorkspace",
-    "ProblemResponse",
     "IsTokenAuthenticated",
     "IsUserAuthenticated",
+    "ProblemResponse",
     "ValidatePermissionsMixin",
 ]
