@@ -13,65 +13,80 @@ import binascii
 import copy
 import logging
 import tempfile
+from collections.abc import Mapping
 from pathlib import Path
-from typing import Any, Generic, TypeVar
+from typing import Any, Generic, IO, Literal, TypeVar
 
 from django.conf import settings
-from django.db import transaction
+from django.core.exceptions import ObjectDoesNotExist
 from django.db.models import Model
-from django.http import HttpResponse
+from django.http import Http404
 from django.utils import timezone
-from rest_framework import status, views
+from rest_framework import status
 from rest_framework.parsers import BaseParser, JSONParser
 from rest_framework.request import Request
 from rest_framework.response import Response
+from rest_framework.serializers import BaseSerializer
 
+from debusine.db.context import context
 from debusine.db.models import (
     Artifact,
     ArtifactRelation,
-    DEFAULT_WORKSPACE_NAME,
     File,
     FileInArtifact,
     FileUpload,
-    Token,
     User,
     WorkRequest,
-    Workspace,
 )
+from debusine.db.models.artifacts import (
+    ArtifactQuerySet,
+    ArtifactRelationQuerySet,
+)
+from debusine.server.exceptions import DebusineAPIException
 from debusine.server.serializers import (
     ArtifactRelationResponseSerializer,
     ArtifactSerializer,
     ArtifactSerializerResponse,
 )
 from debusine.server.views.base import (
+    BaseAPIView,
+    CanDisplayFilterBackend,
     CreateAPIViewBase,
     DestroyAPIViewBase,
+    GenericAPIViewBase,
     IsTokenAuthenticated,
-    IsTokenUserAuthenticated,
-    IsWorkerAuthenticated,
     ListAPIViewBase,
-    ProblemResponse,
 )
 from debusine.utils import parse_content_range_header
 
 logger = logging.getLogger(__name__)
 
 
-class ArtifactView(views.APIView):
+class ArtifactView(GenericAPIViewBase[Artifact], BaseAPIView):
     """View used to get or create artifacts."""
 
-    permission_classes = [IsTokenAuthenticated]
+    filter_backends = [CanDisplayFilterBackend]
+    lookup_url_kwarg = "artifact_id"
 
-    def get(self, request, artifact_id: int):
-        """Return information of artifact_id."""
+    def get_queryset(self) -> ArtifactQuerySet[Any]:
+        """Get the query set for this view."""
+        return Artifact.objects.in_current_scope()
+
+    def get_object(self) -> Artifact:
+        """Override to return more API-friendly errors."""
         try:
-            artifact = Artifact.objects.get(id=artifact_id)
-        except Artifact.DoesNotExist:
-            return ProblemResponse(
-                f"Artifact {artifact_id} not found",
-                status_code=status.HTTP_404_NOT_FOUND,
+            return super().get_object()
+        except Http404 as exc:
+            raise DebusineAPIException(
+                title=str(exc), status_code=status.HTTP_404_NOT_FOUND
             )
 
+    def get(self, request: Request, artifact_id: int) -> Response:  # noqa: U100
+        """Return information of artifact_id."""
+        artifact = self.get_object()
+        self.set_current_workspace(artifact.workspace)
+        self.enforce(artifact.can_display)
+
         artifact_response = ArtifactSerializerResponse.from_artifact(
             artifact, request
         )
@@ -120,72 +135,55 @@ class ArtifactView(views.APIView):
             file_in_artifact.complete = True
             file_in_artifact.save()
 
-    @transaction.atomic
-    def post(self, request: Request) -> HttpResponse:
+    def post(self, request: Request) -> Response:
         """Create a new artifact."""
         data = request.data.copy()
 
-        # "workspace" might not be in data, but it is a mandatory field
-        # in the serializer.
-        # This view might change in the future and use a different "workspace"
-        # depending on the user/token that is creating the Artifact.
-        # Currently, it uses the DEFAULT_WORKSPACE_NAME
-        if "workspace" not in data:
-            data["workspace"] = DEFAULT_WORKSPACE_NAME
-
         artifact_deserialized = ArtifactSerializer(data=data)
-        if not artifact_deserialized.is_valid():
-            return ProblemResponse(
-                "Cannot deserialize artifact",
-                validation_errors=artifact_deserialized.errors,
-            )
+        artifact_deserialized.is_valid(raise_exception=True)
 
-        workspace_name = artifact_deserialized["workspace"].value
-        try:
-            workspace = Workspace.objects.get(name=workspace_name)
-        except Workspace.DoesNotExist:
-            return ProblemResponse(
-                f'Workspace "{workspace_name}" cannot be found'
-            )
+        workspace = artifact_deserialized.validated_data["workspace"]
+        self.set_current_workspace(workspace)
+        self.enforce(workspace.can_create_artifacts)
 
         work_request_id: int | None = artifact_deserialized.validated_data.get(
             "work_request"
         )
+        work_request: WorkRequest | None = None
         created_by: User | None = None
 
         if work_request_id is not None:
-            if not IsWorkerAuthenticated().has_permission(request, self):
-                return ProblemResponse(
-                    "work_request field can only be used by a request with a "
-                    "token from a Worker",
+            if not context.worker_token:
+                raise DebusineAPIException(
+                    title=(
+                        "work_request field can only be used by a request with "
+                        "a token from a Worker"
+                    ),
                     status_code=status.HTTP_403_FORBIDDEN,
                 )
 
             try:
-                WorkRequest.objects.get(id=work_request_id)
+                work_request = WorkRequest.objects.get(id=work_request_id)
             except WorkRequest.DoesNotExist:
-                return ProblemResponse(
-                    f"WorkRequest {work_request_id} does not exist"
+                raise DebusineAPIException(
+                    title=f"WorkRequest {work_request_id} does not exist"
                 )
+        elif context.user is not None and context.user.is_authenticated:
+            created_by = context.user
         else:
-            if not IsTokenUserAuthenticated().has_permission(request, self):
-                return ProblemResponse(
+            raise DebusineAPIException(
+                title=(
                     "If work_request field is empty the WorkRequest must be "
-                    "created using a Token associated to a user",
-                    status_code=status.HTTP_403_FORBIDDEN,
-                )
-            token_key = request.headers.get("token")
-            # The user is authenticated, so there must be a token
-            assert token_key is not None
-            created_by = Token.objects.get(
-                hash=Token._generate_hash(token_key)
-            ).user
+                    "created using a Token associated to a user"
+                ),
+                status_code=status.HTTP_403_FORBIDDEN,
+            )
 
         artifact = Artifact.objects.create(
             category=artifact_deserialized["category"].value,
             workspace=workspace,
             data=artifact_deserialized["data"].value,
-            created_by_work_request_id=work_request_id,
+            created_by_work_request=work_request,
             expiration_delay=artifact_deserialized["expiration_delay"].value,
             created_by=created_by,
         )
@@ -207,21 +205,28 @@ class FileUploadParser(BaseParser):
     media_type = "*/*"
 
     @staticmethod
-    def _raise_error_if_gap_in_file(content_range, temporary_file_path):
-        if content_range is None:
-            return
-
+    def _raise_error_if_gap_in_file(
+        range_start: int, temporary_file_path: Path
+    ) -> None:
         uploaded_size = Path(temporary_file_path).stat().st_size
-        if uploaded_size < content_range["start"]:
+        if uploaded_size < range_start:
             raise ValueError(
                 f"Server had received {uploaded_size} bytes of the file. "
-                f"New upload starts at {content_range['start']}. "
+                f"New upload starts at {range_start}. "
                 f"Please continue from {uploaded_size}"
             )
 
-    def parse(self, stream, media_type=None, parser_context=None):
+    # TODO: Can we avoid overriding the return type of BaseParser.parse?  It
+    # doesn't seem to cause a practical problem, but is a bit odd.
+    def parse(  # type: ignore[override]
+        self,
+        stream: IO[Any],
+        media_type: str | None = None,
+        parser_context: Mapping[str, Any] | None = None,
+    ) -> FileUpload:
         """Upload a file to an artifact."""
         media_type  # fake usage for vulture
+        assert parser_context is not None
         file_in_artifact = parser_context["request"].file_in_artifact
 
         range_start = 0
@@ -232,6 +237,9 @@ class FileUploadParser(BaseParser):
         )
 
         if content_range is not None:
+            # Checked by UploadFileView.put.
+            assert content_range["start"] != "*"
+
             range_start = content_range["start"]
             range_end = content_range["end"]
 
@@ -241,12 +249,13 @@ class FileUploadParser(BaseParser):
                 f"file size ({file_size})"
             )
 
+        uploads_file: IO[bytes]
         try:
             file_upload = FileUpload.objects.get(
                 file_in_artifact=file_in_artifact
             )
             temporary_file_path = file_upload.absolute_file_path()
-            self._raise_error_if_gap_in_file(content_range, temporary_file_path)
+            self._raise_error_if_gap_in_file(range_start, temporary_file_path)
             uploads_file = open(temporary_file_path, "r+b")
         except FileUpload.DoesNotExist:
             uploads_file = tempfile.NamedTemporaryFile(
@@ -295,18 +304,23 @@ class FileUploadParser(BaseParser):
         return file_upload
 
 
-class UploadFileView(views.APIView):
+class UploadFileView(BaseAPIView):
     """View to upload files."""
 
     parser_classes = [FileUploadParser]
+    # TODO: This should be replaced by appropriate debusine permissions.
     permission_classes = [IsTokenAuthenticated]
 
     @staticmethod
-    def _range_header(start, end):
+    def _range_header(start: int, end: int) -> dict[str, str]:
         return {"Range": f"bytes={start}-{end}"}
 
     @classmethod
-    def _file_status_response(cls, file_in_artifact, range_file_size):
+    def _file_status_response(
+        cls,
+        file_in_artifact: FileInArtifact,
+        range_file_size: int | Literal["*"],
+    ) -> Response:
         """
         Return response with the status of a file (completed or missing range).
 
@@ -327,10 +341,12 @@ class UploadFileView(views.APIView):
         if range_file_size != "*" and range_file_size != file_in_db_size:
             # Client reported a size in Content-Range that is not
             # what was expected
-            return ProblemResponse(
-                "Invalid file size in Content-Range header. "
-                f"Expected {file_in_db_size} "
-                f"received {range_file_size}"
+            raise DebusineAPIException(
+                title=(
+                    "Invalid file size in Content-Range header. "
+                    f"Expected {file_in_db_size} "
+                    f"received {range_file_size}"
+                )
             )
 
         current_size = FileUpload.current_size(
@@ -349,29 +365,48 @@ class UploadFileView(views.APIView):
                 headers=cls._range_header(0, current_size),
             )
 
-    @transaction.atomic
-    def put(self, request, artifact_id: int, file_path: str):
-        """Receive file to be uploaded or request uploaded content-range."""
+    def _get_file_in_artifact(
+        self, artifact_id: int, file_path: str
+    ) -> FileInArtifact:
+        """Get file to be uploaded, or raise DebusineAPIException."""
         try:
-            file_in_artifact = _get_file_in_artifact_or_raise_value_error(
-                artifact_id,
-                file_path,
+            artifact = (
+                Artifact.objects.in_current_scope()
+                .can_display(context.require_user())
+                .get(id=artifact_id)
             )
-        except ValueError as exc:
-            return ProblemResponse(
-                str(exc), status_code=status.HTTP_404_NOT_FOUND
+        except Artifact.DoesNotExist:
+            raise DebusineAPIException(
+                title=f"Artifact {artifact_id} does not exist",
+                status_code=status.HTTP_404_NOT_FOUND,
             )
+        assert isinstance(artifact, Artifact)
+
+        try:
+            return FileInArtifact.objects.get(artifact=artifact, path=file_path)
+        except FileInArtifact.DoesNotExist:
+            raise DebusineAPIException(
+                title=f'No file_path "{file_path}" for artifact {artifact.id}',
+                status_code=status.HTTP_404_NOT_FOUND,
+            )
+
+    def put(
+        self, request: Request, artifact_id: int, file_path: str
+    ) -> Response:
+        """Receive file to be uploaded or request uploaded content-range."""
+        file_in_artifact = self._get_file_in_artifact(artifact_id, file_path)
 
         try:
             content_range = parse_content_range_header(request.headers)
         except ValueError as exc:
-            return ProblemResponse(str(exc))
+            raise DebusineAPIException(title=str(exc))
 
         artifact = file_in_artifact.artifact
 
         if content_range is not None and content_range["start"] == "*":
             # Client would like to know the status of a file
             # (which parts have been uploaded)
+            assert content_range["size"] is not None
             return self._file_status_response(
                 file_in_artifact, content_range["size"]
             )
@@ -380,12 +415,13 @@ class UploadFileView(views.APIView):
             # File was already uploaded
             return Response(status=status.HTTP_201_CREATED)
 
-        request.file_in_artifact = file_in_artifact
+        setattr(request, "file_in_artifact", file_in_artifact)
         # request.data use the FileUploadParser
         try:
-            file_upload: FileUpload = request.data
+            file_upload = request.data
         except ValueError as value_error:
-            return ProblemResponse(str(value_error))
+            raise DebusineAPIException(title=str(value_error))
+        assert isinstance(file_upload, FileUpload)
 
         current_size = FileUpload.current_size(
             artifact=artifact, path_in_artifact=file_path
@@ -400,12 +436,13 @@ class UploadFileView(views.APIView):
             # File is uploaded: create the file in the storage
             # Will return HTTP 201 Created (or HTTP 409 Conflict if the
             # expected hash does not match, the client will need to re-upload)
-            response = self.create_file_in_storage(file_upload)
-            file_upload.delete()
-            return response
+            try:
+                return self.create_file_in_storage(file_upload)
+            finally:
+                file_upload.delete()
 
     @staticmethod
-    def create_file_in_storage(file_upload: FileUpload):
+    def create_file_in_storage(file_upload: FileUpload) -> Response:
         """
         Add file_upload to the artifact's default store.
 
@@ -421,10 +458,12 @@ class UploadFileView(views.APIView):
 
         expected_file_hash_digest = file_in_artifact.file.hash_digest
         if file_in_artifact.file.hash_digest.hex() != hash_digest.hex():
-            return ProblemResponse(
-                "Invalid file hash. Expected: "
-                f"{expected_file_hash_digest.hex()} "
-                f"Received: {hash_digest.hex()}",
+            raise DebusineAPIException(
+                title=(
+                    "Invalid file hash. Expected: "
+                    f"{expected_file_hash_digest.hex()} "
+                    f"Received: {hash_digest.hex()}"
+                ),
                 status_code=status.HTTP_409_CONFLICT,
             )
 
@@ -438,35 +477,22 @@ class UploadFileView(views.APIView):
         return Response(status=status.HTTP_201_CREATED)
 
 
-def _get_file_in_artifact_or_raise_value_error(
-    artifact_id: int, file_path: str
-) -> FileInArtifact:
-    try:
-        artifact = Artifact.objects.get(id=artifact_id)
-    except Artifact.DoesNotExist:
-        raise ValueError(
-            f"Artifact {artifact_id} does not exist",
-        )
-
-    try:
-        file_in_artifact = FileInArtifact.objects.get(
-            artifact=artifact, path=file_path
-        )
-    except FileInArtifact.DoesNotExist:
-        raise ValueError(
-            f'No file_path "{file_path}" for artifact {artifact.id}'
-        )
-
-    return file_in_artifact
-
-
 T = TypeVar("T", bound=Model, covariant=True)
 
 
-class GetOrCreateAPIView(CreateAPIViewBase[T], Generic[T]):
+class GetOrCreateAPIView(CreateAPIViewBase[T], BaseAPIView, Generic[T]):
     """Class to create a new object (return 201 created) or return 200."""
 
-    def create(self, request, *args, **kwargs):
+    def get_object(self) -> T:
+        """Override to return more API-friendly errors."""
+        try:
+            return super().get_object()
+        except Http404 as exc:
+            raise DebusineAPIException(
+                title=str(exc), status_code=status.HTTP_404_NOT_FOUND
+            )
+
+    def create(self, request: Request, *args: Any, **kwargs: Any) -> Response:
         """
         Return the object with HTTP 201 (created) or HTTP 200 (already existed).
 
@@ -480,13 +506,15 @@ class GetOrCreateAPIView(CreateAPIViewBase[T], Generic[T]):
         serializer = self.get_serializer(data=request.data)
         serializer.is_valid(raise_exception=True)
 
+        response_status: int
         try:
-            obj = serializer.Meta.model.objects.get(**serializer.validated_data)
+            obj = self.get_queryset().get(**serializer.validated_data)
             response_status = status.HTTP_200_OK
             pk = obj.pk
-        except serializer.Meta.model.DoesNotExist:
+        except ObjectDoesNotExist:
             self.perform_create(serializer)
             response_status = status.HTTP_201_CREATED
+            assert serializer.instance is not None
             pk = serializer.instance.pk
 
         headers = self.get_success_headers(serializer.data)
@@ -501,37 +529,58 @@ class ArtifactRelationsView(
     GetOrCreateAPIView[ArtifactRelation],
     DestroyAPIViewBase[ArtifactRelation],
     ListAPIViewBase[ArtifactRelation],
+    BaseAPIView,
 ):
     """Return, create and delete relations between artifacts."""
 
     serializer_class = ArtifactRelationResponseSerializer
+    filter_backends = [CanDisplayFilterBackend]
     parser_classes = [JSONParser]
     pagination_class = None
-    queryset = ArtifactRelation.objects
 
-    def get(self, request, *args, **kwargs):
-        """Return list of ArtifactRelations for artifact or target_artifact."""
-        artifact_id = request.GET.get("artifact", None)
-        target_artifact_id = request.GET.get("target_artifact", None)
+    def get_queryset(self) -> ArtifactRelationQuerySet[Any]:
+        """Get the query set for this view."""
+        queryset = ArtifactRelation.objects.in_current_scope()
+        filter_kwargs = {}
 
-        if artifact_id is not None:
-            filter_kwargs = {"artifact_id": artifact_id}
-            required_artifact_id = artifact_id
-        elif target_artifact_id is not None:
-            filter_kwargs = {"target_id": target_artifact_id}
-            required_artifact_id = target_artifact_id
-        else:
-            return ProblemResponse(
-                '"artifact" or "target_artifact" parameter are mandatory'
-            )
-        try:
-            Artifact.objects.get(id=required_artifact_id)
-        except Artifact.DoesNotExist:
-            return ProblemResponse(
-                f"Artifact {required_artifact_id} not found",
-                status_code=status.HTTP_404_NOT_FOUND,
-            )
+        if self.request.method == "GET":
+            artifact_id = self.request.GET.get("artifact", None)
+            target_artifact_id = self.request.GET.get("target_artifact", None)
 
-        self.queryset = ArtifactRelation.objects.filter(**filter_kwargs)
+            if artifact_id is not None:
+                filter_kwargs = {"artifact_id": artifact_id}
+                required_artifact_id = artifact_id
+            elif target_artifact_id is not None:
+                filter_kwargs = {"target_id": target_artifact_id}
+                required_artifact_id = target_artifact_id
+            else:
+                raise DebusineAPIException(
+                    title=(
+                        '"artifact" or "target_artifact" parameter are '
+                        'mandatory'
+                    )
+                )
+            try:
+                Artifact.objects.in_current_scope().can_display(
+                    context.require_user()
+                ).get(id=required_artifact_id)
+            except Artifact.DoesNotExist:
+                raise DebusineAPIException(
+                    title=f"Artifact {required_artifact_id} not found",
+                    status_code=status.HTTP_404_NOT_FOUND,
+                )
 
-        return super().get(request, *args, **kwargs)
+        return queryset.filter(**filter_kwargs)
+
+    def perform_create(
+        self, serializer: BaseSerializer[ArtifactRelation]
+    ) -> None:
+        """Create an artifact relation."""
+        artifact = serializer.validated_data["artifact"]
+        target = serializer.validated_data["target"]
+        self.set_current_workspace(artifact.workspace)
+        # TODO: This needs to be based on some kind of write permission
+        # rather than on can_display.
+        self.enforce(artifact.can_display)
+        self.enforce(target.can_display)
+        super().perform_create(serializer)
