@@ -8,12 +8,24 @@
 # contained in the LICENSE file.
 
 """Views for the server application: base infrastructure."""
-from typing import TYPE_CHECKING
+import logging
+from collections.abc import Callable, Iterable, Sequence
+from typing import (
+    Any,
+    Protocol,
+    Self,
+    TYPE_CHECKING,
+    TypeVar,
+    runtime_checkable,
+)
 
 from django.core.exceptions import PermissionDenied
-from django.http import Http404, JsonResponse
+from django.db.models import Model, QuerySet
+from django.http import HttpRequest, HttpResponse, JsonResponse
+from django.views import View
 from rest_framework import status
 from rest_framework.exceptions import PermissionDenied as DRFPermissionDenied
+from rest_framework.filters import BaseFilterBackend
 from rest_framework.generics import (
     CreateAPIView,
     DestroyAPIView,
@@ -23,10 +35,25 @@ from rest_framework.generics import (
     UpdateAPIView,
 )
 from rest_framework.permissions import BasePermission, DjangoModelPermissions
+from rest_framework.request import Request
+from rest_framework.response import Response
+from rest_framework.views import APIView
 
-from debusine.db.models import Artifact, Token
+from debusine.db.context import ContextConsistencyError, context
+from debusine.db.models import Token, Workspace
+from debusine.db.models.permissions import (
+    PermissionUser,
+    format_permission_check_error,
+)
+from debusine.server.exceptions import (
+    DebusineAPIException,
+    raise_workspace_not_found,
+)
 
 if TYPE_CHECKING:
+    from rest_framework.permissions import _PermissionClass
+
+    _PermissionClass  # fake usage for vulture
     CreateAPIViewBase = CreateAPIView
     DestroyAPIViewBase = DestroyAPIView
     GenericAPIViewBase = GenericAPIView
@@ -77,11 +104,11 @@ class ProblemResponse(JsonResponse):
 
     def __init__(
         self,
-        title,
-        detail=None,
-        validation_errors=None,
-        status_code=status.HTTP_400_BAD_REQUEST,
-    ):
+        title: str,
+        detail: str | None = None,
+        validation_errors: Iterable[Any] | None = None,
+        status_code: int = status.HTTP_400_BAD_REQUEST,
+    ) -> None:
         """
         Initialize object.
 
@@ -90,7 +117,7 @@ class ProblemResponse(JsonResponse):
         :param validation_errors: if not None, included in the response data.
         :param status_code: HTTP status code for the response.
         """
-        data = {"title": title}
+        data: dict[str, Any] = {"title": title}
 
         if detail is not None:
             data["detail"] = detail
@@ -106,23 +133,24 @@ class ProblemResponse(JsonResponse):
 class IsTokenAuthenticated(BasePermission):
     """Allow access to requests with a valid token."""
 
-    @staticmethod
-    def _enabled_token(token: Token | None) -> bool:
-        return token is not None and token.enabled
-
-    def has_permission(self, request, view):  # noqa: U100
+    def has_permission(
+        self, request: Request, view: APIView  # noqa: U100
+    ) -> bool:
         """Return True if the request is authenticated with a Token."""
-        return self._enabled_token(request.token)
+        return isinstance(request.auth, Token) and request.auth.enabled
 
 
 class IsTokenUserAuthenticated(IsTokenAuthenticated):
     """Allow access if the request has an enabled Token with associated user."""
 
-    def has_permission(self, request, view):  # noqa: U100
+    def has_permission(
+        self, request: Request, view: APIView  # noqa: U100
+    ) -> bool:
         """Return True if valid token has a User assigned."""
         return (
-            self._enabled_token(request.token)
-            and request.token.user is not None
+            isinstance(request.auth, Token)
+            and request.auth.enabled
+            and request.auth.user is not None
         )
 
 
@@ -134,13 +162,14 @@ class IsTokenUserAuthenticatedDjangoModelPermissions(DjangoModelPermissions):
     checks Token authentication.
     """
 
-    def has_permission(self, request, view):
+    def has_permission(self, request: Request, view: APIView) -> bool:
         """Return True if the user has the right permissions."""
-        token = request.token
+        token = request.auth
         if not token or not token.user:
             return False
 
         queryset = self._queryset(view)
+        assert request.method is not None
         perms = self.get_required_permissions(request.method, queryset.model)
 
         if not token.user.has_perms(perms):
@@ -155,7 +184,9 @@ class IsTokenUserAuthenticatedDjangoModelPermissions(DjangoModelPermissions):
 class IsUserAuthenticated(BasePermission):
     """Allow access to requests with an authenticated user."""
 
-    def has_permission(self, request, view):  # noqa: U100
+    def has_permission(
+        self, request: Request, view: APIView  # noqa: U100
+    ) -> bool:
         """Return True if the request has an authenticated user."""
         return request.user.is_authenticated
 
@@ -163,7 +194,9 @@ class IsUserAuthenticated(BasePermission):
 class IsWorkerAuthenticated(IsTokenAuthenticated):
     """Allow access to requests with a token assigned to a worker."""
 
-    def has_permission(self, request, view):  # noqa: U100
+    def has_permission(
+        self, request: Request, view: APIView  # noqa: U100
+    ) -> bool:
         """
         Return True if the request is an authenticated worker.
 
@@ -173,8 +206,8 @@ class IsWorkerAuthenticated(IsTokenAuthenticated):
             # No token authenticated: no Worker Authenticated
             return False
 
-        if not hasattr(request.token, "worker"):
-            # request.token is None; or it's a Token without a "worker"
+        if not hasattr(request.auth, "worker"):
+            # request.auth is None; or it's a Token without a "worker"
             return False
 
         return True
@@ -183,14 +216,16 @@ class IsWorkerAuthenticated(IsTokenAuthenticated):
 class IsGet(BasePermission):
     """Allow access if the request's method is GET."""
 
-    def has_permission(self, request, view):  # noqa: U100
+    def has_permission(
+        self, request: Request, view: APIView  # noqa: U100
+    ) -> bool:
         """Return True if request.method == "GET"."""
         return request.method == "GET"
 
 
 class ValidatePermissionsMixin:
     """
-    Mixin implementing check_permissions() for the Views.
+    Mixin implementing _check_permissions() for the Views.
 
     To use it:
         class YourView(ValidatePermissionsMixin, View):
@@ -201,7 +236,10 @@ class ValidatePermissionsMixin:
     proceed.
     """
 
-    def check_permissions(self):
+    permission_classes: Sequence["_PermissionClass"]
+    permission_denied_message: str | None = None
+
+    def _check_permissions(self, request: HttpRequest) -> None:
         """
         Raise PermissionDenied() if one permission does not allow access.
 
@@ -212,31 +250,100 @@ class ValidatePermissionsMixin:
         """
         for permission_class in self.permission_classes:
             permission = permission_class()
-            if not permission.has_permission(self.request, self):
-                message = getattr(self, "permission_denied_message", None)
-                raise PermissionDenied(message)
+            # TODO: This arg-type ignore is cheating slightly, because we
+            # also use this mixin in a few places under debusine.web where
+            # it's mixed in with View rather than APIView.  However, it
+            # isn't obvious how to fix that, and it doesn't cause a
+            # practical problem right now.
+            if not permission.has_permission(
+                request, self  # type: ignore[arg-type]
+            ):
+                raise PermissionDenied(self.permission_denied_message)
 
-    def dispatch(self, request, *args, **kwargs):
+    def dispatch(
+        self, request: HttpRequest, *args: Any, **kwargs: Any
+    ) -> HttpResponse:
         """Check permissions for any request method (GET, POST, PUT, etc.)."""
-        self.check_permissions()
+        self._check_permissions(request)
 
-        return super().dispatch(request, *args, **kwargs)
+        assert isinstance(self, View)
+        # The mixin confuses mypy, but it works as long as it's mixed into a
+        # View.
+        return super().dispatch(request, *args, **kwargs)  # type: ignore
 
 
-class ArtifactInPublicWorkspace(BasePermission):
-    """Allows access for Artifacts with public workspace."""
+class BaseAPIView(APIView):
+    """Common base for API views."""
 
-    def has_permission(self, request, view):  # noqa: U100
-        """
-        Return True if request's artifact's workspace is public.
+    def enforce(self, predicate: Callable[[PermissionUser], bool]) -> None:
+        """Enforce a permission predicate."""
+        if predicate(context.user):
+            return
 
-        :raise Http404: if the artifact_id does not exist.
-        """
-        artifact_id = request.resolver_match.kwargs.get("artifact_id")
+        raise DebusineAPIException(
+            title=format_permission_check_error(predicate, context.user),
+            status_code=status.HTTP_403_FORBIDDEN,
+        )
 
+    def set_current_workspace(
+        self,
+        workspace: str | Workspace,
+    ) -> None:
+        """Set the current workspace in context."""
+        if isinstance(workspace, str):
+            try:
+                workspace = Workspace.objects.get_for_context(name=workspace)
+            except Workspace.DoesNotExist:
+                raise_workspace_not_found(workspace)
         try:
-            artifact = Artifact.objects.get(id=artifact_id)
-        except Artifact.DoesNotExist:
-            raise Http404(f"Artifact {artifact_id} does not exist")
+            workspace.set_current()
+        except ContextConsistencyError:
+            # Turn exception in a 404 response.
+            # 404 is used instead of 403 as an attempt to prevent leaking which
+            # private workspace exists that the user cannot see
+            logging.debug("permission denied on %s reported as 404", workspace)
+            raise_workspace_not_found(workspace)
 
-        return artifact.workspace.public
+
+_M = TypeVar("_M", bound=Model, covariant=True)
+
+
+@runtime_checkable
+class SupportsCanDisplay(Protocol[_M]):
+    """A query set that supports `can_display` filtering."""
+
+    def can_display(self, user: PermissionUser) -> Self:
+        """Keep only objects that can be displayed."""
+
+
+class CanDisplayFilterBackend(BaseFilterBackend):
+    """Filter that only allows objects that can be displayed."""
+
+    def filter_queryset(
+        self,
+        request: Request,  # noqa: U100
+        queryset: QuerySet[_M],
+        view: APIView,  # noqa: U100
+    ) -> QuerySet[_M]:
+        """Keep only objects that can be displayed."""
+        assert isinstance(queryset, SupportsCanDisplay)
+        return queryset.can_display(context.require_user())
+
+
+class Whoami(BaseAPIView):
+    """Simple view that returns the authentication status."""
+
+    def get(self, request: Request) -> Response:
+        """Return information about the current user."""
+        user: str
+        if request.user.is_authenticated:
+            user = request.user.username
+        else:
+            user = "<anonymous>"
+        return Response(
+            {
+                "user": user,
+                "auth": bool(request.auth),
+                "worker_token": bool(context.worker_token),
+            }
+        )
