@@ -31,12 +31,16 @@ from rest_framework.views import APIView
 from debusine.db.models import (
     Artifact,
     ArtifactRelation,
-    DEFAULT_WORKSPACE_NAME,
     NotificationChannel,
     TaskDatabase,
     WorkRequest,
 )
-from debusine.db.models.work_requests import CannotRetry, CannotUnblock
+from debusine.db.models.work_requests import (
+    CannotRetry,
+    CannotUnblock,
+    WorkRequestQuerySet,
+)
+from debusine.server.exceptions import DebusineAPIException
 from debusine.server.serializers import (
     WorkRequestExternalDebsignRequestSerializer,
     WorkRequestExternalDebsignSerializer,
@@ -46,13 +50,13 @@ from debusine.server.serializers import (
 )
 from debusine.server.tasks.wait.models import ExternalDebsignDynamicData
 from debusine.server.views.base import (
+    BaseAPIView,
+    CanDisplayFilterBackend,
     GenericAPIViewBase,
     IsGet,
     IsTokenAuthenticated,
     IsTokenUserAuthenticated,
-    IsTokenUserAuthenticatedDjangoModelPermissions,
     ListAPIViewBase,
-    ProblemResponse,
     RetrieveAPIViewBase,
     UpdateAPIViewBase,
 )
@@ -69,7 +73,7 @@ class WorkRequestPagination(CursorPagination):
 
 
 class WorkRequestView(
-    ListAPIViewBase[WorkRequest], UpdateAPIViewBase[WorkRequest]
+    ListAPIViewBase[WorkRequest], UpdateAPIViewBase[WorkRequest], BaseAPIView
 ):
     """View used by the debusine client to get information of a WorkRequest."""
 
@@ -77,38 +81,38 @@ class WorkRequestView(
         IsTokenUserAuthenticated | IsTokenAuthenticated & IsGet
     ]
     serializer_class = WorkRequestSerializer
+    filter_backends = [CanDisplayFilterBackend]
     pagination_class = WorkRequestPagination
-    queryset = WorkRequest.objects.all()
     lookup_url_kwarg = "work_request_id"
 
-    def get(self, request, work_request_id: int | None = None):
+    def get_queryset(self) -> WorkRequestQuerySet[Any]:
+        """Get the query set for this view."""
+        return WorkRequest.objects.in_current_scope()
+
+    def get_object(self) -> WorkRequest:
+        """Override to return more API-friendly errors."""
+        try:
+            return super().get_object()
+        except Http404 as exc:
+            raise NotFound(str(exc))
+
+    def get(
+        self, request: Request, work_request_id: int | None = None
+    ) -> Response:
         """Return status information for WorkRequest or not found."""
         if work_request_id is None:
             return super().get(request)
 
-        try:
-            work_request = WorkRequest.objects.get(pk=work_request_id)
-        except WorkRequest.DoesNotExist:
-            return Response(
-                {'detail': 'Work request id not found'},
-                status=status.HTTP_404_NOT_FOUND,
-            )
-
         return Response(
-            WorkRequestSerializer(work_request).data, status=status.HTTP_200_OK
+            WorkRequestSerializer(self.get_object()).data,
+            status=status.HTTP_200_OK,
         )
 
-    @transaction.atomic
-    def post(self, request):
+    def post(self, request: Request) -> Response:
         """Create a new work request."""
         data = request.data.copy()
 
-        # "workspace" might not be in data, but it is a mandatory field
-        # in the serializer. If not sent by the client use the default workspace
-        if "workspace" not in data:
-            data["workspace"] = DEFAULT_WORKSPACE_NAME
-
-        data["created_by"] = request.token.user.id
+        data["created_by"] = request.auth.user.id
         work_request_deserialized = WorkRequestSerializer(
             data=data,
             only_fields=[
@@ -120,17 +124,21 @@ class WorkRequestView(
             ],
         )
 
-        if not work_request_deserialized.is_valid():
-            errors = work_request_deserialized.errors
-
+        try:
+            work_request_deserialized.is_valid(raise_exception=True)
+        except DebusineAPIException as e:
             logger.debug(
-                "Error creating work request. Could not be deserialized: %s",
-                errors,
+                "Error creating work request. Could not be deserialized: "
+                "%s (detail: %s, validation errors: %s)",
+                e.debusine_title,
+                e.debusine_detail,
+                e.debusine_validation_errors,
             )
+            raise
 
-            return ProblemResponse(
-                "Cannot deserialize work request", validation_errors=errors
-            )
+        workspace = work_request_deserialized.validated_data["workspace"]
+        self.set_current_workspace(workspace)
+        self.enforce(workspace.can_create_work_requests)
 
         # only accept send-notification actions
         for event, reactions in work_request_deserialized.validated_data.get(
@@ -138,8 +146,8 @@ class WorkRequestView(
         ).items():
             for action in reactions:
                 if action["action"] != ActionTypes.SEND_NOTIFICATION:
-                    return ProblemResponse(
-                        "Invalid event_reactions",
+                    raise DebusineAPIException(
+                        title="Invalid event_reactions",
                         detail=(
                             f"Action type {action['action']!r} is not allowed "
                             f"here"
@@ -147,14 +155,14 @@ class WorkRequestView(
                     )
 
         if len(non_existing_channels := self._non_existing_channels(data)) > 0:
-            return ProblemResponse(
-                f"Non-existing channels: {sorted(non_existing_channels)}"
+            raise DebusineAPIException(
+                title=f"Non-existing channels: {sorted(non_existing_channels)}"
             )
 
         task_name = data['task_name']
 
         task_types = [TaskTypes.WORKER]
-        if request.token.user.is_superuser:
+        if request.auth.user.is_superuser:
             task_types.append(TaskTypes.SERVER)
 
         for task_type in task_types:
@@ -173,10 +181,12 @@ class WorkRequestView(
                 "Error creating work request: task name is not registered: %s",
                 task_name,
             )
-            return ProblemResponse(
-                "Cannot create work request: task name is not registered",
-                detail=f'Task name: "{task_name}". '
-                f"Registered task names: {valid_task_names}",
+            raise DebusineAPIException(
+                title="Cannot create work request: task name is not registered",
+                detail=(
+                    f'Task name: "{task_name}". '
+                    f"Registered task names: {valid_task_names}"
+                ),
             )
 
         # Return an error if compute_dynamic_data() raise an exception
@@ -191,8 +201,10 @@ class WorkRequestView(
             w.get_task().compute_dynamic_data(TaskDatabase(w))
         except Exception as exc:
             transaction.set_rollback(True)
-            return ProblemResponse(
-                "Cannot create work request: error computing dynamic data",
+            raise DebusineAPIException(
+                title=(
+                    "Cannot create work request: error computing dynamic data"
+                ),
                 detail=f"Task data: {w.task_data} Error: {exc}",
             )
 
@@ -220,11 +232,7 @@ class WorkRequestView(
 
         return non_existing_channels
 
-    # TODO: We can't declare the return type properly here because the
-    # return type of UpdateAPIView.patch is rest_framework.response.Response
-    # and ProblemResponse isn't a subtype of that.  It works anyway, but
-    # needs to be fixed somehow.
-    def patch(self, request: Request, *args: Any, **kwargs: Any):
+    def patch(self, request: Request, *args: Any, **kwargs: Any) -> Response:
         """Update properties of a work request."""
         # Only a subset of properties may be patched.
         work_request_deserialized = WorkRequestUpdateSerializer(
@@ -240,7 +248,7 @@ class WorkRequestView(
         if priority_adjustment is None:
             pass
         else:
-            token = request.token
+            token = request.auth
             # permission_classes is declared such that we won't get this far
             # unless the request has an enabled token with an associated
             # user.
@@ -257,36 +265,41 @@ class WorkRequestView(
             ):
                 pass
             else:
-                return ProblemResponse(
-                    "You are not permitted to set priority adjustments",
+                raise DebusineAPIException(
+                    title="You are not permitted to set priority adjustments",
                     status_code=status.HTTP_403_FORBIDDEN,
                 )
 
         return super().patch(request, *args, **kwargs)
 
 
-class WorkRequestRetryView(APIView):
+class WorkRequestRetryView(GenericAPIViewBase[WorkRequest], BaseAPIView):
     """View used by the debusine client to get information of a WorkRequest."""
 
     permission_classes = [IsTokenUserAuthenticated]
+    filter_backends = [CanDisplayFilterBackend]
+    lookup_url_kwarg = "work_request_id"
 
-    def post(self, *args, **kwargs):
-        """Retry a work request."""
+    def get_queryset(self) -> WorkRequestQuerySet[Any]:
+        """Get the query set for this view."""
+        return WorkRequest.objects.in_current_scope()
+
+    def get_object(self) -> WorkRequest:
+        """Override to return more API-friendly errors."""
         try:
-            work_request = WorkRequest.objects.get(
-                pk=self.kwargs["work_request_id"]
-            )
-        except WorkRequest.DoesNotExist:
-            return Response(
-                {'detail': 'Work request id not found'},
-                status=status.HTTP_404_NOT_FOUND,
-            )
+            return super().get_object()
+        except Http404 as exc:
+            raise NotFound(str(exc))
+
+    def post(self, *args: Any, **kwargs: Any) -> Response:
+        """Retry a work request."""
+        work_request = self.get_object()
 
         try:
             new_work_request = work_request.retry()
         except CannotRetry as e:
-            return ProblemResponse(
-                "Cannot retry work request",
+            raise DebusineAPIException(
+                title="Cannot retry work request",
                 status_code=status.HTTP_400_BAD_REQUEST,
                 detail=str(e),
             )
@@ -297,32 +310,18 @@ class WorkRequestRetryView(APIView):
         )
 
 
-class WorkRequestUnblockViewPermissions(
-    IsTokenUserAuthenticatedDjangoModelPermissions
-):
-    """Custom permissions for :py:class:`WorkRequestUnblockView`."""
-
-    perms_map = {
-        **IsTokenUserAuthenticatedDjangoModelPermissions.perms_map,
-        # By default DjangoModelPermissions assumes that POST is used to
-        # create new objects, but in this case we're using it to change an
-        # existing object.
-        "POST": ["%(app_label)s.change_%(model_name)s"],
-    }
-
-
-class WorkRequestUnblockView(GenericAPIViewBase[WorkRequest]):
+class WorkRequestUnblockView(GenericAPIViewBase[WorkRequest], BaseAPIView):
     """View used to unblock a work request awaiting manual approval."""
 
-    # TODO: We should replace this with some finer-grained ACL once we have
-    # that sort of thing.  It would be useful to grant the ability to edit
-    # manual unblocks without being able to freely edit work requests, or
-    # the ability to edit manual unblocks only in certain contexts.
-    permission_classes = [WorkRequestUnblockViewPermissions]
+    permission_classes = [IsTokenUserAuthenticated]
     serializer_class = WorkRequestUnblockSerializer
-    queryset = WorkRequest.objects
+    filter_backends = [CanDisplayFilterBackend]
     lookup_url_kwarg = "work_request_id"
 
+    def get_queryset(self) -> WorkRequestQuerySet[Any]:
+        """Get the query set for this view."""
+        return WorkRequest.objects.in_current_scope()
+
     def get_object(self) -> WorkRequest:
         """Override to return more API-friendly errors."""
         try:
@@ -330,17 +329,17 @@ class WorkRequestUnblockView(GenericAPIViewBase[WorkRequest]):
         except Http404 as exc:
             raise NotFound(str(exc))
 
-    def post(
-        self, request: Request, *args: Any, **kwargs: Any
-    ) -> HttpResponseBase:
+    def post(self, request: Request, *args: Any, **kwargs: Any) -> Response:
         """Edit a manual unblock."""
         work_request = self.get_object()
+        self.set_current_workspace(work_request.workspace)
+        self.enforce(work_request.can_unblock)
         serializer = self.get_serializer(data=request.data)
         serializer.is_valid(raise_exception=True)
         notes = serializer.validated_data.get("notes")
         action = serializer.validated_data.get("action")
 
-        token = self.request.token
+        token = self.request.auth
         # permission_classes is declared such that we won't get this far
         # unless the request has an enabled token with an associated user.
         assert token is not None
@@ -351,7 +350,7 @@ class WorkRequestUnblockView(GenericAPIViewBase[WorkRequest]):
                 user=token.user, notes=notes, action=action
             )
         except CannotUnblock as e:
-            return ProblemResponse(str(e))
+            raise DebusineAPIException(title=str(e))
 
         return Response(
             WorkRequestSerializer(work_request).data, status=status.HTTP_200_OK
@@ -365,10 +364,7 @@ class IsWorkRequestCreator(BasePermission):
         self, request: Request, view: APIView, obj: Any  # noqa: U100
     ) -> bool:
         """Return True if the requester created the work request."""
-        if (
-            isinstance(obj, WorkRequest)
-            and request.token.user != obj.created_by
-        ):
+        if isinstance(obj, WorkRequest) and request.auth.user != obj.created_by:
             # Raise an explicit exception in this case for the sake of a
             # better error message.
             raise PermissionDenied(
@@ -379,7 +375,9 @@ class IsWorkRequestCreator(BasePermission):
         return True
 
 
-class WorkRequestExternalDebsignView(RetrieveAPIViewBase[WorkRequest]):
+class WorkRequestExternalDebsignView(
+    RetrieveAPIViewBase[WorkRequest], BaseAPIView
+):
     """View used to handle `ExternalDebsign` wait tasks."""
 
     permission_classes = [
@@ -387,9 +385,13 @@ class WorkRequestExternalDebsignView(RetrieveAPIViewBase[WorkRequest]):
         IsGet | IsWorkRequestCreator,
     ]
     serializer_class = WorkRequestExternalDebsignSerializer
-    queryset = WorkRequest.objects
+    filter_backends = [CanDisplayFilterBackend]
     lookup_url_kwarg = "work_request_id"
 
+    def get_queryset(self) -> WorkRequestQuerySet[Any]:
+        """Get the query set for this view."""
+        return WorkRequest.objects.in_current_scope()
+
     def get_object(self) -> WorkRequest:
         """Override to add constraints and return more API-friendly errors."""
         try:
@@ -417,7 +419,6 @@ class WorkRequestExternalDebsignView(RetrieveAPIViewBase[WorkRequest]):
 
         return work_request
 
-    @transaction.atomic
     def post(
         self, request: Request, *args: Any, **kwargs: Any
     ) -> HttpResponseBase:
@@ -480,6 +481,8 @@ class WorkRequestExternalDebsignView(RetrieveAPIViewBase[WorkRequest]):
             target=unsigned,
             type=ArtifactRelation.Relations.RELATES_TO,
         )
+        signed.created_by_work_request = work_request
+        signed.save()
         work_request.mark_completed(WorkRequest.Results.SUCCESS)
 
         return Response(self.get_serializer(work_request).data)
