@@ -8,6 +8,7 @@
 # contained in the LICENSE file.
 """Sbuild Workflow."""
 
+import re
 from functools import cached_property
 
 from debian.debian_support import DpkgArchTable
@@ -17,18 +18,14 @@ from debusine.artifacts.models import (
     ArtifactCategory,
     BareDataCategory,
     CollectionCategory,
+    DebianSourcePackage,
 )
-from debusine.db.models import (
-    Artifact,
-    CollectionItem,
-    TaskDatabase,
-    WorkRequest,
-)
+from debusine.client.models import LookupChildType
+from debusine.db.models import Artifact, TaskDatabase, WorkRequest
 from debusine.server.collections.lookup import lookup_single
 from debusine.server.workflows.base import Workflow, WorkflowValidationError
 from debusine.server.workflows.models import (
     SbuildWorkflowData,
-    SbuildWorkflowDynamicData,
     WorkRequestWorkflowData,
 )
 from debusine.tasks import get_environment
@@ -37,13 +34,14 @@ from debusine.tasks.models import (
     ActionUpdateCollectionWithArtifacts,
     ActionUpdateCollectionWithData,
     BackendType,
+    BaseDynamicTaskData,
     SbuildBuildComponent,
     SbuildData,
 )
 from debusine.tasks.server import TaskDatabaseInterface
 
 
-class SbuildWorkflow(Workflow[SbuildWorkflowData, SbuildWorkflowDynamicData]):
+class SbuildWorkflow(Workflow[SbuildWorkflowData, BaseDynamicTaskData]):
     """Build a source package for all architectures of a target distribution."""
 
     TASK_NAME = "sbuild"
@@ -54,34 +52,14 @@ class SbuildWorkflow(Workflow[SbuildWorkflowData, SbuildWorkflowDynamicData]):
         if self.data.backend == BackendType.AUTO:
             self.data.backend = BackendType.UNSHARE
 
-    def compute_dynamic_data(
-        self, task_database: TaskDatabaseInterface
-    ) -> SbuildWorkflowDynamicData:
-        """Resolve lookups for this workflow."""
-        return SbuildWorkflowDynamicData(
-            input_source_artifact_id=task_database.lookup_single_artifact(
-                self.data.input.source_artifact
-            ),
-            build_logs_collection_id=(
-                None
-                if self.data.build_logs_collection is None
-                else task_database.lookup_single_collection(
-                    self.data.build_logs_collection,
-                    default_category=CollectionCategory.PACKAGE_BUILD_LOGS,
-                )
-            ),
-        )
-
     @cached_property
     def source_package(self) -> Artifact:
         """Source package artifact."""
-        assert self.workspace is not None
-        assert self.work_request is not None
         artifact = lookup_single(
             self.data.input.source_artifact,
             self.workspace,
             user=self.work_request.created_by,
-            expect_type=CollectionItem.Types.ARTIFACT,
+            expect_type=LookupChildType.ARTIFACT,
         ).artifact
         if artifact.category != ArtifactCategory.SOURCE_PACKAGE:
             raise ValueError(
@@ -90,14 +68,34 @@ class SbuildWorkflow(Workflow[SbuildWorkflowData, SbuildWorkflowDynamicData]):
             )
         return artifact
 
+    @cached_property
+    def source_package_data(self) -> DebianSourcePackage:
+        """Parsed source package artifact data."""
+        return SourcePackage.create_data(self.source_package.data)
+
+    def compute_dynamic_data(
+        self, task_database: TaskDatabaseInterface  # noqa: U100
+    ) -> BaseDynamicTaskData:
+        """Compute dynamic data for this workflow."""
+        source_data = self.source_package_data
+        return BaseDynamicTaskData(
+            parameter_summary=f"{source_data.name}_{source_data.version}"
+        )
+
+    @cached_property
+    def binary_names(self) -> list[str]:
+        """Binary names that may be built."""
+        return re.split(
+            r"\s*,\s*", self.source_package_data.dsc_fields.get("Binary", "")
+        )
+
     @cached_property
     def architectures(self) -> set[str]:
         """Architectures to build."""
         workflow_arches = set(self.data.architectures)
 
-        artifact_data = SourcePackage.create_data(self.source_package.data)
         package_arches = set(
-            artifact_data.dsc_fields.get("Architecture", "").split()
+            self.source_package_data.dsc_fields.get("Architecture", "").split()
         )
         if not package_arches:
             raise WorkflowValidationError("package architecture list is empty")
@@ -136,9 +134,12 @@ class SbuildWorkflow(Workflow[SbuildWorkflowData, SbuildWorkflowDynamicData]):
 
         `architecture` may be "all".
         """
-        # TODO (#358): un-hardcode "amd64" default,
-        # possibly support XS-Build-Indep-Architecture
-        return "amd64" if architecture == "all" else architecture
+        # TODO (#358): possibly support XS-Build-Indep-Architecture
+        return (
+            self.data.arch_all_host_architecture
+            if architecture == "all"
+            else architecture
+        )
 
     def _get_environment_lookup(self) -> str:
         """
@@ -264,10 +265,9 @@ class SbuildWorkflow(Workflow[SbuildWorkflowData, SbuildWorkflowDynamicData]):
         *,
         environment: str | None = None,
         distribution: str | None = None,
-    ):
+    ) -> None:
         """Create a single sbuild WorkRequest."""
         assert self.work_request is not None
-        assert self.dynamic_data is not None
         host_architecture = self._get_host_architecture(architecture)
         task_data = SbuildData(
             input=self.data.input,
@@ -283,11 +283,12 @@ class SbuildWorkflow(Workflow[SbuildWorkflowData, SbuildWorkflowDynamicData]):
                 )
             ],
             build_profiles=self.data.build_profiles,
+            extra_repositories=self.data.extra_repositories,
             binnmu=self.data.binnmu,
         )
         wr = self.work_request.create_child(
             task_name="sbuild",
-            task_data=task_data.dict(),
+            task_data=task_data,
             workflow_data=WorkRequestWorkflowData(
                 display_name=f"Build {architecture}",
                 step=f"build-{architecture}",
@@ -297,17 +298,34 @@ class SbuildWorkflow(Workflow[SbuildWorkflowData, SbuildWorkflowDynamicData]):
         self.provides_artifact(
             wr,
             ArtifactCategory.UPLOAD,
-            f"build-{architecture}",
-            data={"architecture": architecture},
+            f"{self.data.prefix}build-{architecture}",
+            data={
+                "binary_names": self.binary_names,
+                "architecture": architecture,
+            },
         )
 
-        if self.dynamic_data.build_logs_collection_id is not None:
+        try:
+            build_logs_collection_id = self.lookup_singleton_collection(
+                CollectionCategory.PACKAGE_BUILD_LOGS
+            ).id
+        except KeyError:
+            build_logs_collection_id = None
+
+        source_artifact = lookup_single(
+            self.data.input.source_artifact,
+            self.workspace,
+            user=self.work_request.created_by,
+            workflow_root=self.work_request.get_workflow_root(),
+            expect_type=LookupChildType.ARTIFACT,
+        ).artifact
+
+        if build_logs_collection_id is not None:
             vendor, codename = self.data.target_distribution.split(":", 1)
             source_package_data = SourcePackage.create_data(
-                Artifact.objects.get(
-                    id=self.dynamic_data.input_source_artifact_id
-                ).data
+                source_artifact.data
             )
+
             variables = {
                 "work_request_id": wr.id,
                 "vendor": vendor,
@@ -319,14 +337,14 @@ class SbuildWorkflow(Workflow[SbuildWorkflowData, SbuildWorkflowDynamicData]):
             event_reactions = wr.event_reactions
             event_reactions.on_creation.append(
                 ActionUpdateCollectionWithData(
-                    collection=self.dynamic_data.build_logs_collection_id,
+                    collection=build_logs_collection_id,
                     category=BareDataCategory.PACKAGE_BUILD_LOG,
                     data=variables,
                 )
             )
             event_reactions.on_success.append(
                 ActionUpdateCollectionWithArtifacts(
-                    collection=self.dynamic_data.build_logs_collection_id,
+                    collection=build_logs_collection_id,
                     variables=variables,
                     artifact_filters={
                         "category": ArtifactCategory.PACKAGE_BUILD_LOG
@@ -344,8 +362,22 @@ class SbuildWorkflow(Workflow[SbuildWorkflowData, SbuildWorkflowDynamicData]):
             wr.event_reactions = event_reactions
             wr.save()
 
+        for binary_package_name in self.data.signing_template_names.get(
+            architecture, []
+        ):
+            self.provides_artifact(
+                wr,
+                ArtifactCategory.BINARY_PACKAGE,
+                f"{self.data.prefix}signing-template-{architecture}-"
+                f"{binary_package_name}",
+                data={"architecture": architecture},
+                artifact_filters={
+                    "data__deb_fields__Package": binary_package_name
+                },
+            )
+
     def get_label(self) -> str:
         """Return the task label."""
         # TODO: copy the source package information in dynamic task data and
         # use them here if available
-        return "build a package"
+        return f"{self.data.prefix}build a package"
