@@ -15,4 +15,8 @@ from django.core.checks import register
 
 from debusine.django.checks import all_migrations_applied
 
+__all__ = [
+    "all_migrations_applied",
+]
+
 register(partial(all_migrations_applied, "debusine-signing"))
