@@ -10,17 +10,12 @@
 """Database models for the Debusine signing service."""
 
 from pathlib import Path
-from typing import BinaryIO, TYPE_CHECKING, TypeAlias
+from typing import BinaryIO, ClassVar, TYPE_CHECKING, TypeAlias
 
 from django.conf import settings
 from django.db import models
 
-from debusine.signing.models import (
-    BaseProtectedKey,
-    ProtectedKey,
-    ProtectedKeyNaCl,
-    SigningMode,
-)
+from debusine.signing.models import ProtectedKey, ProtectedKeyNaCl, SigningMode
 from debusine.signing.openssl import openssl_generate, x509_fingerprint
 from debusine.signing.sbsign import sbsign
 from debusine.utils import calculate_hash
@@ -41,7 +36,7 @@ class _KeyPurpose(models.TextChoices):
 class AuditLog(models.Model):
     """An audit log entry."""
 
-    objects = models.Manager["AuditLog"]()
+    objects: ClassVar[models.Manager["AuditLog"]] = models.Manager["AuditLog"]()
 
     class Event(models.TextChoices):
         GENERATE = "generate", "Generate"
@@ -93,33 +88,6 @@ class KeyManager(models.Manager["Key"]):
             case _ as unreachable:
                 raise AssertionError(f"Unexpected purpose: {unreachable}")
 
-    def create_key(
-        self,
-        *,
-        purpose: _KeyPurpose,
-        fingerprint: str,
-        private_key: BaseProtectedKey,
-        public_key: bytes,
-        description: str,
-        event: AuditLog.Event = AuditLog.Event.GENERATE,
-        created_by_work_request_id: int | None = None,
-    ) -> "Key":
-        """Create a new key and add an audit log entry for it."""
-        key = self.create(
-            purpose=purpose,
-            fingerprint=fingerprint,
-            private_key=private_key.dict(),
-            public_key=public_key,
-        )
-        AuditLog.objects.create(
-            event=event,
-            purpose=purpose,
-            fingerprint=fingerprint,
-            data={"description": description},
-            created_by_work_request_id=created_by_work_request_id,
-        )
-        return key
-
     def generate(
         self,
         purpose: _KeyPurpose,
@@ -150,17 +118,23 @@ class KeyManager(models.Manager["Key"]):
         fingerprint = self.get_fingerprint(
             purpose=purpose, public_key=public_key, log_file=log_file
         )
-        return self.create_key(
+        key = self.create(
             purpose=purpose,
             fingerprint=fingerprint,
             private_key=ProtectedKeyNaCl.encrypt(
                 settings.DEBUSINE_SIGNING_PRIVATE_KEYS[0].public_key,
                 private_key,
-            ),
+            ).dict(),
             public_key=public_key,
-            description=description,
+        )
+        AuditLog.objects.create(
+            event=AuditLog.Event.GENERATE,
+            purpose=purpose,
+            fingerprint=fingerprint,
+            data={"description": description},
             created_by_work_request_id=created_by_work_request_id,
         )
+        return key
 
 
 class Key(models.Model):
@@ -205,12 +179,13 @@ class Key(models.Model):
         `signature_path` will be overwritten with the resulting signature.
         """
         match (self.purpose, mode):
-            case _KeyPurpose.UEFI, SigningMode.ATTACHED:
+            case _KeyPurpose.UEFI, SigningMode.ATTACHED | SigningMode.DETACHED:
                 sbsign(
                     self.stored_private_key,
                     self.public_key,
                     data_path=data_path,
                     signature_path=signature_path,
+                    detached=(mode == SigningMode.DETACHED),
                     log_file=log_file,
                 )
             case _KeyPurpose.OPENPGP, SigningMode.CLEAR:
