@@ -18,12 +18,11 @@ from pathlib import Path
 from typing import Any, BinaryIO
 from unittest import mock
 
-import django.test
 import gpg
+from django.test import TestCase as DjangoTestCase
 from gpg.results import GenkeyResult, ImportResult, ImportStatus
 from nacl.public import PrivateKey
 
-import debusine.test
 from debusine.signing.db.models import AuditLog, Key
 from debusine.signing.gnupg import GnuPGError
 from debusine.signing.models import (
@@ -32,9 +31,10 @@ from debusine.signing.models import (
     SigningMode,
 )
 from debusine.signing.tests import make_fake_fingerprint
+from debusine.test import TestCase
 
 
-class KeyManagerTests(django.test.TestCase):
+class KeyManagerTests(DjangoTestCase):
     """Unit tests for `KeyManager`."""
 
     def test_generate_uefi(self) -> None:
@@ -289,25 +289,38 @@ class KeyManagerTests(django.test.TestCase):
             Key.objects.generate(Key.Purpose.OPENPGP, "test key", 123, log_file)
 
 
-class KeyTests(debusine.test.TestCase, django.test.TestCase):
+class KeyTests(TestCase, DjangoTestCase):
     """Unit tests for `Key`."""
 
-    def test_sign_uefi(self) -> None:
-        """Sign data using a UEFI key."""
-        # Signing data is slow, so we just test that the correct
-        # subprocesses are executed.
+    def create_nacl_key(
+        self, *, storage_private_key: PrivateKey, purpose: Key.Purpose
+    ) -> tuple[Key, bytes]:
+        """
+        Make a random NaCl-encrypted Key.
+
+        :return: A tuple of the Key itself and its private key.
+        """
         private_key = random.randbytes(64)
         public_key = random.randbytes(64)
         fingerprint = make_fake_fingerprint()
-        storage_private_key = PrivateKey.generate()
         key = Key.objects.create(
-            purpose=Key.Purpose.UEFI,
+            purpose=purpose,
             fingerprint=fingerprint,
             private_key=ProtectedKeyNaCl.encrypt(
                 storage_private_key.public_key, private_key
             ).dict(),
             public_key=public_key,
         )
+        return key, private_key
+
+    def test_sign_uefi_attached(self) -> None:
+        """Sign data using a UEFI key, returning an attached signature."""
+        # Signing data is slow, so we just test that the correct
+        # subprocesses are executed.
+        storage_private_key = PrivateKey.generate()
+        key, private_key = self.create_nacl_key(
+            storage_private_key=storage_private_key, purpose=Key.Purpose.UEFI
+        )
         temp_path = self.create_temporary_directory()
         (data_path := temp_path / "data").write_bytes(b"data to sign")
         signature_path = temp_path / "signed"
@@ -351,7 +364,62 @@ class KeyTests(debusine.test.TestCase, django.test.TestCase):
             stderr=log_file,
         )
         self.assertEqual(key_bytes, private_key)
-        self.assertEqual(certificate_bytes, public_key)
+        self.assertEqual(certificate_bytes, key.public_key)
+        self.assertEqual(signature_path.read_bytes(), signature_bytes)
+
+    def test_sign_uefi_detached(self) -> None:
+        """Sign data using a UEFI key, returning a detached signature."""
+        # Signing data is slow, so we just test that the correct
+        # subprocesses are executed.
+        storage_private_key = PrivateKey.generate()
+        key, private_key = self.create_nacl_key(
+            storage_private_key=storage_private_key, purpose=Key.Purpose.UEFI
+        )
+        temp_path = self.create_temporary_directory()
+        (data_path := temp_path / "data").write_bytes(b"data to sign")
+        signature_path = temp_path / "signed"
+        signature_bytes = random.randbytes(64)
+        log_file = io.BytesIO()
+        key_bytes: bytes | None = None
+        certificate_bytes: bytes | None = None
+
+        def run(
+            args: list[str | os.PathLike[str]], **kwargs: Any
+        ) -> subprocess.CompletedProcess[Any]:
+            nonlocal key_bytes, certificate_bytes
+            key_bytes = Path(args[args.index("--key") + 1]).read_bytes()
+            certificate_bytes = Path(
+                args[args.index("--cert") + 1]
+            ).read_bytes()
+            Path(args[args.index("--output") + 1]).write_bytes(signature_bytes)
+            return subprocess.CompletedProcess(args=args, returncode=0)
+
+        with (
+            mock.patch("subprocess.run", side_effect=run) as mock_run,
+            self.settings(DEBUSINE_SIGNING_PRIVATE_KEYS=[storage_private_key]),
+        ):
+            key.sign(
+                data_path, signature_path, SigningMode.DETACHED, 123, log_file
+            )
+
+        mock_run.assert_called_once_with(
+            [
+                "sbsign",
+                "--key",
+                mock.ANY,
+                "--cert",
+                mock.ANY,
+                "--detached",
+                "--output",
+                signature_path,
+                data_path,
+            ],
+            check=True,
+            stdout=log_file,
+            stderr=log_file,
+        )
+        self.assertEqual(key_bytes, private_key)
+        self.assertEqual(certificate_bytes, key.public_key)
         self.assertEqual(signature_path.read_bytes(), signature_bytes)
 
     def test_sign_openpgp(self) -> None:
@@ -359,20 +427,12 @@ class KeyTests(debusine.test.TestCase, django.test.TestCase):
         # Signing data is slow, so we mock most of this and just test the
         # sequence of operations.  Integration tests will make sure that it
         # actually works.
-        secret_key = random.randbytes(64)
-        public_key = random.randbytes(64)
-        fingerprint = make_fake_fingerprint()
         storage_private_key = PrivateKey.generate()
-        key = Key.objects.create(
-            purpose=Key.Purpose.OPENPGP,
-            fingerprint=fingerprint,
-            private_key=ProtectedKeyNaCl.encrypt(
-                storage_private_key.public_key, secret_key
-            ).dict(),
-            public_key=public_key,
+        key, secret_key = self.create_nacl_key(
+            storage_private_key=storage_private_key, purpose=Key.Purpose.OPENPGP
         )
         import_status = ImportStatus(None)
-        import_status.fpr = fingerprint
+        import_status.fpr = key.fingerprint
         import_result = ImportResult(None)
         import_result.secret_imported = 1
         import_result.imports = [import_status]
@@ -389,7 +449,7 @@ class KeyTests(debusine.test.TestCase, django.test.TestCase):
             data: gpg.Data,
             sink: BinaryIO | None = None,
             mode: int = gpg.constants.SIG_MODE_NORMAL,
-        ):
+        ) -> None:
             self.assertTrue(ctx.armor)
             self.assertEqual(data.read(), data_bytes)
             self.assertEqual(mode, gpg.constants.SIG_MODE_CLEAR)
@@ -416,7 +476,9 @@ class KeyTests(debusine.test.TestCase, django.test.TestCase):
             )
 
         mock_key_import.assert_called_once_with(secret_key)
-        mock_keylist.assert_called_once_with(pattern=fingerprint, secret=True)
+        mock_keylist.assert_called_once_with(
+            pattern=key.fingerprint, secret=True
+        )
         mock_signers.assert_called_with(keylist)
         mock_sign.assert_called_once()
         self.assertEqual(signature_path.read_bytes(), signature_bytes)
@@ -451,20 +513,12 @@ class KeyTests(debusine.test.TestCase, django.test.TestCase):
         # Signing data is slow, so we mock most of this and just test the
         # sequence of operations.  Integration tests will make sure that it
         # actually works.
-        secret_key = random.randbytes(64)
-        public_key = random.randbytes(64)
-        fingerprint = make_fake_fingerprint()
         storage_private_key = PrivateKey.generate()
-        key = Key.objects.create(
-            purpose=Key.Purpose.OPENPGP,
-            fingerprint=fingerprint,
-            private_key=ProtectedKeyNaCl.encrypt(
-                storage_private_key.public_key, secret_key
-            ).dict(),
-            public_key=public_key,
+        key, secret_key = self.create_nacl_key(
+            storage_private_key=storage_private_key, purpose=Key.Purpose.OPENPGP
         )
         import_status = ImportStatus(None)
-        import_status.fpr = fingerprint
+        import_status.fpr = key.fingerprint
         import_result = ImportResult(None)
         import_result.not_imported = 1
         temp_path = self.create_temporary_directory()
@@ -486,20 +540,12 @@ class KeyTests(debusine.test.TestCase, django.test.TestCase):
         # Signing data is slow, so we mock most of this and just test the
         # sequence of operations.  Integration tests will make sure that it
         # actually works.
-        secret_key = random.randbytes(64)
-        public_key = random.randbytes(64)
-        fingerprint = make_fake_fingerprint()
         storage_private_key = PrivateKey.generate()
-        key = Key.objects.create(
-            purpose=Key.Purpose.OPENPGP,
-            fingerprint=fingerprint,
-            private_key=ProtectedKeyNaCl.encrypt(
-                storage_private_key.public_key, secret_key
-            ).dict(),
-            public_key=public_key,
+        key, secret_key = self.create_nacl_key(
+            storage_private_key=storage_private_key, purpose=Key.Purpose.OPENPGP
         )
         import_status = ImportStatus(None)
-        import_status.fpr = fingerprint
+        import_status.fpr = key.fingerprint
         import_result = ImportResult(None)
         import_result.secret_imported = 1
         import_result.imports = [import_status]
@@ -542,7 +588,12 @@ class KeyTests(debusine.test.TestCase, django.test.TestCase):
 
         mock_key_import.assert_called_once_with(secret_key)
         mock_run.assert_any_call(
-            ["debsign", f"-k{fingerprint}", "--re-sign", signed_changes_path],
+            [
+                "debsign",
+                f"-k{key.fingerprint}",
+                "--re-sign",
+                signed_changes_path,
+            ],
             cwd=signed_changes_path.parent,
             env={**os.environ, "GNUPGHOME": mock.ANY},
             check=True,
@@ -589,20 +640,12 @@ class KeyTests(debusine.test.TestCase, django.test.TestCase):
         # Signing data is slow, so we mock most of this and just test the
         # sequence of operations.  Integration tests will make sure that it
         # actually works.
-        secret_key = random.randbytes(64)
-        public_key = random.randbytes(64)
-        fingerprint = make_fake_fingerprint()
         storage_private_key = PrivateKey.generate()
-        key = Key.objects.create(
-            purpose=Key.Purpose.OPENPGP,
-            fingerprint=fingerprint,
-            private_key=ProtectedKeyNaCl.encrypt(
-                storage_private_key.public_key, secret_key
-            ).dict(),
-            public_key=public_key,
+        key, secret_key = self.create_nacl_key(
+            storage_private_key=storage_private_key, purpose=Key.Purpose.OPENPGP
         )
         import_status = ImportStatus(None)
-        import_status.fpr = fingerprint
+        import_status.fpr = key.fingerprint
         import_result = ImportResult(None)
         import_result.not_imported = 1
         unsigned_path = self.create_temporary_directory()
@@ -631,20 +674,12 @@ class KeyTests(debusine.test.TestCase, django.test.TestCase):
         # Signing data is slow, so we mock most of this and just test the
         # sequence of operations.  Integration tests will make sure that it
         # actually works.
-        secret_key = random.randbytes(64)
-        public_key = random.randbytes(64)
-        fingerprint = make_fake_fingerprint()
         storage_private_key = PrivateKey.generate()
-        key = Key.objects.create(
-            purpose=Key.Purpose.OPENPGP,
-            fingerprint=fingerprint,
-            private_key=ProtectedKeyNaCl.encrypt(
-                storage_private_key.public_key, secret_key
-            ).dict(),
-            public_key=public_key,
+        key, secret_key = self.create_nacl_key(
+            storage_private_key=storage_private_key, purpose=Key.Purpose.OPENPGP
         )
         import_status = ImportStatus(None)
-        import_status.fpr = fingerprint
+        import_status.fpr = key.fingerprint
         import_result = ImportResult(None)
         import_result.secret_imported = 1
         import_result.imports = [import_status]
