@@ -16,10 +16,9 @@ from django.core.management import call_command
 from django.test import TestCase
 
 from debusine.tasks.models import WorkerType
-from debusine.test import TestHelpersMixin
 
 
-class SigningWorkerCommandTests(TestHelpersMixin, TestCase):
+class SigningWorkerCommandTests(TestCase):
     """Tests for the signing_worker command."""
 
     def test_no_options(self) -> None:
