@@ -9,6 +9,7 @@
 
 """UEFI Secure Boot signing utility for the Debusine signing service."""
 
+import os
 import subprocess
 from pathlib import Path
 from typing import BinaryIO
@@ -21,19 +22,13 @@ def sbsign(
     public_key: bytes,
     data_path: Path,
     signature_path: Path,
+    detached: bool = False,
     log_file: BinaryIO | None = None,
-):
+) -> None:
     """Sign a UEFI boot image using `sbsign`."""
     with private_key.available(public_key) as available:
-        subprocess.run(
-            [
-                "sbsign",
-                *available.sbsign_args,
-                "--output",
-                signature_path,
-                data_path,
-            ],
-            check=True,
-            stdout=log_file,
-            stderr=log_file,
-        )
+        cmd: list[str | os.PathLike[str]] = ["sbsign", *available.sbsign_args]
+        if detached:
+            cmd.append("--detached")
+        cmd.extend(["--output", signature_path, data_path])
+        subprocess.run(cmd, check=True, stdout=log_file, stderr=log_file)
