@@ -18,10 +18,15 @@ from debusine.signing.tasks.base import BaseSigningTask  # isort: split
 # Sub-tasks need to be imported in order to be available to BaseTask
 # (e.g. for BaseTask.is_valid_task_name). They are registered via
 # BaseTask.__init_subclass__.
-from debusine.signing.tasks.generate_key import GenerateKey  # noqa: F401, I202
-from debusine.signing.tasks.noop import SigningNoop  # noqa: F401, I202
-from debusine.signing.tasks.sign import Sign  # noqa: F401, I202
+from debusine.signing.tasks.debsign import Debsign  # noqa: I202
+from debusine.signing.tasks.generate_key import GenerateKey  # noqa: I202
+from debusine.signing.tasks.noop import SigningNoop  # noqa: I202
+from debusine.signing.tasks.sign import Sign  # noqa: I202
 
 __all__ = [
     "BaseSigningTask",
+    "Debsign",
+    "GenerateKey",
+    "SigningNoop",
+    "Sign",
 ]
