@@ -17,7 +17,8 @@ from tempfile import TemporaryDirectory
 from typing import Any
 from unittest import mock
 
-from django.test import TestCase, TransactionTestCase, override_settings
+from django.test import TestCase as DjangoTestCase
+from django.test import TransactionTestCase, override_settings
 from nacl.public import PrivateKey
 
 from debusine.artifacts import (
@@ -27,20 +28,24 @@ from debusine.artifacts import (
     Upload,
     WorkRequestDebugLogs,
 )
-from debusine.artifacts.models import ArtifactCategory, EmptyArtifactData
+from debusine.artifacts.models import (
+    ArtifactCategory,
+    EmptyArtifactData,
+    KeyPurpose,
+)
 from debusine.client.models import ArtifactResponse, RemoteArtifact
 from debusine.signing.db.models import Key
 from debusine.signing.gnupg import gpg_ephemeral_context
 from debusine.signing.models import SigningMode
-from debusine.signing.tasks.debsign import Debsign
-from debusine.signing.tasks.models import DebsignDynamicData, KeyPurpose
+from debusine.signing.tasks import Debsign
+from debusine.signing.tasks.models import DebsignDynamicData
 from debusine.signing.tasks.tests.test_sign import SignTestMixin
 from debusine.tasks.models import WorkerType
 from debusine.tasks.tests.helper_mixin import FakeTaskDatabase
-from debusine.test import TestHelpersMixin
+from debusine.test import TestCase
 
 
-class DebsignTests(SignTestMixin, TestHelpersMixin, TestCase):
+class DebsignTests(SignTestMixin, TestCase, DjangoTestCase):
     """Unit tests for the :py:class:`Debsign` task."""
 
     def test_analyze_worker(self) -> None:
@@ -261,9 +266,7 @@ class DebsignTests(SignTestMixin, TestHelpersMixin, TestCase):
         )
 
 
-class DebsignIntegrationTests(
-    SignTestMixin, TestHelpersMixin, TransactionTestCase
-):
+class DebsignIntegrationTests(SignTestMixin, TestCase, TransactionTestCase):
     """Integration tests for the :py:class:`Debsign` task."""
 
     def generate_signing_key(
