@@ -8,3 +8,10 @@
 # contained in the LICENSE file.
 
 """Unit tests for the signing application."""
+
+import random
+
+
+def make_fake_fingerprint(length: int = 64) -> str:
+    """Generate a random string in the style of a fingerprint."""
+    return "".join(random.choices("0123456789ABCDEF", k=length))
