@@ -53,7 +53,7 @@ if TYPE_CHECKING:
 from debusine.artifacts import WorkRequestDebugLogs
 from debusine.artifacts.models import CollectionCategory
 from debusine.client.debusine import Debusine
-from debusine.client.models import ArtifactResponse
+from debusine.client.models import ArtifactResponse, RelationType
 from debusine.tasks.executors import (
     ExecutorImageCategory,
     ExecutorInterface,
@@ -82,7 +82,7 @@ class TaskConfigError(Exception):
     """Exception raised when there is an issue with a task configuration."""
 
     def __init__(
-        self, message: str, original_exception: Exception | None = None
+        self, message: str | None, original_exception: Exception | None = None
     ):
         """
         Initialize the TaskConfigError.
@@ -94,7 +94,7 @@ class TaskConfigError(Exception):
         super().__init__(message)
         self.original_exception = original_exception
 
-    def __str__(self):
+    def __str__(self) -> str:
         """
         Return a string representation.
 
@@ -102,12 +102,15 @@ class TaskConfigError(Exception):
         to the message for additional context.
         """
         if self.original_exception:
-            return (
-                f"{self.args[0]} (Original exception: "
-                f"{self.original_exception})"
-            )
-
-        return self.args[0]
+            if self.args[0] is not None:
+                return (
+                    f"{self.args[0]} (Original exception: "
+                    f"{self.original_exception})"
+                )
+            else:
+                return str(self.original_exception)
+        else:
+            return str(self.args[0])
 
 
 TD = TypeVar("TD", bound=BaseTaskData)
@@ -324,7 +327,7 @@ class BaseTask(Generic[TD, DTD], metaclass=ABCMeta):
         }
 
     @classmethod
-    def analyze_worker_all_tasks(cls):
+    def analyze_worker_all_tasks(cls) -> dict[str, Any]:
         """
         Return dictionary with metadata for each task in BaseTask._sub_tasks.
 
@@ -351,6 +354,7 @@ class BaseTask(Generic[TD, DTD], metaclass=ABCMeta):
         """
         task_architecture = getattr(self.data, "host_architecture", None)
         if task_architecture is not None:
+            assert isinstance(task_architecture, str)
             return task_architecture
         return self.worker_host_architecture
 
@@ -422,7 +426,7 @@ class BaseTask(Generic[TD, DTD], metaclass=ABCMeta):
         self,
         task_data: dict[str, Any],
         dynamic_task_data: dict[str, Any] | None = None,
-    ):
+    ) -> None:
         """
         Configure the task with the supplied ``task_data``.
 
@@ -442,7 +446,7 @@ class BaseTask(Generic[TD, DTD], metaclass=ABCMeta):
                 else self.dynamic_task_data_type(**dynamic_task_data)
             )
         except ValueError as exc:
-            raise TaskConfigError(str(exc), original_exception=exc)
+            raise TaskConfigError(None, original_exception=exc)
 
     def execute_logging_exceptions(self) -> bool:
         """Execute self.execute() logging any raised exceptions."""
@@ -485,7 +489,7 @@ class BaseTask(Generic[TD, DTD], metaclass=ABCMeta):
             incompatible with the worker.
         """
 
-    def abort(self):
+    def abort(self) -> None:
         """Task does not need to be executed. Once aborted cannot be changed."""
         self._aborted = True
 
@@ -545,7 +549,7 @@ class BaseTask(Generic[TD, DTD], metaclass=ABCMeta):
         return sorted(BaseTask._sub_tasks[TaskTypes.WORKER].keys())
 
     @abstractmethod
-    def _upload_work_request_debug_logs(self):
+    def _upload_work_request_debug_logs(self) -> None:
         """
         Create a WorkRequestDebugLogs artifact and upload the logs.
 
@@ -623,7 +627,7 @@ class BaseExternalTask(BaseTask[TD, DTD], Generic[TD, DTD], metaclass=ABCMeta):
         self.executor: ExecutorInterface | None = None
         self.executor_instance: InstanceInterface | None = None
 
-    def configure_server_access(self, debusine: Debusine):
+    def configure_server_access(self, debusine: Debusine) -> None:
         """Set the object to access the server."""
         self.debusine = debusine
 
@@ -804,7 +808,7 @@ class BaseExternalTask(BaseTask[TD, DTD], Generic[TD, DTD], metaclass=ABCMeta):
 
             return execution_succeeded
 
-    def _upload_work_request_debug_logs(self):
+    def _upload_work_request_debug_logs(self) -> None:
         """
         Create a WorkRequestDebugLogs artifact and upload the logs.
 
@@ -821,6 +825,7 @@ class BaseExternalTask(BaseTask[TD, DTD], Generic[TD, DTD], metaclass=ABCMeta):
             files=Path(self._debug_log_files_directory.name).glob("*")
         )
 
+        assert self.debusine
         remote_artifact = self.debusine.upload_artifact(
             work_request_debug_logs_artifact,
             workspace=self.workspace_name,
@@ -831,7 +836,7 @@ class BaseExternalTask(BaseTask[TD, DTD], Generic[TD, DTD], metaclass=ABCMeta):
             self.debusine.relation_create(
                 remote_artifact.id,
                 source_artifact_id,
-                "relates-to",
+                RelationType.RELATES_TO,
             )
 
         self._debug_log_files_directory.cleanup()
@@ -958,7 +963,7 @@ class BaseTaskWithExecutor(BaseExternalTask[TDE, DTDE], Generic[TDE, DTDE]):
             image_category=image_category,
         )
 
-    def _prepare_executor(self):
+    def _prepare_executor(self) -> None:
         """
         Prepare the executor.
 
@@ -967,12 +972,15 @@ class BaseTaskWithExecutor(BaseExternalTask[TDE, DTDE], Generic[TDE, DTDE]):
           task's compute_dynamic_data method)
         * Download the image
         """
+        assert self.debusine
+        assert self.dynamic_data
+        assert self.dynamic_data.environment_id
         self.executor = executor_class(self.backend)(
             self.debusine, self.dynamic_data.environment_id
         )
         self.executor.download_image()
 
-    def _prepare_executor_instance(self):
+    def _prepare_executor_instance(self) -> None:
         """
         Create and start an executor instance.
 
@@ -984,6 +992,7 @@ class BaseTaskWithExecutor(BaseExternalTask[TDE, DTDE], Generic[TDE, DTDE]):
         if self.executor is None:
             self._prepare_executor()
 
+        assert self.executor
         self.executor_instance = self.executor.create()
         self.executor_instance.start()
 
@@ -1014,6 +1023,29 @@ class BaseTaskWithExecutor(BaseExternalTask[TDE, DTDE], Generic[TDE, DTDE]):
                 group=non_root_user,
             )
 
+    def run_executor_command(
+        self,
+        cmd: list[str],
+        log_filename: str,
+        run_as_root: bool = False,
+        check: bool = True,
+    ) -> None:
+        """Run cmd within the executor, logging the output to log_name."""
+        assert self.executor_instance
+        with self.open_debug_log_file(log_filename, mode="ab") as cmd_log:
+            msg = f"Executing: {shlex.join(cmd)}\n"
+            cmd_log.write(msg.encode())
+            cmd_log.flush()
+            p = self.executor_instance.run(
+                cmd,
+                run_as_root=run_as_root,
+                stdout=cmd_log,
+                stderr=cmd_log,
+                check=check,
+            )
+            msg = f"Execution completed (exit code {p.returncode})\n"
+            cmd_log.write(msg.encode())
+
     def cleanup(self) -> None:
         """
         Clean up after running the task.
@@ -1066,7 +1098,7 @@ class RunCommandTask(
     CMD_LOG_SEPARATOR = "--------------------"
     CMD_LOG_FILENAME = "cmd-output.log"
 
-    def run(self, execute_directory) -> bool:
+    def run(self, execute_directory: Path) -> bool:
         """
         Run a single command via the executor.
 
@@ -1092,7 +1124,7 @@ class RunCommandTask(
         return self.task_succeeded(returncode, execute_directory)
 
     def task_succeeded(
-        self,  # noqa: U100
+        self,
         returncode: int | None,  # noqa: U100
         execute_directory: Path,  # noqa: U100
     ) -> bool:
@@ -1124,7 +1156,9 @@ class RunCommandTask(
     def _write_utf8(file: BinaryIO, text: str) -> None:
         file.write(text.encode("utf-8", errors="replace") + b"\n")
 
-    def _write_popen_result(self, file: BinaryIO, p: subprocess.Popen[AnyStr]):
+    def _write_popen_result(
+        self, file: BinaryIO, p: subprocess.Popen[AnyStr]
+    ) -> None:
         self._write_utf8(file, f"\naborted: {self.aborted}")
         self._write_utf8(file, f"returncode: {p.returncode}")
 
@@ -1329,11 +1363,13 @@ class RunCommandTask(
 
         return p.returncode
 
-    def _wait_popen(self, popen: subprocess.Popen[AnyStr], timeout: float):
+    def _wait_popen(
+        self, popen: subprocess.Popen[AnyStr], timeout: float
+    ) -> int:
         return popen.wait(timeout)
 
     @staticmethod
-    def _send_signal_pid(pid, signal) -> bool:
+    def _send_signal_pid(pid: int, signal: signal.Signals) -> bool:
         try:
             os.kill(pid, signal)
         except ProcessLookupError:
@@ -1342,7 +1378,7 @@ class RunCommandTask(
         return True
 
     @staticmethod
-    def _send_signal_group(process_group, signal):
+    def _send_signal_group(process_group: int, signal: signal.Signals) -> None:
         """Send signal to the process group."""
         try:
             os.killpg(process_group, signal)
