@@ -33,7 +33,7 @@ from debusine.artifacts.models import (
     CollectionCategory,
     KeyPurpose,
 )
-from debusine.client.models import ArtifactResponse, RelationCreateRequestType
+from debusine.client.models import ArtifactResponse, RelationType
 from debusine.tasks import BaseTaskWithExecutor, RunCommandTask
 from debusine.tasks.models import (
     AssembleSignedSourceData,
@@ -41,7 +41,6 @@ from debusine.tasks.models import (
 )
 from debusine.tasks.server import TaskDatabaseInterface
 
-
 # https://www.debian.org/doc/debian-policy/ch-controlfields.html#s-f-source
 _re_package_name = re.compile(r"^[a-z0-9][a-z0-9+.-]+$")
 
@@ -90,17 +89,17 @@ class AssembleSignedSource(
             return False
 
         executor_available_key = f"executor:{self.backend}:available"
-        return worker_metadata.get(executor_available_key, False)
+        return bool(worker_metadata.get(executor_available_key, False))
 
     def compute_dynamic_data(
         self, task_database: TaskDatabaseInterface
     ) -> AssembleSignedSourceDynamicData:
         """Resolve artifact lookups for this task."""
-        environment = self.get_environment_lookup()
-        assert environment is not None
         return AssembleSignedSourceDynamicData(
-            environment_id=task_database.lookup_single_artifact(
-                environment, default_category=CollectionCategory.ENVIRONMENTS
+            environment_id=self.get_environment(
+                task_database,
+                self.data.environment,
+                default_category=CollectionCategory.ENVIRONMENTS,
             ),
             template_id=task_database.lookup_single_artifact(
                 self.data.template
@@ -128,7 +127,7 @@ class AssembleSignedSource(
         self,
         artifact_id: int,
         target_category: ArtifactCategory,
-        relation_type: RelationCreateRequestType = "relates-to",
+        relation_type: RelationType = RelationType.RELATES_TO,
     ) -> Generator[int, None, None]:
         assert self.debusine is not None
 
@@ -507,7 +506,7 @@ class AssembleSignedSource(
                 self.debusine.relation_create(
                     uploaded_source_package_artifact.id,
                     built_using_id,
-                    "built-using",
+                    RelationType.BUILT_USING,
                 )
             uploaded_upload_artifact = self.debusine.upload_artifact(
                 self._upload_artifact,
@@ -517,7 +516,7 @@ class AssembleSignedSource(
             self.debusine.relation_create(
                 uploaded_upload_artifact.id,
                 uploaded_source_package_artifact.id,
-                "extends",
+                RelationType.EXTENDS,
             )
 
     def cleanup(self) -> None:
