@@ -65,7 +65,7 @@ class Autopkgtest(
         self._autopkgtest_targets: list[Path] = []
 
     @classmethod
-    def analyze_worker(cls):
+    def analyze_worker(cls) -> dict[str, Any]:
         """Report metadata for this task on this worker."""
         metadata = super().analyze_worker()
 
@@ -81,9 +81,10 @@ class Autopkgtest(
 
         executor_available_key = f"executor:{self.backend}:available"
         available_key = self.prefix_with_task_name("available")
-        return worker_metadata.get(
-            executor_available_key, False
-        ) and worker_metadata.get(available_key, False)
+        return bool(
+            worker_metadata.get(executor_available_key, False)
+            and worker_metadata.get(available_key, False)
+        )
 
     def compute_dynamic_data(
         self, task_database: TaskDatabaseInterface
