@@ -13,22 +13,31 @@ Executors for Debusine Tasks.
 Provides containment for tasks using a variety of backends.
 """
 
-from debusine.tasks.executors.base import (  # noqa: F401
+from debusine.tasks.executors.base import (
     ExecutorImageCategory,
     ExecutorInterface,
     InstanceInterface,
     analyze_worker_all_executors,
     executor_class,
 )
-from debusine.tasks.executors.incus import (  # noqa: F401
+from debusine.tasks.executors.incus import (
     IncusInstance,
     IncusLXCExecutor,
     IncusVMExecutor,
 )
-from debusine.tasks.executors.qemu import (  # noqa: F401
-    QemuExecutor,
-)
-from debusine.tasks.executors.unshare import (  # noqa: F401
-    UnshareExecutor,
-    UnshareInstance,
-)
+from debusine.tasks.executors.qemu import QemuExecutor
+from debusine.tasks.executors.unshare import UnshareExecutor, UnshareInstance
+
+__all__ = [
+    "ExecutorImageCategory",
+    "ExecutorInterface",
+    "IncusInstance",
+    "IncusLXCExecutor",
+    "IncusVMExecutor",
+    "InstanceInterface",
+    "QemuExecutor",
+    "UnshareExecutor",
+    "UnshareInstance",
+    "analyze_worker_all_executors",
+    "executor_class",
+]
