@@ -57,14 +57,14 @@ class ExecutorInterface(ABC):
     backend_name: str
     image_category: ClassVar[ExecutorImageCategory]
 
-    def __init_subclass__(cls, /, backend_name, **kwargs):
+    def __init_subclass__(cls, /, backend_name: str, **kwargs: Any) -> None:
         """Register the backend in _backends."""
         super().__init_subclass__(**kwargs)
         cls.backend_name = backend_name
         _backends[backend_name] = cls
 
     @abstractmethod
-    def __init__(self, debusine_api: Debusine, system_image_id: int):
+    def __init__(self, debusine_api: Debusine, system_image_id: int) -> None:
         """
         Instantiate an ExecutorInterface.
 
@@ -131,7 +131,7 @@ class InstanceInterface(ABC):
     _gid_map: dict[str, int]
     non_root_user: str = "_debusine"
 
-    def __init__(self):
+    def __init__(self) -> None:
         """Initialize an InstanceInterface."""
         self._uid_map = {"root": 0}
         self._gid_map = {"root": 0}
@@ -318,7 +318,7 @@ class InstanceInterface(ABC):
         group: str = "root",
         mode: int = 0o755,
         parents: bool = False,
-    ):
+    ) -> None:
         """
         Create directory inside the instance.
 
@@ -370,7 +370,7 @@ class InstanceInterface(ABC):
         *,
         text: Literal[True],
         run_as_root: bool = False,
-        **kwargs,
+        **kwargs: Any,
     ) -> CompletedProcess[str]: ...
 
     @overload
@@ -380,13 +380,13 @@ class InstanceInterface(ABC):
         *,
         text: Literal[False] | None = None,
         run_as_root: bool = False,
-        **kwargs,
+        **kwargs: Any,
     ) -> CompletedProcess[bytes]: ...
 
     def run(
         self,
         args: list[str],
-        **kwargs,
+        **kwargs: Any,
     ) -> CompletedProcess[AnyStr]:
         """
         Run a command in the environment.
@@ -408,7 +408,7 @@ class InstanceInterface(ABC):
         args: list[str],
         text: Literal[True],
         run_as_root: bool = False,
-        **kwargs,
+        **kwargs: Any,
     ) -> CompletedProcess[str]: ...
 
     @overload
@@ -418,7 +418,7 @@ class InstanceInterface(ABC):
         args: list[str],
         text: Literal[False] | None = None,
         run_as_root: bool = False,
-        **kwargs,
+        **kwargs: Any,
     ) -> CompletedProcess[bytes]: ...
 
     @abstractmethod
@@ -427,7 +427,7 @@ class InstanceInterface(ABC):
         args: list[str],
         text: Literal[False] | Literal[True] | None = None,
         run_as_root: bool = False,
-        **kwargs,
+        **kwargs: Any,
     ) -> CompletedProcess[AnyStr]:
         """
         Run a command in the environment.
