@@ -73,7 +73,7 @@ class ImageCache:
             )
         return r
 
-    def _image_lock(self, artifact_id: int):
+    def _image_lock(self, artifact_id: int) -> InterProcessLock:
         """Context Manager to hold a RW lock for working on a cached image."""
         cache_path = self.cache_artifact_path(artifact_id)
         cache_path.mkdir(exist_ok=True)
