@@ -24,7 +24,6 @@ from subprocess import (
     CalledProcessError,
     CompletedProcess,
     DEVNULL,
-    PIPE,
     check_call,
     check_output,
 )
@@ -79,15 +78,17 @@ TEMPLATES = {
 log = logging.getLogger(__name__)
 
 
-def run_incus_cmd(*args: str, suppress_stderr=False) -> str:
+def run_incus_cmd(*args: str, suppress_stderr: bool = False) -> str:
     """Execute an incus CLI command, return the output."""
     cmd = ["incus"]
     cmd.extend(args)
     log.debug("Executing %r", cmd)
-    kwargs: dict[str, Any] = {"encoding": "utf-8"}
-    if suppress_stderr:
-        kwargs["stderr"] = DEVNULL
-    return check_output(cmd, **kwargs)
+    return check_output(
+        cmd,
+        text=True,
+        encoding="utf-8",
+        stderr=DEVNULL if suppress_stderr else None,
+    )
 
 
 class IncusImageCache(ImageCache):
@@ -530,26 +531,45 @@ class IncusInstance(InstanceInterface):
 
     def _spin_until_started(
         self, sleep_time: float = 0.2, timeout: float = 60.0
-    ):
+    ) -> None:
         """Wait for an instance to become ready."""
         assert self._instance is not None
         start_time = time.monotonic()
         end_time = start_time + timeout
         while time.monotonic() < end_time:
             p = subprocess.run(
-                ["incus", "exec", self._instance, "true"],
-                stderr=PIPE,
+                [
+                    "incus",
+                    "exec",
+                    self._instance,
+                    "systemctl",
+                    "is-system-running",
+                ],
+                capture_output=True,
                 text=True,
             )
-            if p.returncode == 0:
+            stdout = p.stdout.strip()
+            stderr = p.stderr.strip()
+            if stdout in ("running", "degraded", "maintenance"):
                 return
-            if (
-                p.returncode != 1
-                or p.stderr.strip() != "Error: VM agent isn't currently running"
+            elif stdout in ("initializing", "starting"):
+                pass
+            elif (
+                p.returncode == 1
+                and stderr == "Error: VM agent isn't currently running"
             ):
-                raise InstanceNotRunning(
-                    f"Unexpected response from exec true: "
-                    f"{p.returncode}: {p.stderr}"
+                pass
+            elif p.returncode == 1 and stderr.startswith(
+                "Failed to connect to system scope bus via local transport:"
+            ):
+                pass
+            else:
+                log.warning(
+                    "Unexpected response from exec systemctl "
+                    "is-system-running: returncode: %i, stdout: %s, stderr: %s",
+                    p.returncode,
+                    p.stdout,
+                    p.stderr,
                 )
             time.sleep(sleep_time)
         raise InstanceNotRunning(
@@ -650,7 +670,7 @@ class IncusInstance(InstanceInterface):
         args: list[str],
         text: Literal[True],
         run_as_root: bool = False,
-        **kwargs,
+        **kwargs: Any,
     ) -> CompletedProcess[str]: ...
 
     @overload
@@ -659,7 +679,7 @@ class IncusInstance(InstanceInterface):
         args: list[str],
         text: Literal[False] | None = None,
         run_as_root: bool = False,
-        **kwargs,
+        **kwargs: Any,
     ) -> CompletedProcess[bytes]: ...
 
     def do_run(
@@ -667,7 +687,7 @@ class IncusInstance(InstanceInterface):
         args: list[str],
         text: Literal[False] | Literal[True] | None = None,
         run_as_root: bool = False,
-        **kwargs,
+        **kwargs: Any,
     ) -> CompletedProcess[AnyStr]:
         """
         Run a command (as root) in the instance.
