@@ -11,6 +11,7 @@
 import os
 import platform
 from pathlib import Path
+from typing import NoReturn
 
 from debusine import utils
 from debusine.client.debusine import Debusine
@@ -73,7 +74,7 @@ class QemuExecutor(ExecutorInterface, backend_name="qemu"):
             raise ImageNotDownloadedError("download_image() not yet called")
         return str(self._local_path)
 
-    def create(self):
+    def create(self) -> NoReturn:
         """
         Create an QemuInstance using the downloaded image.
 
