@@ -70,11 +70,11 @@ class TestInstance(InstanceInterface):
 class ExecutorTests(TestCase):
     """Unit tests for the Executor basic interface."""
 
-    def test_backends_populated(self):
+    def test_backends_populated(self) -> None:
         """Test that we have populated some backends."""
         self.assertGreater(len(_backends), 0)
 
-    def test_analyze_worker_all_executors_no_backends_available(self):
+    def test_analyze_worker_all_executors_no_backends_available(self) -> None:
         """analyze_worker_all_executors copes with no available backends."""
         self.mock_is_command_available({})
         executors_metadata = analyze_worker_all_executors()
@@ -83,7 +83,7 @@ class ExecutorTests(TestCase):
             self.assertRegex(key, r"^executor:.*:available$")
             self.assertFalse(value)
 
-    def test_analyze_worker_all_executors_unshare_available(self):
+    def test_analyze_worker_all_executors_unshare_available(self) -> None:
         """analyze_worker_all_executors reports available backends."""
         self.mock_is_command_available({"unshare": True})
         executors_metadata = analyze_worker_all_executors()
@@ -93,7 +93,7 @@ class ExecutorTests(TestCase):
 class InstanceTests(TestCase):
     """Unit tests for the Instance basic interface."""
 
-    def setUp(self):
+    def setUp(self) -> None:
         """Configure a test instance."""
         self.instance = TestInstance()
 
@@ -103,13 +103,13 @@ class InstanceTests(TestCase):
         self.addCleanup(patcher.stop)
         return patcher.start()
 
-    def test_start(self):
+    def test_start(self) -> None:
         """Test that start calls do_start()."""
         do_start = self.patch_instance("do_start")
         self.instance.start()
         do_start.assert_called_once_with()
 
-    def test_start_started(self):
+    def test_start_started(self) -> None:
         """Test that start requires a stopped instance."""
         self.instance.started = True
         with self.assertRaisesRegex(
@@ -117,35 +117,35 @@ class InstanceTests(TestCase):
         ):
             self.instance.start()
 
-    def test_stop(self):
+    def test_stop(self) -> None:
         """Test that stop calls do_stop()."""
         self.instance.started = True
         do_stop = self.patch_instance("do_stop")
         self.instance.stop()
         do_stop.assert_called_once_with()
 
-    def test_stop_stopped(self):
+    def test_stop_stopped(self) -> None:
         """Test that stop requires a started instance."""
         with self.assertRaisesRegex(
             InstanceNotRunning, "Cannot stop a stopped instance"
         ):
             self.instance.stop()
 
-    def test_restart(self):
+    def test_restart(self) -> None:
         """Test that restart calls do_restart()."""
         self.instance.started = True
         do_restart = self.patch_instance("do_restart")
         self.instance.restart()
         do_restart.assert_called_once_with()
 
-    def test_restart_stopped(self):
+    def test_restart_stopped(self) -> None:
         """Test that restart requires a started instance."""
         with self.assertRaisesRegex(
             InstanceNotRunning, "Cannot restart a stopped instance"
         ):
             self.instance.restart()
 
-    def test_file_push(self):
+    def test_file_push(self) -> None:
         """Test that file_push calls do_file_push()."""
         self.instance.started = True
         do_file_push = self.patch_instance("do_file_push")
@@ -154,14 +154,14 @@ class InstanceTests(TestCase):
         self.instance.file_push(source, dest)
         do_file_push.assert_called_once_with(source, dest, 0, 0, 0o644)
 
-    def test_file_push_stopped(self):
+    def test_file_push_stopped(self) -> None:
         """Test that file_push requires a started instance."""
         with self.assertRaisesRegex(
             InstanceNotRunning, "Cannot push files to a stopped instance"
         ):
             self.instance.file_push(Path("source"), Path("/dest"))
 
-    def test_file_push_relative(self):
+    def test_file_push_relative(self) -> None:
         """Test that file_push requires a relative destination."""
         self.instance.started = True
         with self.assertRaisesRegex(
@@ -169,7 +169,7 @@ class InstanceTests(TestCase):
         ):
             self.instance.file_push(Path("source"), Path("dest"))
 
-    def test_file_pull(self):
+    def test_file_pull(self) -> None:
         """Test that file_pull calls do_file_pull()."""
         self.instance.started = True
         do_file_pull = self.patch_instance("do_file_pull")
@@ -178,14 +178,14 @@ class InstanceTests(TestCase):
         self.instance.file_pull(source, dest)
         do_file_pull.assert_called_once_with(source, dest)
 
-    def test_file_pull_stopped(self):
+    def test_file_pull_stopped(self) -> None:
         """Test that file_pull requires a started instance."""
         with self.assertRaisesRegex(
             InstanceNotRunning, "Cannot pull files from a stopped instance"
         ):
             self.instance.file_pull(Path("/source"), Path("dest"))
 
-    def test_file_pull_relative(self):
+    def test_file_pull_relative(self) -> None:
         """Test that file_pull requires a relative source."""
         self.instance.started = True
         with self.assertRaisesRegex(
@@ -193,7 +193,7 @@ class InstanceTests(TestCase):
         ):
             self.instance.file_pull(Path("source"), Path("dest"))
 
-    def test_directory_push(self):
+    def test_directory_push(self) -> None:
         """Test that directory_push calls do_directory_push()."""
         self.instance.started = True
         do_directory_push = self.patch_instance("do_directory_push")
@@ -202,14 +202,14 @@ class InstanceTests(TestCase):
         self.instance.directory_push(source, dest)
         do_directory_push.assert_called_once_with(source, dest, 0, 0)
 
-    def test_directory_push_stopped(self):
+    def test_directory_push_stopped(self) -> None:
         """Test that directory_push requires a started instance."""
         with self.assertRaisesRegex(
             InstanceNotRunning, "Cannot push files to a stopped instance"
         ):
             self.instance.directory_push(Path("source"), Path("/dest"))
 
-    def test_directory_push_relative(self):
+    def test_directory_push_relative(self) -> None:
         """Test that directory_push requires a relative destination."""
         self.instance.started = True
         with self.assertRaisesRegex(
@@ -217,7 +217,7 @@ class InstanceTests(TestCase):
         ):
             self.instance.directory_push(Path("source"), Path("dest"))
 
-    def test_do_directory_push(self):
+    def test_do_directory_push(self) -> None:
         """do_directory_push creates the directories and copies the files."""
         self.instance.started = True
 
@@ -280,7 +280,7 @@ class InstanceTests(TestCase):
             ]
         )
 
-    def test_run(self):
+    def test_run(self) -> None:
         """Test that run calls do_run()."""
         self.instance.started = True
         do_run = self.patch_instance("do_run")
@@ -289,14 +289,14 @@ class InstanceTests(TestCase):
             ["cmd", "--arg"], stderr=PIPE, stdout=PIPE
         )
 
-    def test_run_stopped(self):
+    def test_run_stopped(self) -> None:
         """Test that run requires a started instance."""
         with self.assertRaisesRegex(
             InstanceNotRunning, "Cannot run commands on a stopped instance"
         ):
             self.instance.run(["foo"])
 
-    def test_run_check_output(self):
+    def test_run_check_output(self) -> None:
         """Test that _run_check_output calls run()."""
         with patch.object(self.instance, "run") as run_mock:
             self.instance._run_check_output(["true"])
@@ -309,7 +309,7 @@ class InstanceTests(TestCase):
             text=True,
         )
 
-    def test_get_uid_uncached(self):
+    def test_get_uid_uncached(self) -> None:
         """Test _get_uid when not cached."""
         with patch.object(self.instance, "_run_check_output") as run_mock:
             run_mock.return_value = "testuser:x:101:105:Test User:/foo:/bin/sh"
@@ -317,13 +317,13 @@ class InstanceTests(TestCase):
         run_mock.assert_called_once_with(["getent", "passwd", "testuser"])
         self.assertEqual(self.instance._uid_map["testuser"], 101)
 
-    def test_get_uid_cached(self):
+    def test_get_uid_cached(self) -> None:
         """Test _get_uid when cached."""
         with patch.object(self.instance, "_run_check_output") as run_mock:
             self.assertEqual(self.instance._get_uid("root"), 0)
         run_mock.assert_not_called()
 
-    def test_get_uid_unknown(self):
+    def test_get_uid_unknown(self) -> None:
         """Test _get_uid when not existent."""
         with patch.object(self.instance, "_run_check_output") as run_mock:
             run_mock.side_effect = CalledProcessError(
@@ -335,7 +335,7 @@ class InstanceTests(TestCase):
                 self.instance._get_uid("testuser")
         self.assertNotIn("testuser", self.instance._uid_map)
 
-    def test_get_uid_unexpected_error(self):
+    def test_get_uid_unexpected_error(self) -> None:
         """Test _get_uid when not existent."""
         with patch.object(self.instance, "_run_check_output") as run_mock:
             run_mock.side_effect = CalledProcessError(
@@ -344,7 +344,7 @@ class InstanceTests(TestCase):
             with self.assertRaises(CalledProcessError):
                 self.instance._get_uid("testuser")
 
-    def test_get_gid_uncached(self):
+    def test_get_gid_uncached(self) -> None:
         """Test _get_gid when not cached."""
         with patch.object(self.instance, "_run_check_output") as run_mock:
             run_mock.return_value = "testgroup:x:102:"
@@ -352,13 +352,13 @@ class InstanceTests(TestCase):
         run_mock.assert_called_once_with(["getent", "group", "testgroup"])
         self.assertEqual(self.instance._gid_map["testgroup"], 102)
 
-    def test_get_gid_cached(self):
+    def test_get_gid_cached(self) -> None:
         """Test _get_gid when cached."""
         with patch.object(self.instance, "_run_check_output") as run_mock:
             self.assertEqual(self.instance._get_gid("root"), 0)
         run_mock.assert_not_called()
 
-    def test_get_gid_unknown(self):
+    def test_get_gid_unknown(self) -> None:
         """Test _get_gid when not existent."""
         with patch.object(self.instance, "_run_check_output") as run_mock:
             run_mock.side_effect = CalledProcessError(
@@ -370,7 +370,7 @@ class InstanceTests(TestCase):
                 self.instance._get_gid("testgroup")
         self.assertNotIn("testgroup", self.instance._gid_map)
 
-    def test_get_gid_unexpected_error(self):
+    def test_get_gid_unexpected_error(self) -> None:
         """Test _get_gid when not existent."""
         with patch.object(self.instance, "_run_check_output") as run_mock:
             run_mock.side_effect = CalledProcessError(
@@ -379,14 +379,14 @@ class InstanceTests(TestCase):
             with self.assertRaises(CalledProcessError):
                 self.instance._get_gid("testgroup")
 
-    def test_create_user_exists(self):
+    def test_create_user_exists(self) -> None:
         """Test create_user when user exists."""
         self.instance._uid_map["testuser"] = 42
         with patch.object(self.instance, "_run_check_output") as run_mock:
             self.assertEqual(self.instance.create_user("testuser"), 42)
         run_mock.assert_not_called()
 
-    def test_create_user_not_exists(self):
+    def test_create_user_not_exists(self) -> None:
         """Test create_user when user doesn't exist."""
 
         def create_user(args: list[str]) -> str:
@@ -406,14 +406,14 @@ class InstanceTests(TestCase):
             ]
         )
 
-    def test_create_user_default_username(self):
+    def test_create_user_default_username(self) -> None:
         """Test create_user when no username is supplied."""
         self.instance._uid_map["_debusine"] = 42
         with patch.object(self.instance, "_run_check_output") as run_mock:
             self.assertEqual(self.instance.create_user(), 42)
         run_mock.assert_not_called()
 
-    def test_mkdir(self):
+    def test_mkdir(self) -> None:
         """Test that mkdir calls do_mkdir()."""
         self.instance.started = True
         do_mkdir = self.patch_instance("do_mkdir")
@@ -421,14 +421,14 @@ class InstanceTests(TestCase):
         self.instance.mkdir(dest)
         do_mkdir.assert_called_once_with(dest, 0, 0, 0o755, False)
 
-    def test_mkdir_stopped(self):
+    def test_mkdir_stopped(self) -> None:
         """Test that mkdir requires a started instance."""
         with self.assertRaisesRegex(
             InstanceNotRunning, "Cannot make directories in a stopped instance"
         ):
             self.instance.mkdir(Path("/dest"))
 
-    def test_mkdir_relative(self):
+    def test_mkdir_relative(self) -> None:
         """Test that mkdir requires a relative target."""
         self.instance.started = True
         with self.assertRaisesRegex(
@@ -436,7 +436,7 @@ class InstanceTests(TestCase):
         ):
             self.instance.mkdir(Path("dest"))
 
-    def test_do_mkdir(self):
+    def test_do_mkdir(self) -> None:
         """do_mkdir() use run() as expected."""
         uid = 101
         gid = 102
@@ -451,7 +451,7 @@ class InstanceTests(TestCase):
         ]
         run_mocked.assert_has_calls(run_expected_calls)
 
-    def test_do_mkdir_parents(self):
+    def test_do_mkdir_parents(self) -> None:
         """do_mkdir() passes --parents."""
         uid = 101
         gid = 102
@@ -466,7 +466,7 @@ class InstanceTests(TestCase):
         ]
         run_mocked.assert_has_calls(run_expected_calls)
 
-    def test_do_mkdir_root(self):
+    def test_do_mkdir_root(self) -> None:
         """do_mkdir() skips chown for root."""
         target = PurePath("/etc/dir1")
 
@@ -475,7 +475,7 @@ class InstanceTests(TestCase):
 
         run_mocked.assert_called_once_with(["mkdir", "--mode=755", str(target)])
 
-    def test_do_mkdir_exists(self):
+    def test_do_mkdir_exists(self) -> None:
         """do_mkdir() raise FileExistsError."""
         self.instance.started = True
         target = PurePath("/some/directory")
@@ -492,7 +492,7 @@ class InstanceTests(TestCase):
             with self.assertRaisesRegex(FileExistsError, msg):
                 self.instance.do_mkdir(target, 0, 0, 0o755, False)
 
-    def test_do_mkdir_unexpected_error(self):
+    def test_do_mkdir_unexpected_error(self) -> None:
         """do_mkdir() raise CalledProcessError on an unknown error."""
         self.instance.started = True
         target = PurePath("/some/directory")
