@@ -15,27 +15,30 @@ from pathlib import Path
 from shutil import rmtree
 from subprocess import CalledProcessError, CompletedProcess, DEVNULL, PIPE
 from tempfile import NamedTemporaryFile, mkdtemp
-from typing import cast
+from typing import Any, cast
 from unittest.mock import ANY, MagicMock, call, create_autospec, patch
 
 import yaml
 
-from debusine.artifacts.models import ArtifactCategory
 from debusine.client.debusine import Debusine
+from debusine.client.models import ArtifactResponse
 from debusine.tasks import Noop
 from debusine.tasks.executors.base import (
+    ExecutorImageCategory,
     ExecutorInterface,
     ImageImportError,
     InstanceNotRunning,
     executor_class,
 )
 from debusine.tasks.executors.incus import (
+    IncusExecutorMixin,
     IncusImageCache,
     IncusInstance,
     IncusLXCExecutor,
     IncusLXCImageCache,
     IncusVMExecutor,
     IncusVMImageCache,
+    log,
     run_incus_cmd,
 )
 from debusine.tasks.tests.helper_mixin import ExternalTaskHelperMixin
@@ -46,30 +49,50 @@ class RunIncusCmdTest(TestCase):
     """Unit tests for run_incus_cmd."""
 
     @patch("debusine.tasks.executors.incus.check_output")
-    def test_run_incus_cmd(self, check_output):
+    def test_run_incus_cmd(self, check_output: MagicMock) -> None:
         """Test that run_incus_cmd is a thin wrapper around check_output."""
         check_output.return_value = "output"
         output = run_incus_cmd("testing", "123")
         self.assertEqual(output, "output")
         check_output.assert_called_with(
-            ["incus", "testing", "123"], encoding="utf-8"
+            ["incus", "testing", "123"],
+            text=True,
+            encoding="utf-8",
+            stderr=None,
         )
 
     @patch("debusine.tasks.executors.incus.check_output")
-    def test_run_incus_cmd_suppress_stderr(self, check_output):
+    def test_run_incus_cmd_suppress_stderr(
+        self, check_output: MagicMock
+    ) -> None:
         """Test that run_incus_cmd can suppress stderr."""
         check_output.return_value = "output"
         output = run_incus_cmd("testing", "123", suppress_stderr=True)
         self.assertEqual(output, "output")
         check_output.assert_called_with(
             ["incus", "testing", "123"],
+            text=True,
             encoding="utf-8",
             stderr=DEVNULL,
         )
 
 
-class IncusImageCacheTestMixin(ExternalTaskHelperMixin[Noop]):
-    """Common setup for all the Incus*ImageCacheTests."""
+class IncusImageCacheCommonTests(ExternalTaskHelperMixin[Noop], TestCase):
+    """Unit tests that apply to all the Incus ImageCache implementations."""
+
+    image_cache_cls: type[IncusImageCache]
+    artifact: ArtifactResponse
+    artifact_base_name: str
+    artifact_category: ExecutorImageCategory
+
+    def setUp(self) -> None:
+        """Mock the image handling for tests."""
+        super().setUp()
+        self.debusine_api = create_autospec(Debusine)
+        self.image_cache = self.image_cache_cls(
+            self.debusine_api, self.artifact_category
+        )
+        self.image_cache.image_cache_path = Path("/tmp/not-used")
 
     def create_tar(
         self,
@@ -137,22 +160,8 @@ class IncusImageCacheTestMixin(ExternalTaskHelperMixin[Noop]):
 
         return path
 
-
-class IncusImageCacheTests(IncusImageCacheTestMixin, TestCase):
-    """Unit tests for IncusImageCache."""
-
-    def setUp(self):
-        """Mock the image handling for tests."""
-        super().setUp()
-        self.debusine_api = create_autospec(Debusine)
-        self.image_cache = IncusImageCache(
-            self.debusine_api, ArtifactCategory.SYSTEM_TARBALL
-        )
-        self.artifact = self.fake_system_tarball_artifact()
-        self.image_cache.image_cache_path = Path("/tmp/not-used")
-
     @patch("debusine.tasks.executors.incus.run_incus_cmd")
-    def test_image_in_cache_uncached(self, run_incus_cmd):
+    def test_image_in_cache_uncached(self, run_incus_cmd: MagicMock) -> None:
         """Test that image_in_cache fails to find a cached image."""
         run_incus_cmd.side_effect = CalledProcessError(
             cmd=["incus"], returncode=1
@@ -163,7 +172,7 @@ class IncusImageCacheTests(IncusImageCacheTestMixin, TestCase):
         )
 
     @patch("debusine.tasks.executors.incus.run_incus_cmd")
-    def test_image_in_cache_cached(self, run_incus_cmd):
+    def test_image_in_cache_cached(self, run_incus_cmd: MagicMock) -> None:
         """Test that image_in_cache finds cached image."""
         run_incus_cmd.return_value = "Fingerprint: abc123\n"
         self.assertTrue(self.image_cache.image_in_cache(self.artifact))
@@ -172,7 +181,7 @@ class IncusImageCacheTests(IncusImageCacheTestMixin, TestCase):
         )
 
     @patch("debusine.tasks.executors.incus.run_incus_cmd")
-    def test_process_downloaded_image(self, run_incus_cmd):
+    def test_process_downloaded_image(self, run_incus_cmd: MagicMock) -> None:
         """
         Test that process_downloaded_image calls build_incus_image and uploads.
 
@@ -188,7 +197,7 @@ class IncusImageCacheTests(IncusImageCacheTestMixin, TestCase):
             self.image_cache.process_downloaded_image(self.artifact)
 
         downloaded_path = self.image_cache.cache_artifact_image_path(
-            42, "system.tar.xz"
+            42, self.artifact_base_name
         )
         build_incus_image.assert_called_with(
             downloaded_path, ANY, self.artifact
@@ -197,7 +206,7 @@ class IncusImageCacheTests(IncusImageCacheTestMixin, TestCase):
             "image", "import", "incus.tar.xz", "--alias", "artifact/42"
         )
 
-    def test_image_metadata(self):
+    def test_image_metadata(self) -> None:
         """Test that _image_metadata returns the correct metadata."""
         metadata = self.image_cache._image_metadata(self.artifact)
 
@@ -209,8 +218,8 @@ class IncusImageCacheTests(IncusImageCacheTestMixin, TestCase):
                 "debian_architecture": "amd64",
                 "debusine_artifact_id": 42,
                 "description": (
-                    "Debusine Artifact #42: "
-                    "debian:system-tarball for debian bookworm amd64"
+                    f"Debusine Artifact #42: "
+                    f"{self.artifact_category} for debian bookworm amd64"
                 ),
                 "os": "debian",
                 "release": "bookworm",
@@ -232,7 +241,7 @@ class IncusImageCacheTests(IncusImageCacheTestMixin, TestCase):
         }
         self.assertEqual(metadata, expected)
 
-    def test_add_tar_directory(self):
+    def test_add_tar_directory(self) -> None:
         """Test that the add_tar_directory helper adds a directory."""
         buffer = BytesIO()
         with tarfile.TarFile(fileobj=buffer, mode="w") as tar:
@@ -250,7 +259,7 @@ class IncusImageCacheTests(IncusImageCacheTestMixin, TestCase):
             self.assertEqual(member.uid, 0)
             self.assertEqual(member.gid, 0)
 
-    def test_add_tar_file(self):
+    def test_add_tar_file(self) -> None:
         """Test that the add_tar_file helper adds a file."""
         buffer = BytesIO()
         with tarfile.TarFile(fileobj=buffer, mode="w") as tar:
@@ -267,9 +276,11 @@ class IncusImageCacheTests(IncusImageCacheTestMixin, TestCase):
             self.assertEqual(member.mode, 0o644)
             self.assertEqual(member.uid, 0)
             self.assertEqual(member.gid, 0)
-            self.assertEqual(tar.extractfile(member).read(), b"bar")
+            member_file = tar.extractfile(member)
+            assert member_file is not None
+            self.assertEqual(member_file.read(), b"bar")
 
-    def test_add_tar_symlink(self):
+    def test_add_tar_symlink(self) -> None:
         """Test that the add_tar_symlink helper adds a symlink."""
         buffer = BytesIO()
         with tarfile.TarFile(fileobj=buffer, mode="w") as tar:
@@ -288,20 +299,19 @@ class IncusImageCacheTests(IncusImageCacheTestMixin, TestCase):
             self.assertEqual(member.linkname, "bar")
 
 
-class IncusLXCImageCacheTests(IncusImageCacheTestMixin, TestCase):
+class IncusLXCImageCacheTests(IncusImageCacheCommonTests):
     """Unit tests for IncusLXCImageCache."""
 
-    def setUp(self):
+    image_cache_cls = IncusLXCImageCache
+    artifact_base_name = "system.tar.xz"
+    artifact_category = ExecutorImageCategory.TARBALL
+
+    def setUp(self) -> None:
         """Mock the image handling for tests."""
         super().setUp()
-        self.debusine_api = create_autospec(Debusine)
-        self.image_cache = IncusLXCImageCache(
-            self.debusine_api, ArtifactCategory.SYSTEM_TARBALL
-        )
         self.artifact = self.fake_system_tarball_artifact()
-        self.image_cache.image_cache_path = Path("/tmp/not-used")
 
-    def test_prepare_incus_image(self):
+    def test_prepare_incus_image(self) -> None:
         """Test that prepare copies over the image and metadata."""
         source = self.create_tar(
             dirs=("./", "./etc", "./sbin"), special_files=True
@@ -352,7 +362,7 @@ class IncusLXCImageCacheTests(IncusImageCacheTestMixin, TestCase):
             tarinfo = tar.getmember("rootfs/special/sym-link")
             self.assertEqual(tarinfo.linkname, "/dev/null")
 
-    def test_build_incus_image(self):
+    def test_build_incus_image(self) -> None:
         """Test that build_incus_image injects basic metadata into a new tar."""
         directory = self.create_temporary_directory()
         source_path = directory / "system.tar.xz"
@@ -368,14 +378,16 @@ class IncusLXCImageCacheTests(IncusImageCacheTestMixin, TestCase):
         with tarfile.open(image) as tar:
             names = set(tar.getnames())
             self.assertIn("metadata.yaml", names)
-            metadata = yaml.safe_load(tar.extractfile("metadata.yaml"))
+            metadata_file = tar.extractfile("metadata.yaml")
+            assert metadata_file is not None
+            metadata = yaml.safe_load(metadata_file)
 
         self.assertNotIn("test-file", names)
         self.assertEqual(
             metadata, self.image_cache._image_metadata(self.artifact)
         )
 
-    def test_prepare_incus_image_requires_init(self):
+    def test_prepare_incus_image_requires_init(self) -> None:
         """Test that prepare only accepts bootable images."""
         source = self.create_tar(files=())
         dest_buf = BytesIO()
@@ -385,7 +397,7 @@ class IncusLXCImageCacheTests(IncusImageCacheTestMixin, TestCase):
                     source, dest_tar, self.artifact
                 )
 
-    def test_prepare_incus_image_accepts_usrmerged_init(self):
+    def test_prepare_incus_image_accepts_usrmerged_init(self) -> None:
         """Test that prepare accepts a /usr/sbin/init."""
         source = self.create_tar(
             dirs=("./", "./usr", "./usr/sbin"),
@@ -398,20 +410,19 @@ class IncusLXCImageCacheTests(IncusImageCacheTestMixin, TestCase):
             )
 
 
-class IncusVMImageCacheTests(IncusImageCacheTestMixin, TestCase):
+class IncusVMImageCacheTests(IncusImageCacheCommonTests):
     """Unit tests for IncusVMImageCache."""
 
-    def setUp(self):
+    image_cache_cls = IncusVMImageCache
+    artifact_base_name = "image.qcow2"
+    artifact_category = ExecutorImageCategory.IMAGE
+
+    def setUp(self) -> None:
         """Mock the image handling for tests."""
         super().setUp()
-        self.debusine_api = create_autospec(Debusine)
-        self.image_cache = IncusVMImageCache(
-            self.debusine_api, ArtifactCategory.SYSTEM_TARBALL
-        )
         self.artifact = self.fake_system_image_artifact()
-        self.image_cache.image_cache_path = Path("/tmp/not-used")
 
-    def test_prepare_qcow2_incus_image(self):
+    def test_prepare_qcow2_incus_image(self) -> None:
         """Test that prepare converts qcow2 images."""
         with NamedTemporaryFile(
             prefix="debusine-test-", suffix=".qcow2", delete=False
@@ -429,7 +440,9 @@ class IncusVMImageCacheTests(IncusImageCacheTestMixin, TestCase):
 
         with tarfile.open(fileobj=dest_buf) as tar:
             names = tar.getnames()
-            image = tar.extractfile("rootfs.img").read()
+            image_file = tar.extractfile("rootfs.img")
+            assert image_file is not None
+            image = image_file.read()
 
         self.assertEqual(
             names,
@@ -444,7 +457,7 @@ class IncusVMImageCacheTests(IncusImageCacheTestMixin, TestCase):
         self.assertEqual(image, b"qcow!")
 
     @patch("debusine.tasks.executors.incus.check_call")
-    def test_prepare_raw_incus_image(self, check_call):
+    def test_prepare_raw_incus_image(self, check_call: MagicMock) -> None:
         """Test that prepare converts raw images."""
         self.artifact.data["image_format"] = "raw"
         self.artifact.data["filename"] = "image.tar.xz"
@@ -458,7 +471,7 @@ class IncusVMImageCacheTests(IncusImageCacheTestMixin, TestCase):
         )
         dest_buf = BytesIO()
 
-        def fake_convert(args: list[str]):
+        def fake_convert(args: list[str]) -> None:
             """Fake for subprocess.check_call."""
             assert args[:2] == ["qemu-img", "convert"]
             with Path(args[-1]).open("wb") as f:
@@ -491,8 +504,12 @@ class IncusVMImageCacheTests(IncusImageCacheTestMixin, TestCase):
 
         with tarfile.open(fileobj=dest_buf) as tar:
             names = tar.getnames()
-            image = tar.extractfile("rootfs.img").read()
-            metadata = yaml.safe_load(tar.extractfile("metadata.yaml"))
+            image_file = tar.extractfile("rootfs.img")
+            assert image_file is not None
+            image = image_file.read()
+            metadata_file = tar.extractfile("metadata.yaml")
+            assert metadata_file is not None
+            metadata = yaml.safe_load(metadata_file)
 
         self.assertEqual(
             names,
@@ -510,13 +527,13 @@ class IncusVMImageCacheTests(IncusImageCacheTestMixin, TestCase):
         )
 
 
-class IncusExecutorCommon(ExternalTaskHelperMixin[Noop]):
+class IncusExecutorCommonTests(ExternalTaskHelperMixin[Noop], TestCase):
     """Unit tests that apply to both Incus Executors."""
 
     executor_cls: type[ExecutorInterface]
     uses_images: bool
 
-    def setUp(self):
+    def setUp(self) -> None:
         """Mock the Debusine API for tests."""
         super().setUp()
         self.debusine_api = MagicMock(spec=Debusine)
@@ -532,7 +549,7 @@ class IncusExecutorCommon(ExternalTaskHelperMixin[Noop]):
 
         self.executor = self.executor_cls(self.debusine_api, 42)
 
-    def test_available(self):
+    def test_available(self) -> None:
         """Test that available() returns True if incus is available."""
         self.mock_is_command_available(
             {
@@ -544,7 +561,7 @@ class IncusExecutorCommon(ExternalTaskHelperMixin[Noop]):
         self.assertTrue(self.executor_cls.available())
         self.run_incus_cmd.assert_called_once_with("info")
 
-    def test_available_no_incus(self):
+    def test_available_no_incus(self) -> None:
         """Test that available() returns False if incus is not available."""
         self.mock_is_command_available(
             {
@@ -555,7 +572,7 @@ class IncusExecutorCommon(ExternalTaskHelperMixin[Noop]):
         self.run_incus_cmd.return_value = "environment:\n  driver: lxc | qemu\n"
         self.assertFalse(self.executor_cls.available())
 
-    def test_available_no_autopkgtest_virt_incus(self):
+    def test_available_no_autopkgtest_virt_incus(self) -> None:
         """Not available() if autopkgtest-virt-incus is not available."""
         self.mock_is_command_available(
             {
@@ -566,7 +583,7 @@ class IncusExecutorCommon(ExternalTaskHelperMixin[Noop]):
         self.run_incus_cmd.return_value = "environment:\n  driver: lxc | qemu\n"
         self.assertFalse(self.executor_cls.available())
 
-    def test_available_no_incus_permission(self):
+    def test_available_no_incus_permission(self) -> None:
         """Test that available() returns False if the worker can't run incus."""
         self.mock_is_command_available(
             {
@@ -579,17 +596,17 @@ class IncusExecutorCommon(ExternalTaskHelperMixin[Noop]):
         )
         self.assertFalse(self.executor_cls.available())
 
-    def test_instantiation_fetches_artifact(self):
+    def test_instantiation_fetches_artifact(self) -> None:
         """Test that instantiating the Executor fetches the artifact."""
         self.assertEqual(self.executor.system_image, self.image_artifact)
 
-    def test_download_image(self):
+    def test_download_image(self) -> None:
         """Test that download_image calls the ImageCache.download_image."""
         self.image_artifact = self.mock_image_download(
             self.debusine_api, create_image=True, system_image=self.uses_images
         )
 
-        def fake_run_incus_cmd(*args: str, **kwargs) -> str:
+        def fake_run_incus_cmd(*args: str, **kwargs: Any) -> str:
             """Fake for run_incus_cmd."""
             if args == ("image", "info", "artifact/42"):
                 raise CalledProcessError(cmd=("incus",) + args, returncode=1)
@@ -610,46 +627,46 @@ class IncusExecutorCommon(ExternalTaskHelperMixin[Noop]):
         expected_name = "artifact/42"
         self.assertEqual(response, expected_name)
 
-    def test_image_name(self):
+    def test_image_name(self) -> None:
         """Test that image_name returns the expected image name."""
         response = self.executor.image_name()
         expected_name = "artifact/42"
         self.assertEqual(response, expected_name)
 
-    def test_autopkgtest_virt_server(self):
+    def test_autopkgtest_virt_server(self) -> None:
         """Test that autopkgtest_virt_server returns incus."""
         self.assertEqual(self.executor.autopkgtest_virt_server(), "incus")
 
-    def test_create(self):
+    def test_create(self) -> None:
         """Test create() return IncusInstance instance."""
-        self.executor._local_path = self.create_temporary_file()
         incus_instance = self.executor.create()
 
         self.assertIsInstance(incus_instance, IncusInstance)
 
     @patch("debusine.tasks.executors.incus.run_incus_cmd")
-    def test_clean_up_image(self, run_incus_cmd):
+    def test_clean_up_image(self, run_incus_cmd: MagicMock) -> None:
         """Test that IncusImageCache.clean_up_image removes incus images."""
+        assert issubclass(self.executor_cls, IncusExecutorMixin)
         self.executor_cls.clean_up_image(42)
         run_incus_cmd.assert_called_once_with("image", "rm", "artifact/42")
 
 
-class IncusLXCExecutorTests(IncusExecutorCommon, TestCase):
+class IncusLXCExecutorTests(IncusExecutorCommonTests):
     """Unit tests for IncusLXCExecutor."""
 
     executor_cls = IncusLXCExecutor
     uses_images = False
 
-    def test_backend_name(self):
+    def test_backend_name(self) -> None:
         """Test that the backend_name attribute was set."""
         self.assertEqual(self.executor_cls.backend_name, "incus-lxc")
 
-    def test_executor_class_finds_incus_lxc(self):
+    def test_executor_class_finds_incus_lxc(self) -> None:
         """Test that executor_class() supports incus-lxc."""
         instance = executor_class("incus-lxc")
         self.assertEqual(instance, self.executor_cls)
 
-    def test_available_no_driver(self):
+    def test_available_no_driver(self) -> None:
         """Test available() returns False if the lxc driver isn't there."""
         self.mock_is_command_available(
             {
@@ -660,7 +677,7 @@ class IncusLXCExecutorTests(IncusExecutorCommon, TestCase):
         self.run_incus_cmd.return_value = "environment:\n  driver: qemu\n"
         self.assertFalse(self.executor_cls.available())
 
-    def test_available_has_driver(self):
+    def test_available_has_driver(self) -> None:
         """Test available() returns True if the lxc driver is there."""
         self.mock_is_command_available(
             {
@@ -671,7 +688,7 @@ class IncusLXCExecutorTests(IncusExecutorCommon, TestCase):
         self.run_incus_cmd.return_value = "environment:\n  driver: lxc\n"
         self.assertTrue(self.executor_cls.available())
 
-    def test_autopkgtest_virt_args(self):
+    def test_autopkgtest_virt_args(self) -> None:
         """Test that autopkgtest_virt_args returns sane arguments."""
         self.assertEqual(
             self.executor.autopkgtest_virt_args(),
@@ -679,28 +696,28 @@ class IncusLXCExecutorTests(IncusExecutorCommon, TestCase):
         )
 
 
-class IncusVMExecutorTests(IncusExecutorCommon, TestCase):
+class IncusVMExecutorTests(IncusExecutorCommonTests):
     """Unit tests for IncusVMExecutor."""
 
     executor_cls = IncusVMExecutor
     uses_images = True
 
-    def test_backend_name(self):
+    def test_backend_name(self) -> None:
         """Test that the backend_name attribute was set."""
         self.assertEqual(self.executor_cls.backend_name, "incus-vm")
 
-    def test_executor_class_finds_incus_vm(self):
+    def test_executor_class_finds_incus_vm(self) -> None:
         """Test that executor_class() supports incus-vm."""
         instance = executor_class("incus-vm")
         self.assertEqual(instance, self.executor_cls)
 
-    def test_available_no_driver(self):
+    def test_available_no_driver(self) -> None:
         """Test available() returns False if the qemu driver isn't there."""
         self.mock_is_command_available({"incus": True})
         self.run_incus_cmd.return_value = "environment:\n  driver: lxc\n"
         self.assertFalse(self.executor_cls.available())
 
-    def test_available_has_driver(self):
+    def test_available_has_driver(self) -> None:
         """Test available() returns True if the qemu driver is there."""
         self.mock_is_command_available(
             {
@@ -711,7 +728,7 @@ class IncusVMExecutorTests(IncusExecutorCommon, TestCase):
         self.run_incus_cmd.return_value = "environment:\n  driver: qemu\n"
         self.assertTrue(self.executor_cls.available())
 
-    def test_autopkgtest_virt_args(self):
+    def test_autopkgtest_virt_args(self) -> None:
         """Test that autopkgtest_virt_args returns sane arguments."""
         self.assertEqual(
             self.executor.autopkgtest_virt_args(),
@@ -722,7 +739,7 @@ class IncusVMExecutorTests(IncusExecutorCommon, TestCase):
 class IncusInstanceTests(TestCase):
     """Tests for IncusInstance class."""
 
-    def setUp(self):
+    def setUp(self) -> None:
         """Initialize test."""
         self.instance = IncusInstance("image-name")
 
@@ -752,13 +769,13 @@ class IncusInstanceTests(TestCase):
 
         self.run_incus_cmd.side_effect = fake_run_incus_cmd
 
-    def patch_spin_until_started(self):
+    def patch_spin_until_started(self) -> None:
         """Patch _spin_until_started() out for testing."""
         patcher = patch.object(self.instance, "_spin_until_started")
         self._spin_until_started_mock = patcher.start()
         self.addCleanup(patcher.stop)
 
-    def test_generate_instance_name(self):
+    def test_generate_instance_name(self) -> None:
         """Test that _generate_instance_name() picks an unused name."""
         rejected = None
 
@@ -788,7 +805,7 @@ class IncusInstanceTests(TestCase):
         self.assertEqual(calls[0], call("info", rejected))
         self.assertEqual(calls[1], call("info", instance_name))
 
-    def test_start(self):
+    def test_start(self) -> None:
         """Test that start() launches an instance."""
         self.patch_spin_until_started()
         self.instance.start()
@@ -802,27 +819,42 @@ class IncusInstanceTests(TestCase):
         self._spin_until_started_mock.assert_called_once_with()
 
     @patch("subprocess.run")
-    def test_spin_until_started(self, run):
-        """Test that spin_until_started() run true until it succeeds."""
+    def test_spin_until_started(self, run: MagicMock) -> None:
+        """Test that spin_until_started() spins until it succeeds."""
         count = 0
 
-        def fake_run(args: list[str], **kwargs) -> CompletedProcess:
+        def fake_run(args: list[str], **kwargs: Any) -> CompletedProcess[str]:
             """
             Fake for subprocess.run().
 
             Pretend to spin once.
             """
             nonlocal count
-            if count == 2:
-                returncode = 0
-                stderr = ""
-            else:
-                count += 1
+            returncode = 0
+            stdout = ""
+            stderr = ""
+            if count == 0:
                 returncode = 1
                 stderr = "Error: VM agent isn't currently running\n"
+            elif count == 1:
+                returncode = 1
+                stderr = (
+                    "Failed to connect to system scope bus via local "
+                    "transport: No such file or directory\n"
+                )
+            elif count == 2:
+                returncode = 1
+                stdout = "initializing\n"
+            elif count == 3:
+                returncode = 1
+                stdout = "starting\n"
+            else:
+                stdout = "running\n"
+            count += 1
             return CompletedProcess(
                 args=args,
                 returncode=returncode,
+                stdout=stdout,
                 stderr=stderr,
             )
 
@@ -832,27 +864,60 @@ class IncusInstanceTests(TestCase):
         self.instance._spin_until_started(sleep_time=0, timeout=2)
 
         run.assert_called_with(
-            ["incus", "exec", self.instance._instance, "true"],
-            stderr=PIPE,
+            [
+                "incus",
+                "exec",
+                self.instance._instance,
+                "systemctl",
+                "is-system-running",
+            ],
+            capture_output=True,
             text=True,
         )
 
     @patch("subprocess.run")
-    def test_spin_until_started_unexpected_failure(self, run):
+    def test_spin_until_started_unexpected_failure(
+        self, run: MagicMock
+    ) -> None:
         """Test that spin_until_started() may raise InstanceNotRunning."""
-        run.return_value = CompletedProcess(
-            args=("...",), returncode=1, stderr="Something weird"
-        )
+        count = 0
+
+        def fake_run(args: list[str], **kwargs: Any) -> CompletedProcess[str]:
+            """
+            Fake for subprocess.run().
+
+            Pretend to spin once.
+            """
+            nonlocal count
+            returncode = 0
+            stdout = ""
+            stderr = ""
+            if count == 0:
+                returncode = 1
+                stderr = "Something weird"
+            else:
+                returncode = 0
+                stdout = "running\n"
+            count += 1
+            return CompletedProcess(
+                args=args,
+                returncode=returncode,
+                stdout=stdout,
+                stderr=stderr,
+            )
+
+        run.side_effect = fake_run
 
         self.instance._instance = "fake-instance"
 
-        with self.assertRaisesRegex(
-            InstanceNotRunning,
-            r"Unexpected response from exec true: 1: Something weird",
+        with self.assertLogsContains(
+            "Unexpected response from exec systemctl is-system-running: "
+            "returncode: 1, stdout: , stderr: Something weird",
+            logger=log,
         ):
-            self.instance._spin_until_started(timeout=5)
+            self.instance._spin_until_started(sleep_time=0, timeout=5)
 
-    def test_spin_until_started_times_out(self):
+    def test_spin_until_started_times_out(self) -> None:
         """Test that spin_until_started() times out."""
         self.instance._instance = "fake-instance"
         with self.assertRaisesRegex(
@@ -862,7 +927,7 @@ class IncusInstanceTests(TestCase):
         ):
             self.instance._spin_until_started(timeout=0)
 
-    def test_restart(self):
+    def test_restart(self) -> None:
         """Test that restart() restarts an instance."""
         self.patch_spin_until_started()
         self.instance.start()
@@ -874,7 +939,7 @@ class IncusInstanceTests(TestCase):
         self.run_incus_cmd.assert_called_with("restart", instance_name)
         self.assertIsNotNone(self.instance._instance)
 
-    def test_stop(self):
+    def test_stop(self) -> None:
         """Test that stop() stops an instance."""
         self.patch_spin_until_started()
         self.instance.start()
@@ -886,7 +951,7 @@ class IncusInstanceTests(TestCase):
         self.run_incus_cmd.assert_called_with("stop", instance_name)
         self.assertIsNone(self.instance._instance)
 
-    def test_file_push(self):
+    def test_file_push(self) -> None:
         """Test that file_push() copies a file into an instance."""
         self.patch_spin_until_started()
         self.instance.start()
@@ -908,7 +973,7 @@ class IncusInstanceTests(TestCase):
             f"{instance_name}/tmp/destination",
         )
 
-    def test_directory_push(self):
+    def test_directory_push(self) -> None:
         """Test that directory_push() copies a directory into an instance."""
         self.patch_spin_until_started()
         self.instance.start()
@@ -939,7 +1004,7 @@ class IncusInstanceTests(TestCase):
             ]
         )
 
-    def test_file_pull(self):
+    def test_file_pull(self) -> None:
         """Test that file_pull() copies a file into an instance."""
         self.patch_spin_until_started()
         self.instance.start()
@@ -953,7 +1018,7 @@ class IncusInstanceTests(TestCase):
         )
 
     @patch("subprocess.run")
-    def test_run(self, run):
+    def test_run(self, run: MagicMock) -> None:
         """Test that run() executes commands as root."""
         self.patch_spin_until_started()
         self.instance.start()
@@ -972,7 +1037,7 @@ class IncusInstanceTests(TestCase):
         )
 
     @patch("subprocess.run")
-    def test_run_as_root(self, run):
+    def test_run_as_root(self, run: MagicMock) -> None:
         """Test that run() executes commands as non-root."""
         self.patch_spin_until_started()
         self.instance.start()
@@ -986,3 +1051,8 @@ class IncusInstanceTests(TestCase):
             stdout=PIPE,
             stderr=PIPE,
         )
+
+
+# Avoid running tests from common base classes.
+del IncusImageCacheCommonTests
+del IncusExecutorCommonTests
