@@ -14,77 +14,72 @@ from pathlib import Path, PurePath
 from shutil import rmtree
 from subprocess import PIPE
 from typing import Any
-from unittest import TestCase, mock
+from unittest import mock
 from unittest.mock import MagicMock, call
 
 from debusine.client.debusine import Debusine
 from debusine.tasks import Noop
 from debusine.tasks.executors.base import (
     ImageNotDownloadedError,
+    InstanceNotRunning,
     executor_class,
 )
-from debusine.tasks.executors.unshare import (
-    InstanceNotRunning,
-    UnshareExecutor,
-    UnshareInstance,
-)
+from debusine.tasks.executors.unshare import UnshareExecutor, UnshareInstance
 from debusine.tasks.tests.helper_mixin import ExternalTaskHelperMixin
-from debusine.test import TestHelpersMixin
+from debusine.test import TestCase
 
 
-class UnshareExecutorTests(
-    TestHelpersMixin, ExternalTaskHelperMixin[Noop], TestCase
-):
+class UnshareExecutorTests(ExternalTaskHelperMixin[Noop], TestCase):
     """Unit tests for UnshareExecutor."""
 
-    def setUp(self):
+    def setUp(self) -> None:
         """Mock the Debusine API for tests."""
         self.debusine_api = MagicMock(spec=Debusine)
         self.image_artifact = self.mock_image_download(self.debusine_api)
 
         self.executor = UnshareExecutor(self.debusine_api, 42)
 
-    def test_backend_name(self):
+    def test_backend_name(self) -> None:
         """Test that the backend_name attribute was set."""
         self.assertEqual(UnshareExecutor.backend_name, "unshare")
 
-    def test_available(self):
+    def test_available(self) -> None:
         """Test that available() returns True if unshare is available."""
         self.mock_is_command_available({"unshare": True})
         self.assertTrue(UnshareExecutor.available())
 
-    def test_available_no_unshare(self):
+    def test_available_no_unshare(self) -> None:
         """Test that available() returns False if unshare is not available."""
         self.mock_is_command_available({"unshare": False})
         self.assertFalse(UnshareExecutor.available())
 
-    def test_instantiation_fetches_artifact(self):
+    def test_instantiation_fetches_artifact(self) -> None:
         """Test that instantiating UnshareExecutor fetches the artifact."""
         self.assertEqual(self.executor.system_image, self.image_artifact)
 
-    def test_download_image(self):
+    def test_download_image(self) -> None:
         """Test that download_image calls ImageCache.download_image."""
         response = self.executor.download_image()
         expected_path = self.image_cache_path / "42/system.tar.xz"
         self.assertEqual(self.executor._local_path, expected_path)
         self.assertEqual(response, str(expected_path))
 
-    def test_image_name(self):
+    def test_image_name(self) -> None:
         """Test that image_name returns the previously downloaded image path."""
         self.executor.download_image()
         response = self.executor.image_name()
         expected_path = self.image_cache_path / "42/system.tar.xz"
         self.assertEqual(response, str(expected_path))
 
-    def test_image_name_not_downloaded(self):
+    def test_image_name_not_downloaded(self) -> None:
         """Test that image_name fails if there is no name."""
         self.assertRaises(ImageNotDownloadedError, self.executor.image_name)
 
-    def test_autopkgtest_virt_server(self):
+    def test_autopkgtest_virt_server(self) -> None:
         """Test that autopkgtest_virt_server returns unshare."""
         self.assertEqual(self.executor.autopkgtest_virt_server(), "unshare")
 
-    def test_autopkgtest_virt_args(self):
+    def test_autopkgtest_virt_args(self) -> None:
         """Test that autopkgtest_virt_args returns sane arguments."""
         self.executor._local_path = Path("/not/used/path/system.tar.xz")
         self.assertEqual(
@@ -99,18 +94,18 @@ class UnshareExecutorTests(
             ],
         )
 
-    def test_autopkgtest_virt_args_before_download(self):
+    def test_autopkgtest_virt_args_before_download(self) -> None:
         """Test that autopkgtest_virt_args requires download."""
         self.assertRaises(
             ImageNotDownloadedError, self.executor.autopkgtest_virt_args
         )
 
-    def test_executor_class_finds_unshare(self):
+    def test_executor_class_finds_unshare(self) -> None:
         """Test that executor_class() supports unshare."""
         instance = executor_class("unshare")
         self.assertEqual(instance, UnshareExecutor)
 
-    def test_create(self):
+    def test_create(self) -> None:
         """Test create() return UnshareInstance instance."""
         self.executor._local_path = self.create_temporary_file()
         unshare_instance = self.executor.create()
@@ -119,17 +114,17 @@ class UnshareExecutorTests(
 
         self.assertEqual(unshare_instance._image, self.executor._local_path)
 
-    def test_create_raise_runtime_error(self):
+    def test_create_raise_runtime_error(self) -> None:
         """Test create() raise ImageNotDownloadedError: _local_path is None."""
         msg_starts_with = r"^UnshareExecutor\.download_image\(\) should have"
         with self.assertRaisesRegex(ImageNotDownloadedError, msg_starts_with):
             self.executor.create()
 
 
-class UnshareInstanceTests(TestHelpersMixin, TestCase):
+class UnshareInstanceTests(TestCase):
     """Tests for UnshareInstance class."""
 
-    def setUp(self):
+    def setUp(self) -> None:
         """Initialize test."""
         self.image_file = self.create_temporary_file()
 
@@ -140,7 +135,7 @@ class UnshareInstanceTests(TestHelpersMixin, TestCase):
             self.run_unshare_command_mocked,
         ) = self.patch_run_unshare_command()
 
-    def tearDown(self):
+    def tearDown(self) -> None:
         """Remove any unpacked instances during tests."""
         if self.instance._extracted:
             rmtree(self.instance._extracted)
@@ -156,7 +151,7 @@ class UnshareInstanceTests(TestHelpersMixin, TestCase):
         self.addCleanup(patcher.stop)
         return patcher, mocked
 
-    def test_start(self):
+    def test_start(self) -> None:
         """start() uncompress the image."""
         _, run_unshare_command_mocked = self.patch_run_unshare_command()
 
@@ -164,6 +159,7 @@ class UnshareInstanceTests(TestHelpersMixin, TestCase):
 
         self.instance.start()
 
+        assert self.instance._extracted is not None
         self.assertEqual(
             stat.S_IMODE(self.instance._extracted.stat().st_mode), 0o755
         )
@@ -192,11 +188,12 @@ class UnshareInstanceTests(TestHelpersMixin, TestCase):
 
         run_unshare_command_mocked.assert_has_calls(expected_calls)
 
-    def test_stop(self):
+    def test_stop(self) -> None:
         """stop() delete the contents using unshare."""
         self.instance.start()
 
         extracted = self.instance._extracted
+        assert extracted is not None
         root = self.instance._root
 
         self.instance.stop()
@@ -208,12 +205,12 @@ class UnshareInstanceTests(TestHelpersMixin, TestCase):
         self.assertIsNone(self.instance._extracted)
         self.assertFalse(extracted.exists())
 
-    def test_restart(self):
+    def test_restart(self) -> None:
         """restart() is noop for a running instance."""
         self.instance.start()
         self.instance.restart()
 
-    def test_do_file_push(self):
+    def test_do_file_push(self) -> None:
         """do_file_push() call _run_unshare_command."""
         self.instance.start()
 
@@ -244,7 +241,7 @@ class UnshareInstanceTests(TestHelpersMixin, TestCase):
             expected_args, stdin=source_open_context, check=True
         )
 
-    def test_file_pull(self):
+    def test_file_pull(self) -> None:
         """file_pull() copies the file."""
         self.instance.start()
 
@@ -266,7 +263,7 @@ class UnshareInstanceTests(TestHelpersMixin, TestCase):
             check=True,
         )
 
-    def test_run_as_non_root(self):
+    def test_run_as_non_root(self) -> None:
         """run(): call _run_unshare_command and return its return value."""
         self.instance.start()
         cmd = ["lintian", "--info"]
@@ -287,7 +284,7 @@ class UnshareInstanceTests(TestHelpersMixin, TestCase):
                         "-c",
                         f"mount --rbind /dev {root}/dev && "
                         f"exec /usr/sbin/chroot {root} "
-                        f"/usr/sbin/runuser -u _debusine -- {' '.join(cmd)}",
+                        f"/sbin/runuser -u _debusine -- {' '.join(cmd)}",
                     ],
                     stderr=PIPE,
                     stdout=PIPE,
@@ -298,7 +295,7 @@ class UnshareInstanceTests(TestHelpersMixin, TestCase):
 
         self.assertEqual(actual, self.run_unshare_command_mocked.return_value)
 
-    def test_run_as_root(self):
+    def test_run_as_root(self) -> None:
         """run(): call _run_unshare_command as root and return."""
         self.instance.start()
         cmd = ["lintian", "--info"]
@@ -329,7 +326,7 @@ class UnshareInstanceTests(TestHelpersMixin, TestCase):
 
         self.assertEqual(actual, self.run_unshare_command_mocked.return_value)
 
-    def patch_unshare_command_args(self, command: str, args: list[str]):
+    def patch_unshare_command_args(self, command: str, args: list[str]) -> None:
         """Patch UnshareExecutor._unshare_command and _unshare_default_args."""
         unshare_command = mock.patch(
             "debusine.tasks.executors.unshare."
@@ -347,7 +344,7 @@ class UnshareInstanceTests(TestHelpersMixin, TestCase):
         unshare_default_args.start()
         self.addCleanup(unshare_default_args.stop)
 
-    def test_run_unshare_command(self):
+    def test_run_unshare_command(self) -> None:
         """run_unshare_command() runs a command, output is as expected."""
         self.run_unshare_command_patcher.stop()
         self.patch_unshare_command_args("echo", ["-n"])
@@ -358,7 +355,7 @@ class UnshareInstanceTests(TestHelpersMixin, TestCase):
         self.assertEqual(completed.stdout, output)
         self.assertEqual(completed.returncode, 0)
 
-    def test_run_unshare_command_with_stderr_stdout(self):
+    def test_run_unshare_command_with_stderr_stdout(self) -> None:
         """run_unshare_command() runs a command with stderr and stdout."""
         self.run_unshare_command_patcher.stop()
 
@@ -384,18 +381,16 @@ class UnshareInstanceTests(TestHelpersMixin, TestCase):
         self.assertEqual(stderr_file_path.read_text(), stderr_contents)
         self.assertEqual(stdout_file_path.read_text(), stdout_contents)
 
-    def test_root(self):
+    def test_root(self) -> None:
         """_root property returns self._extracted / root."""
         self.instance._extracted = self.create_temporary_directory()
 
         self.assertEqual(self.instance._root, self.instance._extracted / "root")
 
-    def test_root_raise_runtime_error(self):
-        """_root property returns self._extracted / root."""
+    def test_root_raise_runtime_error(self) -> None:
+        """start() must be called before using _root property."""
         msg_starts_with = (
             r"^UnshareInstance.start\(\) must be called before using _root$"
         )
         with self.assertRaisesRegex(InstanceNotRunning, msg_starts_with):
-            self.assertEqual(
-                self.instance._root, self.instance._extracted / "root"
-            )
+            self.instance._root
