@@ -258,7 +258,7 @@ class UnshareInstance(InstanceInterface):
         args: list[str],
         text: Literal[True],
         run_as_root: bool = False,
-        **kwargs,
+        **kwargs: Any,
     ) -> CompletedProcess[str]: ...
 
     @overload
@@ -267,7 +267,7 @@ class UnshareInstance(InstanceInterface):
         args: list[str],
         text: Literal[False] | None = None,
         run_as_root: bool = False,
-        **kwargs,
+        **kwargs: Any,
     ) -> CompletedProcess[bytes]: ...
 
     def do_run(
@@ -275,7 +275,7 @@ class UnshareInstance(InstanceInterface):
         args: list[str],
         text: Literal[False] | Literal[True] | None = None,
         run_as_root: bool = False,
-        **kwargs,
+        **kwargs: Any,
     ) -> CompletedProcess[AnyStr]:
         """
         Run a command (as root) in the instance.
@@ -285,7 +285,7 @@ class UnshareInstance(InstanceInterface):
         if not run_as_root:
             username = self.non_root_user
             self.create_user()
-            args = ["/usr/sbin/runuser", "-u", username, "--"] + args
+            args = ["/sbin/runuser", "-u", username, "--"] + args
 
         root = shlex.quote(str(self._root))
         args_quoted = shlex.join(args)
@@ -317,9 +317,8 @@ class UnshareInstance(InstanceInterface):
         cls,
         args: list[str],
         *,
-        stdin_path: Path | None = None,
         text: Literal[True],
-        **kwargs,
+        **kwargs: Any,
     ) -> CompletedProcess[str]: ...
 
     @overload
@@ -328,9 +327,8 @@ class UnshareInstance(InstanceInterface):
         cls,
         args: list[str],
         *,
-        stdin_path: Path | None = None,
         text: Literal[False] | None = None,
-        **kwargs,
+        **kwargs: Any,
     ) -> CompletedProcess[bytes]: ...
 
     @classmethod
