@@ -20,6 +20,7 @@ from typing import Any
 
 from debusine.artifacts import SigningInputArtifact
 from debusine.artifacts.models import ArtifactCategory, CollectionCategory
+from debusine.client.models import RelationType
 from debusine.tasks import BaseTaskWithExecutor, RunCommandTask
 from debusine.tasks.models import (
     ExtractForSigningData,
@@ -74,7 +75,7 @@ class ExtractForSigning(
             return False
 
         executor_available_key = f"executor:{self.backend}:available"
-        return worker_metadata.get(executor_available_key, False)
+        return bool(worker_metadata.get(executor_available_key, False))
 
     def compute_dynamic_data(
         self, task_database: TaskDatabaseInterface
@@ -236,7 +237,11 @@ class ExtractForSigning(
             / "files.json",
             local_files_json_path,
         )
-        return json.loads(local_files_json_path.read_text())
+        manifest = json.loads(local_files_json_path.read_text())
+        # For mypy.  If the manifest isn't a JSON object, we'll get a
+        # run-time error almost immediately afterwards anyway.
+        assert isinstance(manifest, dict)
+        return manifest
 
     def _make_signing_input_artifact(
         self, package: str, metadata: dict[str, Any]
@@ -286,6 +291,7 @@ class ExtractForSigning(
             signing_input_paths,
             local_binary_path.parent,
             trusted_certs=metadata.get("trusted_certs"),
+            binary_package_name=package,
         )
 
     def run(self, execute_directory: Path) -> bool:
@@ -329,7 +335,9 @@ class ExtractForSigning(
                     self._binary_artifacts[package],
                 ):
                     self.debusine.relation_create(
-                        uploaded_artifact.id, related_to_id, "relates-to"
+                        uploaded_artifact.id,
+                        related_to_id,
+                        RelationType.RELATES_TO,
                     )
 
     def cleanup(self) -> None:
