@@ -27,3 +27,7 @@ class Noop(BaseExternalTask[NoopData, BaseDynamicTaskData]):
     def run(self, execute_directory: Path) -> bool:  # noqa: U100
         """Return self.data.result (was sent by the client)."""
         return self.data.result
+
+    def get_label(self) -> str:
+        """Return the task label."""
+        return "noop"
