@@ -11,7 +11,7 @@
 Task to build Debian packages with sbuild.
 
 This task implements the PackageBuild generic task for its task_data:
-https://freexian-team.pages.debian.net/debusine/reference/tasks.html#task-packagebuild
+https://freexian-team.pages.debian.net/debusine/reference/tasks/ontology-generic-tasks.html#task-packagebuild
 """
 
 import email.utils
@@ -41,7 +41,7 @@ from debusine.artifacts.models import (
     CollectionCategory,
     DoseDistCheck,
 )
-from debusine.client.models import RemoteArtifact
+from debusine.client.models import RelationType, RemoteArtifact
 from debusine.tasks import BaseTaskWithExecutor, RunCommandTask, TaskConfigError
 from debusine.tasks.models import SbuildData, SbuildDynamicData
 from debusine.tasks.sbuild_validator_mixin import SbuildValidatorMixin
@@ -65,13 +65,18 @@ class Sbuild(
     ) -> None:
         """Initialize the sbuild task."""
         super().__init__(task_data, dynamic_task_data)
-        self.chroots = None
+        self.chroots: list[str] | None = None
         self.builder = "sbuild"
 
-        # dsc_file Path. Set by self.configure_for_execution()
+        # dsc_file Path. Set by self.fetch_input()
         self._dsc_file: Path | None = None
         self._extra_packages: list[Path] = []
 
+        # Deb822 style sources:
+        self._extra_repository_sources: list[Path] = []
+        # GPG keys (for apt < 2.3.10):
+        self._extra_repository_keys: list[Path] = []
+
     def compute_dynamic_data(
         self, task_database: TaskDatabaseInterface
     ) -> SbuildDynamicData:
@@ -123,7 +128,7 @@ class Sbuild(
         )
 
     @staticmethod
-    def _call_dpkg_architecture():  # pragma: no cover
+    def _call_dpkg_architecture() -> str:  # pragma: no cover
         return (
             subprocess.check_output(["dpkg", "--print-architecture"])
             .decode("utf-8")
@@ -131,7 +136,7 @@ class Sbuild(
         )
 
     @classmethod
-    def analyze_worker(cls):
+    def analyze_worker(cls) -> dict[str, Any]:
         """Report metadata for this task on this worker."""
         metadata = super().analyze_worker()
 
@@ -142,9 +147,6 @@ class Sbuild(
             chroots_key = cls.prefix_with_task_name("chroots")
             metadata[chroots_key] = cls._list_chroots()
 
-        host_arch_key = cls.prefix_with_task_name("host_architecture")
-        metadata[host_arch_key] = cls._call_dpkg_architecture()
-
         return metadata
 
     def can_run_on(self, worker_metadata: dict[str, Any]) -> bool:
@@ -171,7 +173,7 @@ class Sbuild(
         return True
 
     @staticmethod
-    def _call_schroot_list():  # pragma: no cover
+    def _call_schroot_list() -> str:  # pragma: no cover
         return (
             subprocess.check_output(["schroot", "--list"])
             .decode("utf-8")
@@ -195,7 +197,7 @@ class Sbuild(
                 chroots.append(line[7:-7])
         return chroots
 
-    def _update_chroots_list(self):
+    def _update_chroots_list(self) -> None:
         """
         Populate the self.chroots list, if the list is empty.
 
@@ -205,7 +207,7 @@ class Sbuild(
             return
         self.chroots = self._list_chroots()
 
-    def _verify_schroot(self):
+    def _verify_schroot(self) -> bool:
         """Verify a suitable schroot exists."""
         self._update_chroots_list()
 
@@ -266,8 +268,72 @@ class Sbuild(
                 return False
             self.fetch_artifact(artifact_id, destination)
 
+        if self.backend != "schroot":
+            self._prepare_executor()
+
+        self._write_extra_repository_config(destination)
+
         return True
 
+    def _environment_codename(self) -> str:
+        """Return the codename of the environment we're building in."""
+        if self.backend == "schroot":
+            if self.data.distribution is None:
+                raise AssertionError(
+                    "distribution must be set if schroot is in use"
+                )
+            return self.data.distribution
+
+        if self.executor is None:
+            raise ValueError(
+                "self.executor not set, call _prepare_executor() first"
+            )
+
+        return cast(str, self.executor.system_image.data["codename"])
+
+    def _write_extra_repository_config(self, destination: Path) -> None:
+        """Write extra_repositories config to files in destination."""
+        codename = self._environment_codename()
+        # apt < 2.3.10 has no support for keys inline in Signed-By
+        inline_signed_by = codename not in (
+            "jessie",
+            "stretch",
+            "buster",
+            "bullseye",
+            "trusty",
+            "xenial",
+            "bionic",
+            "focal",
+        )
+        for i, extra_repo in enumerate(self.data.extra_repositories or []):
+            if not inline_signed_by:
+                if extra_repo.signing_key:
+                    path = destination / f"extra_apt_key_{i}.asc"
+                    path.write_text(extra_repo.signing_key + "\n")
+                    self._extra_repository_keys.append(path)
+
+            # apt < 1.1 has no support for deb822 sources
+            if codename not in ("jessie", "trusty"):
+                path = destination / f"extra_repository_{i}.sources"
+                source = deb822.Deb822()
+                source["Types"] = "deb"
+                source["URIs"] = extra_repo.url
+                source["Suites"] = extra_repo.suite
+                if extra_repo.components:
+                    source["Components"] = " ".join(extra_repo.components)
+                if extra_repo.signing_key:
+                    if inline_signed_by:
+                        source["Signed-By"] = "\n" + "\n".join(
+                            " " + (line or ".")
+                            for line in extra_repo.signing_key.splitlines()
+                        )
+                    else:
+                        source["Signed-By"] = (
+                            f"/etc/apt/keyrings/extra_apt_key_{i}.asc"
+                        )
+                path.write_text(source.dump())
+                self._extra_repository_sources.append(path)
+
     def _cmdline(self) -> list[str]:
         """
         Build the sbuild command line.
@@ -276,8 +342,10 @@ class Sbuild(
         """
         cmd = [
             self.builder,
-            "--no-clean",
             "--purge-deps=never",
+            # Use a higher-level workflow instead to run both sbuild and
+            # lintian, such as debian_pipeline.
+            "--no-run-lintian",
         ]
         if "any" in self.data.build_components:
             cmd.append("--arch-any")
@@ -293,50 +361,9 @@ class Sbuild(
             cmd.append("--no-source")
 
         cmd.append("--arch=" + self.data.host_architecture)
-
-        if self.backend == "schroot":
-            # Using cast because SbuildTaskData validators enforce that
-            # distribution is not None if backend is schroot
-            cmd.append("--dist=" + cast(str, self.data.distribution))
-        else:
-            # set in configure_for_execution
-            if self.executor is None:
-                raise AssertionError("self.executor not set")
-            distribution = self.executor.system_image.data["codename"]
-            cmd.append(f"--dist={distribution}")
-            if self.backend == "unshare":
-                cmd += [
-                    "--chroot-mode=unshare",
-                    f"--chroot={self.executor.image_name()}",
-                    # Remove any dangling resolv.conf symlink (from
-                    # systemd-resolved installed in the environment, #1071736)
-                    "--chroot-setup-commands=rm -f /etc/resolv.conf",
-                ]
-            else:
-                virt_server = self.executor.autopkgtest_virt_server()
-                cmd += [
-                    "--chroot-mode=autopkgtest",
-                    f"--autopkgtest-virt-server={virt_server}",
-                ] + [
-                    f"--autopkgtest-virt-server-opt={opt.replace('%', '%%')}"
-                    for opt in self.executor.autopkgtest_virt_args()
-                ]
-
-        # binNMU
-        if self.data.binnmu is not None:
-            cmd.append(f"--make-binNMU={self.data.binnmu.changelog}")
-            cmd.append(f"--append-to-version={self.data.binnmu.suffix}")
-            cmd.append("--binNMU=0")
-            if self.data.binnmu.timestamp is not None:
-                rfc5322date = email.utils.format_datetime(
-                    self.data.binnmu.timestamp
-                )
-                cmd.append(f"--binNMU-timestamp={rfc5322date}")
-            maintainer = self.data.binnmu.maintainer
-            if not maintainer:
-                assert self.dynamic_data
-                maintainer = self.dynamic_data.binnmu_maintainer
-            cmd.append(f"--maintainer={maintainer}")
+        cmd += self._cmdline_backend()
+        cmd += self._cmdline_extra_repository()
+        cmd += self._cmdline_binnmu()
 
         if self.data.build_profiles:
             cmd.append(f"--profiles={','.join(self.data.build_profiles)}")
@@ -351,6 +378,90 @@ class Sbuild(
 
         return cmd
 
+    def _cmdline_backend(self) -> list[str]:
+        """Generate command line arguments for the backend and environment."""
+        distribution = self._environment_codename()
+        args: list[str] = [f"--dist={distribution}"]
+        if self.backend == "schroot":
+            return args
+
+        assert self.executor
+        if self.backend == "unshare":
+            args += [
+                "--chroot-mode=unshare",
+                f"--chroot={self.executor.image_name()}",
+                # Remove any dangling resolv.conf symlink (from
+                # systemd-resolved installed in the environment, #1071736)
+                "--chroot-setup-commands=rm -f /etc/resolv.conf",
+            ]
+        else:
+            virt_server = self.executor.autopkgtest_virt_server()
+            args += [
+                "--chroot-mode=autopkgtest",
+                f"--autopkgtest-virt-server={virt_server}",
+            ] + [
+                f"--autopkgtest-virt-server-opt={opt.replace('%', '%%')}"
+                for opt in self.executor.autopkgtest_virt_args()
+            ]
+        return args
+
+    def _cmdline_extra_repository(self) -> list[str]:
+        """Generate the command line arguments for extra_repositories."""
+        args: list[str] = []
+
+        # apt < 1.1 has no support for deb822 sources
+        if self._environment_codename() in ("jessie", "trusty"):
+            for extra_repository in self.data.extra_repositories or []:
+                source = f"deb {extra_repository.url} {extra_repository.suite}"
+                if extra_repository.components:
+                    source += f" {' '.join(extra_repository.components)}"
+                args.append(f"--extra-repository={source}")
+            for key_file in self._extra_repository_keys:
+                args.append(f"--extra-repository-key={key_file}")
+            return args
+
+        # apt < 2.3.10 has no support for keys embedded in Signed-By
+        for path in self._extra_repository_keys:
+            args.append(
+                f"--pre-build-commands=cat {path} | "
+                f"%SBUILD_CHROOT_EXEC sh -c 'mkdir -p /etc/apt/keyrings && "
+                f"cat > /etc/apt/keyrings/{path.name}'"
+            )
+
+        # sbuild has no native deb822 support, yet: #1089735
+        for path in self._extra_repository_sources:
+            args.append(
+                f"--pre-build-commands=cat {path} | "
+                f"%SBUILD_CHROOT_EXEC sh -c 'cat > "
+                f"/etc/apt/sources.list.d/{path.name}'"
+            )
+
+        return args
+
+    def _cmdline_binnmu(self) -> list[str]:
+        """Generate the command line arguments for binnmu."""
+        if self.data.binnmu is None:
+            return []
+
+        args: list[str] = [
+            f"--make-binNMU={self.data.binnmu.changelog}",
+            f"--append-to-version={self.data.binnmu.suffix}",
+            "--binNMU=0",
+        ]
+        if self.data.binnmu.timestamp is not None:
+            rfc5322date = email.utils.format_datetime(
+                self.data.binnmu.timestamp
+            )
+            args.append(f"--binNMU-timestamp={rfc5322date}")
+        maintainer: str | None
+        if self.data.binnmu.maintainer is not None:
+            maintainer = str(self.data.binnmu.maintainer)
+        else:
+            assert self.dynamic_data
+            maintainer = self.dynamic_data.binnmu_maintainer
+        args.append(f"--maintainer={maintainer}")
+        return args
+
     def _cmd_env(self) -> dict[str, str] | None:
         """Set DEB_BUILD_OPTIONS for build profiles."""
         build_options = [
@@ -380,6 +491,35 @@ class Sbuild(
 
         return super().execute()
 
+    def task_succeeded(
+        self,
+        returncode: int | None,  # noqa: U100
+        execute_directory: Path,  # noqa: U100
+    ) -> bool:
+        """
+        Check whether the task succeeded.
+
+        `sbuild` returns 0 for "Status: skipped" (i.e. host architecture not
+        supported by source package); we consider that a failure.
+        """
+        if not super().task_succeeded(returncode, execute_directory):
+            return False
+
+        build_log_path = debusine.utils.find_file_suffixes(
+            execute_directory, [".build"]
+        )
+        if build_log_path is None:
+            return False
+
+        status = ""
+        # Reading the whole build log is inefficient, but shouldn't usually
+        # be too bad in practice.
+        with open(build_log_path, errors="replace") as f:
+            for line in f:
+                if line.startswith("Status: "):
+                    status = line[len("Status: ") :].rstrip("\n")
+        return status == "successful"
+
     @staticmethod
     def _extract_dose3_explanation(
         build_log_path: Path,
@@ -566,7 +706,7 @@ class Sbuild(
         remote_binary_upload: RemoteArtifact | None,
         remote_binary_packages: list[RemoteArtifact],
         remote_signing_input: RemoteArtifact | None,
-    ):
+    ) -> None:
         if not self.debusine:
             raise AssertionError("self.debusine not set")
 
@@ -575,24 +715,26 @@ class Sbuild(
                 self.debusine.relation_create(
                     remote_binary_package.id,
                     source_artifact_id,
-                    "built-using",
+                    RelationType.BUILT_USING,
                 )
 
             if remote_build_log is not None:
                 self.debusine.relation_create(
-                    remote_build_log.id, remote_binary_package.id, "relates-to"
+                    remote_build_log.id,
+                    remote_binary_package.id,
+                    RelationType.RELATES_TO,
                 )
 
             if remote_binary_upload is not None:
                 self.debusine.relation_create(
                     remote_binary_upload.id,
                     remote_binary_package.id,
-                    "extends",
+                    RelationType.EXTENDS,
                 )
                 self.debusine.relation_create(
                     remote_binary_upload.id,
                     remote_binary_package.id,
-                    "relates-to",
+                    RelationType.RELATES_TO,
                 )
 
         if (
@@ -600,7 +742,9 @@ class Sbuild(
             and remote_signing_input is not None
         ):
             self.debusine.relation_create(
-                remote_signing_input.id, remote_binary_upload.id, "relates-to"
+                remote_signing_input.id,
+                remote_binary_upload.id,
+                RelationType.RELATES_TO,
             )
 
     def configure_for_execution(
@@ -618,9 +762,6 @@ class Sbuild(
             )
             return False
 
-        if self.backend != "schroot":
-            self._prepare_executor()
-
         return True
 
     def upload_artifacts(
@@ -650,7 +791,7 @@ class Sbuild(
                     self.debusine.relation_create(
                         remote_build_log.id,
                         source_artifact_id,
-                        "relates-to",
+                        RelationType.RELATES_TO,
                     )
 
             if execution_success:
