@@ -105,7 +105,7 @@ class SimpleSystemImageBuild(SystemImageBuild):
             self.filename = f"{self.data.disk_image.filename}.tar.xz"
 
     @classmethod
-    def analyze_worker(cls):
+    def analyze_worker(cls) -> dict[str, Any]:
         """Report metadata for this task on this worker."""
         metadata = super().analyze_worker()
 
@@ -611,7 +611,7 @@ class SimpleSystemImageBuild(SystemImageBuild):
 
         return True
 
-    def write_overlays(self, destination: Path, overlays: list[str]):
+    def write_overlays(self, destination: Path, overlays: list[str]) -> None:
         """Copy all named overlays into destination."""
         destination.mkdir(exist_ok=True)
         for overlay in overlays:
