@@ -23,21 +23,17 @@ from debusine.client.utils import get_url_contents_sha256sum
 from debusine.tasks import RunCommandTask
 from debusine.tasks.models import (
     BaseDynamicTaskData,
-    BaseTaskData,
+    SystemBootstrapData,
     SystemBootstrapRepository,
     SystemBootstrapRepositoryCheckSignatureWith,
     SystemBootstrapRepositoryKeyring,
 )
 
-TD = TypeVar("TD", bound=BaseTaskData)
+SBD = TypeVar("SBD", bound=SystemBootstrapData)
 
 
-# TODO: this should be generic over SystemBootstrapData instead of
-# BaseTaskData, but we can't currently do that because
-# MmDebstrapVariant can't inherit from
-# SystemBootstrapOptionsVariant
 class SystemBootstrap(
-    abc.ABC, RunCommandTask[TD, BaseDynamicTaskData], Generic[TD]
+    abc.ABC, RunCommandTask[SBD, BaseDynamicTaskData], Generic[SBD]
 ):
     """Implement ontology SystemBootstrap."""
 
@@ -55,7 +51,7 @@ class SystemBootstrap(
             SystemBootstrapRepositoryKeyring, repository.keyring
         )
         keyring, actual_sha256sum = get_url_contents_sha256sum(
-            repo_keyring.url, 100 * 1024 * 1024
+            repo_keyring.url, 100 * 1024 * 1024, allow_file=True
         )
 
         if expected := repo_keyring.sha256sum:
@@ -71,7 +67,9 @@ class SystemBootstrap(
         file = tempfile.NamedTemporaryFile(
             dir=keyring_directory,
             prefix="keyring-repo-",
-            suffix=".asc",
+            # Detect ASCII armoring:
+            # https://www.rfc-editor.org/rfc/rfc4880#section-6.2
+            suffix=".asc" if keyring.startswith(b"-----BEGIN") else ".gpg",
             delete=False,
         )
         Path(file.name).write_bytes(keyring)
@@ -123,7 +121,7 @@ class SystemBootstrap(
         return deb822
 
     @classmethod
-    def _write_deb822s(cls, deb822s: list[Deb822], destination: Path):
+    def _write_deb822s(cls, deb822s: list[Deb822], destination: Path) -> None:
         with destination.open("wb") as f:
             for deb822 in deb822s:
                 deb822.dump(f)
@@ -172,6 +170,7 @@ class SystemBootstrap(
         except KeyError:
             raise ValueError(f"Cannot find components in {release_url}")
 
+        assert isinstance(components, list)
         for component in components:
             if re.search("^[A-Za-z][-/A-Za-z]*$", component) is None:
                 raise ValueError(
