@@ -65,6 +65,7 @@ class LookupTests(TestCase):
                         "category": None,
                         "name_matcher": None,
                         "data_matchers": (),
+                        "lookup_filters": (),
                     },
                 )
             },
@@ -88,6 +89,7 @@ class LookupTests(TestCase):
                         "category": None,
                         "name_matcher": None,
                         "data_matchers": (),
+                        "lookup_filters": (),
                     },
                 )
             },
@@ -111,6 +113,7 @@ class LookupTests(TestCase):
                         "category": ArtifactCategory.SOURCE_PACKAGE,
                         "name_matcher": None,
                         "data_matchers": (),
+                        "lookup_filters": (),
                     },
                 )
             },
@@ -275,6 +278,45 @@ class LookupTests(TestCase):
                 }
             )
 
+    def test_lookup_filter_single(self) -> None:
+        """`lookup__KEY` with a single-lookup value is accepted."""
+        subordinate_lookup = "single"
+        raw_lookup = {
+            "collection": "_@debian:package-build-logs",
+            "lookup__foo": subordinate_lookup,
+        }
+        lookup = task_models.LookupMultiple.parse_obj(raw_lookup)
+        assert isinstance(lookup.__root__[0], task_models.LookupDict)
+        self.assertEqual(
+            lookup.dict()["__root__"][0]["lookup_filters"],
+            (("foo", subordinate_lookup),),
+        )
+        self.assertEqual(lookup.export(), raw_lookup)
+
+    def test_lookup_filter_multiple(self) -> None:
+        """`lookup__KEY` with a multiple-lookup value is accepted."""
+        subordinate_lookup = ["internal@collections/name:build-amd64"]
+        raw_lookup = {
+            "collection": "_@debian:package-build-logs",
+            "lookup__same_work_request": subordinate_lookup,
+        }
+        lookup = task_models.LookupMultiple.parse_obj(raw_lookup)
+        assert isinstance(lookup.__root__[0], task_models.LookupDict)
+        self.assertEqual(
+            lookup.__root__[0].lookup_filters,
+            (
+                (
+                    "same_work_request",
+                    task_models.LookupMultiple.parse_obj(subordinate_lookup),
+                ),
+            ),
+        )
+        self.assertEqual(
+            lookup.dict()["__root__"][0]["lookup_filters"],
+            (("same_work_request", tuple(subordinate_lookup)),),
+        )
+        self.assertEqual(lookup.export(), raw_lookup)
+
     def test_list(self) -> None:
         """Multiple-item lookups accept lists."""
         raw_lookup = [
@@ -294,6 +336,7 @@ class LookupTests(TestCase):
                         "category": None,
                         "name_matcher": None,
                         "data_matchers": (),
+                        "lookup_filters": (),
                     },
                     {
                         "collection": "kali@debian:archive",
@@ -301,6 +344,7 @@ class LookupTests(TestCase):
                         "category": None,
                         "name_matcher": None,
                         "data_matchers": (),
+                        "lookup_filters": (),
                     },
                     "123@artifacts",
                     124,
@@ -343,18 +387,19 @@ class ActionRetryWithDelaysTests(TestCase):
 class EnumTests(TestCase):
     """Tests for SystemImageBuild task data."""
 
-    def test_enums_str(self):
+    def test_enums_str(self) -> None:
         """Test enum stringification."""
         for enum_cls in (
             task_models.AutopkgtestNeedsInternet,
             task_models.BlhcFlags,
+            task_models.DebDiffFlags,
+            task_models.DebootstrapVariant,
             task_models.DiskImageFormat,
             task_models.LintianFailOnSeverity,
             task_models.MmDebstrapVariant,
             task_models.SbuildBuildComponent,
             task_models.SystemBootstrapRepositoryCheckSignatureWith,
             task_models.SystemBootstrapRepositoryType,
-            task_models.SystemBootstrapOptionsVariant,
         ):
             with self.subTest(enum_cls=enum_cls):
                 for el in enum_cls:
@@ -365,39 +410,49 @@ class EnumTests(TestCase):
 class LintianDataTests(TestCase):
     """Tests for Lintian task data."""
 
-    def test_input_validation(self):
+    def test_input_validation(self) -> None:
         """Test LintianInput validation."""
         task_models.LintianInput(source_artifact=1)
-        task_models.LintianInput(binary_artifacts=[1])
-        task_models.LintianInput(binary_artifacts=[1, 2])
+        task_models.LintianInput(
+            binary_artifacts=task_models.LookupMultiple.parse_obj([1])
+        )
+        task_models.LintianInput(
+            binary_artifacts=task_models.LookupMultiple.parse_obj([1, 2])
+        )
 
         error_msg = "One of source_artifact or binary_artifacts must be set"
         with self.assertRaisesRegex(ValueError, error_msg):
             task_models.LintianInput()
         with self.assertRaisesRegex(ValueError, error_msg):
-            task_models.LintianInput(binary_artifacts=[])
+            task_models.LintianInput(
+                binary_artifacts=task_models.empty_lookup_multiple()
+            )
 
 
 class SystemBootstrapDataTests(TestCase):
     """Tests for SystemBootstrap task data."""
 
-    def test_repository_validation(self):
+    def test_repository_validation(self) -> None:
         """Test SystemBootstrapRepository validation."""
         common_kwargs = {
             "mirror": "https://deb.debian.org/deb",
             "suite": "bookworm",
         }
 
-        task_models.SystemBootstrapRepository(**common_kwargs)
+        task_models.SystemBootstrapRepository.parse_obj(common_kwargs)
 
         error_msg = "ensure this value has at least 1 items"
         with self.assertRaisesRegex(ValueError, error_msg):
-            task_models.SystemBootstrapRepository(types=[], **common_kwargs)
+            task_models.SystemBootstrapRepository.parse_obj(
+                dict(types=[], **common_kwargs)
+            )
 
-        task_models.SystemBootstrapRepository(
-            check_signature_with="external",
-            keyring={"url": "https://example.com/keyring_file.txt"},
-            **common_kwargs,
+        task_models.SystemBootstrapRepository.parse_obj(
+            dict(
+                check_signature_with="external",
+                keyring={"url": "https://example.com/keyring_file.txt"},
+                **common_kwargs,
+            )
         )
 
         error_msg = (
@@ -405,34 +460,83 @@ class SystemBootstrapDataTests(TestCase):
             " 'check_signature_with' is set to 'external'"
         )
         with self.assertRaisesRegex(ValueError, error_msg):
-            task_models.SystemBootstrapRepository(
-                check_signature_with="external", **common_kwargs
+            task_models.SystemBootstrapRepository.parse_obj(
+                dict(check_signature_with="external", **common_kwargs)
             )
 
-    def test_repository_validation_duplicate_list_items(self):
+    def test_repository_validation_duplicate_list_items(self) -> None:
         """Test SystemBootstrapRepository validation of duplicate list items."""
         common_kwargs = {
             "mirror": "https://deb.debian.org/deb",
             "suite": "bookworm",
         }
 
-        task_models.SystemBootstrapRepository(**common_kwargs)
+        task_models.SystemBootstrapRepository.parse_obj(common_kwargs)
 
         error_msg = "the list has duplicated items"
         with self.assertRaisesRegex(ValueError, error_msg):
-            task_models.SystemBootstrapRepository(
-                types=["deb", "deb"], **common_kwargs
+            task_models.SystemBootstrapRepository.parse_obj(
+                dict(types=["deb", "deb"], **common_kwargs)
             )
         with self.assertRaisesRegex(ValueError, error_msg):
-            task_models.SystemBootstrapRepository(
-                components=["main", "main"], **common_kwargs
+            task_models.SystemBootstrapRepository.parse_obj(
+                dict(components=["main", "main"], **common_kwargs)
+            )
+
+    def test_keyring_file_url_under_usr_share_keyrings(self) -> None:
+        """Keyring file:// URLs under /usr/share/keyrings/ are allowed."""
+        task_models.SystemBootstrapRepository.parse_obj(
+            {
+                "mirror": "https://deb.debian.org/deb",
+                "suite": "bookworm",
+                "check_signature_with": "external",
+                "keyring": {
+                    "url": (
+                        "file:///usr/share/keyrings/debian-archive-keyring.gpg"
+                    )
+                },
+            }
+        )
+
+    def test_keyring_file_url_not_under_usr_share_keyrings(self) -> None:
+        """Keyring file:// URLs not under /usr/share/keyrings/ are rejected."""
+        common_kwargs = {
+            "mirror": "https://deb.debian.org/deb",
+            "suite": "bookworm",
+            "check_signature_with": "external",
+        }
+
+        error_msg = (
+            "file:// URLs for keyrings must be under /usr/share/keyrings/"
+        )
+        with self.assertRaisesRegex(ValueError, error_msg):
+            task_models.SystemBootstrapRepository.parse_obj(
+                {"keyring": {"url": "file:///etc/passwd", **common_kwargs}}
+            )
+        with self.assertRaisesRegex(ValueError, error_msg):
+            task_models.SystemBootstrapRepository.parse_obj(
+                {
+                    "keyring": {
+                        "url": "file:///usr/share/keyringssuffix",
+                        **common_kwargs,
+                    }
+                }
+            )
+        with self.assertRaisesRegex(ValueError, error_msg):
+            task_models.SystemBootstrapRepository.parse_obj(
+                {
+                    "keyring": {
+                        "url": "file:///usr/share/keyrings/../escape",
+                        **common_kwargs,
+                    }
+                }
             )
 
 
 class AutopkgtestDataTests(TestCase):
     """Tests for Autopkgtest task data."""
 
-    def test_validation(self):
+    def test_validation(self) -> None:
         """Test AutopkgtestData validation."""
         common_kwargs = {
             "input": {
@@ -443,7 +547,7 @@ class AutopkgtestDataTests(TestCase):
             "environment": 1,
         }
 
-        task_models.AutopkgtestData(**common_kwargs)
+        task_models.AutopkgtestData.parse_obj(common_kwargs)
 
         for field in ("global", "factor", "short", "install", "test", "copy"):
             with self.subTest(field=field):
@@ -452,21 +556,104 @@ class AutopkgtestDataTests(TestCase):
                     f"  ensure this value is greater than or equal to 0"
                 )
                 with self.assertRaisesRegex(ValueError, error_msg):
-                    task_models.AutopkgtestData(
-                        timeout={field: -1}, **common_kwargs
+                    task_models.AutopkgtestData.parse_obj(
+                        dict(timeout={field: -1}, **common_kwargs)
                     )
 
 
+class SbuildExtraRepositoryTests(TestCase):
+    """Tests for SbuildExtraRepository."""
+
+    def test_valid_sources(self) -> None:
+        """Accept a typical valid source."""
+        task_models.SbuildExtraRepository.parse_obj(
+            {
+                "url": "http://deb.debian.org/debian",
+                "suite": "bookworm",
+                "components": ["main", "non-free-firmware"],
+            }
+        )
+
+    def test_source_with_key(self) -> None:
+        """Accept a valid flat source with a key."""
+        task_models.SbuildExtraRepository.parse_obj(
+            {
+                "url": "http://example.com/repo",
+                "suite": "./",
+                "signing_key": "PUBLIC KEY",
+            }
+        )
+
+    def test_file_url(self) -> None:
+        """Reject a file:// URL."""
+        with self.assertRaisesRegex(
+            ValueError, "invalid or missing URL scheme"
+        ):
+            task_models.SbuildExtraRepository.parse_obj(
+                {
+                    "url": "file:/etc/passwd",
+                    "suite": "bookworm",
+                    "components": ["main"],
+                }
+            )
+
+    def test_crazy_suite(self) -> None:
+        """Reject an unreasonable suite."""
+        with self.assertRaisesRegex(ValueError, "Invalid suite b@d_suite"):
+            task_models.SbuildExtraRepository.parse_obj(
+                {
+                    "url": "http://example.com/",
+                    "suite": "b@d_suite",
+                    "components": ["main"],
+                }
+            )
+
+    def test_flat_repository_with_component(self) -> None:
+        """Reject an flat repository with a component."""
+        with self.assertRaisesRegex(
+            ValueError, "Components cannot be specified for a flat repository"
+        ):
+            task_models.SbuildExtraRepository.parse_obj(
+                {
+                    "url": "http://example.com/",
+                    "suite": "flat/",
+                    "components": ["main"],
+                }
+            )
+
+    def test_crazy_component(self) -> None:
+        """Reject an unreasonable component."""
+        with self.assertRaisesRegex(ValueError, "Invalid component foo/bar"):
+            task_models.SbuildExtraRepository.parse_obj(
+                {
+                    "url": "http://example.com/",
+                    "suite": "bookworm",
+                    "components": ["foo/bar"],
+                }
+            )
+
+
+class SbuildDynamicDataTests(TestCase):
+    """Tests for SbuildDynamicData."""
+
+    def test_binnmu_maintainer_not_strict(self) -> None:
+        """binnmu_maintainer does not have to be a deliverable email address."""
+        task_models.SbuildDynamicData(
+            input_source_artifact_id=1,
+            binnmu_maintainer="Debusine <noreply@debusine-dev>",
+        )
+
+
 class ImageCacheUsageLogEntryTests(TestCase):
     """Tests for ImageCacheUsageLogEntry."""
 
-    def test_timestamp_validation_succeeds(self):
+    def test_timestamp_validation_succeeds(self) -> None:
         """Test ImageCacheUsageLogEntry accepts aware timestamp."""
         task_models.ImageCacheUsageLogEntry(
             filename="foo", timestamp=datetime.now(timezone.utc)
         )
 
-    def test_timestamp_validation_fails(self):
+    def test_timestamp_validation_fails(self) -> None:
         """Test ImageCacheUsageLogEntry rejects naive timestamp."""
         with self.assertRaisesRegex(ValueError, "timestamp is TZ-naive"):
             task_models.ImageCacheUsageLogEntry(
@@ -477,11 +664,11 @@ class ImageCacheUsageLogEntryTests(TestCase):
 class ImageCacheUsageLogTests(TestCase):
     """Tests for ImageCacheUsageLog."""
 
-    def test_version_validation_succeeds(self):
+    def test_version_validation_succeeds(self) -> None:
         """Test ImageCacheUsageLog accepts version 1."""
         task_models.ImageCacheUsageLog(version=1)
 
-    def test_version_validation_fails(self):
+    def test_version_validation_fails(self) -> None:
         """Test ImageCacheUsageLog rejects version 99."""
         with self.assertRaisesRegex(ValueError, "Unknown usage log version 99"):
             task_models.ImageCacheUsageLog(version=99)
@@ -490,64 +677,78 @@ class ImageCacheUsageLogTests(TestCase):
 class ExtractForSigningDataTests(TestCase):
     """Tests for ExtractForSigningData."""
 
-    def test_backend_validation_succeeds(self):
+    def test_backend_validation_succeeds(self) -> None:
         """Test ExtractForSigningData accepts no backend or backend=auto."""
-        task_models.ExtractForSigningData(
-            environment=1,
-            input={"template_artifact": 2, "binary_artifacts": [3]},
+        task_models.ExtractForSigningData.parse_obj(
+            {
+                "environment": 1,
+                "input": {"template_artifact": 2, "binary_artifacts": [3]},
+            }
         )
-        task_models.ExtractForSigningData(
-            environment=1,
-            backend=task_models.BackendType.AUTO,
-            input={"template_artifact": 2, "binary_artifacts": [3]},
+        task_models.ExtractForSigningData.parse_obj(
+            {
+                "environment": 1,
+                "backend": task_models.BackendType.AUTO,
+                "input": {"template_artifact": 2, "binary_artifacts": [3]},
+            }
         )
 
-    def test_backend_validation_fails(self):
+    def test_backend_validation_fails(self) -> None:
         """Test ExtractForSigningData rejects backends other than auto."""
         with self.assertRaisesRegex(
             ValueError,
             'ExtractForSigning only accepts backend "auto", not "qemu"',
         ):
-            task_models.ExtractForSigningData(
-                environment=1,
-                backend=task_models.BackendType.QEMU,
-                input={"template_artifact": 2, "binary_artifacts": [3]},
+            task_models.ExtractForSigningData.parse_obj(
+                {
+                    "environment": 1,
+                    "backend": task_models.BackendType.QEMU,
+                    "input": {"template_artifact": 2, "binary_artifacts": [3]},
+                }
             )
 
 
 class AssembleSignedSourceDataTests(TestCase):
     """Tests for AssembleSignedSourceData."""
 
-    def test_backend_validation_succeeds(self):
+    def test_backend_validation_succeeds(self) -> None:
         """Test AssembleSignedSourceData accepts no backend or backend=auto."""
-        task_models.AssembleSignedSourceData(
-            environment=1, template=2, signed=[3]
+        task_models.AssembleSignedSourceData.parse_obj(
+            {
+                "environment": 1,
+                "template": 2,
+                "signed": [3],
+            }
         )
-        task_models.AssembleSignedSourceData(
-            environment=1,
-            backend=task_models.BackendType.AUTO,
-            template=2,
-            signed=[3],
+        task_models.AssembleSignedSourceData.parse_obj(
+            {
+                "environment": 1,
+                "backend": task_models.BackendType.AUTO,
+                "template": 2,
+                "signed": [3],
+            }
         )
 
-    def test_backend_validation_fails(self):
+    def test_backend_validation_fails(self) -> None:
         """Test AssembleSignedSourceData rejects backends other than auto."""
         with self.assertRaisesRegex(
             ValueError,
             'AssembleSignedSource only accepts backend "auto", not "qemu"',
         ):
-            task_models.AssembleSignedSourceData(
-                environment=1,
-                backend=task_models.BackendType.QEMU,
-                template=2,
-                signed=[3],
+            task_models.AssembleSignedSourceData.parse_obj(
+                {
+                    "environment": 1,
+                    "backend": task_models.BackendType.QEMU,
+                    "template": 2,
+                    "signed": [3],
+                }
             )
 
 
 class EventReactionsTests(TestCase):
     """Tests for EventReactions."""
 
-    def test_deserialize(self):
+    def test_deserialize(self) -> None:
         """Test basic deserialization."""
         task_models.EventReactions.parse_raw(
             """
@@ -562,8 +763,8 @@ class EventReactionsTests(TestCase):
                         },
                         "name_template": "{package}_{version}",
                         "variables": {
-                            "package": "deb_fields.Package",
-                            "version": "deb_fields.Version"
+                            "$package": "deb_fields.Package",
+                            "$version": "deb_fields.Version"
                         }
                     }
                 ],
