@@ -11,25 +11,29 @@
 
 from unittest import TestCase
 
-from debusine.tasks import TaskConfigError
-from debusine.tasks.noop import Noop
+from debusine.tasks import Noop, TaskConfigError
 
 
 class NoopTaskTests(TestCase):
     """Test the noop task."""
 
-    def test_execute_true(self):
+    def test_execute_true(self) -> None:
         """Assert execute() return True (result is True)."""
         noop = Noop({"result": True})
         self.assertTrue(noop.execute())
 
-    def test_execute_false(self):
+    def test_execute_false(self) -> None:
         """Assert execute() return False (result is False)."""
         noop = Noop({"result": False})
         self.assertFalse(noop.execute())
 
-    def test_no_additional_properties(self):
+    def test_no_additional_properties(self) -> None:
         """Assert no additional properties."""
         error_msg = "extra fields not permitted"
         with self.assertRaisesRegex(TaskConfigError, error_msg):
             Noop({"result": "True", "input": {}})
+
+    def test_label(self) -> None:
+        """Test get_label."""
+        noop = Noop({"result": True})
+        self.assertEqual(noop.get_label(), "noop")
