@@ -10,17 +10,19 @@
 """Unit tests for the SbuildValidatorMixin class."""
 
 from pathlib import Path
-from unittest import TestCase
 
 from debusine.tasks import BaseExternalTask
 from debusine.tasks.models import BaseDynamicTaskData, BaseTaskData
 from debusine.tasks.sbuild_validator_mixin import SbuildValidatorMixin
-from debusine.test import TestHelpersMixin
+from debusine.tasks.tests.helper_mixin import TestTaskMixin
+from debusine.test import TestCase
 from debusine.utils import calculate_hash
 
 
 class TaskWithSbuildValidatorMixin(
-    SbuildValidatorMixin, BaseExternalTask[BaseTaskData, BaseDynamicTaskData]
+    TestTaskMixin,
+    SbuildValidatorMixin,
+    BaseExternalTask[BaseTaskData, BaseDynamicTaskData],
 ):
     """Minimum class to test SbuildValidatorMixin."""
 
@@ -29,14 +31,14 @@ class TaskWithSbuildValidatorMixin(
         raise NotImplementedError()
 
 
-class SbuildValidatorMixinTests(TestHelpersMixin, TestCase):
+class SbuildValidatorMixinTests(TestCase):
     """Tests for SbuildValidatorMixin class."""
 
-    def setUp(self):
+    def setUp(self) -> None:
         """Set up test."""
         self.sbuild_validator_mixin = TaskWithSbuildValidatorMixin({})
 
-    def test_check_directory_for_consistency_errors_no_errors(self):
+    def test_check_directory_for_consistency_errors_no_errors(self) -> None:
         """check_directory_for_consistency_errors() return no errors."""
         sbuild_directory = self.create_temporary_directory()
 
@@ -65,7 +67,9 @@ class SbuildValidatorMixinTests(TestHelpersMixin, TestCase):
             [],
         )
 
-    def test_check_directory_for_consistency_errors_no_changes_file(self):
+    def test_check_directory_for_consistency_errors_no_changes_file(
+        self,
+    ) -> None:
         """check_directory_for_consistency_errors() handles missing .changes."""
         sbuild_directory = self.create_temporary_directory()
 
@@ -73,7 +77,7 @@ class SbuildValidatorMixinTests(TestHelpersMixin, TestCase):
             sbuild_directory
         )
 
-    def test_check_directory_for_consistency_errors_missing_file(self):
+    def test_check_directory_for_consistency_errors_missing_file(self) -> None:
         """check_directory_for_consistency_errors() return one file missing."""
         sbuild_directory = self.create_temporary_directory()
 
@@ -101,7 +105,9 @@ class SbuildValidatorMixinTests(TestHelpersMixin, TestCase):
             expected_errors,
         )
 
-    def test_check_directory_for_consistency_errors_files_in_dsc_no_error(self):
+    def test_check_directory_for_consistency_errors_files_in_dsc_no_error(
+        self,
+    ) -> None:
         """
         check_directory_for_consistency_errors(): no error.
 
@@ -123,7 +129,7 @@ class SbuildValidatorMixinTests(TestHelpersMixin, TestCase):
             [],
         )
 
-    def test_check_directory_for_consistency_errors_invalid_hash(self):
+    def test_check_directory_for_consistency_errors_invalid_hash(self) -> None:
         """
         check_directory_for_consistency_errors() return an error.
 
@@ -155,7 +161,7 @@ class SbuildValidatorMixinTests(TestHelpersMixin, TestCase):
             actual_hash: str,
             expected_size: int,
             actual_size: int,
-        ):
+        ) -> str:
             return (
                 f'{hash_algo} for file {filename} does not match: '
                 f'expected: {expected_hash} actual: {actual_hash}. '
@@ -196,7 +202,7 @@ class SbuildValidatorMixinTests(TestHelpersMixin, TestCase):
             expected_errors,
         )
 
-    def test_check_directory_for_consistency_errors_extra_file(self):
+    def test_check_directory_for_consistency_errors_extra_file(self) -> None:
         """
         check_directory_for_consistency_errors() return an error.
 
