@@ -10,15 +10,18 @@
 """Unit tests for SystemBootstrap class."""
 import copy
 import hashlib
+import io
 import textwrap
 from pathlib import Path
-from unittest import TestCase, mock
+from typing import Any, cast
+from unittest import mock
 
 try:
     import pydantic.v1 as pydantic
 except ImportError:
     import pydantic as pydantic  # type: ignore
 
+import requests
 import responses
 
 from debusine.client.exceptions import ContentValidationError
@@ -32,7 +35,7 @@ from debusine.tasks.models import (
 )
 from debusine.tasks.systembootstrap import SystemBootstrap
 from debusine.tasks.tests.helper_mixin import ExternalTaskHelperMixin
-from debusine.test import TestHelpersMixin
+from debusine.test import TestCase
 
 
 class SystemBootstrapImpl(SystemBootstrap[SystemBootstrapData]):
@@ -43,7 +46,7 @@ class SystemBootstrapImpl(SystemBootstrap[SystemBootstrapData]):
 
 
 class SystemBootstrapTests(
-    TestHelpersMixin, ExternalTaskHelperMixin[SystemBootstrapImpl], TestCase
+    ExternalTaskHelperMixin[SystemBootstrapImpl], TestCase
 ):
     """Tests SystemBootstrap class."""
 
@@ -70,16 +73,19 @@ class SystemBootstrapTests(
         ],
     }
 
-    def setUp(self):
+    def setUp(self) -> None:
         """Initialize test."""
         self.configure_task()
 
-    def test_configure_task(self):
+    def test_configure_task(self) -> None:
         """Ensure self.SAMPLE_TASK_DATA is valid."""
         self.configure_task(self.SAMPLE_TASK_DATA)
 
         # Test default options
-        sample_repositories = self.SAMPLE_TASK_DATA["bootstrap_repositories"]
+        sample_repositories = cast(
+            list[dict[str, Any]],
+            self.SAMPLE_TASK_DATA["bootstrap_repositories"],
+        )
         actual_repositories = self.task.data.bootstrap_repositories
 
         # First sample repository does not have "types" nor
@@ -97,12 +103,15 @@ class SystemBootstrapTests(
             actual_repositories[1].check_signature_with, "no-check"
         )
 
-    def test_configure_task_raise_value_error_missing_keyring(self):
+    def test_configure_task_raise_value_error_missing_keyring(self) -> None:
         """configure_task() raise ValueError: missing keyring parameter."""
         task_data = copy.deepcopy(self.SAMPLE_TASK_DATA)
-        task_data["bootstrap_repositories"][0][
-            "check_signature_with"
-        ] = "external"
+        bootstrap_repo = cast(
+            list[dict[str, Any]],
+            copy.deepcopy(self.SAMPLE_TASK_DATA["bootstrap_repositories"]),
+        )[0]
+        bootstrap_repo["check_signature_with"] = "external"
+        task_data["bootstrap_repositories"] = [bootstrap_repo]
 
         msg = (
             r"repository requires 'keyring':"
@@ -123,7 +132,9 @@ class SystemBootstrapTests(
         keyring: SystemBootstrapRepositoryKeyring | None = None
         if keyring_url is not None:
             keyring = SystemBootstrapRepositoryKeyring(
-                url=pydantic.parse_obj_as(pydantic.AnyUrl, keyring_url),
+                url=pydantic.parse_obj_as(
+                    pydantic.AnyUrl | pydantic.FileUrl, keyring_url
+                ),
                 sha256sum=keyring_sha256sum,
             )
         repository = SystemBootstrapRepository(
@@ -137,7 +148,7 @@ class SystemBootstrapTests(
 
         return repository
 
-    def test_deb822_source(self):
+    def test_deb822_source(self) -> None:
         """Generate a correct Deb822 sources file."""
         repository = self.create_and_validate_repo(
             SystemBootstrapRepositoryCheckSignatureWith.SYSTEM
@@ -158,7 +169,7 @@ class SystemBootstrapTests(
 
         self.assertEqual(actual, expected)
 
-    def test_deb822_source_no_components(self):
+    def test_deb822_source_no_components(self) -> None:
         """Use components returned by _list_components_for_suite."""
         repository = self.create_and_validate_repo(
             SystemBootstrapRepositoryCheckSignatureWith.NO_CHECK
@@ -182,7 +193,7 @@ class SystemBootstrapTests(
 
         mocked.assert_called_with(repository.mirror, repository.suite)
 
-    def test_deb822_source_signature_no_check(self):
+    def test_deb822_source_signature_no_check(self) -> None:
         """Generate Deb822 with "trusted"."""
         repository = self.create_and_validate_repo(
             SystemBootstrapRepositoryCheckSignatureWith.NO_CHECK
@@ -197,10 +208,10 @@ class SystemBootstrapTests(
         self.assertEqual(actual["Trusted"], "yes")
 
     @responses.activate
-    def test_deb822_source_file_download_signature(self):
-        """Generate Deb822 and downloads a signature file."""
-        keyring_contents = b"some-keyring"
-        keyring_url = "https://example.com/keyring_file.txt"
+    def test_deb822_source_file_download_keyring_binary(self) -> None:
+        """Generate Deb822 and downloads a binary keyring file."""
+        keyring_contents = b"\x99\x02"
+        keyring_url = "https://example.com/keyring_file.gpg"
         keyring_directory = self.create_temporary_directory()
 
         repository = self.create_and_validate_repo(
@@ -208,6 +219,8 @@ class SystemBootstrapTests(
             keyring_url=keyring_url,
         )
 
+        self.assertIsNotNone(repository.keyring)
+        assert repository.keyring
         responses.add(
             responses.GET, repository.keyring.url, body=keyring_contents
         )
@@ -217,13 +230,43 @@ class SystemBootstrapTests(
         )
 
         signed_by_keyring = Path(actual["Signed-By"])
+        self.assertTrue(signed_by_keyring.name.endswith(".gpg"))
         self.assertEqual(signed_by_keyring.read_bytes(), keyring_contents)
 
         # It was created in the correct directory
         self.assertTrue(signed_by_keyring.is_relative_to(keyring_directory))
 
     @responses.activate
-    def test_deb822_source_keyring_hash_mismatch(self):
+    def test_deb822_source_file_download_keyring_ascii(self) -> None:
+        """Generate Deb822 and downloads an ASCII-armored keyring file."""
+        keyring_contents = b"-----BEGIN PGP PUBLIC KEY BLOCK-----\n"
+        keyring_url = "https://example.com/keyring_file.asc"
+        keyring_directory = self.create_temporary_directory()
+
+        repository = self.create_and_validate_repo(
+            SystemBootstrapRepositoryCheckSignatureWith.EXTERNAL,
+            keyring_url=keyring_url,
+        )
+
+        self.assertIsNotNone(repository.keyring)
+        assert repository.keyring
+        responses.add(
+            responses.GET, repository.keyring.url, body=keyring_contents
+        )
+
+        actual = self.task._deb822_source(
+            repository, keyring_directory=keyring_directory, use_signed_by=True
+        )
+
+        signed_by_keyring = Path(actual["Signed-By"])
+        self.assertTrue(signed_by_keyring.name.endswith(".asc"))
+        self.assertEqual(signed_by_keyring.read_bytes(), keyring_contents)
+
+        # It was created in the correct directory
+        self.assertTrue(signed_by_keyring.is_relative_to(keyring_directory))
+
+    @responses.activate
+    def test_deb822_source_keyring_hash_mismatch(self) -> None:
         """Method raise ValueError: keyring sha256 mismatch."""
         keyring_contents = b"some-keyring"
         keyring_url = "https://example.com/keyring_file.txt"
@@ -237,6 +280,9 @@ class SystemBootstrapTests(
             keyring_sha256sum=expected_sha256,
         )
 
+        self.assertIsNotNone(repository.keyring)
+        assert repository.keyring
+
         responses.add(
             responses.GET, repository.keyring.url, body=keyring_contents
         )
@@ -256,9 +302,8 @@ class SystemBootstrapTests(
             )
 
     @responses.activate
-    def test_deb822_source_keyring_hash_correct(self):
+    def test_deb822_source_keyring_hash_correct(self) -> None:
         """Keyring sha256 is correct."""
-        """Method raise ValueError: keyring sha256 mismatch."""
         keyring_contents = b"some-keyring"
         keyring_url = "https://example.com/keyring_file.txt"
         keyring_directory = self.create_temporary_directory()
@@ -271,6 +316,9 @@ class SystemBootstrapTests(
             keyring_sha256sum=actual_sha256,
         )
 
+        self.assertIsNotNone(repository.keyring)
+        assert repository.keyring
+
         responses.add(
             responses.GET, repository.keyring.url, body=keyring_contents
         )
@@ -279,19 +327,56 @@ class SystemBootstrapTests(
             repository, keyring_directory=keyring_directory, use_signed_by=True
         )
 
+    def test_deb822_source_keyring_file(self) -> None:
+        """The keyring may be read from a file in /usr/share/keyrings/."""
+        keyring_contents = b"some-keyring"
+        keyring_url = "file:///usr/share/keyrings/debian-archive-keyring.gpg"
+        keyring_directory = self.create_temporary_directory()
+
+        repository = self.create_and_validate_repo(
+            SystemBootstrapRepositoryCheckSignatureWith.EXTERNAL,
+            keyring_url=keyring_url,
+        )
+
+        self.assertIsNotNone(repository.keyring)
+        assert repository.keyring
+
+        response = requests.Response()
+        response.raw = io.BytesIO(keyring_contents)
+        response.status_code = requests.codes.ok
+        response.url = keyring_url
+        with mock.patch(
+            "requests_file.FileAdapter.send", return_value=response
+        ):
+            actual = self.task._deb822_source(
+                repository,
+                keyring_directory=keyring_directory,
+                use_signed_by=True,
+            )
+
+        signed_by_keyring = Path(actual["Signed-By"])
+        self.assertEqual(signed_by_keyring.read_bytes(), keyring_contents)
+
+        # It was created in the correct directory
+        self.assertTrue(signed_by_keyring.is_relative_to(keyring_directory))
+
     @responses.activate
-    def test_generate_deb822_sources_file(self):
+    def test_generate_deb822_sources_file(self) -> None:
         """Assert _generate_deb822_sources_file() return the expected files."""
         keyring_url = "https://example.com/keyring_1.gpg"
         responses.add(responses.GET, keyring_url)
 
         repository = SystemBootstrapRepository(
-            types=["deb"],
+            types=[SystemBootstrapRepositoryType.DEB],
             mirror="https://deb.debian.org/deb",
             suite="bookworm",
             components=["main", "contrib"],
-            check_signature_with="external",
-            keyring={"url": keyring_url},
+            check_signature_with=(
+                SystemBootstrapRepositoryCheckSignatureWith.EXTERNAL
+            ),
+            keyring=SystemBootstrapRepositoryKeyring(
+                url=pydantic.parse_obj_as(pydantic.AnyUrl, keyring_url)
+            ),
         )
 
         for use_signed_by in [True, False]:
@@ -355,7 +440,7 @@ class SystemBootstrapTests(
         return mirror_url, suite, release_url
 
     @responses.activate
-    def test_list_components_for_suite(self):
+    def test_list_components_for_suite(self) -> None:
         """_list_components_for_suite get Release URL and return components."""
         expected_components = [
             "main",
@@ -376,7 +461,7 @@ class SystemBootstrapTests(
         self.assertEqual(actual_components, expected_components)
 
     @responses.activate
-    def test_list_components_for_suite_raise_value_error(self):
+    def test_list_components_for_suite_raise_value_error(self) -> None:
         """
         _list_components_for_suite get Release URL and raise exception.
 
@@ -398,7 +483,9 @@ class SystemBootstrapTests(
             SystemBootstrap._list_components_for_suite(mirror_url, "bookworm")
 
     @responses.activate
-    def test_list_components_raise_value_error_invalid_character_dot(self):
+    def test_list_components_raise_value_error_invalid_character_dot(
+        self,
+    ) -> None:
         """
         _list_components_for_suite raise ValueError: invalid component name.
 
@@ -418,7 +505,9 @@ class SystemBootstrapTests(
             SystemBootstrap._list_components_for_suite(mirror_url, suite)
 
     @responses.activate
-    def test_list_components_raise_value_error_invalid_start_with_slash(self):
+    def test_list_components_raise_value_error_invalid_start_with_slash(
+        self,
+    ) -> None:
         """
         _list_components_for_suite raise ValueError: invalid component name.
 
