@@ -176,7 +176,9 @@ class TestCase(unittest.TestCase):
         return Path(temp_dir.name)
 
     @contextlib.contextmanager
-    def assertRaisesSystemExit(self, exit_code) -> Generator[None, None, None]:
+    def assertRaisesSystemExit(
+        self, exit_code: int
+    ) -> Generator[None, None, None]:
         """Assert that raises SystemExit with the specific exit_code."""
         with self.assertRaisesRegex(
             SystemExit,
@@ -297,6 +299,7 @@ class TestCase(unittest.TestCase):
         cls,
         path: Path,
         files: list[Path],
+        binaries: list[str] | None = None,
         version: str = "2.10-5",
         binnmu: bool = False,
     ) -> dict[str, Any]:
@@ -305,7 +308,7 @@ class TestCase(unittest.TestCase):
             deb822.Changes,
             path,
             files,
-            format_version="1.8",
+            binaries=binaries,
             version=version,
             binnmu=binnmu,
         )
@@ -349,7 +352,6 @@ class TestCase(unittest.TestCase):
         self.assertEqual(source.name, name)
         self.assertEqual(source.version, version)
         self.assertEqual(source.type, "dpkg")
-        self.maxDiff = None
 
         def csum(name: str, text: str) -> str:
             hasher = hashlib.new(name)
