@@ -60,7 +60,7 @@ class Playground(playground.Playground, contextlib.ExitStack):
             self.file_store.configuration = local_config.dict()
             self.file_store.save()
 
-    def __deepcopy__(self, memo) -> Self:
+    def __deepcopy__(self, memo: dict[int, Any]) -> Self:
         """Deep copy of Playground, as used by Django's TestCase."""
         # Standard deepcopy
         cls = self.__class__
@@ -79,7 +79,7 @@ class Playground(playground.Playground, contextlib.ExitStack):
                 )
         return result
 
-    def _deepcopy_memory(self, memo) -> None:
+    def _deepcopy_memory(self, memo: dict[int, Any]) -> None:
         """Preserve the storage of MemoryFileBackend."""
         if self.initial_storages is None:
             self.initial_storages = copy.deepcopy(
@@ -90,7 +90,7 @@ class Playground(playground.Playground, contextlib.ExitStack):
                 self.initial_storages, memo
             )
 
-    def _deepcopy_files(self, memo) -> None:  # noqa: U100
+    def _deepcopy_files(self, memo: dict[int, Any]) -> None:  # noqa: U100
         """Preserve the storage of LocalFileBackend."""
         if not self.saved_store_path.exists():
             self.current_store_path.rename(self.saved_store_path)
@@ -108,7 +108,7 @@ class Playground(playground.Playground, contextlib.ExitStack):
             check=True,
         )
 
-    def create_file_in_backend(self, *args, **kwargs) -> File:
+    def create_file_in_backend(self, *args: Any, **kwargs: Any) -> File:
         """
         Create a temporary file and adds it in the backend.
 
