@@ -11,12 +11,52 @@
 
 from typing import ClassVar
 
+import debusine.db.playground
+from debusine.db.context import context
 from debusine.db.models import Artifact, FileInArtifact, FileStore, Workspace
-from debusine.test.django import TestCase, TransactionTestCase
+from debusine.db.playground import scenarios
+from debusine.test.django import PlaygroundMixin, TestCase, TransactionTestCase
 from debusine.test.playground import Playground
 
 
-class MemoryPlaygroundTestCaseTest(TestCase):
+class IntScenario(scenarios.Scenario):
+    """Scenario used to test modification isolation between test cases."""
+
+    value: int
+
+    def build(self, playground: debusine.db.playground.Playground) -> None:
+        """Build the scenario."""
+        super().build(playground)
+        self.value = 0
+
+
+class ScenarioTestMixins(PlaygroundMixin):
+    """Common scenario-related tests."""
+
+    scenario = scenarios.DefaultContext()
+    int_scenario = IntScenario()
+
+    def test_scenario_instantiated(self) -> None:
+        """Test that the scenario has been instantiated."""
+        self.assertIsNot(self.scenario, self.__class__.scenario)
+        self.assertEqual(
+            getattr(self, "playground").scenarios["scenario"], self.scenario
+        )
+        self.assertIsNotNone(self.scenario.scope)
+        self.assertIsNone(context.scope)
+
+    def test_scenario_local_changes1(self) -> None:
+        """Test local changes isolation between methods."""
+        self.assertEqual(self.int_scenario.value, 0)
+        self.int_scenario.value = 1
+
+    def test_scenario_local_changes2(self) -> None:
+        """Test local changes isolation between methods."""
+        self.assertEqual(self.int_scenario.value, 0)
+        self.int_scenario.value = 1
+
+
+class MemoryPlaygroundTestCaseTest(ScenarioTestMixins, TestCase):
     """Test instantiation of playground in normal test case."""
 
     def test_instantiated(self) -> None:
@@ -27,10 +67,11 @@ class MemoryPlaygroundTestCaseTest(TestCase):
         )
 
 
-class LocalPlaygroundTestCaseTest(TestCase):
+class LocalPlaygroundTestCaseTest(ScenarioTestMixins, TestCase):
     """Test instantiation of playground in normal test case."""
 
     playground_memory_file_store = False
+    scenario = scenarios.DefaultContext()
 
     def test_instantiated(self) -> None:
         """Playground is instantiated."""
@@ -44,15 +85,25 @@ class NoPlaygroundTestCaseTest(TestCase):
     """Test instantiation of playground in normal test case."""
 
     playground_needed = False
+    scenario = scenarios.DefaultContext()
 
     def test_instantiated(self) -> None:
         """Playground is instantiated."""
         self.assertFalse(hasattr(self, "playground"))
 
+    def test_scenario_not_instantiated(self) -> None:
+        """Test that the scenario has not been instantiated."""
+        self.assertFalse(hasattr(self.scenario, "scope"))
+        self.assertIsNone(context.scope)
 
-class MemoryPlaygroundTransactionTestCaseTest(TransactionTestCase):
+
+class MemoryPlaygroundTransactionTestCaseTest(
+    ScenarioTestMixins, TransactionTestCase
+):
     """Test instantiation of playground in normal test case."""
 
+    scenario = scenarios.DefaultContext()
+
     def test_instantiated(self) -> None:
         """Playground is instantiated."""
         self.assertIsInstance(self.playground, Playground)
@@ -61,10 +112,13 @@ class MemoryPlaygroundTransactionTestCaseTest(TransactionTestCase):
         )
 
 
-class LocalPlaygroundTransactionTestCaseTest(TransactionTestCase):
+class LocalPlaygroundTransactionTestCaseTest(
+    ScenarioTestMixins, TransactionTestCase
+):
     """Test instantiation of playground in normal test case."""
 
     playground_memory_file_store = False
+    scenario = scenarios.DefaultContext()
 
     def test_instantiated(self) -> None:
         """Playground is instantiated."""
@@ -78,11 +132,67 @@ class NoPlaygroundTransactionTestCaseTest(TransactionTestCase):
     """Test instantiation of playground in normal test case."""
 
     playground_needed = False
+    scenario = scenarios.DefaultContext()
 
     def test_instantiated(self) -> None:
         """Playground is instantiated."""
         self.assertFalse(hasattr(self, "playground"))
 
+    def test_scenario_not_instantiated(self) -> None:
+        """Test that the scenario has not been instantiated."""
+        self.assertFalse(hasattr(self.scenario, "scope"))
+        self.assertIsNone(context.scope)
+
+
+class _InheritedScenarioTestBase(TestCase):
+    scenario1 = scenarios.DefaultContext()
+
+
+class InheritedScenarioTest(_InheritedScenarioTestBase):
+    """Test inheritance of scenario definitions."""
+
+    scenario2 = scenarios.DefaultContext()
+
+    def test_list_playground_scenarios(self) -> None:
+        """Test list_playground_scenarios method."""
+        self.assertEqual(
+            list(_InheritedScenarioTestBase.list_playground_scenarios()),
+            [("scenario1", self.__class__.scenario1)],
+        )
+        self.assertEqual(
+            list(self.list_playground_scenarios()),
+            [
+                ("scenario1", self.__class__.scenario1),
+                ("scenario2", self.__class__.scenario2),
+            ],
+        )
+
+    def test_scenario_instantiated(self) -> None:
+        """Test that the scenario has been instantiated."""
+        self.assertIsNotNone(self.scenario1.scope)
+        self.assertIsNotNone(self.scenario2.scope)
+        self.assertIsNone(context.scope)
+
+
+class SetCurrentScenarioTestCaseTest(TestCase):
+    """Test scenario annotations."""
+
+    scenario = scenarios.DefaultContext(set_current=True)
+
+    def test_scenario_instantiated(self) -> None:
+        """Test that the scenario has been instantiated."""
+        self.assertEqual(context.scope, self.scenario.scope)
+
+
+class SetCurrentScenarioTransactionTestCaseTest(TransactionTestCase):
+    """Test scenario annotations."""
+
+    scenario = scenarios.DefaultContext(set_current=True)
+
+    def test_scenario_instantiated(self) -> None:
+        """Test that the scenario has been instantiated."""
+        self.assertEqual(context.scope, self.scenario.scope)
+
 
 class PlaygroundTransactionalTestMixin(TestCase):
     """Test restoring state across test methods."""
@@ -101,6 +211,7 @@ class PlaygroundTransactionalTestMixin(TestCase):
     art2: ClassVar[Artifact]
 
     @classmethod
+    @context.disable_permission_checks()
     def setUpTestData(cls) -> None:
         """Create a playground to setup test data."""
         super().setUpTestData()
@@ -129,6 +240,7 @@ class PlaygroundTransactionalTestMixin(TestCase):
         with backend.get_stream(files2[1]) as fd:
             self.assertEqual(fd.read(), b"testtarxz")
 
+    @context.disable_permission_checks()
     def test_alter_file_storage_1(self) -> None:
         """First part of testing transactional file storage behaviour."""
         self.assertTestFixtureUnchanged()
@@ -170,4 +282,5 @@ class PlaygroundTransactionalTestFileStore(PlaygroundTransactionalTestMixin):
 
 
 # Avoid running tests from the common base
+del ScenarioTestMixins
 del PlaygroundTransactionalTestMixin
