@@ -11,7 +11,9 @@
 
 import contextlib
 import copy
-from collections.abc import Generator, Iterator
+import sys
+import traceback
+from collections.abc import Callable, Generator, Iterator
 from datetime import datetime, timedelta, timezone
 from itertools import count
 from typing import Any
@@ -77,7 +79,7 @@ def date_time_to_isoformat_rest_framework(value: datetime) -> str:
     return value_str
 
 
-def create_file_response(**kwargs) -> FileResponse:
+def create_file_response(**kwargs: Any) -> FileResponse:
     """Return a FileResponse. Use defaults for certain fields."""
     defaults: dict[str, Any] = {
         "size": 0,
@@ -90,7 +92,7 @@ def create_file_response(**kwargs) -> FileResponse:
     return FileResponse(**defaults)
 
 
-def create_artifact_response(**kwargs) -> ArtifactResponse:
+def create_artifact_response(**kwargs: Any) -> ArtifactResponse:
     """Return an ArtifactResponse. Use defaults for certain fields."""
     kwargs = kwargs.copy()
     defaults: dict[str, Any] = {
@@ -107,7 +109,7 @@ def create_artifact_response(**kwargs) -> ArtifactResponse:
     return ArtifactResponse(**defaults)
 
 
-def create_work_request_response(**kwargs) -> WorkRequestResponse:
+def create_work_request_response(**kwargs: Any) -> WorkRequestResponse:
     """Return a WorkRequestResponse. Use defaults for certain fields."""
     defaults: dict[str, Any] = {
         "id": 11,
@@ -127,7 +129,9 @@ def create_work_request_response(**kwargs) -> WorkRequestResponse:
     return WorkRequestResponse(**defaults)
 
 
-def create_workflow_template_response(**kwargs) -> WorkflowTemplateResponse:
+def create_workflow_template_response(
+    **kwargs: Any,
+) -> WorkflowTemplateResponse:
     """Return a WorkflowTemplateResponse. Use defaults for certain fields."""
     defaults: dict[str, Any] = {
         "id": 11,
@@ -166,3 +170,34 @@ def preserve_task_registry() -> Generator[None, None, None]:
         yield
     finally:
         BaseTask._sub_tasks = orig
+
+
+@contextlib.contextmanager
+def print_sql_and_stack() -> Generator[None, None, None]:
+    """Print each SQL query executed, with python stack trace."""
+    from django.db import connection
+
+    class Printer:
+        """Print numbered SQL queries and stack traces."""
+
+        def __init__(self) -> None:
+            """Set the initial counter."""
+            self.index = 0
+
+        def __call__(
+            self,
+            execute: Callable[[str, Any, bool, dict[str, Any]], Any],
+            sql: str,
+            params: Any,
+            many: bool,
+            context: dict[str, Any],
+        ) -> Any:
+            """Interface with the django query executor to print traces."""
+            self.index += 1
+            print(f"#{self.index}:", "-" * 10, "8<", "-" * 10, file=sys.stderr)
+            traceback.print_stack(file=sys.stderr)
+            print("  SQL Query:", sql, file=sys.stderr)
+            return execute(sql, params, many, context)
+
+    with connection.execute_wrapper(Printer()):
+        yield
