@@ -23,3 +23,15 @@ def server_info(
     return {
         "DEBUSINE_FQDN": settings.DEBUSINE_FQDN,
     }
+
+
+def application_context(
+    request: "django.http.HttpRequest",  # noqa: U100
+) -> dict[str, Any]:
+    """Add general server information to the request context."""
+    from debusine.db.context import context
+
+    return {
+        "scope": lambda: context.scope,
+        "workspace": lambda: context.workspace,
+    }
