@@ -9,13 +9,17 @@
 
 """Template tag override_query_parameters."""
 
+from typing import Any
+
 from django import template
 
 register = template.Library()
 
 
 @register.simple_tag(takes_context=True)
-def override_query_parameters(context, **kwargs) -> str:
+def override_query_parameters(
+    context: template.RequestContext, **kwargs: Any
+) -> str:
     """
     Return query with overridden query parameters.
 
