@@ -22,7 +22,7 @@ from debusine.web.templatetags.override_query_parameters import (
 class OverrideQueryParametersTests(TestCase):
     """Tests for override_query_parameters() function."""
 
-    def test_override_query_parameters(self):
+    def test_override_query_parameters(self) -> None:
         """Test override_query_parameters() return modified query parameters."""
         context = MagicMock()
         context.request.GET = QueryDict(query_string="page=1&order=id&asc=1")
