@@ -16,7 +16,6 @@ from typing import Any, ClassVar
 from django import forms
 from django.contrib.auth import get_user_model
 from django.core.files.uploadedfile import SimpleUploadedFile
-from django.test import TestCase
 from django.utils.datastructures import MultiValueDict
 
 try:
@@ -27,16 +26,18 @@ except ImportError:
 import yaml
 
 from debusine.artifacts import PackageBuildLog
-from debusine.artifacts.models import ArtifactCategory
-from debusine.db.models import Artifact, Token, User, Workspace
+from debusine.artifacts.models import ArtifactCategory, DebianPackageBuildLog
+from debusine.db.context import context
+from debusine.db.models import Artifact, Token, User, WorkRequest, Workspace
+from debusine.db.playground import scenarios
 from debusine.tasks import BaseTask
-from debusine.tasks.models import TaskTypes
-from debusine.test import TestHelpersMixin
+from debusine.test.django import TestCase
 from debusine.web.forms import (
     ArtifactForm,
     BootstrapMixin,
     TokenForm,
     WorkRequestForm,
+    WorkflowFilterForm,
     WorkspaceChoiceField,
 )
 
@@ -73,19 +74,19 @@ class BootstrapMixinTests(TestCase):
     form: ClassVar[BootstrapMixinForm]
 
     @classmethod
-    def setUpTestData(cls):
+    def setUpTestData(cls) -> None:
         """Set up common data for tests."""
         super().setUpTestData()
         cls.form = BootstrapMixinForm()
 
-    def test_charfield_bootstrap_class(self):
+    def test_charfield_bootstrap_class(self) -> None:
         """CharField has the correct Bootstrap class."""  # noqa: D403
         char_widget_class = self.form.fields["char_field"].widget.attrs.get(
             "class"
         )
         self.assertEqual(char_widget_class, "form-control")
 
-    def test_charfield_bootstrap_class_add(self):
+    def test_charfield_bootstrap_class_add(self) -> None:
         """CharField has the correct classes."""  # noqa: D403
         char_widget_class = self.form.fields[
             "char_field_extra_class"
@@ -95,14 +96,14 @@ class BootstrapMixinTests(TestCase):
             f"{BootstrapMixinForm.existing_class} form-control",
         )
 
-    def test_choicefield_bootstrap_class(self):
+    def test_choicefield_bootstrap_class(self) -> None:
         """ChoiceField has the correct Bootstrap class."""  # noqa: D403
         choice_widget_class = self.form.fields["choice_field"].widget.attrs.get(
             "class"
         )
         self.assertEqual(choice_widget_class, "form-select")
 
-    def test_choicefield_bootstrap_class_add(self):
+    def test_choicefield_bootstrap_class_add(self) -> None:
         """ChoiceField has the correct classes."""  # noqa: D403
         choice_widget_class = self.form.fields[
             "choice_field_extra_class"
@@ -112,14 +113,14 @@ class BootstrapMixinTests(TestCase):
             f"{BootstrapMixinForm.existing_class} form-select",
         )
 
-    def test_filefield_bootstrap_class(self):
+    def test_filefield_bootstrap_class(self) -> None:
         """FileField has the correct Bootstrap class."""  # noqa: D403
         file_widget_class = self.form.fields["file_field"].widget.attrs.get(
             "class"
         )
         self.assertEqual(file_widget_class, "form-control")
 
-    def test_datetimefield_bootstrap_class(self):
+    def test_datetimefield_bootstrap_class(self) -> None:
         """Test DateField retains its original class."""  # noqa: D403
         date_widget_class = self.form.fields["datetime_field"].widget.attrs.get(
             "class"
@@ -129,7 +130,7 @@ class BootstrapMixinTests(TestCase):
             f"{BootstrapMixinForm.existing_class} form-control",
         )
 
-    def test_datefield_retain_existing_class(self):
+    def test_datefield_retain_existing_class(self) -> None:
         """
         Test DateField retains its original class.
 
@@ -142,12 +143,12 @@ class BootstrapMixinTests(TestCase):
             date_widget_class, f"{BootstrapMixinForm.existing_class}"
         )
 
-    def test_asterisk_set_if_required(self):
+    def test_asterisk_set_if_required(self) -> None:
         """Test asterisk is set if field is required."""
         char_field_required = self.form.fields["char_field_required"]
         self.assertEqual(char_field_required.label_suffix, " *")
 
-    def test_asterisk_added_if_required(self):
+    def test_asterisk_added_if_required(self) -> None:
         """Test asterisk is added if field is required."""
         char_field_required = self.form.fields[
             "char_field_required_with_suffix"
@@ -161,20 +162,20 @@ class TokenFormTests(TestCase):
     user: ClassVar[User]
 
     @classmethod
-    def setUpTestData(cls):
+    def setUpTestData(cls) -> None:
         """Initialize test objects."""
         super().setUpTestData()
         cls.user = get_user_model().objects.create_user(
             username="testuser", password="testpass"
         )
 
-    def test_form_initialization(self):
+    def test_form_initialization(self) -> None:
         """Form initialization set form.user."""
         form = TokenForm(user=self.user)
         self.assertEqual(form.user, self.user)
         self.assertTrue(form.fields["enabled"].initial)
 
-    def test_form_save(self):
+    def test_form_save(self) -> None:
         """Form save the Token."""
         form = TokenForm({"comment": "Test Comment"}, user=self.user)
         self.assertTrue(form.is_valid())
@@ -184,7 +185,7 @@ class TokenFormTests(TestCase):
         self.assertEqual(token.user, self.user)
         self.assertEqual(token.comment, "Test Comment")
 
-    def test_form_save_commit_false(self):
+    def test_form_save_commit_false(self) -> None:
         """Form does not save the token: commit=False."""
         form = TokenForm({"comment": "Test Comment"}, user=self.user)
         self.assertTrue(form.is_valid())
@@ -197,8 +198,9 @@ class TokenFormTests(TestCase):
         with self.assertRaises(Token.DoesNotExist):
             Token.objects.get(comment="Test Comment")
 
-    def test_form_validation(self):
+    def test_form_validation(self) -> None:
         """Form return an error if the "comment" is not valid: too long."""
+        assert Token.comment.field.max_length is not None
         form = TokenForm(
             {"comment": "x" * (Token.comment.field.max_length + 1)},
             user=self.user,
@@ -206,7 +208,7 @@ class TokenFormTests(TestCase):
         self.assertFalse(form.is_valid())
         self.assertIn("comment", form.errors)
 
-    def test_comment_allowed_empty(self):
+    def test_comment_allowed_empty(self) -> None:
         """Form validates if comment is empty."""
         form = TokenForm({"comment": ""}, user=self.user)
         self.assertTrue(form.is_valid())
@@ -219,7 +221,7 @@ class WorkRequestFormTests(TestCase):
     workspace: ClassVar[Workspace]
 
     @classmethod
-    def setUpTestData(cls):
+    def setUpTestData(cls) -> None:
         """Initialize test objects."""
         super().setUpTestData()
         cls.user = get_user_model().objects.create_user(
@@ -227,12 +229,13 @@ class WorkRequestFormTests(TestCase):
         )
         cls.workspace = Workspace.objects.earliest("id")
 
-    def test_form_initialization(self):
+    def test_form_initialization(self) -> None:
         """Form initialization set form.user."""
-        form = WorkRequestForm(user=self.user)
+        form = WorkRequestForm(user=self.user, workspace=self.workspace)
         self.assertEqual(form.user, self.user)
+        self.assertEqual(form.workspace, self.workspace)
 
-    def test_form_save_no_commit(self):
+    def test_form_save_no_commit(self) -> None:
         """Save method is called without committing to the DB."""
         task_name = "sbuild"
         task_data = textwrap.dedent(
@@ -250,11 +253,11 @@ class WorkRequestFormTests(TestCase):
 
         form = WorkRequestForm(
             {
-                "workspace": self.workspace,
                 "task_name": task_name,
                 "task_data": task_data,
             },
             user=self.user,
+            workspace=self.workspace,
         )
         self.assertTrue(form.is_valid())
 
@@ -266,29 +269,31 @@ class WorkRequestFormTests(TestCase):
         self.assertEqual(work_request.task_data, yaml.safe_load(task_data))
         self.assertIsNone(work_request.id)
 
-    def test_form_save_invalid_yaml(self):
+    def test_form_save_invalid_yaml(self) -> None:
         """Try to save the work request, YAML is invalid."""
         form = WorkRequestForm(
             {
-                "workspace": self.workspace,
                 "task_data": "sbuild",
                 "data_yaml": ":",
             },
             user=self.user,
+            workspace=self.workspace,
         )
 
         with self.assertRaises(ValueError):
             # The WorkRequest could not be created: data_yaml is invalid
             form.save(commit=False)
 
-    def test_clean_data_yaml_raise_validation_error(self):
+    def test_clean_data_yaml_raise_validation_error(self) -> None:
         """Raise forms.ValidationError: invalid data."""
-        form = WorkRequestForm({"task_data": ":"}, user=self.user)
+        form = WorkRequestForm(
+            {"task_data": ":"}, user=self.user, workspace=self.workspace
+        )
 
         self.assertFalse(form.is_valid())
-        self.assertRegex(form.errors["task_data"][0], "^Invalid YAML")
+        self.assertRegex(str(form.errors["task_data"][0]), "^Invalid YAML")
 
-    def test_clean_task_name_must_be_valid(self):
+    def test_clean_task_name_must_be_valid(self) -> None:
         """Raise forms.ValidationError: invalid task name."""
         form = WorkRequestForm(
             {
@@ -296,15 +301,16 @@ class WorkRequestFormTests(TestCase):
                 "task_data": "",
             },
             user=self.user,
+            workspace=self.workspace,
         )
 
         self.assertFalse(form.is_valid())
         self.assertRegex(
-            form.errors["task_name"][0],
+            str(form.errors["task_name"][0]),
             "does-not-exist is not one of the available choices.",
         )
 
-    def test_clean_data_task_data_must_be_dict(self):
+    def test_clean_data_task_data_must_be_dict(self) -> None:
         """Raise forms.ValidationError: task data not a dict."""
         form = WorkRequestForm(
             {
@@ -312,15 +318,16 @@ class WorkRequestFormTests(TestCase):
                 "task_data": "[]",
             },
             user=self.user,
+            workspace=self.workspace,
         )
 
         self.assertFalse(form.is_valid())
         self.assertRegex(
-            form.errors["task_data"][0],
+            str(form.errors["task_data"][0]),
             "task data must be a dictionary",
         )
 
-    def test_clean_data_task_data_raise_validation_error(self):
+    def test_clean_data_task_data_raise_validation_error(self) -> None:
         """Raise forms.ValidationError: invalid task data."""
         task_data_yaml = textwrap.dedent(
             """
@@ -339,15 +346,16 @@ class WorkRequestFormTests(TestCase):
                 "task_data": task_data_yaml,
             },
             user=self.user,
+            workspace=self.workspace,
         )
 
         self.assertFalse(form.is_valid())
         self.assertRegex(
-            form.errors["task_data"][0],
+            str(form.errors["task_data"][0]),
             'The backend "auto" requires "environment" to be set',
         )
 
-    def test_clean_data_yaml_return_dictionary(self):
+    def test_clean_data_yaml_return_dictionary(self) -> None:
         """clean_data_yaml return parsed YAML in a dictionary."""
         task_data = {
             "input": {
@@ -363,6 +371,7 @@ class WorkRequestFormTests(TestCase):
                 "task_name": "sbuild",
             },
             user=self.user,
+            workspace=self.workspace,
         )
 
         form.is_valid()
@@ -371,31 +380,24 @@ class WorkRequestFormTests(TestCase):
 
         self.assertEqual(form.cleaned_data["task_data"], task_data)
 
-    def test_clean_task_data_return_empty_dictionary(self):
+    def test_clean_task_data_return_empty_dictionary(self) -> None:
         """clean_data_yaml return empty dictionary."""
         task_data = ""
         form = WorkRequestForm(
-            {"task_name": "noop", "task_data": task_data}, user=self.user
+            {"task_name": "noop", "task_data": task_data},
+            user=self.user,
+            workspace=self.workspace,
         )
 
         form.is_valid()
         self.assertEqual(form.cleaned_data["task_data"], {})
 
-    def test_form_use_workspace_choice_field(self):
-        """
-        Workspace field is of instance WorkspaceChoiceField.
-
-        Sorting, label, etc. are tested in WorkspaceChoiceFieldTests.
-        """
-        form = ArtifactForm(user=self.user)
-        self.assertIsInstance(form.fields["workspace"], WorkspaceChoiceField)
-
-    def test_task_name(self):
+    def test_task_name(self) -> None:
         """The field "task_name" is a choice field with the expected names."""
-        form = WorkRequestForm(user=self.user)
+        form = WorkRequestForm(user=self.user, workspace=self.workspace)
 
         task_names = [
-            [name, name]
+            (name, name)
             for name in sorted(BaseTask.worker_task_names())
             if "internal" not in name
         ]
@@ -404,66 +406,48 @@ class WorkRequestFormTests(TestCase):
             list(form.fields["task_name"].widget.choices), task_names
         )
 
-    def test_task_name_superuser(self):
-        """The field "task_name" is a choice field with the expected names."""
-        user = get_user_model().objects.create_superuser(
-            username="testsuperuser",
-            email="superuser@mail.none",
-            password="testsuperpass",
-        )
-        form = WorkRequestForm(user=user)
 
-        task_names = [
-            [name, name]
-            for name in sorted(
-                BaseTask.task_names(TaskTypes.WORKER)
-                + BaseTask.task_names(TaskTypes.SERVER)
-            )
-        ]
-
-        self.assertEqual(
-            list(form.fields["task_name"].widget.choices), task_names
-        )
-
-
-class ArtifactFormTests(TestHelpersMixin, TestCase):
+class ArtifactFormTests(TestCase):
     """Tests for ArtifactForm."""
 
     user: ClassVar[User]
+    workspace: ClassVar[Workspace]
     artifact_category: ClassVar[ArtifactCategory]
     form_data: ClassVar[dict[str, Any]]
 
     @classmethod
-    def setUpTestData(cls):
+    def setUpTestData(cls) -> None:
         """Initialize test objects."""
         super().setUpTestData()
+        cls.workspace = cls.playground.get_default_workspace()
         cls.user = get_user_model().objects.create_user(
             username="testuser", password="testpass"
         )
         cls.artifact_category = ArtifactCategory.WORK_REQUEST_DEBUG_LOGS
         cls.form_data = {
             "category": cls.artifact_category,
-            "workspace": Workspace.objects.earliest("id"),
         }
 
-    def test_form_initialization(self):
+    def test_form_initialization(self) -> None:
         """Form initialization set form.user."""
-        form = ArtifactForm(user=self.user)
+        form = ArtifactForm(user=self.user, workspace=self.workspace)
         self.assertEqual(form.user, self.user)
 
-    def test_form_save_set_created_by(self):
+    def test_form_save_set_created_by(self) -> None:
         """Form save(commit=False) set created_by."""
-        form = ArtifactForm(data=self.form_data, user=self.user)
+        form = ArtifactForm(
+            data=self.form_data, user=self.user, workspace=self.workspace
+        )
         self.addCleanup(form.cleanup)
         form.is_valid()
         instance = form.save(commit=False)
 
         self.assertEqual(instance.created_by, self.user)
 
-    def test_form_save_commit(self):
+    @context.disable_permission_checks()
+    def test_form_save_commit(self) -> None:
         """Form save(commit=True) set created_by and saves into the DB."""
         category = self.artifact_category
-        workspace = Workspace.objects.earliest("id")
 
         files_to_upload = {"file1.txt": b"contents1", "file2.txt": b"contents2"}
 
@@ -474,14 +458,15 @@ class ArtifactFormTests(TestHelpersMixin, TestCase):
         form_data = {
             "data": {
                 "category": category,
-                "workspace": workspace.id,
                 "data": {},
                 "expiration_delay_in_days": 1,
             },
             "files": MultiValueDict({"files": simple_uploaded_files}),
         }
 
-        form = ArtifactForm(**form_data, user=self.user)
+        form = ArtifactForm(
+            **form_data, user=self.user, workspace=self.workspace
+        )
         self.addCleanup(form.cleanup)
 
         self.assertTrue(form.is_valid())
@@ -491,14 +476,14 @@ class ArtifactFormTests(TestHelpersMixin, TestCase):
         self.assertEqual(Artifact.objects.get(id=artifact.id), artifact)
 
         self.assertEqual(artifact.category, category)
-        self.assertEqual(artifact.workspace, workspace)
+        self.assertEqual(artifact.workspace, self.workspace)
 
         self.assertEqual(
             artifact.fileinartifact_set.count(), len(files_to_upload)
         )
         self.assertEqual(artifact.expiration_delay, timedelta(days=1))
 
-        file_backend = workspace.default_file_store.get_backend_object()
+        file_backend = self.workspace.default_file_store.get_backend_object()
 
         for file_in_artifact, file_uploaded in zip(
             artifact.fileinartifact_set.order_by("id"), files_to_upload
@@ -509,10 +494,8 @@ class ArtifactFormTests(TestHelpersMixin, TestCase):
             with file_backend.get_stream(file_in_artifact.file) as file:
                 self.assertEqual(file.read(), files_to_upload[file_uploaded])
 
-    def test_form_save_no_commit(self):
+    def test_form_save_no_commit(self) -> None:
         """Form save(commit=False): not saved to the DB."""
-        workspace = Workspace.objects.earliest("id")
-
         files_to_upload = {"file1.txt": b"contents1", "file2.txt": b"contents2"}
 
         simple_uploaded_files = []
@@ -522,13 +505,14 @@ class ArtifactFormTests(TestHelpersMixin, TestCase):
         form_data = {
             "data": {
                 "category": self.artifact_category,
-                "workspace": workspace.id,
                 "data": {},
             },
             "files": MultiValueDict({"files": simple_uploaded_files}),
         }
 
-        form = ArtifactForm(**form_data, user=self.user)
+        form = ArtifactForm(
+            **form_data, user=self.user, workspace=self.workspace
+        )
         self.addCleanup(form.cleanup)
 
         self.assertTrue(form.is_valid())
@@ -541,21 +525,11 @@ class ArtifactFormTests(TestHelpersMixin, TestCase):
         # Nothing is saved
         self.assertEqual(Artifact.objects.count(), 0)
 
-    def test_form_use_workspace_choice_field(self):
-        """
-        Workspace field is of instance WorkspaceChoiceField.
-
-        Sorting, label, etc. are tested in WorkspaceChoiceFieldTests.
-        """
-        form = ArtifactForm(user=self.user)
-        self.assertIsInstance(form.fields["workspace"], WorkspaceChoiceField)
-
-    def test_form_expire_at_past_error(self):
+    def test_form_expire_at_past_error(self) -> None:
         """Field expire_at is in the past: return an error."""
         form_data = {
             "data": {
                 "category": self.artifact_category,
-                "workspace": Workspace.objects.earliest("id").id,
                 "data": {},
                 "expiration_delay_in_days": -1,
             },
@@ -564,7 +538,9 @@ class ArtifactFormTests(TestHelpersMixin, TestCase):
             },
         }
 
-        form = ArtifactForm(**form_data, user=self.user)
+        form = ArtifactForm(
+            **form_data, user=self.user, workspace=self.workspace
+        )
         self.addCleanup(form.cleanup)
         self.assertFalse(form.is_valid())
         self.assertEqual(
@@ -576,16 +552,17 @@ class ArtifactFormTests(TestHelpersMixin, TestCase):
             },
         )
 
-    def test_categories(self):
+    def test_categories(self) -> None:
         """Check expected category choices."""
-        form = ArtifactForm(user=self.user)
+        form = ArtifactForm(user=self.user, workspace=self.workspace)
         choices = []
         for category in sorted(ArtifactCategory):
             choices.append((str(category), str(category)))
 
+        assert isinstance(form.fields["category"], forms.ChoiceField)
         self.assertEqual(form.fields["category"].choices, choices)
 
-    def test_file_zero_bytes(self):
+    def test_file_zero_bytes(self) -> None:
         """The form is valid with a file of zero bytes."""
         category = ArtifactCategory.WORK_REQUEST_DEBUG_LOGS
         file_name = "build.txt"
@@ -593,18 +570,19 @@ class ArtifactFormTests(TestHelpersMixin, TestCase):
         form_data = {
             "data": {
                 "category": category,
-                "workspace": Workspace.objects.earliest("id").id,
                 "data": {},
                 "expire_at": None,
             },
             "files": {"files": [SimpleUploadedFile(file_name, b"")]},
         }
 
-        form = ArtifactForm(**form_data, user=self.user)
+        form = ArtifactForm(
+            **form_data, user=self.user, workspace=self.workspace
+        )
         self.addCleanup(form.cleanup)
         self.assertTrue(form.is_valid())
 
-    def test_files_invalid(self):
+    def test_files_invalid(self) -> None:
         """
         User tries to create an Artifact with invalid data.
 
@@ -624,32 +602,30 @@ class ArtifactFormTests(TestHelpersMixin, TestCase):
         form_data = {
             "data": {
                 "category": category,
-                "workspace": Workspace.objects.earliest("id").id,
                 "data": data,
                 "expire_at": None,
             },
             "files": {"files": [SimpleUploadedFile(file_name, b"content.txt")]},
         }
 
-        form = ArtifactForm(**form_data, user=self.user)
+        form = ArtifactForm(
+            **form_data, user=self.user, workspace=self.workspace
+        )
         self.addCleanup(form.cleanup)
         self.assertFalse(form.is_valid())
 
-        expected_error = None
-        try:
+        with self.assertRaises(pydantic.ValidationError) as raised:
             PackageBuildLog(
                 category=category,
-                data=data,
+                data=DebianPackageBuildLog.parse_obj(data),
                 files={file_name: Path("not-used.txt")},
             )
-        except pydantic.ValidationError as exc:
-            expected_error = exc
 
         self.assertEqual(
-            form.errors, {"files": [str(expected_error.args[0][0].exc)]}
+            form.errors, {"files": [str(raised.exception.args[0][0].exc)]}
         )
 
-    def test_data_invalid(self):
+    def test_data_invalid(self) -> None:
         """
         User tries to create an Artifact with invalid data.
 
@@ -669,46 +645,44 @@ class ArtifactFormTests(TestHelpersMixin, TestCase):
         form_data = {
             "data": {
                 "category": category,
-                "workspace": Workspace.objects.earliest("id").id,
                 "data": data,
                 "expire_at": None,
             },
             "files": {"files": [SimpleUploadedFile(file_name, b"content.txt")]},
         }
 
-        form = ArtifactForm(**form_data, user=self.user)
+        form = ArtifactForm(
+            **form_data, user=self.user, workspace=self.workspace
+        )
         self.addCleanup(form.cleanup)
         self.assertFalse(form.is_valid())
 
-        expected_error = None
-        try:
+        with self.assertRaises(pydantic.ValidationError) as raised:
             PackageBuildLog(
                 category=category,
-                data=data,
+                data=DebianPackageBuildLog.parse_obj(data),
                 files={file_name: Path("not-used.txt")},
             )
-        except pydantic.ValidationError as exc:
-            expected_error = exc
 
         self.assertEqual(
-            form.errors, {"files": [str(expected_error.args[0][0].exc)]}
+            form.errors, {"files": [str(raised.exception.args[0][0].exc)]}
         )
 
 
-class WorkspaceChoiceFieldTests(TestHelpersMixin, TestCase):
+class WorkspaceChoiceFieldTests(TestCase):
     """Tests for WorkspaceChoiceField."""
 
     user: ClassVar[User]
 
     @classmethod
-    def setUpTestData(cls):
+    def setUpTestData(cls) -> None:
         """Initialize test objects."""
         super().setUpTestData()
         cls.user = get_user_model().objects.create_user(
             username="testuser", password="testpass"
         )
 
-    def test_label(self):
+    def test_label(self) -> None:
         """The label is the workspace name."""
         workspace_choice_field = WorkspaceChoiceField(user=self.user)
 
@@ -724,21 +698,25 @@ class WorkspaceChoiceFieldTests(TestHelpersMixin, TestCase):
             ],
         )
 
-    def test_order_by(self):
+    def test_order_by(self) -> None:
         """The field's queryset is ordered by name."""
         workspace_choice_field = WorkspaceChoiceField(user=None)
 
+        assert workspace_choice_field.queryset is not None
         self.assertEqual(
             workspace_choice_field.queryset.query.order_by, ("name",)
         )
 
-    def test_choices_no_user(self):
+    def test_choices_no_user(self) -> None:
         """Choices for if no user: only the public workspaces."""
         default_workspace = Workspace.objects.earliest("id")
         default_workspace.public = False
         default_workspace.save()
 
-        workspace = self.create_workspace(name="It is public", public=True)
+        with context.disable_permission_checks():
+            workspace = self.playground.create_workspace(
+                name="It is public", public=True
+            )
 
         workspace_choice_field = WorkspaceChoiceField(user=None)
 
@@ -751,7 +729,7 @@ class WorkspaceChoiceFieldTests(TestHelpersMixin, TestCase):
             ],
         )
 
-    def test_choices_with_user(self):
+    def test_choices_with_user(self) -> None:
         """Private workspace included in the choices: user has access."""
         workspace = Workspace.objects.earliest("id")
         workspace.public = False
@@ -766,3 +744,110 @@ class WorkspaceChoiceFieldTests(TestHelpersMixin, TestCase):
                 (workspace.pk, workspace.name),
             ],
         )
+
+
+class WorkflowFilterFormTests(TestCase):
+    """Tests for WorkflowFilterForm."""
+
+    scenario = scenarios.DefaultContext(set_current=True)
+
+    def setUp(self) -> None:
+        """Set up tests."""
+        super().setUp()
+
+        self.form = WorkflowFilterForm()
+
+    def test_workflow_templates_field(self) -> None:
+        """Test workflow_templates field."""
+        self.playground.create_workflow_template(
+            "name-1", "noop", workspace=self.scenario.workspace
+        )
+
+        self.playground.create_workflow_template(
+            "name-2",
+            "sbuild",
+            workspace=self.playground.create_workspace(
+                name="unused", public=False
+            ),
+        )
+
+        self.playground.create_workflow(task_name="noop")
+        self.playground.create_workflow(task_name="sbuild", validate=False)
+
+        self.form = WorkflowFilterForm()
+
+        self.assertEqual(
+            self.form.fields["workflow_templates"].widget.choices,
+            [
+                ("noop-template", "noop-template"),
+                ("sbuild-template", "sbuild-template"),
+            ],
+        )
+
+    def test_statuses_field(self) -> None:
+        """Test statuses field."""
+        self.assertEqual(
+            self.form.fields["statuses"].widget.choices,
+            sorted(WorkRequest.Statuses.choices),
+        )
+
+    def test_runtime_statuses_field(self) -> None:
+        """Test runtime_statuses field."""
+        self.assertEqual(
+            self.form.fields["runtime_statuses"].widget.choices,
+            [
+                ("Needs_Input", "Needs Input"),
+                ("Running", "Running"),
+                ("Waiting", "Waiting"),
+                ("Pending", "Pending"),
+                ("Aborted", "Aborted"),
+                ("Completed", "Completed"),
+                ("Blocked", "Blocked"),
+            ],
+        )
+
+    def test_results_field(self) -> None:
+        """Test results field."""
+        self.assertEqual(
+            self.form.fields["results"].widget.choices,
+            [
+                ("error", "Error"),
+                ("failure", "Failure"),
+                ("success", "Success"),
+            ],
+        )
+
+    def test_started_by_field(self) -> None:
+        """Test started_by field."""
+        # "other-1" created a workflow in another workspace:
+        # not listed by started_by field
+        other_1 = self.playground.create_user(username="other-1")
+        workflow_1 = self.playground.create_workflow(created_by=other_1)
+        workflow_1.workspace = self.playground.create_workspace(
+            name="new-workspace"
+        )
+        workflow_1.save()
+
+        # "other-2" has not created any workflow (only a work request):
+        # not listed by started_by field
+        other_2 = self.playground.create_user(username="other-2")
+        self.playground.create_work_request(created_by=other_2)
+
+        # Creates a workflow with self.playground.get_default_user()
+        self.playground.create_workflow()
+
+        self.form = WorkflowFilterForm()
+
+        self.assertEqual(
+            self.form.fields["started_by"].widget.choices,
+            [
+                (
+                    self.scenario.user.username,
+                    self.scenario.user.username,
+                )
+            ],
+        )
+
+    def test_with_failed_work_requests_failed_field(self) -> None:
+        """Test with_failed_work_requests field."""
+        self.assertIn("with_failed_work_requests", self.form.fields)
