@@ -18,6 +18,6 @@ from debusine.web.utils import ui_prototype
 class TestUtils(TestCase):
     """Test view utilities."""
 
-    def test_ui_prototype(self):
+    def test_ui_prototype(self) -> None:
         """Check that it returns its argument."""
         self.assertIs(ui_prototype(TestCase), TestCase)
