@@ -8,33 +8,29 @@
 # contained in the LICENSE file.
 """Django pages URL's for Debusine."""
 from django.urls import include, path
+from django.views.generic import RedirectView
 
+# Scoped URLs
 urlpatterns = [
-    path("", include("debusine.web.urls.homepage", namespace="homepage")),
+    # TODO Preserved for compatibility redirects only
     path(
-        "accounts/",
-        include("debusine.web.urls.accounts", namespace="accounts"),
+        "accounts/oidc_callback/<name>/",
+        RedirectView.as_view(
+            pattern_name="signon:oidc_callback", permanent=True
+        ),
     ),
     path(
-        "work-request/",
-        include("debusine.web.urls.work_requests", namespace="work-requests"),
+        "accounts/bind_identity/<name>/",
+        RedirectView.as_view(
+            pattern_name="signon:bind_identity", permanent=True
+        ),
     ),
     path(
-        "workspace/",
+        "",
+        include("debusine.web.urls.scopes", namespace="scopes"),
+    ),
+    path(
+        "",
         include("debusine.web.urls.workspaces", namespace="workspaces"),
     ),
-    path(
-        "artifact/",
-        include(
-            "debusine.web.urls.artifacts",
-            namespace="artifacts",
-        ),
-    ),
-    path(
-        "user/",
-        include(
-            "debusine.web.urls.user",
-            namespace="user",
-        ),
-    ),
 ]
