@@ -11,7 +11,8 @@
 
 from django.urls import path
 
-from debusine.web.views import (
+from debusine.web.views.auth import (
+    LogoutView,
     UserTokenCreateView,
     UserTokenDeleteView,
     UserTokenListView,
@@ -41,4 +42,12 @@ urlpatterns = [
         UserTokenDeleteView.as_view(),
         name="token-delete",
     ),
+    path(
+        "logout/",
+        LogoutView.as_view(
+            template_name="account/logged_out.html",
+            extra_context={"is_logout_view": True},
+        ),
+        name="logout",
+    ),
 ]
