@@ -31,6 +31,11 @@ urlpatterns = [
         views.WorkRequestRetryView.as_view(),
         name="retry",
     ),
+    path(
+        "<int:pk>/unblock/",
+        views.WorkRequestUnblockView.as_view(),
+        name="unblock",
+    ),
     path(
         "create/",
         views.WorkRequestCreateView.as_view(),
