@@ -10,14 +10,15 @@
 """View for Lintian work request."""
 import json
 import uuid
+from collections.abc import Iterable
+from functools import cached_property
 from typing import Any
 
-from django.urls import reverse
-
 from debusine.artifacts import LintianArtifact
 from debusine.artifacts.models import BaseArtifactDataModel
-from debusine.db.models import Artifact, FileInArtifact, WorkRequest
+from debusine.db.models import Artifact, FileInArtifact
 from debusine.tasks import Lintian
+from debusine.tasks.models import TaskTypes
 from debusine.web.views.work_request import WorkRequestPlugin
 
 
@@ -42,11 +43,19 @@ class LintianView(WorkRequestPlugin):
     """View for Lintian work request."""
 
     template_name = "web/lintian-detail.html"
+    task_type = TaskTypes.WORKER
     task_name = "lintian"
 
+    @cached_property
+    def task(self) -> Lintian:
+        """Return the task to display."""
+        return Lintian(
+            self.work_request.task_data, self.work_request.dynamic_task_data
+        )
+
     @classmethod
     def _artifacts_to_tags(
-        cls, artifacts
+        cls, artifacts: Iterable[Artifact]
     ) -> dict[str, dict[str, LintianAnalysisTag]]:
         """
         Receive a list of artifacts and return dict with file names and tags.
@@ -124,18 +133,20 @@ class LintianView(WorkRequestPlugin):
                 tags_with_information[filename][tag_name] = LintianAnalysisTag(
                     **tags_information[tag_name],
                     uuid=uuid.uuid4(),
-                    occurrences=occurrences
+                    occurrences=occurrences,
                 )
 
         return tags_with_information
 
     @classmethod
-    def _get_summaries(cls, artifacts) -> list[dict[str, Any]]:
+    def _get_summaries(
+        cls, artifacts: Iterable[Artifact]
+    ) -> list[dict[str, Any]]:
         """Return list of all the artifacts.data["summary"] in artifacts."""
         return [artifact.data["summary"] for artifact in artifacts]
 
     @staticmethod
-    def _get_analyses(artifacts) -> list[dict[str, Any]]:
+    def _get_analyses(artifacts: Iterable[Artifact]) -> list[dict[str, Any]]:
         """Return a list with all the parsed analysis.json in artifacts."""
         analyses: list[dict[str, Any]] = []
 
@@ -158,7 +169,7 @@ class LintianView(WorkRequestPlugin):
         return analyses
 
     @staticmethod
-    def _find_lintian_txt_url_path(artifacts) -> str | None:
+    def _find_lintian_txt_url_path(artifacts: Iterable[Artifact]) -> str | None:
         """
         Return the URL path for the lintian.txt file.
 
@@ -168,17 +179,16 @@ class LintianView(WorkRequestPlugin):
             for file_in_artifact in artifact.fileinartifact_set.order_by("id"):
                 if file_in_artifact.path == Lintian.CAPTURE_OUTPUT_FILENAME:
                     return (
-                        reverse(
-                            "artifacts:download",
-                            kwargs={"artifact_id": artifact.id},
-                        )
+                        artifact.get_absolute_url_download()
                         + Lintian.CAPTURE_OUTPUT_FILENAME
                     )
 
         return None
 
     @classmethod
-    def _count_by_severity(cls, artifacts) -> dict[str, int]:
+    def _count_by_severity(
+        cls, artifacts: Iterable[Artifact]
+    ) -> dict[str, int]:
         tags_count_severity = Lintian.generate_severity_count_zero()
         tags_count_severity["overridden"] = 0
 
@@ -230,6 +240,7 @@ class LintianView(WorkRequestPlugin):
                         {
                             "id": source_artifact.id,
                             "files": cls._list_files(source_artifact),
+                            "artifact": source_artifact,
                         }
                     )
 
@@ -245,15 +256,16 @@ class LintianView(WorkRequestPlugin):
                     {
                         "id": binary_artifact_id,
                         "files": cls._list_files(binary_artifact),
+                        "artifact": binary_artifact,
                     }
                 )
 
         return request_data
 
-    def get_context_data(self, work_request: WorkRequest):
+    def get_context_data(self) -> dict[str, Any]:
         """Return the context."""
-        task = Lintian(work_request.task_data, work_request.dynamic_task_data)
-        artifacts = work_request.artifact_set.filter(
+        task = self.task
+        artifacts = self.work_request.artifact_set.filter(
             category=LintianArtifact._category
         ).order_by("id")
 
@@ -261,7 +273,7 @@ class LintianView(WorkRequestPlugin):
 
         return {
             "request_data": self._get_request_data(task),
-            "result": work_request.result,
+            "result": self.work_request.result,
             "lintian_txt_path": lintian_txt_path,
             "fail_on_severity": task.data.fail_on_severity,
             "tags": self._artifacts_to_tags(artifacts),
