@@ -20,7 +20,6 @@ from functools import cached_property
 
 from django.db.models import Model
 from django.template.context import BaseContext
-from django.urls import reverse
 from django.utils.formats import date_format
 from django.utils.html import format_html
 from django.utils.safestring import SafeString
@@ -36,6 +35,7 @@ from debusine.db.models import (
     Workspace,
 )
 from debusine.db.models.artifacts import ARTIFACT_CATEGORY_SHORT_NAMES
+from debusine.web.icons import Icons
 from debusine.web.views.base import Widget
 
 
@@ -135,10 +135,7 @@ class SidebarAction(SidebarItem):
 def create_workspace(workspace: Workspace) -> SidebarItem:
     """Create workspace information."""
     return SidebarAction(
-        url=reverse(
-            "workspaces:detail",
-            kwargs={"wname": workspace.name},
-        ),
+        url=workspace.get_absolute_url(),
         icon="person-workspace",
         label="Workspace",
         value=workspace.name,
@@ -150,7 +147,7 @@ def create_work_request(
 ) -> SidebarItem:
     """Create work request information."""
     kwargs = {
-        "icon": "hammer",
+        "icon": Icons.WORK_REQUEST,
         "label": "Work request",
     }
 
@@ -164,10 +161,7 @@ def create_work_request(
         return SidebarElement(**kwargs)
 
     return SidebarAction(
-        url=reverse(
-            "work_requests:detail",
-            kwargs={"pk": work_request.pk},
-        ),
+        url=work_request.get_absolute_url(),
         **kwargs,
     )
 
@@ -185,7 +179,7 @@ def create_user(
             title = "User"
 
     return SidebarElement(
-        icon="person-circle",
+        icon=Icons.USER,
         label=title,
         value=str(user) if user else None,
     )
@@ -194,7 +188,7 @@ def create_user(
 def create_created_at(at: datetime) -> SidebarItem:
     """Create created_at information."""
     return SidebarElement(
-        icon="clock",
+        icon=Icons.CREATED_AT,
         label="Created",
         value=timesince(at) + " ago",
         detail=date_format(at, "DATETIME_FORMAT"),
@@ -204,7 +198,7 @@ def create_created_at(at: datetime) -> SidebarItem:
 def create_expire_at(at: datetime | None) -> SidebarItem:
     """Create expire_at information."""
     kwargs = {
-        "icon": "hourglass-split",
+        "icon": Icons.EXPIRE_AT,
         "label": "Expiration date",
     }
     if at:
@@ -220,7 +214,7 @@ def create_expire_at(at: datetime | None) -> SidebarItem:
 def create_work_request_status(work_request: WorkRequest) -> SidebarItem:
     """Create information about a work request status and result."""
     kwargs = {
-        "icon": "heart-pulse",
+        "icon": Icons.WORK_REQUEST_STATUS,
         "label": "Status and result",
     }
 
@@ -272,16 +266,13 @@ def create_work_request_status(work_request: WorkRequest) -> SidebarItem:
 def create_work_request_superseded(work_request: WorkRequest) -> SidebarItem:
     """Create a link to the work request that supersedes this one."""
     kwargs = {
-        "icon": "arrow-repeat",
+        "icon": Icons.WORK_REQUEST_SUPERSEDED,
         "label": "Superseded by",
     }
     if (superseded := getattr(work_request, "superseded", None)) is None:
         return SidebarElement(**kwargs)
     return SidebarAction(
-        url=reverse(
-            "work_requests:detail",
-            kwargs={"pk": superseded.id},
-        ),
+        url=superseded.get_absolute_url(),
         value=str(superseded),
         **kwargs,
     )
@@ -290,17 +281,14 @@ def create_work_request_superseded(work_request: WorkRequest) -> SidebarItem:
 def create_work_request_supersedes(work_request: WorkRequest) -> SidebarItem:
     """Create a link to the work request superseded by this one."""
     kwargs = {
-        "icon": "shuffle",
+        "icon": Icons.WORK_REQUEST_SUPERSEDES,
         "label": "Supersedes",
     }
     if (supersedes := getattr(work_request, "supersedes", None)) is None:
         return SidebarElement(**kwargs)
 
     return SidebarAction(
-        url=reverse(
-            "work_requests:detail",
-            kwargs={"pk": supersedes.id},
-        ),
+        url=supersedes.get_absolute_url(),
         value=str(supersedes),
         **kwargs,
     )
@@ -310,14 +298,14 @@ def create_workflow(workflow: WorkRequest | None) -> SidebarItem:
     """Create a link to the associated workflow."""
     if workflow is None:
         return SidebarElement(
-            icon="diagram-3",
+            icon=Icons.WORKFLOW,
             label="Workflow",
         )
 
     return SidebarAction(
         icon="diagram-3",
         label="Workflow",
-        url=reverse("work_requests:detail", kwargs={"pk": workflow.pk}),
+        url=workflow.get_absolute_url(),
         value=str(workflow),
     )
 
@@ -325,7 +313,7 @@ def create_workflow(workflow: WorkRequest | None) -> SidebarItem:
 def create_worker(worker: Worker | None) -> SidebarItem:
     """Create a link to a worker."""
     kwargs = {
-        "icon": "pc-horizontal",
+        "icon": Icons.WORKER,
         "label": "Worker",
     }
     if worker is None:
@@ -336,7 +324,7 @@ def create_worker(worker: Worker | None) -> SidebarItem:
 def create_work_request_started_at(work_request: WorkRequest) -> SidebarItem:
     """Show the time the work request spent in a queue before being started."""
     kwargs = {
-        "icon": "rocket-takeoff",
+        "icon": Icons.STARTED_AT,
         "label": "Time from creation to start",
     }
     if not work_request.started_at:
@@ -352,7 +340,7 @@ def create_work_request_started_at(work_request: WorkRequest) -> SidebarItem:
 def create_work_request_duration(work_request: WorkRequest) -> SidebarItem:
     """Show the duration of the work request run."""
     kwargs = {
-        "icon": "stopwatch",
+        "icon": Icons.DURATION,
         "label": "Duration",
     }
 
@@ -378,7 +366,7 @@ def create_work_request_duration(work_request: WorkRequest) -> SidebarItem:
 def create_artifact_category(artifact: Artifact) -> SidebarItem:
     """Show an artifact's category."""
     return SidebarElement(
-        icon="postcard",
+        icon=Icons.CATEGORY,
         label="Artifact category",
         detail=artifact.category,
         value=ARTIFACT_CATEGORY_SHORT_NAMES.get(
@@ -391,7 +379,7 @@ def create_collection(collection: Collection) -> SidebarItem:
     """Link to a collection."""
     return SidebarAction(
         url=collection.get_absolute_url(),
-        icon="collection",
+        icon=Icons.COLLECTION,
         label="Collection",
         detail=f"{collection.name}@{collection.category}",
         value=collection.name,
