@@ -16,16 +16,14 @@ from typing import ClassVar, NamedTuple, cast
 from unittest import mock
 
 from django.http.request import HttpHeaders
-from django.http.response import HttpResponseBase
+from django.http.response import HttpResponseBase, HttpResponseRedirectBase
+from django.template import engines
+from django.test import override_settings
 from django.utils.datastructures import CaseInsensitiveMapping
-
 from rest_framework import status
 
-from debusine.db.models import (
-    File,
-    FileInArtifact,
-    FileStore,
-)
+from debusine.db.context import context
+from debusine.db.models import File, FileInArtifact, FileStore
 from debusine.server.file_backend.interface import FileBackendInterface
 from debusine.server.views import ProblemResponse
 from debusine.test.django import TestCase
@@ -40,9 +38,23 @@ from debusine.web.views.files import (
 from debusine.web.views.tests.utils import ViewTestMixin
 
 
-class FileUITests(unittest.TestCase):
+class FileUITests(TestCase):
     """Tests for the FileUI class."""
 
+    def assertRenders(
+        self, file_ui: FileUI, file_in_artifact: FileInArtifact
+    ) -> None:
+        """Test that the given widget renders without errors."""
+        template = engines["django"].from_string(
+            "{% load debusine %}{% widget widget %}"
+        )
+        setattr(template, "engine", engines["django"])
+        widget = file_ui.widget_class(file_in_artifact, file_ui)
+        # Set DEBUG=True to have render raise exceptions instead of logging
+        # them
+        with override_settings(DEBUG=True):
+            template.render({"widget": widget})
+
     def test_from_file_in_artifact(self) -> None:
         """Test from_file_in_artifact."""
 
@@ -97,11 +109,16 @@ class FileUITests(unittest.TestCase):
 
         for sub_test in sub_tests:
             with self.subTest(sub_test.filename):
-                file_in_artifact = FileInArtifact(path=sub_test.filename)
-                file_in_artifact.file = File(size=sub_test.size)
+                with context.disable_permission_checks():
+                    artifact, _ = self.playground.create_artifact(
+                        paths={sub_test.filename: b"test"}, create_files=True
+                    )
+                file_in_artifact = FileInArtifact.objects.get(artifact=artifact)
+                file_in_artifact.file.size = sub_test.size
                 fileui = FileUI.from_file_in_artifact(file_in_artifact)
                 self.assertEqual(fileui.content_type, sub_test.content_type)
                 self.assertEqual(fileui.widget_class, sub_test.widget_class)
+                self.assertRenders(fileui, file_in_artifact)
 
     def test_compressed(self) -> None:
         """Test from_file_in_artifact with a compressed file."""
@@ -149,6 +166,7 @@ class FileDownloadMixinTests(ViewTestMixin, TestCase):
     file: ClassVar[FileInArtifact]
 
     @classmethod
+    @context.disable_permission_checks()
     def setUpTestData(cls) -> None:
         """Set up the common test fixture."""
         super().setUpTestData()
@@ -203,7 +221,7 @@ class FileDownloadMixinTests(ViewTestMixin, TestCase):
         range_start: int = 0,
         range_end: int | None = None,
         disposition: str = "attachment",
-    ):
+    ) -> None:
         """Assert that response has the expected headers and content."""
         if range_start == 0 and range_end is None:
             self.assertEqual(response.status_code, status.HTTP_200_OK)
@@ -237,7 +255,7 @@ class FileDownloadMixinTests(ViewTestMixin, TestCase):
         assert streaming_content is not None
         self.assertEqual(b"".join(streaming_content), expected_contents)
 
-    def test_get_file(self):
+    def test_get_file(self) -> None:
         """Get return the file."""
         response = self.get_stream_response(self.file)
         self.assertFileResponse(response, self.file)
@@ -245,7 +263,7 @@ class FileDownloadMixinTests(ViewTestMixin, TestCase):
             response.headers["content-type"], "text/markdown; charset=utf-8"
         )
 
-    def test_get_file_inline(self):
+    def test_get_file_inline(self) -> None:
         """Get return the file."""
         response = self.get_stream_response(self.file, download=False)
         self.assertFileResponse(response, self.file, disposition="inline")
@@ -253,7 +271,7 @@ class FileDownloadMixinTests(ViewTestMixin, TestCase):
             response.headers["content-type"], "text/markdown; charset=utf-8"
         )
 
-    def test_get_empty_file(self):
+    def test_get_empty_file(self) -> None:
         """Test empty downloadable file (which mmap doesn't support)."""
         response = self.get_stream_response(self.empty)
         self.assertFileResponse(response, self.empty)
@@ -262,7 +280,7 @@ class FileDownloadMixinTests(ViewTestMixin, TestCase):
             "application/octet-stream",
         )
 
-    def test_get_file_range(self):
+    def test_get_file_range(self) -> None:
         """Get return part of the file (based on Range header)."""
         start, end = 10, 20
         response = self.get_stream_response(
@@ -270,7 +288,7 @@ class FileDownloadMixinTests(ViewTestMixin, TestCase):
         )
         self.assertFileResponse(response, self.file, start, end)
 
-    def test_get_file_content_range_to_end_of_file(self):
+    def test_get_file_content_range_to_end_of_file(self) -> None:
         """Server returns a file from a position to the end."""
         start, end = 5, len(self.contents["file.md"]) - 1
         response = self.get_stream_response(
@@ -278,7 +296,7 @@ class FileDownloadMixinTests(ViewTestMixin, TestCase):
         )
         self.assertFileResponse(response, self.file, start, end)
 
-    def test_get_file_content_range_invalid(self):
+    def test_get_file_content_range_invalid(self) -> None:
         """Get return an error: Range header was invalid."""
         invalid_range_header = "invalid-range"
         response = self.get_stream_response(
@@ -288,7 +306,7 @@ class FileDownloadMixinTests(ViewTestMixin, TestCase):
             response, f'Invalid Range header: "{invalid_range_header}"'
         )
 
-    def test_get_file_range_start_greater_file_size(self):
+    def test_get_file_range_start_greater_file_size(self) -> None:
         """Get return 400: client requested an invalid start position."""
         file_size = len(self.contents["file.md"])
         start, end = file_size + 10, file_size + 20
@@ -300,7 +318,7 @@ class FileDownloadMixinTests(ViewTestMixin, TestCase):
             f"Invalid Content-Range start: {start}. " f"File size: {file_size}",
         )
 
-    def test_get_file_range_end_is_file_size(self):
+    def test_get_file_range_end_is_file_size(self) -> None:
         """Get return 400: client requested and invalid end position."""
         end = len(self.contents["file.md"])
         response = self.get_stream_response(self.file, range_header=(0, end))
@@ -309,7 +327,7 @@ class FileDownloadMixinTests(ViewTestMixin, TestCase):
             f"Invalid Content-Range end: {end}. File size: {end}",
         )
 
-    def test_get_file_range_end_greater_file_size(self):
+    def test_get_file_range_end_greater_file_size(self) -> None:
         """Get return 400: client requested an invalid end position."""
         file_size = len(self.contents["file.md"])
         end = file_size + 10
@@ -319,7 +337,7 @@ class FileDownloadMixinTests(ViewTestMixin, TestCase):
             f"Invalid Content-Range end: {end}. " f"File size: {file_size}",
         )
 
-    def test_get_file_url_redirect(self):
+    def test_get_file_url_redirect(self) -> None:
         """
         Get file response: redirect if get_url for the file is available.
 
@@ -340,6 +358,7 @@ class FileDownloadMixinTests(ViewTestMixin, TestCase):
         ):
             response = self.get_stream_response(self.file)
             self.assertEqual(response.status_code, status.HTTP_302_FOUND)
+            assert isinstance(response, HttpResponseRedirectBase)
             self.assertEqual(response.url, destination_url)
 
 
@@ -354,6 +373,7 @@ class FileWidgetTests(ViewTestMixin, TestCase):
     text: ClassVar[FileInArtifact]
 
     @classmethod
+    @context.disable_permission_checks()
     def setUpTestData(cls) -> None:
         """Set up the common test fixture."""
         super().setUpTestData()
