@@ -11,88 +11,119 @@
 import json
 import uuid
 from pathlib import Path
+from typing import ClassVar
 
+from django.http import HttpResponseBase
 from django.template.loader import get_template
-from django.test import TestCase
-from django.urls import reverse
 
 from debusine.artifacts import LintianArtifact, PackageBuildLog
-from debusine.db.models import Artifact, FileInArtifact, WorkRequest
+from debusine.db.context import context
+from debusine.db.models import Artifact, FileInArtifact, WorkRequest, Workspace
 from debusine.tasks.models import (
+    LintianData,
     LintianDynamicData,
+    LintianFailOnSeverity,
+    LintianInput,
     LookupMultiple,
     LookupSingle,
 )
-from debusine.test import TestHelpersMixin
+from debusine.test.django import TestCase
 from debusine.web.views.lintian import LintianView
 
 
-class LintianViewTests(TestHelpersMixin, TestCase):
+class LintianViewTests(TestCase):
     """Tests for LintianView."""
 
-    def setUp(self) -> None:
-        """Set up common objects."""
-        self.fail_on_severity = "warning"
+    fail_on_severity: ClassVar[LintianFailOnSeverity]
+    package_name: ClassVar[str]
+    workspace: ClassVar[Workspace]
+    binary_artifact: ClassVar[Artifact]
+    source_artifact: ClassVar[Artifact]
+    work_request: ClassVar[WorkRequest]
+    package_source_file: ClassVar[str]
+    package_binary_file_1: ClassVar[str]
+    package_binary_file_2: ClassVar[str]
 
-        source_artifact, _ = self.create_artifact(
-            paths=["hello.dsc"], create_files=True
-        )
-        binary_artifact_1, _ = self.create_artifact(
-            paths=["hello-1.deb"], create_files=True
-        )
-        binary_artifact_2, _ = self.create_artifact(
-            paths=["hello-2.deb"], create_files=True
+    @classmethod
+    @context.disable_permission_checks()
+    def setUpTestData(cls) -> None:
+        """Set up common data for tests."""
+        super().setUpTestData()
+        cls.fail_on_severity = LintianFailOnSeverity.WARNING
+        cls.workspace = cls.playground.create_workspace(
+            name="Public", public=True
         )
 
-        self.work_request: WorkRequest = self.create_work_request(
+        source_artifact, _ = cls.create_artifact(
+            paths=["hello.dsc"],
+            create_files=True,
+            workspace=cls.workspace,
+        )
+        binary_artifact_1, _ = cls.create_artifact(
+            paths=["hello-1.deb"],
+            create_files=True,
+            workspace=cls.workspace,
+        )
+        binary_artifact_2, _ = cls.create_artifact(
+            paths=["hello-2.deb"],
+            create_files=True,
+            workspace=cls.workspace,
+        )
+
+        cls.work_request = cls.playground.create_work_request(
             result=WorkRequest.Results.SUCCESS,
-            task_data={
-                "input": {
-                    "source_artifact": source_artifact.id,
-                    "binary_artifacts": [
-                        binary_artifact_1.id,
-                        binary_artifact_2.id,
-                    ],
-                },
-                "fail_on_severity": self.fail_on_severity,
-            },
+            task_data=LintianData(
+                input=LintianInput(
+                    source_artifact=source_artifact.id,
+                    binary_artifacts=LookupMultiple.parse_obj(
+                        [binary_artifact_1.id, binary_artifact_2.id]
+                    ),
+                ),
+                fail_on_severity=cls.fail_on_severity,
+            ),
             task_name="lintian",
+            workspace=cls.workspace,
         )
-        self.package_name = "hello"
+        cls.package_name = "hello"
 
-        self.package_source_file = "hello_2.10-3.dsc"
-        self.package_binary_file_1 = "hello_2.10-3.deb"
-        self.package_binary_file_2 = "libhello_2.10-3.deb"
+        cls.package_source_file = "hello_2.10-3.dsc"
+        cls.package_binary_file_1 = "hello_2.10-3.deb"
+        cls.package_binary_file_2 = "libhello_2.10-3.deb"
         summary_binary_package = {
             "summary": {
                 "package_filename": {
-                    "hello": self.package_binary_file_1,
-                    "libhello": self.package_binary_file_2,
+                    "hello": cls.package_binary_file_1,
+                    "libhello": cls.package_binary_file_2,
                 }
             }
         }
-        self.binary_artifact, _ = self.create_artifact(
-            paths=["summary.json", "lintian.txt"], create_files=True
+        cls.binary_artifact, _ = cls.create_artifact(
+            paths=["summary.json", "lintian.txt"],
+            create_files=True,
+            workspace=cls.workspace,
         )
-        self.binary_artifact.category = LintianArtifact._category
-        self.binary_artifact.data = summary_binary_package
-        self.binary_artifact.created_by_work_request = self.work_request
-        self.binary_artifact.save()
+        cls.binary_artifact.category = LintianArtifact._category
+        cls.binary_artifact.data = summary_binary_package
+        cls.binary_artifact.created_by_work_request = cls.work_request
+        cls.binary_artifact.save()
 
         summary_source_package = {
-            "summary": {"package_filename": {"hello": self.package_source_file}}
+            "summary": {"package_filename": {"hello": cls.package_source_file}}
         }
-        self.source_artifact, _ = self.create_artifact(
+        cls.source_artifact, _ = cls.create_artifact(
             paths=["lintian.txt"],
             create_files=True,
+            workspace=cls.workspace,
         )
-        self.source_artifact.category = LintianArtifact._category
-        self.source_artifact.data = summary_source_package
-        self.source_artifact.created_by_work_request = self.work_request
-        self.source_artifact.save()
+        cls.source_artifact.category = LintianArtifact._category
+        cls.source_artifact.data = summary_source_package
+        cls.source_artifact.created_by_work_request = cls.work_request
+        cls.source_artifact.save()
 
-        build_log_artifact, _ = self.create_artifact(
-            paths=["some-file.log"], create_files=True
+        build_log_artifact, _ = cls.create_artifact(
+            paths=["some-file.log"],
+            create_files=True,
+            workspace=cls.workspace,
         )
         build_log_artifact.category = PackageBuildLog._category
         build_log_artifact.save()
@@ -154,18 +185,13 @@ class LintianViewTests(TestHelpersMixin, TestCase):
         ).encode("utf-8")
         return self.create_temporary_file(contents=contents)
 
-    def test_get_context_data(self):
+    def test_get_context_data(self) -> None:
         """Test get_context_data() return the expected context data."""
         lintian_txt_path = (
-            reverse(
-                "artifacts:download",
-                kwargs={"artifact_id": self.binary_artifact.id},
-            )
-            + "lintian.txt"
+            self.binary_artifact.get_absolute_url_download() + "lintian.txt"
         )
-        lintian_view = LintianView()
-
-        context_data = lintian_view.get_context_data(self.work_request)
+        lintian_view = LintianView(self.work_request)
+        context_data = lintian_view.get_context_data()
 
         # Verify "tags" and "tags_count_severity" in separate methods
         context_data.pop("tags")
@@ -181,7 +207,7 @@ class LintianViewTests(TestHelpersMixin, TestCase):
             },
         )
 
-    def test_get_context_data_verify_tag_count_severity(self):
+    def test_get_context_data_verify_tag_count_severity(self) -> None:
         """Test get_context_data(): verify "tags_count_severity"."""
         self.binary_artifact.data["summary"] = {
             "package_filename": {"hello": "hello_2.10-3.deb"},
@@ -211,9 +237,8 @@ class LintianViewTests(TestHelpersMixin, TestCase):
         }
         self.source_artifact.save()
 
-        lintian_view = LintianView()
-
-        context_data = lintian_view.get_context_data(self.work_request)
+        lintian_view = LintianView(self.work_request)
+        context_data = lintian_view.get_context_data()
 
         self.assertEqual(
             context_data["tags_count_severity"],
@@ -228,7 +253,7 @@ class LintianViewTests(TestHelpersMixin, TestCase):
             },
         )
 
-    def test_get_context_data_verify_tags(self):
+    def test_get_context_data_verify_tags(self) -> None:
         """Test get_context_data(): verify "tags"."""
         self.add_file_in_artifact(
             self.binary_artifact,
@@ -236,9 +261,8 @@ class LintianViewTests(TestHelpersMixin, TestCase):
             self.create_analysis().read_bytes(),
         )
 
-        lintian_view = LintianView()
-
-        context_data = lintian_view.get_context_data(self.work_request)
+        lintian_view = LintianView(self.work_request)
+        context_data = lintian_view.get_context_data()
 
         expected = {
             "hello_2.10-3.dsc": {
@@ -287,7 +311,7 @@ class LintianViewTests(TestHelpersMixin, TestCase):
 
         self.assertEqual(actual, expected)
 
-    def test_get_context_data_verify_request(self):
+    def test_get_context_data_verify_request(self) -> None:
         """Test get_context_data(): verify "request_data"."""
         task_data_input = self.work_request.task_data["input"]
         self.work_request.dynamic_task_data = LintianDynamicData(
@@ -296,15 +320,17 @@ class LintianViewTests(TestHelpersMixin, TestCase):
         ).dict(exclude_unset=True)
         self.work_request.save()
 
-        lintian_view = LintianView()
-
-        context_data = lintian_view.get_context_data(self.work_request)
+        lintian_view = LintianView(self.work_request)
+        context_data = lintian_view.get_context_data()
 
         expected = {
             "source_artifact": {
                 "files": ["hello.dsc"],
                 "id": task_data_input["source_artifact"],
                 "lookup": task_data_input["source_artifact"],
+                "artifact": Artifact.objects.get(
+                    id=task_data_input["source_artifact"]
+                ),
             },
             "binary_artifacts": {
                 "lookup": task_data_input["binary_artifacts"],
@@ -312,10 +338,16 @@ class LintianViewTests(TestHelpersMixin, TestCase):
                     {
                         "files": ["hello-1.deb"],
                         "id": task_data_input["binary_artifacts"][0],
+                        "artifact": Artifact.objects.get(
+                            id=task_data_input["binary_artifacts"][0]
+                        ),
                     },
                     {
                         "files": ["hello-2.deb"],
                         "id": task_data_input["binary_artifacts"][1],
+                        "artifact": Artifact.objects.get(
+                            id=task_data_input["binary_artifacts"][1]
+                        ),
                     },
                 ],
             },
@@ -323,13 +355,12 @@ class LintianViewTests(TestHelpersMixin, TestCase):
 
         self.assertEqual(context_data["request_data"], expected)
 
-    def test_get_context_data_no_dynamic_task_data(self):
+    def test_get_context_data_no_dynamic_task_data(self) -> None:
         """If there is no dynamic data, context data omits artifact info."""
         task_data_input = self.work_request.task_data["input"]
 
-        lintian_view = LintianView()
-
-        context_data = lintian_view.get_context_data(self.work_request)
+        lintian_view = LintianView(self.work_request)
+        context_data = lintian_view.get_context_data()
 
         self.assertEqual(
             context_data["request_data"]["source_artifact"],
@@ -344,7 +375,7 @@ class LintianViewTests(TestHelpersMixin, TestCase):
             {"lookup": task_data_input["binary_artifacts"], "artifacts": []},
         )
 
-    def test_get_context_data_missing_source_artifact(self):
+    def test_get_context_data_missing_source_artifact(self) -> None:
         """If the source artifact is missing, context data omits its files."""
         task_data_input = self.work_request.task_data["input"]
         self.work_request.dynamic_task_data = LintianDynamicData(
@@ -356,16 +387,15 @@ class LintianViewTests(TestHelpersMixin, TestCase):
         source_artifact = Artifact.objects.get(id=source_artifact_id)
         source_artifact.files.clear()
         source_artifact.delete()
-        lintian_view = LintianView()
-
-        context_data = lintian_view.get_context_data(self.work_request)
+        lintian_view = LintianView(self.work_request)
+        context_data = lintian_view.get_context_data()
 
         self.assertEqual(
             context_data["request_data"]["source_artifact"],
             {"lookup": source_artifact_id, "id": None, "files": []},
         )
 
-    def test_get_context_data_missing_binary_artifact(self):
+    def test_get_context_data_missing_binary_artifact(self) -> None:
         """If a binary artifact is missing, context data omits its files."""
         task_data_input = self.work_request.task_data["input"]
         self.work_request.dynamic_task_data = LintianDynamicData(
@@ -377,19 +407,25 @@ class LintianViewTests(TestHelpersMixin, TestCase):
         binary_artifact_0 = Artifact.objects.get(id=binary_artifact_0_id)
         binary_artifact_0.files.clear()
         binary_artifact_0.delete()
-        lintian_view = LintianView()
-
-        context_data = lintian_view.get_context_data(self.work_request)
+        lintian_view = LintianView(self.work_request)
+        context_data = lintian_view.get_context_data()
 
         self.assertEqual(
             context_data["request_data"]["binary_artifacts"],
             {
                 "lookup": task_data_input["binary_artifacts"],
                 "artifacts": [
-                    {"files": [], "id": task_data_input["binary_artifacts"][0]},
+                    {
+                        "files": [],
+                        "id": task_data_input["binary_artifacts"][0],
+                        "artifact": None,
+                    },
                     {
                         "files": ["hello-2.deb"],
                         "id": task_data_input["binary_artifacts"][1],
+                        "artifact": Artifact.objects.get(
+                            id=task_data_input["binary_artifacts"][1]
+                        ),
                     },
                 ],
             },
@@ -409,13 +445,10 @@ class LintianViewTests(TestHelpersMixin, TestCase):
             self.create_analysis().read_bytes(),
         )
 
-        response = self.client.get(
-            reverse("work-requests:detail", kwargs={"pk": self.work_request.id})
-        )
+        response = self.client.get(self.work_request.get_absolute_url())
 
         work_request_generic_path = (
-            reverse("work_requests:detail", kwargs={"pk": self.work_request.id})
-            + "?view=generic"
+            self.work_request.get_absolute_url() + "?view=generic"
         )
         self.assertContains(
             response,
@@ -431,11 +464,7 @@ class LintianViewTests(TestHelpersMixin, TestCase):
         )
 
         lintian_txt_path = (
-            reverse(
-                "artifacts:download",
-                kwargs={"artifact_id": self.binary_artifact.id},
-            )
-            + "lintian.txt"
+            self.binary_artifact.get_absolute_url_download() + "lintian.txt"
         )
         self.assertContains(
             response,
@@ -481,20 +510,17 @@ class LintianViewTests(TestHelpersMixin, TestCase):
 
     def assert_contains_artifact_information(
         self,
-        response,
+        response: HttpResponseBase,
         description: str,
         artifact_lookup: LookupSingle,
         artifact_id: int,
-    ):
+    ) -> None:
         """Assert contains information of an artifact: link and files."""
-        files = (
-            Artifact.objects.get(id=artifact_id)
-            .fileinartifact_set.order_by("path")
-            .values_list("path", flat=True)
-        )
-        artifact_path = reverse(
-            "artifacts:detail", kwargs={"artifact_id": artifact_id}
+        artifact = Artifact.objects.get(id=artifact_id)
+        files = artifact.fileinartifact_set.order_by("path").values_list(
+            "path", flat=True
         )
+        artifact_path = artifact.get_absolute_url()
         artifact_link = f'<a href="{artifact_path}">#{artifact_id}</a>'
         files_li = "".join([f"<li>{file}</li>" for file in files])
         self.assertContains(
@@ -507,31 +533,28 @@ class LintianViewTests(TestHelpersMixin, TestCase):
 
     def assert_contains_artifacts_information(
         self,
-        response,
+        response: HttpResponseBase,
         description: str,
         artifact_lookup: LookupMultiple,
         artifact_ids: list[int],
-    ):
+    ) -> None:
         """Assert contains information of a set of artifacts."""
         expected_response = (
             f"<li>{description} ({artifact_lookup.export()})<ul>"
         )
         for artifact_id in artifact_ids:
-            files = (
-                Artifact.objects.get(id=artifact_id)
-                .fileinartifact_set.order_by("path")
-                .values_list("path", flat=True)
-            )
-            artifact_path = reverse(
-                "artifacts:detail", kwargs={"artifact_id": artifact_id}
+            artifact = Artifact.objects.get(id=artifact_id)
+            files = artifact.fileinartifact_set.order_by("path").values_list(
+                "path", flat=True
             )
+            artifact_path = artifact.get_absolute_url()
             artifact_link = f'<a href="{artifact_path}">#{artifact_id}</a>'
             files_li = "".join([f"<li>{file}</li>" for file in files])
             expected_response += f"<li>{artifact_link}<ul>{files_li}</ul></li>"
         expected_response += "</ul></li>"
         self.assertContains(response, expected_response, html=True)
 
-    def test_template_output_include_note_pointer_comment(self):
+    def test_template_output_include_note_pointer_comment(self) -> None:
         """Template output of _lintian_tag.html include all the information."""
         tag = {
             "note": "invariant part is:\nwith no invariant",
@@ -546,7 +569,9 @@ class LintianViewTests(TestHelpersMixin, TestCase):
         self.assertIn("Comment:", rendered)
         self.assertIn(f"<pre>{tag['comment']}</pre>", rendered)
 
-    def test_template_output_do_not_include_non_needed_information(self):
+    def test_template_output_do_not_include_non_needed_information(
+        self,
+    ) -> None:
         """Template _lintian_tag.html does not output note, comment, pointer."""
         tag = {"note": "", "pointer": "", "comment": ""}
         rendered = get_template("web/_lintian_tag.html").render({"tag": tag})
@@ -556,10 +581,7 @@ class LintianViewTests(TestHelpersMixin, TestCase):
         self.assertNotIn("Comment:", rendered)
         self.assertNotIn("<pre></pre>", rendered)
 
-    def test_template_output_lintian_tag_include_pointer(self):
-        """."""
-
-    def test_template_output_no_source_artifact(self):
+    def test_template_output_no_source_artifact(self) -> None:
         """The view renders OK if there was no source artifact in the input."""
         del self.work_request.task_data["input"]["source_artifact"]
         self.work_request.dynamic_task_data = LintianDynamicData(
@@ -569,13 +591,11 @@ class LintianViewTests(TestHelpersMixin, TestCase):
         ).dict(exclude_unset=True)
         self.work_request.save()
 
-        response = self.client.get(
-            reverse("work-requests:detail", kwargs={"pk": self.work_request.id})
-        )
+        response = self.client.get(self.work_request.get_absolute_url())
 
         self.assertNotContains(response, "Source artifact", html=True)
 
-    def test_template_output_no_binary_artifacts(self):
+    def test_template_output_no_binary_artifacts(self) -> None:
         """The view renders OK if there was no binary artifact in the input."""
         del self.work_request.task_data["input"]["binary_artifacts"]
         self.work_request.dynamic_task_data = LintianDynamicData().dict(
@@ -583,29 +603,24 @@ class LintianViewTests(TestHelpersMixin, TestCase):
         )
         self.work_request.save()
 
-        response = self.client.get(
-            reverse("work-requests:detail", kwargs={"pk": self.work_request.id})
-        )
-
+        response = self.client.get(self.work_request.get_absolute_url())
         self.assertNotContains(response, "Binary artifact", html=True)
 
-    def test_find_lintian_txt_url_path_no_artifacts(self):
+    def test_find_lintian_txt_url_path_no_artifacts(self) -> None:
         """_find_lintian_txt_url_path() with no artifacts: return None."""
         self.assertIsNone(LintianView._find_lintian_txt_url_path([]))
 
-    def test_find_lintian_txt_url_path_artifact_no_files(self):
+    def test_find_lintian_txt_url_path_artifact_no_files(self) -> None:
         """_find_lintian_txt_url_path(), artifact no files: return None."""
-        artifact, _ = self.create_artifact()
+        with context.disable_permission_checks():
+            artifact, _ = self.create_artifact()
 
         self.assertIsNone(LintianView._find_lintian_txt_url_path([artifact]))
 
-    def test_template_output_no_lintian_txt(self):
+    def test_template_output_no_lintian_txt(self) -> None:
         """The view renders OK if no lintian.txt was output."""
         self.source_artifact.fileinartifact_set.all().delete()
         self.binary_artifact.fileinartifact_set.all().delete()
 
-        response = self.client.get(
-            reverse("work-requests:detail", kwargs={"pk": self.work_request.id})
-        )
-
+        response = self.client.get(self.work_request.get_absolute_url())
         self.assertNotContains(response, "Lintian output", html=True)
