@@ -16,12 +16,13 @@ import pygments.formatters
 import pygments.lexers
 import yaml
 from django.utils.safestring import SafeString
+from django.views.generic.base import ContextMixin
 
 
-class PaginationMixin:
+class PaginationMixin(ContextMixin):
     """Add elided_page_range in "context" via get_context_data()."""
 
-    def get_context_data(self, *args, **kwargs):
+    def get_context_data(self, *args: Any, **kwargs: Any) -> dict[str, Any]:
         """Return super().get_context_data with elided_page_range."""
         context = super().get_context_data(*args, **kwargs)
 
@@ -36,7 +37,9 @@ class PaginationMixin:
 class UIDumper(yaml.SafeDumper):
     """A YAML dumper that represents multi-line strings in the literal style."""
 
-    def represent_scalar(self, tag, value, style=None):
+    def represent_scalar(
+        self, tag: str, value: Any, style: str | None = None
+    ) -> yaml.ScalarNode:
         """Represent multi-line strings in the literal style."""
         if style is None and "\n" in value:
             style = "|"
