@@ -67,10 +67,13 @@ from debusine.client.exceptions import TokenDisabledError
 from debusine.client.models import WorkRequestResponse
 from debusine.tasks import BaseExternalTask, BaseTask, TaskConfigError
 from debusine.tasks.executors import analyze_worker_all_executors
-from debusine.tasks.models import TaskTypes
+from debusine.tasks.models import TaskTypes, WorkerType
 from debusine.worker.config import ConfigHandler
 from debusine.worker.debusine_async_http_client import DebusineAsyncHttpClient
-from debusine.worker.system_information import WorkerType, system_metadata
+from debusine.worker.system_information import (
+    host_architecture,
+    system_metadata,
+)
 
 
 class Worker:
@@ -535,6 +538,7 @@ class Worker:
         )
         self._task_running.work_request_id = work_request.id
         self._task_running.workspace_name = work_request.workspace
+        self._task_running.worker_host_architecture = host_architecture()
 
         self._task_name = self._task_running.name
 
@@ -563,7 +567,9 @@ class Worker:
         """Exit current thread, mocked in tests."""
         raise SystemExit(1)
 
-    def _send_task_result(self, work_request_id: int, task_exec_future) -> None:
+    def _send_task_result(
+        self, work_request_id: int, task_exec_future: Future[bool]
+    ) -> None:
         """
         Send the result of the task to the debusine server.
 
