@@ -67,7 +67,7 @@ class ConfigHandler(ConfigParser):
             self._fail(f'Cannot read {config_filename}')
 
     @staticmethod
-    def _fail(message) -> NoReturn:
+    def _fail(message: str) -> NoReturn:
         """
         Print message and fail (exits) the execution of the Worker.
 
