@@ -8,11 +8,10 @@
 # contained in the LICENSE file.
 
 """Tests DebusineAsyncHttpClient."""
-import asyncio
+import asyncio.log
 import itertools
 import logging
 
-
 from debusine.test import TestCase
 from debusine.worker.config import ConfigHandler
 from debusine.worker.debusine_async_http_client import DebusineAsyncHttpClient
@@ -22,7 +21,7 @@ from debusine.worker.tests import server
 class TestDebusineAsyncHttpClient(server.DebusineAioHTTPTestCase, TestCase):
     """Tests for DebusineAsyncHttpClient."""
 
-    async def setUpAsync(self):
+    async def setUpAsync(self) -> None:
         """Set up tests."""
         # asyncio/base_events.py print "<some information> is serving" in the
         # debug log. Unit tests can be executed with environment variables:
@@ -37,11 +36,9 @@ class TestDebusineAsyncHttpClient(server.DebusineAioHTTPTestCase, TestCase):
 
         self.config = self.build_basic_config()
 
-        self.debusine_async_http_client = DebusineAsyncHttpClient(
-            logging, self.config
-        )
+        self.debusine_async_http_client = DebusineAsyncHttpClient(self.config)
 
-    async def tearDownAsync(self):
+    async def tearDownAsync(self) -> None:
         """Tear down."""
         await super().tearDownAsync()
         await self.debusine_async_http_client.close()
@@ -61,7 +58,7 @@ class TestDebusineAsyncHttpClient(server.DebusineAioHTTPTestCase, TestCase):
 
         return config
 
-    def setup_valid_token(self, include_token: bool):
+    def setup_valid_token(self, include_token: bool) -> None:
         """Set a valid token in the test server and client configuration."""
         self.config = self.build_basic_config()
 
@@ -74,14 +71,14 @@ class TestDebusineAsyncHttpClient(server.DebusineAioHTTPTestCase, TestCase):
 
         self.debusine_async_http_client._config = self.config
 
-    async def test_make_http_request_raise_path_must_be_absolute(self):
+    async def test_make_http_request_raise_path_must_be_absolute(self) -> None:
         """Worker._make_http_request raise exception for relative paths."""
         with self.assertRaisesRegex(ValueError, 'Path must be absolute'):
             self.debusine_async_http_client._make_http_request(
                 'PUT', 'api/endpoint'
             )
 
-    async def test_make_http_request_raise_invalid_method(self):
+    async def test_make_http_request_raise_invalid_method(self) -> None:
         """Worker._make_http_request raise exception for unknown HTTP method."""
         with self.assertRaisesRegex(
             ValueError, '^Method must be one of: PUT, POST, GET$'
@@ -111,8 +108,12 @@ class TestDebusineAsyncHttpClient(server.DebusineAioHTTPTestCase, TestCase):
         return data
 
     async def assert_http_request(
-        self, method: str, path: str, include_token: bool, data: dict[str, str]
-    ):
+        self,
+        method: str,
+        path: str,
+        include_token: bool,
+        data: dict[str, str] | None,
+    ) -> None:
         """
         Assert that Worker._make_http_request sends the data and token.
 
@@ -139,7 +140,7 @@ class TestDebusineAsyncHttpClient(server.DebusineAioHTTPTestCase, TestCase):
         # Path that the request was made is the path that it was received
         self.assertEqual(self.server_latest_request.path, path)
 
-    async def test_make_http_request(self):
+    async def test_make_http_request(self) -> None:
         """Worker._make_http_request makes the request without token."""
         path = '/collect_request_information/'
 
@@ -153,7 +154,9 @@ class TestDebusineAsyncHttpClient(server.DebusineAioHTTPTestCase, TestCase):
                     method, path, include_token=include_token, data=data
                 )
 
-    async def test_make_http_request_get_with_data_raises_value_error(self):
+    async def test_make_http_request_get_with_data_raises_value_error(
+        self,
+    ) -> None:
         """_make_http_request raises ValueError if data passed in HTTP GET."""
         with self.assertRaisesRegex(
             ValueError, '^data not allowed in GET requests$'
