@@ -9,17 +9,17 @@
 
 """Tests for the system_information."""
 import subprocess
-from unittest import TestCase, mock
+from unittest import mock
 from unittest.mock import MagicMock
 
 import psutil
 
 from debusine.tasks.models import WorkerType
-from debusine.test import TestHelpersMixin
+from debusine.test import TestCase
 from debusine.worker import system_information
 
 
-class TestSystemInformation(TestHelpersMixin, TestCase):
+class TestSystemInformation(TestCase):
     """Test for system_information.py functions."""
 
     def setUp(self) -> None:
@@ -51,6 +51,11 @@ class TestSystemInformation(TestHelpersMixin, TestCase):
         # Check that it matches the underlying implementation
         self.assertEqual(cpu_count, psutil.cpu_count())
 
+    def test_total_cpu_count_undefined(self) -> None:
+        """Test cpu_count function when undefined."""
+        with mock.patch("psutil.cpu_count", return_value=None):
+            self.assertEqual(system_information.cpu_count(), 1)
+
     def test_host_architecture(self) -> None:
         """Test host_architecture() function."""
         architecture = system_information.host_architecture()
